#  BEEN: Benchmarking Environment
#  ==============================
#
#  File author: Jakub Lehotsky
#
#  GNU Lesser General Public License Version 2.1
#  ---------------------------------------------
#  Copyright (C) 2004-2006 Distributed Systems Research Group,
#  Faculty of Mathematics and Physics, Charles University in Prague
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License version 2.1, as published by the Free Software Foundation.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA  02111-1307  USA

# Note: these functions are documented in .hlp files, that come together with whole R package, and
#		can be browsed via BEEN UI. See BEEN web packages for more R packages to download


# basic R functions for drawing multi-colored graphs (general)


# Draws one colored graph and stores it into the file
#
# graphsListFilename - file of list of graphs, where to add it
# graphNumber - number of the graph (should be unique from previously added graphs)
# graphDirectory - directory of entity, when graph will be put
# graphFilename - filename of .png (again, must be unique for a given entity)
# values - vector of values to display
# xCount - count of values (on x axis)
# graphTitle - title of graph to be displayed in GUI
# vertLabel - vertical label
# horLabel - horizontal label
# color - string describing color ("red", "blue", etc...)

base.generateOneColoredGraph <- function (graphsListFilename, graphNumber, graphDirectory, graphFilename,
	 values, graphTitle = "", vertLabel = "", horLabel = "", grWidth = 550, grHeight = 360, color = "black", graphLabels = "",
	 regression = FALSE) {

		cat(c(graphNumber, graphTitle, grWidth, grHeight, graphFilename, "\n"), 
			file=graphsListFilename, sep = "\t", append = TRUE);

		fname <- paste(graphDirectory, "/", graphFilename, sep="");
		
		if (file.exists(graphFilename)) {
			file.remove(graphFilename);	
		}
		
		png(fname, width = grWidth, height=grHeight);

		if (regression) {
			par(mar = c(13, 5, 2, 2) + 0.7);
			par(mgp = c(10.3, 1, 0));
		} else {
				par(mar = c(3, 5, 1, 1) + 0.7);
		}
	
		plot(values, type="o", pch=20, xlab = vertLabel, 
			ylab=horLabel, axes=FALSE, col = color);
			
		axis(1, at=c(1:length(values)), labels = graphLabels, las = 2);
		axis(2, las = 1);		
		
		box();

		dev.off();
}

# Draws multi colored graph and stores it into the file
#
# graphsListFilename - file of list of graphs, where to add it
# graphNumber - number of the graph (should be unique from previously added graphs)
# graphDirectory - directory of entity, when graph will be put
# graphFilename - filename of .png (again, must be unique for a given entity)
# values - matrix of values to display
# graphTitle - title of graph to be displayed in GUI
# vertLabel - vertical label
# horLabel - horizontal label
# colors - vector of strings describing color ("red", "blue", etc...), or defaults are used
# legends - vector of legends to be displayed

base.generateMultiColoredGraph <- function (graphsListFilename, graphNumber, graphDirectory, 
	graphFilename, values, graphLabels = "", graphTitle = "", vertLabel = "", horLabel = "", 
	height = 360, colors = "", legends = "") {
	 
	 	if (colors == "") {
	 		color = c("red", "blue", "green", "yellow", "cyan", "magenta", "brown", "darkblue",
	 			"grey", "pink", "azure", sample(colors()));
	 	} else {
	 		color = colors;
	 	}
	 	
	 	legcol <- 1;
	 	if (length(legends) > 4) {
	 		legcol = ceiling(length(legends)/20);
			legrow = length(legends)/legcol;
		 	graphHeight = floor(height + 25 * (legrow - 4) + 5 * length(legends));
		} else {
			graphHeight = height;
		}
	 	
		cat(c(graphNumber, graphTitle, 550, graphHeight, graphFilename, "\n"),
			file=graphsListFilename, sep = "\t", append = TRUE);

		fname <- paste(graphDirectory, "/", graphFilename, sep="");

		if (file.exists(graphFilename)) {
			file.remove(graphFilename);
		}

		png(fname, width=550, height=graphHeight);
		par(mar = c(3, 4, 1, 1) + 0.7);

		x <- range(c(values));
		x[1] = x[1] * 0.9;
		x[2] = x[2] * 1.2;

		plot(values[,1], xlab = vertLabel, ylab=horLabel, type="o", pch=20, col = color[1], 
			ylim = c(x[1],x[2]), xlim = c(1, dim(values)[1]), axes=FALSE);

		axis(1, at=c(1:dim(values)[1]), labels = graphLabels, las = 2);
		axis(2, las = 1);

		box();

		for (graphIdx in 2:dim(values)[2]) {
			points(values[,graphIdx], type="o", pch=20, col = color[graphIdx])
		}
		
		legend("top", ncol = legcol, legends, lty=1:2, pch=c(20,20), col=color);
		dev.off();
		
}
