/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map.Entry;

import cz.cuni.mff.been.benchmarkmanager.gui.SchedulerInfo;
import cz.cuni.mff.been.benchmarkmanager.plugins.TaskGenerator;
import cz.cuni.mff.been.benchmarkmanager.plugins.VersionProvider;
import cz.cuni.mff.been.benchmarkmanager.scheduler.PastDateException;
import cz.cuni.mff.been.benchmarkmanager.scheduler.ScheduledJobListener;
import cz.cuni.mff.been.benchmarkmanager.scheduler.Scheduler;
import cz.cuni.mff.been.benchmarkmanager.scheduler.SchedulerEntry;
import cz.cuni.mff.been.common.WorkQueue;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Delete;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.PEID;
import cz.cuni.mff.been.hostmanager.HostManagerInterface;
import cz.cuni.mff.been.hostmanager.HostManagerService;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryService;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.data.TaskState;

public class AnalysisManager implements ScheduledJobListener  {

	/** Interval in which new experiments is scanned for */
	private static final long EXPERIMENT_START_INTERVAL = 20*1000; // millis
	
	/** Interval for checking experiment status */
	private static final long EXPERIMENT_MONITOR_INTERVAL = 10*1000; // millis

	/** Maximum of regression experiments that may run in paralell for a single analysis */
	private static final int MAX_RUNNING_EXPERIMENTS = 2;
	
	/** Root under which all persistency-related data will be stored */
	private static final String RESTORE_SUBDIRECTORY = "analysismanager";  
	
	private class GeneratorThread extends Thread {
		
		public GeneratorThread() {
			super("GeneratorThread");
			start();
		}
		
		@Override
		public void run() {
			while (true) {
				ExperimentMetadata experiment = scheduledExperiments.getWork();
				try {
					/*
					 * 0. metadata of configured experiment are stored
					 * 1. start generator
					 * 2. rewrite metadata so that tasks are stored as well
					 * 3. queue experiment
					 * 
					 * We can't do more to inform user when experiment fails, because
					 * this is being done offline so we update the logs at least
					 */
					bm.logInfo("Starting generator for experiment " 
							+ EID.toNamedString(experiment.getID()));
					startGenerator(experiment);
					reportResultsMetadata(experiment);
					bm.logInfo("Generator finished");
					dispatchExperiment(experiment);
				} catch (BenchmarkManagerException e) {
					e.printStackTrace();
					bm.logError("Experiment could not be dispatched " 
							+ EID.toNamedString(experiment.getID()) + ": " + e.getMessage());
				} 
			}
		}
		
	}
	
	private class DispatcherThread extends Thread {
		private HashMap<AID,Integer> running = new HashMap<AID,Integer>();
		
		public DispatcherThread() {
			super("DispatcherThread");
			start();
		}
		
		/*
		 * Refreshes list of currently running experiments/analyses 
		 * from monitor thread queue
		 */
		private void refreshRunningAnalyses() {
			running.clear();
			for (ExperimentMetadata exp : monitorQueue){
				updateRunCount(exp.getAnalysis());
			}
		}
		
		private void updateRunCount(AID aid) {
			if (running.containsKey(aid)) {
				running.put(aid,running.get(aid)+1);
			} else {
				running.put(aid,1);
			}
		}
		
		/*
		 * Determines whether an experiment may be started for analyses
		 * Comparison experiments are created by user so we will start them ASAP
		 * as user is eager to see them running.
		 * For regression experiment there is a defined limit, because they will
		 * be colliding in benchmarking hosts so there is no good reason to run
		 * a lot of them in parallel.
		 */
		private boolean canStart(ExperimentMetadata exp) {
			if (! getAnalysis(exp.getAnalysis()).isRegression()) {
				// comparison - run asap
				return true;
			} else {
				/*
				 * regression - check if not too many experiments are running
				 */
				Integer runningCount = running.get(exp.getAnalysis());
				if (runningCount == null || runningCount < MAX_RUNNING_EXPERIMENTS) {
					return true;
				}
				
				return false;
					
			}
		}
		
		@Override
		public void run() {
			boolean tmNotAvailable = false;
						
			while (true) {
				try {
				// refresh list of running analyses from monitor queue
				refreshRunningAnalyses();
				// go over entries starting experiments
				for (Entry<AID,List<ExperimentMetadata>> entry : analysisQueues.entrySet()) {
					
					// check if we have some experiments to run
					if (entry.getValue().isEmpty()) {
						continue;
					}
					
					// fire found experiments
					for (ListIterator<ExperimentMetadata> iter = entry.getValue().listIterator();  iter.hasNext(); ) {
						try {
							ExperimentMetadata exp = iter.next();
							if (canStart(exp)) {
								bm.logInfo("Starting experiment " + EID.toNamedString(exp.getID()) + ", analysis " + AID.toNamedString(exp.getAnalysis()) 
										+": " +  exp.getAllTasks().length + " tasks");
								bm.runExperiment(exp);
								exp.setStatus(EntityStatus.SUBMITTED);
								storeExperiment(exp);
								iter.remove();
								monitorQueue.add(exp);
								updateRunCount(exp.getAnalysis());
							}
						} catch (BenchmarkManagerException e) {
							e.printStackTrace();
							// task manager is not available, we will skip the experiment and try starting it later
							bm.logError("Unable to start experiment: " + e.getMessage());
							// set flag so that we don't try starting more experiments, instead we go sleeping
							tmNotAvailable = true;
							break;
						} catch (IOException e) {
							e.printStackTrace();
							bm.logError("Experiment was started, but restore log could not be written: "  + e.getMessage());
							bm.logError("When benchmark manager will be restared, experiment will have incorrect status");
						}
					}
					
					if (tmNotAvailable) {
						// task manager is not available, go sleeping
						tmNotAvailable = false;
						break;
					}
				}
				
				} catch (Exception e) {
					e.printStackTrace();
					// keep running
				}
					
				
				try {
					sleep(AnalysisManager.EXPERIMENT_START_INTERVAL);
				} catch (InterruptedException e) {
					// ignored
				}
				
			}
		}
	}

	private class MonitorThread extends Thread {
		
		public MonitorThread() {
			super("MonitorThread");
			start();
		}
		
		@Override
		public void run() {
			while (true) {
				try {
					for (ListIterator iter = monitorQueue.listIterator(); iter.hasNext();) {
						ExperimentMetadata exp = (ExperimentMetadata) iter.next();
						TaskDescriptor[] tasks = exp.getAllTasks();
						try {
							for (TaskDescriptor desc : tasks) {
								TaskState state = bm.getTaskStatus(desc.getTaskTid(),desc.getContextId());
								/*
								 * 1. TM deleted info about the task (state == null) -> flag task as finished
								 * 2. Task is finished/aborted -> flag as finished
								 * 3. otherwise flag as running
								 */
								if (state == null || state == TaskState.ABORTED || state == TaskState.FINISHED) {
									exp.getTaskSequence().setTaskStatus(desc.getTaskTid(),EntityStatus.FINISHED);
								}  else{
									exp.getTaskSequence().setTaskStatus(desc.getTaskTid(),EntityStatus.RUNNING);
								}
							}
	
							EntityStatus expStatus = exp.getStatus();
							bm.logDebug("Status " + EID.toNamedString(exp.getID()) + " - " +  expStatus );
							
							// we don't need to monitor finished experiments
							if (expStatus == EntityStatus.FINISHED) {
								bm.logDebug("Experiment " + EID.toNamedString(exp.getID()) + " finished - removing from queue");
								storeExperiment(exp);
								iter.remove();
							}
						} catch (BenchmarkManagerException e) {
							e.printStackTrace();
							bm.logError("Can't update status for experiment: " + e.getMessage());
						} catch (IOException e) {
							e.printStackTrace();
							bm.logError("Can't write status for experiment: " + e.getMessage());
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
					// keep running
				}
				
				try {
					sleep(EXPERIMENT_MONITOR_INTERVAL);
				} catch (InterruptedException e) {
					// ignored
					e.printStackTrace();
				}
			}
		}
		
	}
	
	
	private BenchmarkManager bm;
	private final HashMap<AID, AnalysisMetadata> analyses =	new HashMap<AID, AnalysisMetadata>();
	private final HashMap<AID, List<ExperimentMetadata>> analysisQueues = new HashMap<AID, List<ExperimentMetadata>>();
	private final List<ExperimentMetadata> monitorQueue = Collections.synchronizedList(new LinkedList<ExperimentMetadata>());
	private WorkQueue<ExperimentMetadata> scheduledExperiments = new WorkQueue<ExperimentMetadata>();
	private final Scheduler scheduler = new Scheduler();
	
	private final File restoreRootDir; 
	
	
	public AnalysisManager(BenchmarkManager bm, File restoreRootDir) {
		this.bm = bm;
		this.restoreRootDir = restoreRootDir;
	}
	
	
	public static AnalysisManager createInstance(BenchmarkManager benchmarkManager, File restoreRootDir)
	throws IOException, ClassNotFoundException {
		File restoreDir = new File(restoreRootDir,RESTORE_SUBDIRECTORY);
		AnalysisManager instance = new AnalysisManager(benchmarkManager,restoreDir);
		if (restoreDir.exists()) {
			restore(instance);
		} else {
			instance.bm.logDebug("Restore directory not found... creating");
			if (!restoreDir.mkdirs()) {
				throw new IOException("Unable to create restore directory");
			}
		}
		
		// create worker threads for instance once the restore is finished
		instance.new GeneratorThread();
		instance.new DispatcherThread();
		instance.new MonitorThread();
		
		return instance;
	}

	/**
	 * Creates new analysis with given name and description
	 * Created analysis is immediately registered with benchmark manager (unique identifier is assigned)
	 * and reported to the ResultsRepository
	 * 
	 * @param name	Name of analysis
	 * @param description Description of the analysis
	 * @param isRegression	Regression flag
	 * 
	 * @return container object for newly created metadata
	 * @throws BenchmarkManagerException if analysis can't be reported to ResultsRepository
	 */
	public AnalysisEntity createAnalysis(String name, String description, boolean isRegression) 
	throws  BenchmarkManagerException {
		ResultsRepositoryInterface resultsRepository = (ResultsRepositoryInterface)bm.lookup(ResultsRepositoryService.SERVICE_NAME);
		
		final AnalysisMetadata metadata = 
				isRegression ? 
				new RegressionAnalysisMetadata(getNextAID()) : 
				new AnalysisMetadata(getNextAID());
		metadata.setName(name);
		metadata.setDescription(description);

		// create lightweight transport object
		AnalysisEntity entity = EntityFactory.createEntity(metadata);
		try {
			/*
			 * Act as in transaction:
			 * 1. update our local log to avoid contaminating RR in case od disk-write failure
			 * 2. add analysis to live
			 * 3. report RR the analysis
			 * 
			 * In case of failure in 1. the RR is not contaminated, when 3. fails, we delete the analysis
			 * from live data and delete its record from log
			 */
			storeAnalysis(metadata);
			analyses.put(metadata.getID(), metadata);
			resultsRepository.addAnalysis(entity);
		} catch (ResultsRepositoryException e) {
			// failed reporting analysis to RR -> delete from local log
			deleteAnalysis(metadata.getID());
			throw new BenchmarkManagerException(e);
		} catch (RemoteException e) {
			// failed reporting analysis to RR -> delete from local log
			deleteAnalysis(metadata.getID());
			throw new BenchmarkManagerException(e);
		} catch (IOException e) {
			throw new BenchmarkManagerException(e);
		}

		
		return entity;
	}
		
	/**
	 * Deletes analysis identified by <code>aid</code>
	 * @param id	identifier of analysis 
	 */
	public void deleteAnalysis(AID id)
	throws BenchmarkManagerException {
		try {
			/*
			 * 1. Check experiments - if any of them is active, we can't delete
			 * 2. Delete restore log
			 * 3. Unschedule regression analysis
			 * 4. Remove analysis manager entry 
			 */
			AnalysisMetadata analysis = getAnalysis(id);
			
			for (ExperimentMetadata experiment : analysis.getExperiments()) {
				if (experiment.getStatus() != EntityStatus.FINISHED){
					throw new BenchmarkManagerException("Analysis has active experiment!");
				}
			}
			
			deleteAnalysisRecord(id);

			// remove (possibly) scheduled regression analysis
			if (analysis.isRegression()) {
				RegressionAnalysisMetadata reg = (RegressionAnalysisMetadata)analysis;
				if (reg.getSchedule() != null && ! reg.getSchedule().isPaused()) {
					scheduler.removeEntry(createSchedulerEntry(id,reg.getSchedule()));
				}
			}
			
			analyses.remove(id);
			
		} catch (AntTaskException e) {
			e.printStackTrace();
			throw new BenchmarkManagerException(e);
		}
		
	}
	
	/**
	 * Lists analyses available to manager 
	 * @return	List of analysis IDs currently available to manager 
	 */
	public AnalysisMetadata[] listAnalyses() 
	throws RemoteException {
		Collection<AnalysisMetadata> values = analyses.values();
		return (AnalysisMetadata[]) values.toArray(new AnalysisMetadata[values.size()]);
	}
	
	/**
	 * Registers new experiment with Benchmark manager.
	 * Adds the experiment under corresponding analysis 
	 * and reports metadata to ResultsRepository
	 * 
	 * @param experiment	Experiment to register
	 * @throws BenchmarkManagerException	when generator can't be started or fails while running
	 */
	public void addExperiment(ExperimentMetadata experiment) 
	throws BenchmarkManagerException {
		AnalysisMetadata ad = analyses.get(experiment.getAnalysis());
		if (ad == null) {
			throw new BenchmarkManagerException("Analysis AID(" +
					experiment.getAnalysis() + ") does not exist");
		}
		
		try {
			storeExperiment(experiment);
			ad.addExperiment(experiment);
		} catch (IOException e) {
			throw new BenchmarkManagerException("Can't register experiment",e);
		} 
	}
	
	public void registerModelExperiment(ExperimentMetadata experiment, SchedulerInfo schedule) 
	throws BenchmarkManagerException {
		RegressionAnalysisMetadata ad = (RegressionAnalysisMetadata)getAnalysis(experiment.getAnalysis());
		try {
			ad.setModelExperiment(experiment);
			ad.setSchedule(schedule);
			storeAnalysis(ad);
			scheduleExperiment(ad);
		} catch (IOException e) {
			throw new BenchmarkManagerException("Can't schedule experiment",e);
		}
	}
	
	
	/**
	 * @param analysis 	analysis identifier
	 * @return	Analysis configuration registered in the manager under given <code>id</code>
	 * @throws IllegalArgumentException when analysis with given identifier 
	 * 			does not exist 
	 */
	public AnalysisMetadata getAnalysis(AID analysis)
	throws IllegalArgumentException {
		AnalysisMetadata ad = analyses.get(analysis);
		if (ad == null) {
			throw new IllegalArgumentException("Analysis AID(" + analysis + ") does not exist");
		}
		return ad;
	}
	
	/**
	 * Creates new experiment with provided name and description. The experiment is not registered anywhere though.
	 * @param analysis	Analysis identifier under which the experiment should be created
	 * @param name	Name of the experiment
	 * @param description	Description of the experiment
	 * @param pluginID	Plugin which the experiment uses
	 * 
	 * @return new ExperimentMetadata object
	 */
	public ExperimentMetadata createExperiment(AID analysis, String name, String description, PEID pluginID)
	throws BenchmarkManagerException {
		String pluginName = getPluginLoader().getPluginName(pluginID);
		ExperimentMetadata exp = new ExperimentMetadata(analysis, getNextEID(), name, description, pluginID, pluginName);
		return exp;
	}
	
	public synchronized void startGenerator(ExperimentMetadata experiment)
	throws BenchmarkManagerException {
		TaskGenerator generator = bm.getPluginLoader().createGenerator(experiment.getPlugin());
		generator.setBenchmarkManager(bm);
		final HostManagerInterface hostManager = (HostManagerInterface)bm.lookup(HostManagerService.SERVICE_NAME);
		try {
			// expand rsl to hosts
			for (StandardRole role : experiment.getRoles()) {
				role.expand(hostManager);
			}
		} catch (Exception e) {
			throw new BenchmarkManagerException("Unable to expand RSL into host info", e);
		}
		generator.setExperiment(experiment);
		generator.setRegression(getAnalysis(experiment.getAnalysis()).isRegression());
		generator.startGenerator();
	}
	
	/**
	 * Reschedules and/or pauses the sched
	 * @param analysis analysis to reschedule
	 * @param newSchedule	new scheduling information
	 * @throws IllegalArgumentException	when analysis is not registered with benchmark manager
	 * @throws IOException 	
	 */
	public void reschedule(AID analysis, SchedulerInfo newSchedule)
	throws IllegalArgumentException, BenchmarkManagerException {

		RegressionAnalysisMetadata metadata = (RegressionAnalysisMetadata)getAnalysis(analysis);
		assert metadata.isRegression() : "Analysis is not a regression analysis";

		SchedulerInfo oldSchedule  = metadata.getSchedule();
		
		if (oldSchedule.equals(newSchedule)) {
			// nothing to do, everything is the same, see SchedulerInfo.equals
			return;
		}

		// the infos have something different - we won't investigate what just reschedule and check for a pause

		// try to remove old entry (may not be present if it was paused before
		SchedulerEntry oldEntry = createSchedulerEntry(analysis, oldSchedule);
		scheduler.removeEntry(oldEntry);
		
		try {
			/*
			 * 1. try to create new entry to see if it is valid
			 * 2. change metadata 
			 * 3. check for pause and pause/unpause
			 */
			SchedulerEntry newEntry = createSchedulerEntry(analysis, newSchedule);
			metadata.setSchedule(newSchedule);
			storeAnalysis(metadata);
			if (! newSchedule.isPaused()) {
				scheduler.addEntry(newEntry);
				metadata.setNextRunDate(newEntry.getAlarmTime());
			} else {
				metadata.setNextRunDate(VersionProvider.FIRST_RUN);
			}
		} catch (IOException e) {
			metadata.setSchedule(oldSchedule);
			throw new BenchmarkManagerException("Can't reschedule analysis",e);
		}
	}
	
	public void runJob(SchedulerEntry entry) {
		RegressionAnalysisMetadata analysis = (RegressionAnalysisMetadata)getAnalysis((AID)entry.getData());
		bm.logInfo("Starting version provider for '" + analysis.getName() + "'");
		PEID plugin = analysis.getModelExperiment().getPlugin();
		VersionProvider provider;
		try {
			provider = getPluginLoader().createVersionProvider(plugin);
		} catch (BenchmarkManagerException e) {
			bm.logError("Unable to create version provider instance for experiment:");
			bm.logError("Analysis: " + analysis.getName());
			bm.logError("Plugin: " + analysis.getModelExperiment().getPluginName());
			bm.logError("Error: " + e.getMessage());
			// we can't do anything more here
			return;
		}
		ExperimentMetadata[] experiments = null;
		long currentTime = System.currentTimeMillis();
		provider.setModel(analysis.getModelExperiment());
		provider.setLastRun(analysis.getLastRunDate());
		provider.setProcessedVersions(analysis.getProcessedVersions());
		provider.setHistoricalStart(analysis.getHistoricalStart());
		provider.setHistoricalEnd(analysis.getHistoricalEnd());
		provider.setBenchmarkManager(bm);
		provider.setSchedule(analysis.getSchedule());
		try {
			experiments = provider.getVersions();
		} catch (Exception e) {
			bm.logWarning("Version provider encountered error during scheduled execution");
			bm.logWarning("No experiments will be created for this run");
			bm.logWarning("Analysis: " + analysis.getName());
			return;
		}
		
		bm.logInfo("Version provider '" + analysis.getName() + "' created " + experiments.length + " experiments");
		
		/*
		 * register newly created experiments under the analysis
		 * add experiments to the queue to have generator started on them
		 */ 
		for (ExperimentMetadata exp : experiments) {
			try {
				exp.setStatus(EntityStatus.CONFIGURED);
				addExperiment(exp);
				scheduledExperiments.addWork(exp);
			} catch (BenchmarkManagerException e) {
				/* XXX: need to make somehow consistency with ResultsRepository	 */
				bm.logError("Unable to report metadata to ResultsRepository - experiment skipped: " + e.getMessage());
			}
		}
		
		/*
		 * All experiments were successfully reported and scheduled, now we can update
		 * time based metadata. If anything goes wrong 
		 */
		analysis.setLastRunDate(currentTime);
		analysis.setNextRunDate(entry.getAlarmTime());
		analysis.addProcessedVersions(provider.getProcessedVersions());
		try {
			storeAnalysis(analysis);
		} catch (IOException e) {
			// XXX: we are seriously doomed here, correct behaviour would require transactions
			bm.logError("Unable to report metadata to ResultsRepository - experiment skipped: " + e.getMessage());
		}
	}
	
	
	void reportResultsMetadata(ExperimentMetadata experiment) 
	throws BenchmarkManagerException {

			ResultsRepositoryInterface resultsRepository = (ResultsRepositoryInterface)bm.lookup(ResultsRepositoryService.SERVICE_NAME);
			try {
				// register basic metadata
				resultsRepository.addExperiment(new ExperimentEntity(experiment));
				for (BID bid : experiment.getBinaryIds()) {
					resultsRepository.addBinary(new BinaryEntity(experiment,bid));
				}
				// register callback scripts
				resultsRepository.setExperimentCallbackRScripts(
						experiment.getAnalysis(),
						experiment.getID(),
						experiment.getScripts());
			} catch (Exception e) {
				bm.logError("Unable to report experiment metadata to results repository for experiment" + 
						EID.toNamedString(experiment.getID()));
				throw new BenchmarkManagerException(e);
			}
	}

	
	private JPFPluginManager getPluginLoader() {
		return bm.getPluginLoader();
	}

	private EID getNextEID() {
		return bm.getIDManager().getNext(EID.class);
	}
	
	private AID getNextAID() {
		return bm.getIDManager().getNext(AID.class);
	}
	
	private SchedulerEntry scheduleExperiment(RegressionAnalysisMetadata analysis) 
	throws BenchmarkManagerException {
		SchedulerInfo schedule = analysis.getSchedule();
		SchedulerEntry entry = createSchedulerEntry(analysis.getID(), schedule);
		scheduler.addEntry(entry);
		analysis.setNextRunDate(entry.getAlarmTime());
		return entry;
	}
	
	void dispatchExperiment(ExperimentMetadata experiment) throws BenchmarkManagerException {
		// experiment is dispatched
		try {
			bm.logInfo("Experiment " + EID.toNamedString(experiment.getID()) + " dispatched - waiting for execution");
			experiment.setStatus(EntityStatus.DISPATCHED);
			storeExperiment(experiment);
			List<ExperimentMetadata> queue = getOrCreateDispatchQueue(experiment.getAnalysis());
			queue.add(experiment);
		} catch (IOException e) {
			bm.logDebug("Can't dispatch experiment: " + e.getMessage());
			e.printStackTrace();
			throw new BenchmarkManagerException("Can't dispatch experiment",e);
		}
	}

	private List<ExperimentMetadata> getOrCreateDispatchQueue(AID analysis) {
		List<ExperimentMetadata> queue = analysisQueues.get(analysis);
		if (queue == null) {
			/*
			 * must be synchronized, because experiments will be added on 
			 * scheduler events/user requests and other thread will be managing
			 * experiment execution 
			 */ 
			queue = Collections.synchronizedList(new ArrayList<ExperimentMetadata>());
			analysisQueues.put(analysis,queue);
		}
		
		return queue;
	}
	
	private SchedulerEntry createSchedulerEntry(AID analysis, SchedulerInfo schedule) {
		SchedulerEntry entry;
		try {
			entry = new SchedulerEntry(
					analysis,
					schedule.getMinutes(),
					schedule.getHours(),
					schedule.getDaysOfMonth(),
					schedule.getMonths(),
					schedule.getDaysOfWeek(),
					schedule.getYear(),
					this
					);
			return entry;
		} catch (PastDateException e) {
			assert false : "This can't happen: PastDateException " + e.getMessage();
			return null;
		}
	}
	
	
	/*
	 * -------------------- Persistency related methods ---------------------------
	 */
	private void storeAnalysis(AnalysisMetadata metadata) 
	throws IOException {
		bm.logTrace("Writing restore file for analysis " + AID.toNamedString(metadata.getID()));
		// use the id as root for storing metadata
		File analysisDir = new File(restoreRootDir,metadata.getID().toString());
		if (!analysisDir.exists() && !analysisDir.mkdirs()) {
			bm.logError("Unable to create restore directory for analysis " + AID.toNamedString(metadata.getID()));
			throw new IOException("Unable to create root directory for analysis " + analysisDir.getPath());
		}
		
		File metadataFile = new File(analysisDir,"metadata.dat");
		ObjectOutputStream oos = null;
		try {
			oos = new ObjectOutputStream(new FileOutputStream(metadataFile));
		oos.writeObject(metadata);
		oos.flush();
		} catch (IOException e ) {
			bm.logError("Unable to write analysis restore file " + AID.toNamedString(metadata.getID()));
		} finally {
			if (oos != null) {
				oos.close();
			}
		}
		
		bm.logTrace("Analysis restore file written");
	}
	
	private void deleteAnalysisRecord(AID analysisID) 
	throws AntTaskException {
		bm.logTrace("Deleting restore file for analysis " + AID.toNamedString(analysisID));
		try {
			Delete.deleteDirectory(new File(restoreRootDir,analysisID.toString()).getAbsolutePath());
		} catch (AntTaskException e) {
			bm.logFatal("Unable to delete analysis restore directory : " + AID.toNamedString(analysisID));
			bm.logFatal("Stop benchmark manager and remove directory manually before starting again," +
					" otherwise phantom analysis will be created");
			throw e;
		}
	}
	
	private void storeExperiment(ExperimentMetadata metadata) throws IOException {
		bm.logTrace("Writing restore file for experiment " + EID.toNamedString(metadata.getID()));
		File analysisDir = new File(restoreRootDir,metadata.getAnalysis().toString());
		if (! analysisDir.exists()) {
			bm.logError("Restore directory for analysis " + AID.toNamedString(metadata.getAnalysis()) + " does not exist!");
			throw new FileNotFoundException("Restore directory for analysis " + analysisDir.getPath() + " does not exist!");
		}

		ObjectOutputStream oos = null;
		try {
			File experimentFile = new File(analysisDir,metadata.getID().toString());
			oos = new ObjectOutputStream(new FileOutputStream(experimentFile));
			oos.writeObject(metadata);
			oos.flush();
		} catch (IOException e ) {
			bm.logError("Unable to write experiment restore file " + EID.toNamedString(metadata.getID()));
			throw e;
		} finally {
			if (oos != null) {
				oos.close();
			}
		}
		
		bm.logTrace("Experiment file written");
	}
	
//	private void deleteExperimentFile(AID analysis, EID experiment) throws IOException {
//		File experimentFile = new File(new File(restoreRootDir,analysis.toString()),experiment.toString());
//		if (experimentFile.exists() && !experimentFile.delete()) {
//			bm.logFatal("Unable to delete experiment restire file " + EID.toNamedString(experiment) + 
//						"in analysis " + AID.toNamedString(analysis));
//			bm.logFatal("Stop benchmark manager and delete restore file manually before restarting," +
//					"otherwise phantom experiment will be created");
//			throw new IOException("Unable to delete experiment restore file " + experimentFile.getPath());
//		}
//	}
	

	/*
	 * Every analysis has its own directory - AID
	 * When experiment is created it is added to analysis metadata, but the metadata is not written.
	 * We delete created/delete directly experiment file and construct list of experiments in analysis from
	 * files under analysis directory
	 * 
	 * Scheduling information is extracted from analysis metadata. Scheduler entries are created and registered
	 * from analysis scheduling info and AIDs.
	 *
	 * When reading experiments, they are assigned to lists according their statuses:
	 * CONFIGURED - experiment was created by version provider but generator was not started for it
	 * 			  - it must be added to scheduledExperiments
	 * 
	 * SUBMITTING - experiment was started generator for but was not submitted
	 * 			  - it must be added to proper analysis queue
	 * 
	 * SCHEDULED - experiment was submitted to TM
	 * 			 - it must be added to monitorQueue to periodically check its status
	 * 
	 * FINISHED - experiment finished running in TM
	 * 			- must be placed only in analysis queue
	 */
	
	public static void restore(AnalysisManager instance) throws IOException, ClassNotFoundException {
		for (File analysisDir : instance.restoreRootDir.listFiles()) {
			if (!analysisDir.isDirectory()) {
				// skip garbage
				continue;
			}
			
			instance.bm.logDebug("Restoring analysis " + analysisDir.getName());
			/*
			 * 1. Restore basic metadata of analysis
			 * 2. Restore experiments under analysis:
			 * 		a. Add experiment to list of analysis experiments
			 * 		b. Put experiment to corresponding queue according to its status
			 */
			instance.bm.logTrace("Reading analysis metadata");
			File metadataFile = new File(analysisDir,"metadata.dat");
			ObjectInputStream ois = new ObjectInputStream(new FileInputStream(metadataFile));
			AnalysisMetadata analysis = (AnalysisMetadata)ois.readObject();
			ois.close();
			instance.analyses.put(analysis.getID(),analysis);
			instance.bm.logDebug("Metadata file read successfully");
			
			if (analysis.isRegression()) {
				instance.bm.logDebug("Scheduling regression analysis");
				RegressionAnalysisMetadata regression = (RegressionAnalysisMetadata)analysis;
				if (regression.getSchedule() != null) {
					// analysis has schedule and model experiment -> schedule it
					try {
						instance.scheduleExperiment(regression);
					} catch (BenchmarkManagerException e) {
						instance.bm.logWarning("Analysis " + AID.toNamedString(regression.getID()) + 
								" has overdue scheduling time and can't be scheduled again");
					}
				} else {
					instance.bm.logDebug("No schedule found for analysis, scheduling skipped");
				}
			}
			
			for (File experimentFile : analysisDir.listFiles()) {
				if ("metadata.dat".equals(experimentFile.getName())) {
					// skip metadata file of analysis
					continue;
				}
				
				instance.bm.logDebug("Restoring experiment " + experimentFile.getName());
				ois = new ObjectInputStream(new FileInputStream(experimentFile));
				ExperimentMetadata experiment = (ExperimentMetadata)ois.readObject();
				ois.close();
				analysis.addExperiment(experiment);
				switch (experiment.getStatus()) {
					case CONFIGURED:
						instance.bm.logInfo("Adding experiment " + experiment.getID() + " to configured queue");
						instance.scheduledExperiments.addWork(experiment);
						break;
					case DISPATCHED:
						instance.bm.logInfo("Adding experiment " + experiment.getID() + " to dispatcher queue");
						List<ExperimentMetadata> queue = instance.getOrCreateDispatchQueue(experiment.getAnalysis());
						queue.add(experiment);
						break;
					case SUBMITTED:
						instance.bm.logInfo("Adding experiment " + experiment.getID() + " to monitor queue");
						instance.monitorQueue.add(experiment);
						break;
				}
			}
		}
	}
	
}


