/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.rmi.Remote;
import java.rmi.RemoteException;

import cz.cuni.mff.been.taskmanager.TaskDescriptor;

/**
 * Interface encapsulating operations with given entity.
 * 
 * To avoid <code>BenchmarkmanagerGUIInterface</code> clutter and code
 * duplication, each Benchmark Manager entity (analysis, experiment, binary
 * and run) has its own interface. This interface excapsulates all operations
 * which can be done with entities. The <code>BenchmarkmanagerGUIInterface</code>
 * has only one method, which gets the interface for analyses (top-level
 * entity).
 * 
 * The interface is typed by the identifier class of the entity which it
 * encapsulates.
 * 
 * If the concrete implementation of this interface doesn't support some
 * particular method, it should throw <code>UnsupportedOperationException</code>.
 * methods which can throw this exceptions are marked as such using
 * <code>throws</code> statement and in JavaDoc. Described behavior is instance
 * of the Composite design pattern.
 *  
 * Note the interface encapsulates operation with all entities of given type,
 * not just one. This is to allow more flexibility and is (maybe?) a bit nicer
 * as many methods aren't parametrized by concrete entity.
 * 
 * Entities are sent to the web interface using <code>Entity&lt;ID&ht;</code>
 * class, which is only a lightweight container, not an remote interface. This
 * is to avoid frequent RMI calls, which has probably better performance
 * characteristics (not tested, though). 
 * 
 * @author David Majda
 *
 * @param <ID> identifier of encapsulated entity
 */
public interface EntityInterface<ID extends cz.cuni.mff.been.common.id.ID>
		extends Remote {
  /**
   * Deletes an entity.
   * 
   * @param id entity identifier of the entity to delete
   * @throws IllegalArgumentException if the entity identified by the entity
   *                                   identifier doesn't exist 
   * @throws UnsupportedOperationException if this operation isn't supported by
   *                                        encapsulated entity type 
	 * @throws RemoteException if something in RMI goes bad
   */
  void deleteEntity(ID id) throws RemoteException, BenchmarkManagerException;
  
  /**
   * Returns ID's of tasks for given entity <b>recursively</b>. Works on all levels of the entity
   * hierarchy.
   * 
   * @param id entity identifier of the entity
   * @return descriptors of tasks for given entity and its subentities
   * @throws IllegalArgumentException if the entity identified by the entity
   *                                   identifier doesn't exist 
   * @throws RemoteException if something in RMI goes bad
   */
  TaskDescriptor[] getAllTasks(ID id) throws RemoteException, IllegalArgumentException;
  
  
  /**
   * Returns ID's of tasks for given entity. Works on all level or hierarchy
   * @param id	identifier of entity
   * @return	descriptor of tasks for given entity
   * @throws RemoteException if something in RMI goes bad
   * @throws IllegalArgumentExceptionf the entity identified by the entity
   *                                   identifier doesn't exist 
   */
  TaskDescriptor[] getTasks(ID id) throws RemoteException, IllegalArgumentException;
  
}

