/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.java.plugin.JpfException;
import org.java.plugin.ObjectFactory;
import org.java.plugin.PluginClassLoader;
import org.java.plugin.PluginLifecycleException;
import org.java.plugin.PluginManager;
import org.java.plugin.PluginManager.PluginLocation;
import org.java.plugin.boot.DefaultPluginsCollector;
import org.java.plugin.registry.Extension;
import org.java.plugin.registry.IntegrityCheckReport;
import org.java.plugin.registry.ManifestProcessingException;
import org.java.plugin.registry.PluginAttribute;
import org.java.plugin.registry.PluginDescriptor;
import org.java.plugin.registry.PluginRegistry;
import org.java.plugin.registry.IntegrityCheckReport.ReportItem;
import org.java.plugin.util.ExtendedProperties;
import org.java.plugin.util.IoUtil;

import cz.cuni.mff.been.benchmarkmanager.PluginsIntegrityReport.IntegrityErrorSeverity;
import cz.cuni.mff.been.benchmarkmanager.plugins.Configurator;
import cz.cuni.mff.been.benchmarkmanager.plugins.TaskGenerator;
import cz.cuni.mff.been.benchmarkmanager.plugins.VersionProvider;
import cz.cuni.mff.been.common.BeenException;
import cz.cuni.mff.been.common.ComponentInitializationException;
import cz.cuni.mff.been.common.StringUtils;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.common.id.PEID;

/**
 * @author mtomcanyi
 */
public class JPFPluginManager {

	/** Name of plugins repository root */
	private static final String PLUGIN_REPOSITORY_ROOT = "plugins";
	/** Identifier of core plugin */
	static final String CORE_PLUGIN = "cz.cuni.mff.been.benchmarkmanager.core";
	
	private static final String RESTORE_SUBDIRECTORY = "pluginmanager";
	
	private final ExtendedProperties bootProperties;
	private BenchmarkManagerLocalInterface benchManager = null;
	
	
	/* JPF Plugin manager instance */
	private /* final */ PluginManager pluginManager;
	
	private final HashSet<URL> rejectedPlugins = new HashSet<URL>();
	
	
	/**
	 * Holds latest report of integrity. Modified on every refresh of registry
	 * @see #refreshRegistry()
	 */
	private PluginsIntegrityReport integrityReport;
	
	/** Directory where all plugins are stored */
	private File pluginsRepository;
	private String defaultPluginsDirectory;
	private String workingDirectory;
	private File restoreRootDir;
	
	
	// NOT AVAILABLE
	private JPFPluginManager() {
		bootProperties = null;
		pluginsRepository = null;
	}
	
	JPFPluginManager(BenchmarkManagerLocalInterface manager, ExtendedProperties bootProperties, 
			String defaultPluginsDirectory,
			String workingDirectory,
			File restoreRootDir) 
	throws ComponentInitializationException {
		this.benchManager = manager;
		this.bootProperties = bootProperties;
		this.pluginsRepository = null;
		this.defaultPluginsDirectory = defaultPluginsDirectory;
		this.workingDirectory = workingDirectory;
		this.restoreRootDir = restoreRootDir;
	}
	
	public static JPFPluginManager createInstance(BenchmarkManagerLocalInterface manager, 
			ExtendedProperties bootProperties,
			String defaultPluginsDirectory,
			String workingDirectory,
			File restoreRootDirectory)
	throws ComponentInitializationException {
		File restoreDir = new File(restoreRootDirectory,RESTORE_SUBDIRECTORY);
			JPFPluginManager instance = new JPFPluginManager(manager, bootProperties, defaultPluginsDirectory,workingDirectory, restoreDir);
			if (restoreDir.exists()) { 
				restore(instance);
			} else {
				if (!restoreDir.mkdirs()) {
					throw new ComponentInitializationException("Unable to create working directory :" + restoreDir.getPath());
				}
			}
			instance.initialize();
			instance.refreshRegistry(true);

		return instance;
	}
		
	/**
	 * @return list of plugins available to plugin loader
	 */
	public BeenPluginDescriptor[] getAvailablePlugins()  {
		Collection extensions = pluginManager.getRegistry().getExtensionPoint(CORE_PLUGIN, "benchmark").getConnectedExtensions();
		BeenPluginDescriptor[] plugins = new BeenPluginDescriptor[extensions.size()];
		int i = 0;
		for (Iterator iter = extensions.iterator(); iter.hasNext(); ) {
			try {
				Extension ext = (Extension) iter.next();
				final String jpfPluginID = ext.getDeclaringPluginDescriptor().getId();
				final PEID pluginID = new PEID(jpfPluginID);
				final BeenPluginDescriptor desc = new BeenPluginDescriptor(pluginID);
				desc.setName(getPluginName(pluginID));
				if (ext.getDeclaringPluginDescriptor().getDocumentation() != null) {
					desc.setDescription(ext.getDeclaringPluginDescriptor().getDocumentation().getText());
				}
				String pluginVersion = ext.getDeclaringPluginDescriptor().getVersion().toString();
				if (! StringUtils.isEmpty(pluginVersion)) {
					desc.setVersion(pluginVersion);
				}
				plugins[i++] = desc;
			} catch (BenchmarkManagerException e) {
				// skip the problematic plugin
				continue;
			}
		}
		return plugins;
	}
	
	/**
	 * @param pluginID	identifier of plugin to search for
	 * @return	name of given plugin
	 */
	public String getPluginName(PEID pluginID) throws BenchmarkManagerException {
		PluginDescriptor desc;
		try {
			desc = pluginManager.getPlugin((String)pluginID.value()).getDescriptor();
		} catch (PluginLifecycleException e) {
			throw new BenchmarkManagerException(e);
		}
		PluginAttribute attName = desc.getAttribute("pluginName");
		if (attName == null || StringUtils.isEmpty(attName.getValue())) {
			return (String)pluginID.value();
		} else {
			return attName.getValue();
		}
	}
	
	/**
	 * @return directory where plugins are stored
	 */
	File getPluginsRepository() {
		return pluginsRepository;
	}
	
	/**
	 * @param pluginID plugin id for which the configurator should be created
	 * @return new configurator instance for given plugin
	 * 		Note that this method never returns <code>null</code>
	 * @throws BenchmarkManagerException when configurator instance can't be created for any reason
	 */
	public Configurator createConfigurator(PEID pluginID) throws BenchmarkManagerException {
		String id = (String) pluginID.value();
		return (Configurator) loadClassForPlugin(id, "configurator");
	}
	
	/**
	 * @param pluginID plugin id for which the generator should be created
	 * @return new generator instance for given plugin
	 * 		Note that this method never returns <code>null</code>
	 * @throws BenchmarkManagerException when generator instance can't be created for any reason
	 */
	public TaskGenerator createGenerator(PEID pluginID) 
	throws BenchmarkManagerException {
		String id = (String) pluginID.value();
		return (TaskGenerator) loadClassForPlugin(id, "generator");
	}

	public VersionProvider createVersionProvider(PEID pluginID) 
	throws BenchmarkManagerException {
		String id = (String) pluginID.value();
		VersionProvider p = (VersionProvider) loadClassForPlugin(id, "versionprovider");
		p.setBenchmarkManager(benchManager);
		return p;
	}
	
	private Object loadClassForPlugin(String pluginID, String paramName) throws BenchmarkManagerException {
		try {
			PluginDescriptor pluginDesc = pluginManager.getPlugin(pluginID).getDescriptor();
			// only one extension is allowed for plugin on benchmark extension point
			Extension benchmarkExtension = null;
			for (Iterator iter = pluginDesc.getExtensions().iterator(); iter.hasNext(); ) {
				Extension ext = (Extension) iter.next();
				if (ext.getExtendedPointId().equals("benchmark")) {
					benchmarkExtension = ext;
					break;
				}
			}
			
			if (benchmarkExtension == null) {
				throw new BenchmarkManagerException("Plugin has no extension for 'benchmark' point");
			}
			
			final String className = benchmarkExtension.getParameter(paramName).valueAsString();
			PluginClassLoader pluginLoader = pluginManager.getPluginClassLoader(pluginDesc);
			Class instanceClass = pluginLoader.loadClass(className);
			return instanceClass.newInstance();
		} catch (Exception e) {
			throw new BenchmarkManagerException(e);
		}
		
	}
	
	
	/**
	 * Validates plugins detected in repository.
	 * Uses standard JPF validation, then checks required classes needed by BM.
	 * Creates new integrity reports and use it to store the validation result.
	 * 
	 * Invalid plugins will be disabled in JPF
	 * @throws ComponentInitializationException When core plugin is invalid (no other plugins can be activated then)
	 * 
	 * @see #getIntegrityReport
	 */
	private void validatePlugins() throws ComponentInitializationException {
			
		integrityReport = new PluginsIntegrityReport();
		
		final List<String> pluginsToDisable = new LinkedList<String>();
		
		// check plugins from BM's point of view
		Collection extensions = pluginManager.getRegistry().getExtensionPoint(CORE_PLUGIN, "benchmark").getConnectedExtensions();
		for (Iterator iter = extensions.iterator(); iter.hasNext(); ) {
			Extension ext = (Extension) iter.next();
			String errorMessage = validatePlugin(ext);
			final String pluginID = ext.getDeclaringPluginDescriptor().getId();
			String pluginName;
			try {
				// try to fill in the plugin name
				pluginName = getPluginName(new PEID(pluginID));
			} catch (BenchmarkManagerException e) {
				/*
				 *  as we are reporting plugin status, the previous call may fail
				 *  ignore exception and use pluginID instead
				 */
				pluginName = pluginID;
				if (errorMessage == null) {
					errorMessage = e.getMessage();
				}
			} catch (RuntimeException e) {
				// JPF throws IllegalArgumentException when plugin is disabled
				pluginName = pluginID;
				if (errorMessage == null) {
					errorMessage = e.getMessage();
				}
				
			}
			addIntegrityEntry(pluginName, errorMessage);
			if (errorMessage != null) {
				pluginsToDisable.add(pluginID);
			}
		}
	
		// unregister and try to delete invalid plugins
		updateRejectionList(pluginsToDisable);

		benchManager.logDebug(integrityReport.toString());
		
	}
	
	private synchronized void updateRejectionList(List<String> rejected) throws ComponentInitializationException {
		
		// get contexts for plugins
		for (String id : rejected) {
			try {
				URL rejectedURL = pluginManager.getPathResolver().getRegisteredConext(id);
				rejectedPlugins.add(rejectedURL);
			} catch (IllegalArgumentException e ) {
				benchManager.logDebug("Can't get context for damaged plugin '" + id + "'");
			}
		}
		
		// unregister damaged plugins
		pluginManager.getRegistry().unregister(rejected.toArray(new String[rejected.size()]));
		
		/*
		 *   URLClassLoader keeps locking jar files, but hopefully it will be garbage-collected 
		 *   once and we will be able to delete the file.
		 *   
		 *   Go throuh the files, check them and try to delete them, then remove the entry.
		 */
		List<URL> successfullyDeleted = new LinkedList<URL>();
		for (URL location : rejectedPlugins) {
			String fname =location.getFile();
			String pluginFile = fname.substring(0,fname.length()-2);
			if (deleteRejectedURL(location)) {
				benchManager.logInfo("Deleted invalid plugin file: " + pluginFile);
				successfullyDeleted.add(location);
			} else {
				benchManager.logInfo("Can't delete invalid plugin file: " + pluginFile);
			}
		}
		
		rejectedPlugins.removeAll(successfullyDeleted);
		
		try {
			saveRejectionList();
		} catch (IOException e) {
			throw new ComponentInitializationException(e);
		}
	}
	
	
	/**
	 * Checks if plugin is valid from BM's point of view - whether required classes are present
	 * @param ext	Extension to validate
	 * @return <code>null</code> if plugin is OK. Error message if plugin is invalid
	 */
	private String validatePlugin(Extension ext) {
			// try to load entry classes
			final String pluginID = ext.getDeclaringPluginDescriptor().getId();
			
			for (String paramName : new String[]{"configurator", "generator"}) {
				try {
					loadClassForPlugin(pluginID, paramName);
				} catch (BenchmarkManagerException e) {
					benchManager.logDebug(e.getMessage());
					return "Can't create " + paramName + " for plugin ";
				}
			}
			
		// plugin ok
		return null;
	}
	
	
	public void refreshRegistry(boolean firstTime) throws ComponentInitializationException {

		benchManager.logInfo("Refreshing registry");
		
		PluginRegistry pluginRegistry = pluginManager.getRegistry();
		
		DefaultPluginsCollector collector = new DefaultPluginsCollector();
		try {
			collector.configure(bootProperties);
		} catch (Exception e) {
			throw new ComponentInitializationException(e);
		}
		Collection locations = collector.collectPluginLocations();
		List<PluginLocation> newLocations = new ArrayList<PluginLocation>();
		// remove entries that are in the rejection list
		for (Iterator iter = locations.iterator(); iter.hasNext();) {
			PluginLocation element = (PluginLocation) iter.next();
			if (! rejectedPlugins.contains(element.getContextLocation())) {
				newLocations.add(element);
			}
		}
		
		/*
		 * We will check old plugins for existence only when CORE_PLUGIN is available to JPF 
		 * (i.e. when we are not running for the first time)
		 */
		if (pluginManager.getRegistry().isPluginDescriptorAvailable(CORE_PLUGIN)) {
			Collection extensions = pluginManager.getRegistry().getExtensionPoint(CORE_PLUGIN, "benchmark").getConnectedExtensions();
			ArrayList<String> toRemove = new ArrayList<String>(); 
			for (Iterator iter = extensions.iterator(); iter.hasNext(); ) {
				Extension ext = (Extension) iter.next();
				final String pluginId = ext.getDeclaringPluginDescriptor().getId();
				if (!pluginRegistry.isPluginDescriptorAvailable(pluginId)) {
					// plugin has been damaged or removed
					pluginManager.deactivatePlugin(pluginId);
					benchManager.logWarning("Plugin '" + pluginId + "' is not available. Deactivating");
					toRemove.add(pluginId);
				}
			}
			
			// remove damaged plugins from registry
			updateRejectionList(toRemove);
		}

		// register new entries & activate them afterwards
		try {
			@SuppressWarnings("unchecked")
			PluginLocation[] locationArray = (PluginLocation[]) newLocations.toArray(new PluginLocation[newLocations.size()]);
			benchManager.logInfo("Detected " + locationArray.length + " plugins in repository");
			pluginManager.publishPlugins(locationArray);
			
			for (Iterator iter = pluginManager.getRegistry().getPluginDescriptors().iterator(); iter.hasNext();) {
				PluginDescriptor plugin = (PluginDescriptor) iter.next();
				if (! pluginManager.isPluginActivated(plugin)) {
					benchManager.logInfo("Activated: " + plugin.getId());
					pluginManager.activatePlugin(plugin.getId());
				}
			}
		} catch (JpfException e) {
			throw new ComponentInitializationException(e);
		}
		
		// check if we have core plugin available - if not, we are in trouble and won't do anything more
		if (! pluginManager.getRegistry().isPluginDescriptorAvailable(CORE_PLUGIN)) {
			IntegrityCheckReport report  = pluginManager.getRegistry().checkIntegrity(pluginManager.getPathResolver());
			
			// let's blame JPF
			String reason = "JPF library error";

			// is at least the plugin file available in the registry?
			if (! new File(pluginsRepository,"benchmarkmanager.jar").exists()) {

				reason = "Core plugin file (benchmarkmanager.jar) not found in plugins repository: " + pluginsRepository.getAbsolutePath(); 
			}
			// try to get reason from report
			for (Iterator iter = report.getItems().iterator(); iter.hasNext();) {
				ReportItem item = (ReportItem) iter.next();
				if (item.getSource() != null && CORE_PLUGIN.equals(item.getSource().getId()) && 
						(item.getSeverity() == ReportItem.SEVERITY_ERROR || item.getSeverity() == ReportItem.SEVERITY_WARNING)) {
					reason = item.getMessage();
					break;
				}
			}
			benchManager.logFatal("Core plugin is not available: " + reason);
			throw new ComponentInitializationException("Core plugin is not available. No plugins will work." + '\n' +
					"Please check its integrity and restart BenchmarkManager" + '\n' +
					"Reason: " + reason
					);
		}

		// perform validation and disable invalid plugins
		validatePlugins();
		
	}
	
	public void registerPlugin(File sourceFile) 
	throws IOException, BeenException {

		DefaultPluginsCollector collector = new DefaultPluginsCollector();
		ExtendedProperties props = new  ExtendedProperties();
		props.putAll(this.bootProperties);
		// we will scan only the new file
		props.put("org.java.plugin.boot.pluginsRepositories", sourceFile.getAbsolutePath());
		
		try {
			collector.configure(props);
		} catch (Exception e) {
			benchManager.logError("Unable to configure plugins collector: " + e.getMessage());
			throw new BenchmarkManagerException("Unable to deploy plugin");
		}

		Collection collectedLocations = collector.collectPluginLocations();
		
		// we have no new entry -> probably some strange file
		if (collectedLocations.isEmpty()) {
			throw new BenchmarkManagerException("Plugin file is not a ZIP/JAR file or plugin is missing the descriptor");
		}
		
		// ok, detected the file, let's validate it
		PluginLocation loc = (PluginLocation)collectedLocations.iterator().next();
		try {
			pluginManager.getRegistry().readManifestInfo(loc.getManifestLocation());
		} catch (ManifestProcessingException e) {
			throw new BenchmarkManagerException("Plugin descriptor is invalid");
		}
		
		// copy file to repository and refresh
		Copy.copyToDir(sourceFile.getAbsolutePath(),getPluginsRepository().getAbsolutePath());
		refreshRegistry(false);
		File pluginFile = new File(getPluginsRepository().getAbsolutePath(),sourceFile.getName());

		// check if the plugin was rehected
		if (isRejected(pluginFile)) {
			throw new BenchmarkManagerException("Plugin is missing required parameters or classes");
		}
		
		
		// we have new entry not in error -> plugin is OK
		benchManager.logInfo("New plugin installed: " + sourceFile.getName());
	}
	
	
	/**
	 * Performs standard initialization of manager instance.
	 * 
	 * <i>
	 * The initialization steps must be performed in this manner and were split into
	 * more methods for purpose of testing
	 * </i> 
	 */
	void initialize() throws ComponentInitializationException {
		createPluginsRepository();
		deployCorePlugin();
		deployDefaultPlugins();
		createPluginManager();
	}
	
	/**
	 * Prepares directory where benchmark plugins will be stored.
	 * Copies default plugins distributed with benchmark manager package to the repository
	 * @throws IOException 
	 */
	void createPluginsRepository() 
	throws ComponentInitializationException {
		
		pluginsRepository = new File(workingDirectory, PLUGIN_REPOSITORY_ROOT);
		// create plugins directory if not exists
		if (!pluginsRepository.exists()) {
			if (!pluginsRepository.mkdir()) {
				throw new ComponentInitializationException("Unable to create plugins directory: " + 
						pluginsRepository.getAbsolutePath());
			}
		}
		
		// add repository key for the DefaultPluginsCollector
		bootProperties.put("org.java.plugin.boot.pluginsRepositories", pluginsRepository.getAbsolutePath());

	}
	
	void deployCorePlugin() 
	throws ComponentInitializationException {
		// copy also benchmarkmanager.jar as it serves as plugin core and must be detected by JPF
		final String pluginCoreFile = defaultPluginsDirectory + File.separator + "benchmarkmanager.jar";
		if (!new File(pluginsRepository, "benchmarkmanager.jar").exists()) {
			try {
				Copy.copyToDir(pluginCoreFile, pluginsRepository.getAbsolutePath());
			} catch (AntTaskException e) {
				benchManager.logFatal("Core plugin not found in package: " + e.getMessage());
				throw new ComponentInitializationException("Core plugin is not available!" + '\n' + 
						" Plug-in system will not work. Check package integrity. " + '\n' +  
						e.getMessage());
			}
		}
	}
	
	void deployDefaultPlugins() {
		// if set, copy default plugins to repository
		final File defaultDir = new File(defaultPluginsDirectory);
		for (String fileName : defaultDir.list()) {
			try {
				// do not overwrite existing plugins
				if (new File(pluginsRepository, fileName).exists()) {
					continue;
				}
				
				// otherwise copy default plugin to repository
				Copy.copyToDir(defaultPluginsDirectory + File.separator + fileName, pluginsRepository.getAbsolutePath());
			} catch (AntTaskException e) {
				benchManager.logWarning("Can't copy default plugin " + fileName 
					+ " to plugins repository. Reason: " + e.getMessage());
			}
		}
	}
	
	void createPluginManager() {
		// create new plugin manager, scan plugins and register them
		pluginManager = ObjectFactory.newInstance(bootProperties).createManager();
	}
	
	boolean isRejected(File pluginFile) {
		
		try {
			return rejectedPlugins.contains(new URL("jar:" 
			            + IoUtil.file2url(pluginFile).toExternalForm() + "!/"));
		} catch (MalformedURLException e) {
			assert false : "Unable to convert plugin file to URL";
		}
		
		return true;
	}
	
	/**
	 * Creates new integrity entry in current error report.
	 * If errorMessage is <code>null</code>, creates entry with OK severity, otherwise with ERROR severity
	 * @param pluginID	plugin ID the integrity entry relates to	
	 * @param errorMessage	error message or <code>null</code> if plugin is ok
	 */
	private void addIntegrityEntry(String pluginID, String errorMessage) {
		integrityReport.addErrorEntry(pluginID, errorMessage, 
				errorMessage == null ? IntegrityErrorSeverity.OK : IntegrityErrorSeverity.ERROR);
	}
	
	
	/**
	 * @return Integrity report of plugins registered with this manager instance
	 */
	public PluginsIntegrityReport getIntegrityReport() {
		return integrityReport;
	}
	
	
	private static boolean deleteRejectedURL(URL location) {
		String fileName = location.getFile();
		// remove "file:/" prefix and "/!" suffix because JarURL returns file like that
		fileName = fileName.substring(6,fileName.length()-2);
		File pluginFile = new File(fileName);
		if (pluginFile.exists()) {
			return pluginFile.delete();
		}
		
		return false;
	}
	
	// --------------------------- Persistency related methods -------------------
	
	private void saveRejectionList() throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(new File(restoreRootDir,"rejected.dat")));
		for (URL url : rejectedPlugins) {
			writer.write(url.toExternalForm());
			writer.newLine();
		}
		writer.flush();
		writer.close();
	}
	
	private static void restore(JPFPluginManager instance) throws ComponentInitializationException {
		instance.benchManager.logDebug("Restoring plugins rejections list");
		File rejected = new File(instance.restoreRootDir,"rejected.dat");
		try {
		BufferedReader reader = new BufferedReader(new FileReader(rejected));
		String rejectURL;
		while ((rejectURL = reader.readLine()) != null) {
			URL reject = new URL(rejectURL);
			String fileName = reject.getFile();
			// remove file:/ prefix and /! suffix
			fileName  = fileName.substring(6,fileName.length()-2);
			// try to delete rejects - if can't delete, add them to rejection list
			if (!deleteRejectedURL(reject)) {
				instance.benchManager.logInfo("Can't delete rejected plugin " + fileName);
				instance.rejectedPlugins.add(reject);
			} else {
				instance.benchManager.logInfo("Deleted rejected plugin " + fileName);
			}
		}
		reader.close();
		instance.benchManager.logDebug("Rejection list restored");
		} catch (MalformedURLException e) {
			throw new ComponentInitializationException("Malformed rejection URL occured during restore",e);
		} catch (IOException e) {
			throw new ComponentInitializationException("I/O error occured during resotre",e);
		}
	}
	
	
	// ----------------------------- TEST METHODS --------------------------------
	
	/*
	 * These methods are needed for JUnit testing. Don't use them for other reasons
	 */
	PluginManager internalGetManager() {
		return pluginManager; 
	}
	
	
	
	
}


