/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;


/**
 * Class represents report of validity for plugins detected by BenchmarkManager.
 * @author mtomcanyi
 *
 */
public class PluginsIntegrityReport implements Serializable {

	public enum IntegrityErrorSeverity {

		OK, // plugin ok
		ERROR; // plugin is not available
		
		@Override
		public String toString() {
			switch (this) {
				case OK: return "OK";
				case ERROR: return "error";
				default: 
						assert false : "toString requested for an unknown enum value"; // can never get here
						return null;
			}
		}
	}
	
	/**
	 * Class holds validity information for a single plugin
	 */
	public static class IntegrityEntry implements Serializable {
		private String pluginName;
		private String errorMessage;
		private IntegrityErrorSeverity severity;
		
		/**
		 * Allocates entry for invalid  plugin
		 * @param pluginName	Name of plugin
		 * @param errorMessage	Error message or <code>null</code> if none
		 */
		public IntegrityEntry(String pluginName, String errorMessage, IntegrityErrorSeverity severity) {
			this.pluginName = pluginName;
			this.errorMessage = errorMessage;
			this.severity = severity;
		}
		
		
		/**
		 * Allocates entry for valid plugin
		 * @param pluginName
		 */
		public IntegrityEntry(String pluginName) {
			this(pluginName, "Valid plugin", IntegrityErrorSeverity.OK);
		}

		
		public IntegrityErrorSeverity getSeverity() {
			return severity;
		}
		
		public String getErrorMessage() {
			return errorMessage;
		}
		
		public String getPluginName() {
			return pluginName;
		}
		
		public String toString() {
			return "IntegrityEntry(pluginID=" + pluginName
				+ ", severity=" + severity 
				+ "," + errorMessage + ")";
		}
		
		@Override
		/**
		 * Entries are equal if denote same plugin with the same error and
		 * error message
		 */
		public boolean equals(Object obj) {
			if (obj instanceof IntegrityEntry) {
				IntegrityEntry other = (IntegrityEntry)obj;
				return pluginName.equals(other.pluginName) &&
						severity.equals(other.severity) && 
						(errorMessage == null ? other.errorMessage == null :
							errorMessage.equals(other.errorMessage));
			}
			
			return false;
		}
		
		
		@Override
		public int hashCode() {
			int result = 7*pluginName.hashCode();
			result += 23*severity.hashCode();
			result +=  errorMessage == null ? 0 : 31*errorMessage.hashCode();
			return result;
		}
	}

	/**
	 * Report entries
	 */
	private final List<IntegrityEntry> entries = new ArrayList<IntegrityEntry>();
	
	
	public PluginsIntegrityReport() {
	}
	
	
	public IntegrityEntry[] getEntries() {
		return (IntegrityEntry[]) entries.toArray(new IntegrityEntry[entries.size()]); 
	}
	
	void addErrorEntry(String pluginName, String errorMessage, IntegrityErrorSeverity severity) {
		entries.add(new IntegrityEntry(pluginName, errorMessage, severity));
	}
	
	void addOKEntry(String pluginName) {
		entries.add(new IntegrityEntry(pluginName, "Plugin is valid", IntegrityErrorSeverity.OK));
	}

	@Override
	public String toString() {
		String message = "Integrity report:\n";
		for (IntegrityEntry entry : entries) {
			message = message + entry + "\n";
		}
		
		return message;
		
	}
	
}
