/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.Serializable;
import java.rmi.RemoteException;

import cz.cuni.mff.been.common.rsl.Condition;
import cz.cuni.mff.been.hostmanager.HostManagerException;
import cz.cuni.mff.been.hostmanager.HostManagerInterface;
import cz.cuni.mff.been.hostmanager.ValueNotFoundException;
import cz.cuni.mff.been.hostmanager.ValueTypeIncorrectException;
import cz.cuni.mff.been.hostmanager.database.HostInfoInterface;
import cz.cuni.mff.been.hostmanager.database.RSLRestriction;
import cz.cuni.mff.been.hostmanager.database.RestrictionInterface;

public class StandardRole implements Serializable {

	private String name;
	private Condition rsl;
	private HostInfoInterface[] hosts;
	private int count;
	
	/**
	 * Creates new benchmark role with name and RSL-based specification of host
	 * @param name of benchmarking role
	 * @param rsl	RSL-based host specification
	 * @param count	expected count of hosts for the role
	 */
	public StandardRole(String name, Condition rsl, int count) {
		this.name = name;
		this.rsl = rsl;
		this.count = count;
	}

	/*
	 * Method called by BenchmarkManger to expand RSL into matching hosts
	 */
	void expand(HostManagerInterface hostManager)
	throws RemoteException, HostManagerException, ValueNotFoundException, ValueTypeIncorrectException {
		RSLRestriction rslQuery = new RSLRestriction(rsl);
		hosts = hostManager.queryHosts(new RestrictionInterface[]{rslQuery});
	}

	
	/**
	 * @return name of the role
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns RSL specification of host. 
	 * More hosts may match the RSL specification and TaskManager
	 * is the one to choose the best suited for the purpose of load balancing
	 * @param indexOfHost	index of matching host - the parameters is ignored for
	 * 						non-benchmarking role
	 * @return RSL specification of host
	 */
	public Condition getRSL(int indexOfHost) {
		return rsl;
	}
	
	/**
	 * @return hosts matching role's RSL
	 */
	public HostInfoInterface[] getHostDetails() {
		return hosts;
	}
	
	/**
	 * @return host names of hosts matching role's RSL
	 */
	public String[] getHostNames() {
		String[] names = new String[hosts.length];
		int i=0;
		for (HostInfoInterface hostInfo : hosts) {
			names[i++] = hostInfo.getHostName();
		}
		
		return names;
	}
	
	/**
	 * Indication whether the host's role is direct part of the benchmark or
	 * just a supporting role e.g. compilation, download
	 * 
	 * @return true if host directly acts as a part of the benchmark
	 */
	public boolean isBenchmarking() {
		return false;
	}
	
	/**
	 * @return expected count of hosts for the role
	 */
	public int getCount() {
		return count;
	}
}
