/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.OID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;

/**
 * Class represents sequence of BEEN tasks for a single experiment.
 */
public class TaskSequence implements Serializable {

	/**
	 * Comparator used for searching for SequenceItem with given ID
	 */
	private static Comparator<SequenceItem> ID_COMPARATOR = new Comparator<SequenceItem>() {
		
		/**
		 * OID.compareTo performs only numeric comparison of OIDs.
		 * But we are always comparing only IDs of the same level of hierarchy
		 * so numeric comparison is enough
		 * 
		 * @see OID#compareTo(OID)
		 */
		public int compare(SequenceItem o1, SequenceItem o2) {
			return o1.getID().compareTo(o2.getID());
		}
	};
	
	
	private class TaskItem implements Serializable {
		private TaskDescriptor descriptor;
		private EntityStatus status = EntityStatus.CONFIGURED;
		
		public TaskItem(TaskDescriptor descriptor) {
			this.descriptor = descriptor;
		}

		public TaskDescriptor getDescriptor() {
			return descriptor;
		}
		
		public EntityStatus getStatus() {
			return status;
		}
		
		public void setStatus(EntityStatus status) {
			this.status = status;
		}
		
		@Override
		public String toString() {
			return descriptor.getTaskTid() + "=" + status;
		}
		
	}
	
	/**
	 * Single item in hierarchy
	 * Experiment -> Binary -> Run
	 * The run level only contains task descriptors
	 */
	private class SequenceItem implements Serializable {
		
		private final OID id;
		private EntityStatus status = EntityStatus.CONFIGURED;
		
		/**
		 * Task descriptors on this level of hierarchy
		 */
		private LinkedList<TaskItem> tasks = new LinkedList<TaskItem>();
		
		/**
		 * Lower level of hierarchy:
		 * Experiment -> Binaries
		 * Binary -> Runs
		 * Runs level contain only tasks
		 * 
		 * This is lazily initialized, because runs won't contain lower levels
		 */
		private ArrayList<SequenceItem> children;
		
		
		public SequenceItem(OID objectID) {
			this.id = objectID;
		}
		
		public OID getID(){
			return id;
		}
		
		/**
		 * Adds task descriptor to this level of hierarchy
		 * @param td TaskDescriptor to add
		 */
		public TaskItem addTask(TaskDescriptor td) {
			TaskItem item = new TaskItem(td);
			tasks.add(item);
			return item;
		}

		/**
		 * Adds a child item to the end of children list.
		 * <b>Invariant: Keeps children sorted by IDs</b>
		 * @param child	child item to add
		 */
		public void addChild(SequenceItem child) {
			if (children == null) {
				children = new ArrayList<SequenceItem>();
			}
			children.add(child);
		}

		/**
		 * @return true if both items carry the same ID
		 * 		   false otherwise
		 */
		public boolean equals(Object obj) {
			if (obj instanceof SequenceItem == false) {
				return false;
			}
			
			SequenceItem otherItem = (SequenceItem)obj;
			return getID().equals(otherItem.getID());
		}
		
		/**
		 * Searches for an item with given ID in the tree hierarchy.
		 * The method uses few assumptions:
		 *  - only one type of id is on each level of hierarchy:
		 *    i.e. EID -> BID -> RID -> no children
		 *  - all levels of IDs are generated by autoincrement
		 *  	=> children of one item are sorted by their IDs
		 *  
		 *  Using the assumptions following algorithm was deduced:
		 *  <nl>
		 *  <li>Check type and value of item itself, return if match</li>
		 *  <li>If no item has no children, return null</li>
		 *  <li>Check type of IDs of item's children</li>
		 *  	<ul>
		 *  	<li>correct type -> perform binary search as children are sorted</li>
		 *  	<li>incorrect type -> immediately drill-down to next level of hierarchy</li>
		 *  	</ul>
		 *  <li>If no match on children level -> return null</li>
		 *  </nl>
		 * @param id ID to search for
		 * @return item represented by given ID or <code>null</code> when
		 * 			none matches
		 */
		public SequenceItem findByID(OID id) {
			
			/*
			 * check self
			 * this also check for correct type
			 */ 
			if (this.id.equals(id)) {
				return this;
			}
			
			// if no children -> no match immediately
			if (children == null) {
				return null;
			}

			/*
			 *  if children are of correct 'ID level' perform binary search 
			 *  on children
			 */ 
			if (id.getClass().isInstance(children.get(0).getID())) {
				SequenceItem tmp = new SequenceItem(id);
				int matchIndex = Collections.binarySearch(children,tmp,ID_COMPARATOR);
				return matchIndex < 0 ? null : children.get(matchIndex);
			}
			
			/*
			 * if children don't have correct level of IDs, go to the next
			 * level of hierarchy
			 */ 
			for (SequenceItem child : children) {
				SequenceItem result = child.findByID(id);
				if (result != null) {
					return result;
				}
			}
			
			/*
			 * child with given ID was not found neither in this item
			 * nor in the subtree of this item
			 */
			return null;
		}

		/**
		 * @param id	identifier of item in the hierarchy
		 * @return	tasks from item identified by <code>id</code> and all its descendants
		 * 			e.g. if <code>id</code> refers to a binary, tasks from binary and all its runs are returned
		 * 			if entity is not found or has no tasks TaskDescriptor[0] is returned 
		 */
		public TaskDescriptor[] getAllTasks(OID id) {
			SequenceItem item = findByID(id);
			if (item == null) {
				return new TaskDescriptor[0];
			}
			
			List<TaskDescriptor> result = new LinkedList<TaskDescriptor>();
			item.getAllTasks(result);
			return (TaskDescriptor[]) result.toArray(new TaskDescriptor[result.size()]);
		}
		
		/**
		 * @return all tasks in the sequence
		 */
		public TaskDescriptor[] getAllTasks() {
			return getAllTasks(getID());
		}
		
		/**
		 * @param id	identifier od item in sequence
		 * @return	tasks of only the item identified by <code>id</code>
		 * 			if entity has no tasks or is not found TaskDescriptor[0] is returned
		 */
		public TaskDescriptor[] getTasks(OID id) {
			SequenceItem item = findByID(id);
			if (item == null) {
				return new TaskDescriptor[0];
			}
			
			return item.getTasks();
		}
		
		/**
		 * @return tasks for this entity or empty array if entity has no tasks
		 */
		public TaskDescriptor[] getTasks() {
			List<TaskDescriptor> taskList = getItemsTasks();
			return (TaskDescriptor[]) taskList.toArray(new TaskDescriptor[taskList.size()]);
		}
		
		/**
		 * @return IDs of children in this item or empty array if no children
		 * 	are present
		 */
		public OID[] getChildrenIds() {
			List<SequenceItem> chlds = getChildren();
			OID[] result = new OID[chlds.size()];
			int i=0;
			for (SequenceItem child : chlds) {
				result[i++] = child.getID();
			}
			
			return result;
		}
		
		
		public String toString() {
			StringBuffer buf = new StringBuffer();
			buf.append("(").append(id.toString()).append(",");
			buf.append(tasks.toString());
			buf.append(",");
			buf.append(hasChildren() ? children.toString() : "[]" );
			buf.append(")");
			return buf.toString();
		}
		
		
		/**
		 * Merges children and descriptors of this item with other item.
		 * <code>item</code> must have 'equals' ID with this item
		 * Algorithm:
		 * <nl>
		 * 	<li>If item has equal ID as we have, take all its task descriptors</li>
		 * 	<li>Go through items children
		 * 		- if it has equal children as we have -> merge them recursively
		 *      - if the child is not equal to any of our children, append to our children
		 *      	<b>new children are created with AUTOINCREMENT IDs, so appending
		 *      		preserves sorted manner of our children</b>
		 *   </li>   
		 * </nl>
		 * @param item to merge with
		 */
		public void mergeWith(SequenceItem item) {
			/* 
			 * this should not happen as tasksequence starts merge on experiment
			 * level and mergeWith() implementation preserves this
			 */
			assert getID().getClass().isInstance(item.getID())  :
					   "Merging items with incompatible ID class!!!";
			
			if (getID().equals(item.getID())) {
				/*
				 *  the other item is 'the same' as we are
				 *  -> all its tasks and children belong to us!
				 */
				if (item.hasTasks()) {
					tasks.addAll(item.tasks);
				}
				for (SequenceItem otherChild : item.getChildren()) {
					SequenceItem myChild = findByID(otherChild.getID());
					if (myChild == null) {
						// we don't have this child among ours -> take it!
						children.add(otherChild);
					} else {
						// we have this child already -> merge it with us
						myChild.mergeWith(otherChild);
					}
				}
			} else {
				assert false : "Merging items with different values: " +
					getID().getClass().getCanonicalName() + "(" + getID() + ") and "
					+ item.getID().getClass().getCanonicalName() + "(" + item.getID() + ")";
			}
				  
		}
		
		
		public EntityStatus getStatus() {
			
			/*
			 * When finished, our status can't change
			 */
			if (status == EntityStatus.FINISHED) {
				return EntityStatus.FINISHED;
			}
			
			/* No children and no tasks -> finished immediately */
			if (!hasChildren() && !hasTasks()) {
				this.status = EntityStatus.FINISHED;
				return EntityStatus.FINISHED;
			}
			
			/*
			 * Compute status from 
			 * 1. tasks 
			 * 2. children (causes recursive computation of status)
			 * 
			 * If any of tasks/children is running, return running
			 * Check also, whether all tasks and children are finished.
			 * Once finished, our state can't change
			 */
			boolean allFinished = true;
			
			if (hasTasks()) {
				for (TaskItem item : tasks) {
					EntityStatus itemStatus = item.getStatus();
					if (itemStatus == EntityStatus.RUNNING) {
						this.status = EntityStatus.RUNNING;
						return EntityStatus.RUNNING;
					}
					allFinished = allFinished && itemStatus == EntityStatus.FINISHED;
				}
			}
			
			for (SequenceItem item : getChildren()) {
				EntityStatus itemStatus = item.getStatus();
				if (itemStatus == EntityStatus.RUNNING) {
					this.status = EntityStatus.RUNNING;
					return EntityStatus.RUNNING;
				}
				allFinished = allFinished && itemStatus == EntityStatus.FINISHED;
			}
			
			// all children are finished -> this item is also finished
			if (allFinished) {
				this.status = EntityStatus.FINISHED;
			}

			return status;

		}
		
		
		private boolean hasChildren() {
			return children != null;
		}
		
		private boolean hasTasks() {
			return tasks != null;
		}
		
		private List<SequenceItem> getChildren() {
			return hasChildren() ? children : Collections.<SequenceItem>emptyList();
		}

		private void getAllTasks(List<TaskDescriptor> result) {
			result.addAll(getItemsTasks());
			
			for (SequenceItem child : getChildren()) {
				child.getAllTasks(result);
			}
		}
		
		private List<TaskDescriptor> getItemsTasks() {
			if (!hasTasks()) {
				return Collections.<TaskDescriptor>emptyList();
			}
			
			LinkedList<TaskDescriptor> result = new LinkedList<TaskDescriptor>();
			for (TaskItem item : tasks) {
				result.add(item.getDescriptor());
			}
			
			return result;
		}

		/**
		 * Sets status of this item and all its tasks/subitems
		 * @param status status to set
		 */
		public void setStatus(EntityStatus status) {
			this.status = status;
			
			if (hasTasks()) {
				for (TaskItem item : tasks) {
					item.setStatus(status);
				}
			}
			
			for (SequenceItem child : getChildren()) {
				child.setStatus(status);
			}
		}
		
	}
	
	private LinkedList<SequenceItem> stack = new LinkedList<SequenceItem>();
	
	/** Cross index from TID to TaskItem stored in the sequence */
	private HashMap<String,TaskItem> tasks = new HashMap<String,TaskItem>();
	
	
	/**
	 * Creates new sequence representing tasks of an experiment 
	 * @param experimentID	ID of experiment to be represented by the sequence
	 */
	public TaskSequence(EID experimentID) {
		openItem(new SequenceItem(experimentID));
	}

	/**
	 * Opens new binary under given experiment. Tasks will be added to
	 * this binary while not closed. Binary may also be reopen.
	 * @param binaryID	ID of binary to (re)open
	 */
	public void openBinary(BID binaryID) throws IllegalStateException {
		// opening currently open binary -> nothing to do
		if (getOpenItem().getID().equals(binaryID)) {
			return;
		}
		
		// close possibly open previous binary
		closeBinary();
		
		// check whether we are reopening an existing binary
		SequenceItem binary = getOpenItem().findByID(binaryID);
		
		// not found -> create new binary
		if (binary == null) {
			binary = new SequenceItem(binaryID);
			getOpenItem().addChild(binary);
		}
		
		openItem(binary);
	}
	
	/**
	 * Opens new run in currently open binary.
	 * Task will be added to this run while not closed
	 * Run may be reopen, operation is relative to currently open binary.
	 * @param runID		ID of run to (re)open
	 * @throws IllegalStateException	when no binary was open before calling
	 * 									this method
	 * 
	 * @see #openBinary(BID)
	 */
	public void openRun(RID runID) throws IllegalStateException {
		// trying to open current run
		if (getOpenItem().getID().equals(runID)) {
			return;
		}
		
		// close possible previous run -> last open binary will be on top
		closeRun();

		// check whether the last open item is an binary
		if (getOpenItem().getID() instanceof BID == false) {
			throw new IllegalStateException("The last open item is not a binary");
		}
		
		// check whether we are reopening an existing run within this binary
		SequenceItem run = getOpenItem().findByID(runID);
		
		// not found -> create new run
		if (run == null) {
			run = new SequenceItem(runID);
			getOpenItem().addChild(run);
		}
		
		openItem(run);
	}
	
	/**
	 * Closes last open run
	 */
	public void closeRun() {
		// last item open is not a run
		if ( getOpenItem().getID() instanceof RID == false) {
			return;
		}

		closeTopItem();
	}
	
	/**
	 * Closes last open binary
	 */
	public void closeBinary() {
		// close previous run if open
		closeRun();
		
		// last item open is not a binary
		if (getOpenItem().getID() instanceof BID == false) {
			return;
		}
		
		closeTopItem();
	}
	
	/**
	 * @return identifier of last open binary 
	 * @throws IllegalStateException	when no binary was open
	 * @see #openBinary(BID)
	 */
	public BID getOpenBinary() throws IllegalStateException {
		// only binary is open
		if (getOpenItem().getID() instanceof BID) {
			return (BID)getOpenItem().getID();
		} 
		
		// run is open -> binary is behind it
		if (getOpenItem().getID() instanceof RID) {
			return (BID)stack.get(1).getID();
		}
		
		// no binary is open ...
		throw new IllegalStateException("No binary is open");
	}
	
	/**
	 * @return identifier of last open run
	 * @throws IllegalStateException	when no run is open
	 * @see #openRun(RID)
	 */
	public RID getOpenRun() throws IllegalStateException {
		if (getOpenItem().getID() instanceof RID) {
			return (RID)getOpenItem().getID();
		}
		
		throw new IllegalStateException("No run is open");
	}
	
	
	/**
	 * Adds task to the sequence
	 * @param task
	 */
	public void addTask(TaskDescriptor task) {
		TaskItem item = getOpenItem().addTask(task);
		tasks.put(task.getTaskTid(),item);
	}
	
	
	/**
	 * @return all tasks in the sequence or empty array if no tasks are in the sequence
	 */
	public TaskDescriptor[] getAllTasks() {
		List<TaskDescriptor> result = new LinkedList<TaskDescriptor>();
		for(TaskItem item : tasks.values()) {
			result.add(item.getDescriptor());
		}
		
		return result.toArray(new TaskDescriptor[result.size()]);
	}
	
	/**
	 * @param id to search for
	 * @return tasks of item with given <code>id</code> and its children or empty array if 
	 * 	no tasks are found
	 */
	public TaskDescriptor[] getAllTasks(OID id) {
		return getExperiment().getAllTasks(id);
	}
	
	
	/**
	 * @param id	identifier of entity in experiment
	 * @return	tasks of entity identified by <code>id</code>
	 */
	public TaskDescriptor[] getTasks(OID id) {
		return getExperiment().getTasks(id);
	}
	
	/**
	 * @return Identifiers of binaries stored in the sequence
	 */
	public BID[] getBinaries() {
		OID[] children = getExperiment().getChildrenIds();
		BID[] binaries = new BID[children.length];
		System.arraycopy(children,0,binaries,0,binaries.length);
		return binaries;
	}
	
	/**
	 * @return Identifier of runs stored in the sequence for given binary
	 */
	public RID[] getRuns(BID id) {
		SequenceItem item = getExperiment().findByID(id);
		if (item == null) {
			return new RID[0];
		}
		OID[] ids = item.getChildrenIds();
		RID[] result = new RID[ids.length];
		System.arraycopy(ids,0,result,0,result.length);
		return result;
	}
	
	public RID[] getRuns() {
		LinkedList<RID> result = new LinkedList<RID>();
		for (OID id : getBinaries()) {
			result.addAll(Arrays.asList(getRuns((BID)id)));
		}
		
		return result.toArray(new RID[result.size()]);
	}
	
	public String toString() {
		return getExperiment().toString();
	}
	
	public void merge(TaskSequence otherSequence) {
		getExperiment().mergeWith(otherSequence.getExperiment());
	}
	
	public boolean contains(OID id) {
		return getExperiment().findByID(id) != null;
	}
	
	public EntityStatus getStatus(OID id) {
		SequenceItem item = getExperiment().findByID(id);
		assert item != null  : "Entity with given id does not exist";
		
		return item.getStatus();
	}
	
	void setTaskStatus(String tid, EntityStatus status) {
		tasks.get(tid).setStatus(status);
	}
	
	void setGlobalStatus(EntityStatus status) {
		getExperiment().setStatus(status);
	}

	private void openItem(SequenceItem item) {
		stack.addFirst(item);
	}
	
	private SequenceItem getOpenItem() {
		return stack.getFirst();
	}
	
	private void closeTopItem() {
		stack.removeFirst();
	}

	
	/*
	 * Item of the experiment is allways the last, as hierarchy items
	 * are pushed on top of it on the stack
	 */
	private SequenceItem getExperiment() {
		return stack.getLast();
	}
	
	
//	public static void main(String[] args) {
//		TaskSequence seq = new TaskSequence(new EID(1));
//		seq.openBinary(new BID(1));
//		seq.openRun(new RID(1));
//		seq.addTask(TaskDescriptor.createTask("1","prva","context", new EqualsCondition<String>("aaa","bbb")));
//		seq.addTask(TaskDescriptor.createTask("2","druha","context", new EqualsCondition<String>("aaa","bbb")));
//		
//		seq.openRun(new RID(2));
//		seq.addTask(TaskDescriptor.createTask("3","prva","context", new EqualsCondition<String>("aaa","bbb")));
//		seq.addTask(TaskDescriptor.createTask("4","druha","context", new EqualsCondition<String>("aaa","bbb")));
//		
//		seq.setTaskStatus("1", EntityStatus.RUNNING);
//		System.out.println(seq.getStatus(new RID(1)));
//		System.out.println(seq.getStatus(new RID(2)));
//		System.out.println(seq.getStatus(new EID(1)));
//		seq.setTaskStatus("1",EntityStatus.FINISHED);
//		seq.setTaskStatus("2",EntityStatus.FINISHED);
//		System.out.println(seq.getStatus(new EID(1)));
//		System.out.println(seq.getStatus(new EID(1)));
//	}
}
