/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager.gui;

import java.net.InetAddress;
import java.rmi.Remote;
import java.rmi.RemoteException;

import cz.cuni.mff.been.benchmarkmanager.AnalysisEntity;
import cz.cuni.mff.been.benchmarkmanager.AnalysisEntityInterface;
import cz.cuni.mff.been.benchmarkmanager.BeenPluginDescriptor;
import cz.cuni.mff.been.benchmarkmanager.BenchmarkManagerException;
import cz.cuni.mff.been.benchmarkmanager.ExperimentEntity;
import cz.cuni.mff.been.benchmarkmanager.PluginsIntegrityReport;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BRID;
import cz.cuni.mff.been.common.id.PEID;
import cz.cuni.mff.been.hostmanager.HostManagerException;
import cz.cuni.mff.been.hostmanager.ValueNotFoundException;
import cz.cuni.mff.been.hostmanager.ValueTypeIncorrectException;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;

/**
 * RMI interface to the Benchmark Manager, intended for the GUI (web interface).
 *  
 * @author Michal Tomcanyi
 * @author David Majda
 */
public interface BenchmarkManagerGUIInterface extends Remote {
	/** Size of buffer used when uploading plugins in the GUI. */
	int UPLOAD_BUFFER_SIZE = 4096; 

	/**
	 * Main entry point to entity hierarchy for GUI
	 * @return Entity interface for analysis
	 * @throws RemoteException if something in RMI goes bad
	 */
	AnalysisEntityInterface getAnalysisInterface() throws RemoteException;

	/**
	 * Creates new metadata entry of benchmark analysis
	 * @param name	Name of analysis
	 * @param description	Description of the analysis
	 * @param isRegression	Regression flag
	 * @throws BenchmarkManagerException when analysis can't be registered for
	 *          any reason
	 * @throws RemoteException when something in RMI goes bad
	 */
	AnalysisEntity createAnalysis(String name, String description, boolean isRegression)
		throws BenchmarkManagerException, RemoteException;
      
	/**
	 * Returns first plugin configurator screen for given plugin in the session.
	 * Benchmark experiment instance is created implicitely, however not
	 * registered anywhere.
	 * 
	 * @param sessionId session identifier
	 * @param analysisID analysis identifier for which to start configurator 
	 * @param name human readable experiment name
	 * @param description description of experiment
	 * @param pluginID plugin which the experiment uses
	 * @return first plugin configurator screen, or <code>null</code> if there
	 *          are no (more) screens for this plugin
	 * 
	 * @throws RemoteException if something in RMI goes bad
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          occurs
	 */
	Screen getFirstConfiguratorScreen(BRID sessionId, AID analysisID, String name,
		String description, PEID pluginID)
		throws RemoteException, BenchmarkManagerException;
	
	/**
	 * Returns next plugin configurator screen, given current screen and session.
	 * 
	 * @param sessionId session identifier
	 * @param currentScreen current plugin configurator screen
	 * @return next plugin configurator screen, or <code>null</code> if there are
	 *          no more configurator screens for plugin configured in given
	 *          session
	 * @throws RemoteException if something in RMI goes bad
	 * @throws IllegalScreenSequenceException if the method is called out of
	 *          sequence - see <code>IllegalScreenSequenceException</code> for
	 *          description of the contract and behavior
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          occurs
	 */
	Screen getNextConfiguratorScreen(BRID sessionId, Screen currentScreen)
		throws RemoteException, IllegalScreenSequenceException, BenchmarkManagerException;
	
	/**
	 * Returns previous plugin configurator screen, given current screen and session.
	 * 
	 * @param sessionId session identifier
	 * @param currentScreen current plugin configurator screen
	 * @return previous plugin configurator screen, or <code>null</code> if there
	 *          are no more configurator screens for plugin configured in given
	 *          session
	 * @throws RemoteException if something in RMI goes bad
	 * @throws IllegalScreenSequenceException if the method is called out of
	 *          sequence - see <code>IllegalScreenSequenceException</code> for
	 *          description of the contract and behavior
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          occurs
	 */
	Screen getPreviousConfiguratorScreen(BRID sessionId, Screen currentScreen)
		throws RemoteException, IllegalScreenSequenceException, BenchmarkManagerException;

	/**
	 * Return description of roles in the experiment configured under given
	 * session ID. Roles have filled list of possible assigned hosts. 
	 * 
	 * @param sessionId session identifier
	 * @return description of <b>benchmark</b>roles in the experiment
	 * @throws RemoteException when something in RMI goes bad
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          occurs
	 * @throws HostManagerException in case of problems with invalid RSL
	 * @throws ValueNotFoundException in case of problems with invalid RSL
	 * @throws ValueTypeIncorrectException in case of problems with invalid RSL
	 */
	Role[] getBenchmarkRoles(BRID sessionId) 
		throws RemoteException, HostManagerException, BenchmarkManagerException, 
		ValueNotFoundException, ValueTypeIncorrectException;

	/**
	 * Sets user-specified hosts for experiment stored under given session
	 * @param sessionID	identifier of session
	 * @param roles	roles to set
	 * @throws RemoteException	when something in RMI goes wrong 
	 */
	void setBenchmarkRoles(BRID sessionID, Role[] roles) throws RemoteException;
	
	/**
	 * Returns configured experiment fr given session ID.
	 * 
	 * @param sessionId session identifier
	 * @return configured experiment
	 * @throws RemoteException when something in RMI goes bad
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          occurs
	 */
	ExperimentEntity getConfiguredExperiment(BRID sessionId) 
		throws RemoteException, BenchmarkManagerException;

	/**
	 * Starts task generation procedure in experiment's plugin task generator.
	 *  
	 * @param sessionId session identifier
	 * @return set of task descriptors for generated tasks
	 * @throws RemoteException when something in RMI goes bad
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          of the task generator occurs
	 */
	TaskDescriptor[] beginTaskGeneration(BRID sessionId) 
		throws RemoteException, BenchmarkManagerException;

	/**
	 * Starts the experiment configured under given session ID.
	 * 
	 * @param sessionId session identifier
	 * @param schedule information about scheduling of the experiment or <code>null</code>
	 * 		 	when experiment is not regression
	 * @param scripts	R Callback scripts or <code>null</code> when analysis is not regression
	 * @throws BenchmarkManagerException if some error in the Benchmark Manager
	 *          occurs
	 * @throws RemoteException when something in RMI goes bad
	 */
	void startExperiment(BRID sessionId, SchedulerInfo schedule, CallbackRScripts scripts)
		throws RemoteException, BenchmarkManagerException;

	/**
	 * Retrieves the list of available benchmarks.
	 *
	 * @return list of available benchmarks
	 * @throws RemoteException when something in RMI goes bad
	 */
	BeenPluginDescriptor[] getAvailablePlugins() throws RemoteException;

	/**
	 * @return Benchmark plugins integrity report
	 * @throws RemoteException if something in RMI goes bad
	 */
	PluginsIntegrityReport getPluginsIntegrityReport() throws RemoteException;

	/**
	 * Uploads a plugin to the Benchmark Manager. The client must open the socket
	 * on some port before calling this function. Then it calls the
	 * <code>beginPluginUpload</code> function.
	 * 
	 * This function work asynchronously and returns immediately. Before return it
	 * starts a new thread that will receive the plugin contents from the socket
	 * open on given host and port.
	 * 
	 * If there is any error when sending the plugin contents, the socket is
	 * closed. 
	 * 
	 * Calling this function doesn't guarantee that this plugin will be added to
	 * the Benchmark Manager - if it arrives damaged, it is invalid, or some error
	 * during processing occurs, Benchmark Manager rejects the plugin. Client has
	 * an opportunity to check (via the <code>getPluginUploadStatus</code> method
	 * whether the plugin arrived complete and undamaged.
	 *  
	 * After the upload finishes, client should close the returned handle
	 * via the <code>endPluginUpload</code> method.  
	 *
	 * @param ip IP address of the machine prepared for sending the plugin (not
	 *         necessairly the caller itself)
	 * @param port port open for sending the plugin contents
	 * @throws RemoteException when something in RMI goes bad
	 */
	UploadHandle beginPluginUpload(InetAddress ip, int port)
		throws RemoteException;
	
	/**
	 * Returns the status of the plugin upload. Result is one of the values listed
	 * in the <code>UploadStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>UPLOADING</code> mean that
	 * Benchmark Manager still works on the package upload. Values
	 * <code>ACCEPTED</code>, <code>REJECTED</code> and <code>ERROR</code>
	 * mean that Benchmark Manager has finished its work (either successfully
	 * or unsuccessfully) and the upload handle is ready to be closed by the
	 * <code>endPluginUpload</code> method.   
	 * 
	 * @param handle plugin upload handle
	 * @return upload status
	 * @throws RemoteException when something in RMI goes bad
	 * @throws IllegalArgumentException when uploader with given handle does not exist
	 */
	UploadStatus getPluginUploadStatus(UploadHandle handle)
		throws RemoteException, IllegalArgumentException;
	
	/**
	 * Returns the error messages associated with rejected plugin upload. These
	 * messages contain human-readable messages describing the reason of the
	 * rejection and are suitable for displaying to the user.
	 * 
	 * @param handle package upload handle
	 * @return error messages associated with rejected plugin upload
	 * @throws IllegalStateException if the plugin upload status is different
	 *          from <code>REJECTED</code>
	 * @throws RemoteException when something in RMI goes bad
	 */
	String[] getRejectedPluginUploadErrorMessages(UploadHandle handle)
		throws RemoteException;

	/**
	 * Closes the upload handle. Client should call this method after the upload
	 * status returned by <code>getPluginUploadStatus</code> has value
	 * <code>ACCEPTED</code>, <code>REJECTED</code> or <code>ERROR</code>.
	 * 
	 * If the client doesn't close the upload handle, Benchmark Manager will leak
	 * a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle. 
	 * 
	 * @param handle upload handle
	 * @throws RemoteException when something in RMI goes bad
	 */
	void endPluginUpload(UploadHandle handle) throws RemoteException;
	
	/**
	 * Deletes the plugin form the Benchmark Manager.
	 * 
	 * @param name name of the deleted plugin
	 * @return <code>true</code> if the deletion was successfull;
	 *          <code>false</code> otherwise
	 * @throws RemoteException when something in RMI goes bad
	 */
	boolean deletePlugin(String name) throws RemoteException;

	
	/**
	 * @param brid session identifier
	 * @return default plugin-specific callback scripts for Results Repository that will be invoked 
	 * 			on collected results
	 */
	CallbackRScripts getAnalysisCallbackRScripts(BRID brid) throws RemoteException;	
}