package cz.cuni.mff.been.benchmarkmanager.gui;

import java.io.Serializable;
import java.util.Arrays;

import cz.cuni.mff.been.benchmarkmanager.StandardRole;

/**
 * Class storing information about selection of hosts for one role.
 * 
 * @author David Majda
 */
public class Role implements Serializable {
	/** Role name. */
	private String name;
	/** Number of hosts that should be selected by the user to this role. */
	private int count;
	/** Hostnames of hosts allowed to be selected to this role. */
	private String[] hosts;
	/**
	 * Hostnames of hosts which user selected to this role. Does not need to
	 * match the <code>count</code> attribute, as the user can select more or less
	 * hosts in the user interface (this will be followed by the error message). 
	 */
	private String[] selectedHosts;
	
	/** @return role name */
	public String getName() {
		return name;
	}

	/**
	 * @return number of hosts that should be selected by the user to this role
	 */
	public int getCount() {
		return count;
	}
	
	/** @return hostnames of hosts allowed to be selected to this role */
	public String[] getHosts() {
		return hosts;
	}
	
	/** @return hostnames of hosts which user selected to this role. Does not
	 *           need to match the <code>count</code> attribute, as the user can
	 *           select more or less hosts in the user interface (this will be
	 *           followed by the error message)
	 */
	public String[] getSelectedHosts() {
		return selectedHosts;
	}

	/**
	 * Allocates a new <code>Role</code> object.
	 * 
	 * @param name role name
	 * @param count number of hosts that should be selected by the user to this
	 *         role
	 * @param hosts hostnames of hosts allowed to be selected to this role
	 * @param selectedHosts hostnames of hosts which user selected to this role.
	 *         Does not need to match the <code>count</code> attribute, as the
	 *         user can select more or less hosts in the user interface (this will
	 *         be followed by the error message)
	 */
	public Role(String name, int count, String[] hosts, String[] selectedHosts) {
		this.name = name;
		this.count = count;
		this.hosts = hosts;
		this.selectedHosts = selectedHosts;
		Arrays.sort(this.selectedHosts);
	}
	
	/**
	 * Allocates a new <code>Role</code> object using Benchmark Manager's
	 * <code>BenchmarkRole</code> object.
	 * 
	 * @param benchmarkRole Benchmark Manager's object storing information about
	 *         the role
	 * @param selectedHosts hostnames of hosts which user selected to this role.
	 *         Does not need to match the <code>count</code> attribute, as the
	 *         user can select more or less hosts in the user interface (this will
	 *         be followed by the error message)
	 */
	public Role(StandardRole benchmarkRole, String[] selectedHosts) {
		name = benchmarkRole.getName();
		count = benchmarkRole.getCount();
		this.hosts = benchmarkRole.getHostNames();
		this.selectedHosts = selectedHosts;
		Arrays.sort(this.selectedHosts);
	}
	
	/**
	 * Checks if the host is between selected hosts for this role.
	 * 
	 * @param host hostname of host to check
	 * @return <code>true</code> if the host is between selected hosts for this
	 *          role;
	 *          <code>false</code> otherwise
	 */
	public boolean isHostSelected(String host) {
		return Arrays.binarySearch(selectedHosts, host) >= 0;
	}
}
