/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager.gui;

import java.io.Serializable;
import java.util.Arrays;

import cz.cuni.mff.been.benchmarkmanager.plugins.VersionProvider;

/**
 * Class which stores information about timing of the experiments in the
 * regression analysis.
 * 
 * The class is immutable and especially Misho should never ever try to make it
 * mutable :-)
 * 
 * @author David Majda
 */
public class SchedulerInfo implements Serializable {
	/** Valid minutes. Allowed values 0-59, or {-1} for all. */
	private int[] minutes;
	/** Valid hours. Allowed values 0-23, or {-1} for all. */
	private int[] hours;
	/** Valid days of month. Allowed values 1-31, or {-1} for all. */
	private int[] daysOfMonth;
	/**
	 * Valid months. Allowed values 0-11 (0 = January, 1 = February, ...), or
	 * {-1} for all. <code>java.util.Calendar</code> constants can be used.
	 */
	private int[] months;
	/**
	 * Valid days of week. This attribute is exclusive with
	 * <code>dayOfMonth</code>. Allowed values 1-7 (1 = Sunday, 2 = Monday, ...),
	 * or {-1} for all. <code>java.util.Calendar</code> constants can be used.
	 */
	private int[] daysOfWeek;
	/** 
	 * Year. When this field is not set (i.e. -1) the event is repetitive (i.e.
	 * it is rescheduled when reached).
	 */ 
	private int year;
	/**
	 * Flag indicating that the experiment in regession analysis is paused. 
	 */
	private boolean isPaused;
	/**
	 * Interval in which version provider should detect versions
	 */
	private long historicalStartDate = VersionProvider.NO_START;
	private long historicalEndDate = VersionProvider.NO_END;
	
	/** @return valid minutes */
	public int[] getMinutes() {
		return minutes;
	}

	/** @return valid hours */
	public int[] getHours() {
		return hours;
	}

	/** @return valid days of month */
	public int[] getDaysOfMonth() {
		return daysOfMonth;
	}
	
	/** @return valid months */
	public int[] getMonths() {
		return months;
	}

	/** @return valid days of week */
	public int[] getDaysOfWeek() {
		return daysOfWeek;
	}

	/** @return year */
	public int getYear() {
		return year;
	}

	/**
	 * @return flag indicating that the experiment in regession analysis is
	 *          paused
	 */
	public boolean isPaused() {
		return isPaused;
	}
	
	/**
	 * @return timestamp before which the version provider should detect new
	 *          versions (it will ignore versions before this timestamp). Value
	 *          {@link VersionProvider#NO_START} indicates to start now.
	 */
	public long getHistoricalStartDate() {
		return historicalStartDate;
	}
	
	/**
	 * @return timestamp after which version provider should not detect new 
	 * 			versions (it will ignore versions after this timestamp). Value
	 * 			{@link VersionProvider#NO_END} indicates "infinity".
	 */
	public long getHistoricalEndDate() {
		return historicalEndDate;
	}
	
	
	
	/**
	 * Allocates a new <code>SchedulerInfo</code> object.
	 * 
	 * @param minutes valid minute. Allowed values 0-59, or {-1} for all
	 * @param hours valid hours. Allowed values 0-23, or {-1} for all
	 * @param daysOfMonth valid days. Allowed values 1-31, or {-1} for all
	 * @param months valid months. Allowed values 0-11 (0 = January, 1 =
	 *         February, ...), or {-1} for all. <code>java.util.Calendar</code>
	 *         constants can be used
	 * @param daysOfWeek valid days of week. This attribute is exclusive with
	 *         <code>dayOfMonth</code>. Allowed values 1-7 (1 = Sunday, 2 =
	 *         Monday, ...), or {-1} for all. <code>java.util.Calendar</code>
	 *         constants can be used
	 * @param year year. When this field is not set (i.e. -1) the event is
	 *         repetitive (i.e. it is rescheduled when reached).
	 * @param isPaused
	 * @param historicalStartDate timestamp after which the version provider
	 *         should detect new versions (it will ignore versions before this
	 *         timestamp). Value <code>0</code> means "ignore"
	 */
	public SchedulerInfo(int[] minutes, int[] hours, int[] daysOfMonth,
			int[] months, int[] daysOfWeek, int year, boolean isPaused,
			long historicalStartDate, long historicalEndDate) {
		this.minutes = minutes;
		this.hours = hours;
		this.daysOfMonth = daysOfMonth;
		this.months = months;
		this.daysOfWeek = daysOfWeek;
		this.year = year;
		this.isPaused = isPaused;
		this.historicalStartDate = historicalStartDate;
		this.historicalEndDate = historicalEndDate;
		sortArrayFields();
	}
	
	/**
	 * Sorts all array-based attributes so that <code>equals</code> method works correctly.
	 */
	private void sortArrayFields() {
		for (int[] arrayField: new int[][] {
			minutes, hours, daysOfMonth, months, daysOfWeek
		}) {
			if (arrayField != null) {
				Arrays.sort(arrayField);
			}
		}
	}	

	/**
	 * Compares this <code>SchedulerInfo</code> to the specified object. The
	 * result is <code>true</code> if and only if the argument is not
	 * <code>null</code> and is an <code>SchedulerInfo</code> object with equal
	 * fileds set in the constructor.
	 * 
	 * @throws ClassCastException if the argument is not an <code>AndCondition</code>.
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final SchedulerInfo other = (SchedulerInfo) obj;
		if (isPaused != other.isPaused)
			return false;
		if (year != other.year)
			return false;
		if (historicalStartDate != other.historicalStartDate)
			return false;
		if (!Arrays.equals(daysOfMonth, other.daysOfMonth))
			return false;
		if (!Arrays.equals(daysOfWeek, other.daysOfWeek))
			return false;
		if (!Arrays.equals(hours, other.hours))
			return false;
		if (!Arrays.equals(minutes, other.minutes))
			return false;
		if (!Arrays.equals(months, other.months))
			return false;
		return true;
	}

	/**
	 * Returns a hash code value for this object.
	 * 
	 * The hash code is computed using all the fields set in the constructor.  
	 * 
	 * @return a hash code value for this object
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + Arrays.hashCode(daysOfMonth);
		result = PRIME * result + Arrays.hashCode(daysOfWeek);
		result = PRIME * result + (int) (historicalStartDate ^ (historicalStartDate >>> 32));
		result = PRIME * result + Arrays.hashCode(hours);
		result = PRIME * result + (isPaused ? 1231 : 1237);
		result = PRIME * result + Arrays.hashCode(minutes);
		result = PRIME * result + Arrays.hashCode(months);
		result = PRIME * result + year;
		return result;
	}
}
