/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager.plugins;

import java.rmi.Remote;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import cz.cuni.mff.been.benchmarkmanager.BenchmarkManager;
import cz.cuni.mff.been.benchmarkmanager.BenchmarkManagerException;
import cz.cuni.mff.been.benchmarkmanager.BinaryMetadata;
import cz.cuni.mff.been.benchmarkmanager.ExperimentMetadata;
import cz.cuni.mff.been.benchmarkmanager.GeneratorException;
import cz.cuni.mff.been.benchmarkmanager.StandardRole;
import cz.cuni.mff.been.benchmarkmanager.TaskSequence;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.common.id.TID;
import cz.cuni.mff.been.common.rsl.Condition;
import cz.cuni.mff.been.hostmanager.HostManagerInterface;
import cz.cuni.mff.been.hostmanager.HostManagerService;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryInterface;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryService;
import cz.cuni.mff.been.softwarerepository.PackageMetadata;
import cz.cuni.mff.been.softwarerepository.RSLPackageQueryCallback;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryService;
import cz.cuni.mff.been.task.Task;
import cz.cuni.mff.been.taskmanager.Dependency;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;

public abstract class TaskGenerator implements TaskGeneratorInterface {

	// Benchmark manager reference
	private BenchmarkManager bm;
	private ExperimentMetadata experiment;
	private TaskDescriptor lastAdded;
	
	/** Indication whether generator is running for regression experiment */
	private boolean isRegression;
	
	// Active role
	private String activeRole;
	
	
	/**
	 * Allocates new generator
	 * No-arg constructor must exist so that generator can be created without
	 * using Reflections
	 */
	public TaskGenerator() {
		
	}
	
	public void startGenerator() throws GeneratorException {
		checkRemoteReferences();
		dumpProperties();
		setupResults();
		generate();
		generateLogCollection();
	}
	
	
	// -------------------- API to BenchmarkManager ----------------------
	/**
	 * Sets benchmark manager instance which the generator may use
	 * e.g. logging, service detection
	 */
	public void setBenchmarkManager(BenchmarkManager bm) {
		this.bm = bm;
	}
	
	/**
	 * Sets experiment metadata holding description of experiment
	 */
	public void setExperiment(ExperimentMetadata experiment) {
		this.experiment = experiment;
	}
	
	/**
	 * @return	experiment updated with generator run
	 */
	public ExperimentMetadata getExperiment() {
		return experiment;
	}
	
	/**
	 * Sets regression flag
	 * @param isRegression
	 */
	public void setRegression(boolean isRegression) {
		this.isRegression = isRegression;
	}
	

	// ----------------------- Main method ----------------------------
	
	/**
	 * Performs real generation work. Will be called in separate thread.
	 * Child should update its status correctly if it wants to inform user about its progress
	 * Children must override.
	 * 
	 * @see #startGenerator()
	 * @throws GeneratorException when generation was unsuccessfull for any reason
	 * 		(incorrect parameters, BEEN component error)
	 */
	
	protected abstract void generate() throws GeneratorException;
	
	
	protected abstract void registerResults(StandardRole[] roles) throws GeneratorException;
	
	
	// ---------------------- Utility methods ---------------------
	
	/**
	 * @return indication whether generator is running for regression run
	 */
	public boolean isRegression() {
		return isRegression;
	}
	
	/** 
	 * Activates role for host lookups when creating task descriptors
	 * @param activeRole	role to activate
	 */
	protected void setActiveRole(String activeRole) {
		this.activeRole = activeRole;
	}
	
	/**
	 * Utility function which creates new task descriptor which starts task
	 * identified by <code>taskName</code>. Host runtimes are prefilled in the descriptor
	 * will be determined according to active role.
	 * @see #setActiveRole(String)
	 * 
	 * @param taskName name of task to start with task descriptor
	 * @return task descriptor for given task
	 */
	protected TaskDescriptor createTaskDescriptor(String taskName, int indexOfHost) {

		assert experiment.getRole(getActiveRole()).getHostNames().length > 0 :
			"No host available for '" + getActiveRole() + "'";

		final Condition hostRSL = experiment.getRole(getActiveRole()).getRSL(indexOfHost);
		
		return TaskDescriptor.createTask(getNextTID().toString(), taskName, 
				getExperiment().getContextID(), hostRSL);
	}
	
	/**
	 * Utility method to create task descriptor for a standard role.
	 * This method is equal to calling <code>createTaskDescriptor(taskName,0)</code>.
	 * @see #createTaskDescriptor(String, int)
	 * 
	 * @param taskName	name of task to start with task descriptor
	 * @return task descriptor for given task
	 */
	protected TaskDescriptor createTaskDescriptor(String taskName) {
		return createTaskDescriptor(taskName,0);
	}
	

	/**
	 * @return SoftwareRepository remote interface
	 */
	protected SoftwareRepositoryInterface getSoftwareRepository() 
	throws GeneratorException {
		return (SoftwareRepositoryInterface) lookupBeenService(
				SoftwareRepositoryService.SERVICE_NAME);
	}

	/**
	 * @return	HostManager remote interface
	 * @throws GeneratorException 
	 */
	protected HostManagerInterface getHostManager() 
	throws GeneratorException {
		return (HostManagerInterface) lookupBeenService(
				HostManagerService.SERVICE_NAME);
	}

	/**
	 * @return	ResultsRepository remote interface
	 * @throws GeneratorException 
	 */
	protected ResultsRepositoryInterface getResultsRepository() 
	throws GeneratorException {
		return (ResultsRepositoryInterface) lookupBeenService(
				ResultsRepositoryService.SERVICE_NAME);
	}

	
	protected void logDebug(String errorMessage) {
		bm.logDebug(errorMessage);
	}
	
	protected void logError(String errorMessage) {
		bm.logError(errorMessage);
	}
	
	protected void logFatal(String errorMessage) {
		bm.logFatal(errorMessage);
	}
	
	protected void logInfo(String infoMessage) {
		bm.logInfo(infoMessage);
	}

	protected void logTrace(String infoMessage) {
		bm.logTrace(infoMessage);
	}

	protected void logWarning(String warningMessage) {
		bm.logWarning(warningMessage);
	}
	
	/**
	 * @return true if this is reprise run, false otherwise
	 */
	protected boolean isRerun() {
		return getExperiment().isRerun();
	}
	
	/**
	 * @return number of binaries to create in current run
	 */
	protected int getNewBinaryCount() {
		return getExperiment().getBinaryCount();
	}
	
	/**
	 * @return default number of runs to creat in new binary
	 */
	protected int getDefaultRunCount() {
		return getExperiment().getRunCount();
	}

	/**
	 * @return true if (at least one) new binary should be created in generator run
	 */
	protected boolean createNewBinaries() {
		return getNewBinaryCount() > 0;
	}
	
	/**
	 * @return true when new runs should be created for any existing binary, false otherwise
	 */
	protected boolean createNewRuns() {
		for (BinaryMetadata binary : getExperiment().getBinaries()) {
			if (binary.getNewRunCount() > 0) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Tests package availability in software repository
	 * @param condition	RSL condition for package
	 * @return	<code>true</code> if package is available
	 * 			<code>false</code> otherwise
	 * @throws GeneratorException when an error in communicaton with SoftwareRepositry occurs
	 */
	protected boolean isPackageAvailable(Condition condition)  
	throws GeneratorException {
		return availablePackages(condition).length > 0;
	}
	
	/**
	 * @param condition	RSL condition for package
	 * @return	packages available in SoftwareRepository matching given condition
	 * @throws GeneratorException when an error in communicaton with SoftwareRepositry occurs
	 */
	protected PackageMetadata[] availablePackages(Condition condition) 
	throws GeneratorException {
			
		SoftwareRepositoryInterface swRepo = getSoftwareRepository();
		if (swRepo == null){
			throw new GeneratorException("Software repository not available");
		}
		
		try {
			RSLPackageQueryCallback ri = new RSLPackageQueryCallback(condition);
			return swRepo.queryPackages(ri);
		} catch (Exception e) {
			throw new GeneratorException("Can't query software repository",e);
		}
		
	}

	/**
	 * @return BEEN-wide unique binary identifier represented by BID object
	 */
	protected BID getNextBinaryID() {
		return bm.getIDManager().getNext(BID.class);
	}
	
	/**
	 * @return BEEN-wide unique run identifier represented by RID object
	 */
	protected RID getNextRunID() {
		return bm.getIDManager().getNext(RID.class);
	}

	/**
	 * Searches for unused binary packages fitting provided <code>query</code>.
	 * Used packages are determined according to the metadata flag stored
	 * in binary metadata under <code>packageName</code> key.
	 * 
	 * The binary is <b>NOT</b> used whenits metadata flag is not found among flags
	 * from used binaries. 
	 * 
	 * <code>null</code> value can't be used as metadata flag and used binaries with
	 * such flag are skipped.
	 * 
	 * @param query 	RSL query to use with SoftwareRepository to find all available packages
	 * @param packageName	name of key to binary metadata under which metadata flag is found
	 * @return	array of unused binary packages 
	 * @throws GeneratorException when SoftwareRepository is not available
	 */
	protected PackageMetadata[] findUnusedBinaries(Condition query, String packageName) throws GeneratorException {
		// find used metadata flags for given package 
		HashSet<String> usedMetadataFlags = new HashSet<String>();
		for (BinaryMetadata binary : getExperiment().getBinaries()) {
			String flag = binary.getPackage(packageName);
			if (flag != null) {
				usedMetadataFlags.add(flag);
			}
		}
		
		// no binaries need to be filtered - return all matching packages
		if (usedMetadataFlags.size() == 0) {
			return availablePackages(query);
		}
		
		List<PackageMetadata> result = new LinkedList<PackageMetadata>();
		
		for (PackageMetadata available : availablePackages(query)) {
			if (! usedMetadataFlags.contains(available.getBinaryIdentifier())) {
				result.add(available);
			}
		}
		
		return (PackageMetadata[]) result.toArray(new PackageMetadata[result.size()]);
		
	}
	
	/**
	 * Creates and opens new binary.
	 * @return ID of newly created binary
	 */
	protected BID createNewBinary() {
		BID id = getNextBinaryID();
		experiment.getTaskSequence().openBinary(id);
		return id;
	}
	
	
	/**
	 * @see TaskSequence#getOpenBinary()
	 */
	protected BID getOpenBinary() throws IllegalStateException {
		return experiment.getTaskSequence().getOpenBinary();
	}
	
	
	/**
	 * Closes last open binary (as well as run if any open)
	 */
	protected void closeBinary() {
		experiment.getTaskSequence().closeBinary();
	}
	
	/**
	 * Creates and opens new run within currently open binary
	 * @return ID of newly created run
	 */
	protected RID createNewRun() {
		RID id = getNextRunID();
		experiment.getTaskSequence().openRun(id);
		return id;
	}
	
	/**
	 * @see TaskSequence#getOpenRun()
	 */
	protected RID getOpenRun() throws IllegalStateException {
		return experiment.getTaskSequence().getOpenRun();
	}
	
	/**
	 * Closes last open run (leaves open last binary)
	 */
	protected void closeRun() {
		experiment.getTaskSequence().closeRun();
	}
	

	/**
	 * Utility method which adds task to the sequence with dependency on
	 * specific <code>checkPoint</code> of given <code>dependency</code> task
	 * 
	 * Task descriptor will be modfied in the manner that new dependency will
	 * be added via addDependency() method
	 * 
	 * @param task	task descriptor to add
	 * @param dependency	unique TID of dependency
	 * @param checkPoint	name of checkpoint to wait for
	 */
	protected void addWithDependency(TaskDescriptor task, String dependency, String checkPoint) {
		task.addDependencyCheckPoint(new Dependency(dependency,checkPoint));
		addTask(task);
	}
	
	/**
	 * Adds task to last open hierarchy level in the experiment's sequence
	 * @param task	task to add
	 */
	protected void addTask(TaskDescriptor task) {
		lastAdded = task;
		experiment.getTaskSequence().addTask(task);
	}
	
	/**
	 * Adds task to last open hierarchy level in sequence with success dependency
	 * on the last task added to the sequence
	 * 
	 * Call to this method is equivalent to
	 * 
	 * @param task	task to add
	 * @throws NullPointerException when no task was added to the sequence
	 */
	protected void addAfterLast(TaskDescriptor task) {
		task.addDependencyCheckPoint(success(lastAdded.getTaskTid()));
		addTask(task);
	}
	
	/**
	 * Adds new results record to experiment metadata.
	 * Results repository will be informed about results and will check
	 * their arrival.
	 * 
	 * @param role	client-defined name of role
	 * @param host	host from which results will arrive
	 */
	protected void addResultsMapping(String role, String host) {
		getExperiment().addRoleMapping(role,host);
	}
	
	
	/**
	 * @return last task added to the sequence by any of <code>add*</code>
	 * 			methods or <code>null</code> when no task was added yet
	 */
	public TaskDescriptor getLastAdded() {
		return lastAdded;
	}
	
	
	/**
	 * Performs lookup of (main) remote interface of a BEEN service
	 * @param serviceName name of service to look-up
	 * @return	main remote service interface or <code>null</code> if service could not be found
	 */
	private Remote lookupBeenService(String serviceName)
	throws GeneratorException {
		try {
			return bm.lookup(serviceName);
		} catch (BenchmarkManagerException e) {
			throw new GeneratorException("Service lookup failed",e);
		}
	}
	
	private void setupResults() throws GeneratorException {
		registerResults(experiment.getRoles());
	}
	
	/*
	 * Checks whether BEEN components are available
	 */
	private void checkRemoteReferences() throws GeneratorException {
		if (getSoftwareRepository() == null) {
			throw new GeneratorException("Software repository reference invalid");
		}
		if (getResultsRepository() == null) {
			throw new GeneratorException("Results repository reference invalid");
		}
		
		if (getHostManager() == null) {
			throw new GeneratorException("Host manager reference invalid");
		}
	}
	
	/*
	 * @return BEEN-unique task identifier represented by TID object
	 */
	private TID getNextTID() {
		return bm.getIDManager().getNext(TID.class);
	}
	
	/*
	 * @return active role which will be used for creating task descriptors
	 */
	private String getActiveRole() {
		return this.activeRole;
	}

	/**
	 * 
	 * @param taskId task ID of the task which is the dependency.
	 * @return a dependency on a successfull finish of a task.
	 */
	protected static Dependency success(String taskId) {
		return new Dependency(taskId, 
				Task.CHECKPOINT_NAME_FINISHED,
				Task.EXIT_CODE_SUCCESS);
	}

	/**
	 * 
	 * @param taskId task ID of the task which is the dependency.
	 * @return a dependency on an unsuccessfull finish of a task.
	 */
	protected static Dependency error(String taskId) {
		return new Dependency(taskId, 
				Task.CHECKPOINT_NAME_FINISHED,
				Task.EXIT_CODE_ERROR);
	}
	
	/**
	 * 
	 * @param taskId task ID of the task which is the dependency. 
	 * @return a dependency on a successfull or unsuccessfull finish of a task.
	 */
	protected static Dependency finish(String taskId) {
		return new Dependency(taskId, 
				Task.CHECKPOINT_NAME_FINISHED,
				null);
	}

	/**
	 * Generates log collection task and make dependency on finish of all previous tasks,
	 * which will tell Results Repository to collect all the rest of the logs   
	 */
	private void generateLogCollection() {
			TaskDescriptor logUploadTask = createTaskDescriptor("log-upload");
		
			TaskDescriptor[] allTasks = experiment.getTaskSequence().getAllTasks();		
		
			for (TaskDescriptor taskIterator : allTasks) {
				logUploadTask.addDependencyCheckPoint(finish(taskIterator.getTaskTid()));
			}
			
			logUploadTask.addTaskProperties("analysis.id", getExperiment()
					.getAnalysis().toString());
			logUploadTask.addTaskProperties("experiment.id", getExperiment().getID()
					.toString());
			addTask(logUploadTask);
			
	}
	
	private void dumpProperties() {
		String[] properties = experiment.getPropertyNames();
		String logMessage = "Benchmark properties:\n";
		for (String property : properties) {
			logMessage += "  " + property + "=" + experiment.getProperty(property) + "\n";
		}
		logDebug(logMessage);
	}
}
