package cz.cuni.mff.been.benchmarkmanager.plugins;

import cz.cuni.mff.been.benchmarkmanager.BenchmarkManagerLocalInterface;
import cz.cuni.mff.been.benchmarkmanager.ExperimentMetadata;
import cz.cuni.mff.been.benchmarkmanager.gui.SchedulerInfo;
import cz.cuni.mff.been.common.Version;
import cz.cuni.mff.been.common.id.EID;

public abstract class VersionProvider {

	/** Constant to indicate that version provider is running for the first run */
	public static final long FIRST_RUN = -1;

	/** Constant to indicate that user did not specified any upper boundary value */
	public static final long NO_END = -1;
	
	/** Constant to indicate that user did not specified any lower boundary value */
	public static final long NO_START = -1;
	
	
	/** Experiment model from which subsequent experiments are derived by version provider run */
	protected ExperimentMetadata model;
	
	/** List of processed versions */
	protected Version[] processedVersions;
	
	/**
	 * Timestamp of last <b>valid</b> run
	 * @see #getVersions()
	 */
	protected long lastRun;
	
	/** Scheduling information */
	protected SchedulerInfo schedule;

	/** Time interval in which the version provider should detect versions. (Versions out of this interval should be detected) */
	protected long historicalStart; // millis
	protected long historicalEnd; // millis

	/** BenchmarkManager instance mainly for logging purposes */
	protected BenchmarkManagerLocalInterface benchmarkManager;


	public VersionProvider() {
		// hidden
	}
	
	
	/**
	 * Queries version provider for list of versions.
	 * 
	 * Expected functionality:
	 * <p> 
	 * return list of (experiments for) versions that
	 * were added to the repository since the last run
	 * </p>
	 * 
	 * Experiments created by the version provider will be
	 * derived from the model experiment, the version provider
	 * is expected to perform plugin-dependent modification of
	 * the basic configuration. It also may use some properties
	 * created by plugin configurator for its own use.
	 * 
	 * <code>historicalStart</code> designates a point in history
	 * before which versions from repository should be not detected 
	 * (i.e. ignored) by the version provider
	 * 
	 * <p>
	 * To assure consistency (i.e. no version is skipped), when version provider
	 * is not able to provide all versions from repository (e.g. network error),
	 * it should throw an exception. In such case BenchmarkManager will
	 * restart the version provider according to schedule, but will not update
	 * the historical start, so that version provider has a chance to detect
	 * previously missed versions of software.
	 * </p>
	 * 
	 * <code>processedVersions</code> parameter is read-write list of versions that
	 * version provider already detected. During its execution, it should add
	 * newly detected versions to the list.
	 * 
	 * <code>lastRun</code> argument is the timestamp of the last execution of
	 * version provider. When it is running for the first time, the argument is
	 * set to {@link #FIRST_RUN} constant.
	 * 
	 * The list of experiments should be ordered by software version
	 * ascending (e.g. 1.4, 1.5)
	 * 
	 * 
	 * 
	 * @return list of unprocessed versions, sorted by version
	 * @throws Exception	exception indicating that version provider could not fetch
	 * 						all available versions
	 */
	public abstract ExperimentMetadata[] getVersions() throws Exception;
	
	public void setBenchmarkManager(BenchmarkManagerLocalInterface bm) {
		this.benchmarkManager = bm;
	}
	
	/**
	 * @return list of versions processed in this run
	 */
	public abstract Version[] getProcessedVersions();
	
	protected ExperimentMetadata duplicateModel() {
		return new ExperimentMetadata(benchmarkManager.getIDManager().getNext(EID.class),model);
	}

	public void setHistoricalStart(long historicalStart) {
		this.historicalStart = historicalStart;
	}
	
	public void setHistoricalEnd(long historicalEnd) {
		this.historicalEnd = historicalEnd;
	}
	
	// ------------------- SETTERS AS WE DON'T WANT TO USE PARAMETRIC CONSTRUCTOR-----------------------
	
	public void setLastRun(long lastRun) {
		this.lastRun = lastRun;
	}
	
	public void setModel(ExperimentMetadata model) {
		this.model = model;
	}
	
	/**
	 * Sets version that were successfully processed by the provider in all its previous runs
	 * Will be set to an empty array, in the first run (no versions were processed)
	 * 
	 * @param processedVersions	list of processed versions or <code>Version[0]</code> in the first run
	 */
	public void setProcessedVersions(Version[] processedVersions) {
		this.processedVersions = processedVersions;
	}
	
	public void setSchedule(SchedulerInfo schedule) {
		this.schedule = schedule;
	}

	// ---------------- LOGGING METHODS ------------------
	protected void logDebug(String debugMessage) {
		benchmarkManager.logDebug(debugMessage);
	}


	protected void logError(String errorMessage) {
		benchmarkManager.logError(errorMessage);
	}


	protected void logInfo(String infoMessage) {
		benchmarkManager.logInfo(infoMessage);
	}


	protected void logWarning(String warningMessage) {
		benchmarkManager.logWarning(warningMessage);
	}
	
	
}
