/*
 * Created on Nov 7, 2006
 *
 * BEEN
 * @author Jaroslav Urban
 */
package cz.cuni.mff.been.benchmarkmanager.plugins.simpletest;

import cz.cuni.mff.been.benchmarkmanager.GeneratorException;
import cz.cuni.mff.been.benchmarkmanager.StandardRole;
import cz.cuni.mff.been.benchmarkmanager.plugins.TaskGenerator;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.data.TaskExclusivity;

/**
 * Tak generator for a simple benchmark plugin test.
 * 
 * This plugin runs 4 tasks:
 * 
 * <b>Task 1</b>: gets the value of the task property "test.property". The value
 * is a String array with 2 elements, "Hello" and "World". It also sets a 
 * checkpoint with the name "data" and with a String array as value, with "BEEN"
 * and "DSRG" elements.
 * 
 * <b>Task 2</b>: Starts after the first task successfully finishes. It gets the 
 * value of the checkpoint "data", which was set by the first task. It also 
 * sleeps for a the time which was set by the user in the experiment 
 * configuration.
 * 
 * <b>Task 3</b>: Starts after the second task. It sets a checkpoint with the 
 * name "go" and with a String array as value, with "BEEN" and "DSRG" elements.
 * 
 * <b>Task 4</b>: Starts immediately, and blocks on the "go" checkpoint which
 * will be set by the third task.
 * 
 * @author Jaroslav Urban
 */
public class SimpleTestGenerator extends TaskGenerator {
	/** Benchmark property specifying the wait time of one task. */
	public static final String WAIT_TIME = "wait.time";
	/** Benchmark property specifying whether exclusive tasks should be used. */
	public static final String EXCLUSIVITY = "exclusivity";
	/** Name of the benchmark role where this test will run. */
	public static final String TEST_ROLE = "Test";
	
	/**
	 * Exclusivity of tasks as set by the user.
	 * 
	 * @author Jaroslav Urban
	 */
	public enum Exclusivity {
		/** Tasks will no be exclusive. */
		NONE("none"),
		/** Tasks will be context exclusive. */
		CONTEXT_EXCLUSIVE("context exclusive"),
		/** Tasks will be exclusive. */
		EXCLUSIVE("exclusive");
		
		private String name;
		
		private Exclusivity(String name) {
			this.name = name;
		}
		
		public String toString() {
			return name;
		}
	}
	
	private Exclusivity exclusivity;
	
	
	@Override
	protected void registerResults(StandardRole[] roles) throws GeneratorException {
		// no results will be uploaded by this generator
	}
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.benchmarkmanager.plugins.TaskGenerator#generate()
	 */
	@Override
	protected void generate() throws GeneratorException {
		setActiveRole(TEST_ROLE);
		
		if (getExperiment().getProperty(EXCLUSIVITY).
				equals(Exclusivity.NONE.toString())) {
			exclusivity = Exclusivity.NONE;
		}
		if (getExperiment().getProperty(EXCLUSIVITY).
				equals(Exclusivity.CONTEXT_EXCLUSIVE.toString())) {
			exclusivity = Exclusivity.CONTEXT_EXCLUSIVE;
		}
		if (getExperiment().getProperty(EXCLUSIVITY).
				equals(Exclusivity.EXCLUSIVE.toString())) {
			exclusivity = Exclusivity.EXCLUSIVE;
		}
		
		for (int i = 0; i < getExperiment().getRole(TEST_ROLE).getCount(); i++) {
			TaskDescriptor task1 = createTaskDescriptor("testworker", i);
			task1.addTaskProperties("do.property.value", "true");
			task1.addTaskProperties("property.name", "test.property");
			task1.addTaskPropertyObject("test.property", new String[] {"Hello", "World"});
			task1.addTaskProperties("do.checkpoint.set", "true");
			task1.addTaskProperties("checkpoint.name", "data");
			//setExclusivity(task1);
			addTask(task1);

			TaskDescriptor taskSleeper = createTaskDescriptor("testworker", i);
			taskSleeper.addTaskProperties("do.wait", "true");
			taskSleeper.addTaskProperties("wait.time", getExperiment().getProperty(WAIT_TIME));
			taskSleeper.addTaskProperties("do.checkpoint.value", "true");
			taskSleeper.addTaskProperties("checkpoint.name", "data");
			taskSleeper.addTaskProperties("checkpoint.task", task1.getTaskTid());
			taskSleeper.addDependencyCheckPoint(
					success(task1.getTaskTid()));
			setExclusivity(taskSleeper);
			addTask(taskSleeper);

			TaskDescriptor task3 = createTaskDescriptor("testworker", i);
			task3.addTaskProperties("do.checkpoint.set", "true");
			task3.addTaskProperties("checkpoint.name", "go");
			task3.addDependencyCheckPoint(
					success(taskSleeper.getTaskTid()));
			//setExclusivity(task3);
			addTask(task3);

			TaskDescriptor taskWaiter = createTaskDescriptor("testworker", i);
			taskWaiter.addTaskProperties("do.checkpoint.block", "true");
			taskWaiter.addTaskProperties("checkpoint.name", "go");
			taskWaiter.addTaskProperties("checkpoint.task", task3.getTaskTid());
			//setExclusivity(taskWaiter);
			addTask(taskWaiter);
			
			TaskDescriptor previousbinary = null;
			for (int j = 0; j < getExperiment().getBinaryCount(); j++) {
				createNewBinary();
				TaskDescriptor taskForBinary = createTaskDescriptor("testworker", i);
				taskForBinary.addTaskProperties("do.wait", "true");
				taskForBinary.addTaskProperties("wait.time", 
						getExperiment().getProperty(WAIT_TIME));
				if (j == 0) {
					taskForBinary.addDependencyCheckPoint(
							success(taskWaiter.getTaskTid()));
				} else {
					taskForBinary.addDependencyCheckPoint(
							success(previousbinary.getTaskTid()));
				}
				addTask(taskForBinary);
				previousbinary = taskForBinary; // must do this when run count is 0

				TaskDescriptor previousRun = null;
				for (int k = 0; k < getExperiment().getRunCount(); k++) {
					createNewRun();
					TaskDescriptor taskForRun = createTaskDescriptor("testworker", i);
					taskForRun.addTaskProperties("do.wait", "true");
					taskForRun.addTaskProperties("wait.time", 
							getExperiment().getProperty(WAIT_TIME));
					if (k == 0) {
						taskForRun.addDependencyCheckPoint(
								success(taskForBinary.getTaskTid()));
					} else {
						taskForRun.addDependencyCheckPoint(
								finish(previousRun.getTaskTid()));
					}
					setExclusivity(taskForRun);
					addTask(taskForRun);
					previousRun = taskForRun;
					closeRun();
				}
				previousbinary = previousRun;
				closeBinary();
			}
		}
	}
	
	/**
	 * Sets tasks' exclusivity according to user's choice in the configurator.
	 * 
	 * @param task
	 */
	private void setExclusivity(TaskDescriptor task) {
		if (exclusivity == Exclusivity.CONTEXT_EXCLUSIVE) {
			task.setTaskExclusive(TaskExclusivity.CONTEXT_EXCLUSIVE);
		} else if (exclusivity == Exclusivity.EXCLUSIVE) {
			task.setTaskExclusive(TaskExclusivity.EXCLUSIVE);
		} 
	}

}
