/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.benchmarkmanager.plugins.xampler;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import cz.cuni.mff.been.benchmarkmanager.BenchmarkRole;
import cz.cuni.mff.been.benchmarkmanager.GeneratorException;
import cz.cuni.mff.been.benchmarkmanager.StandardRole;
import cz.cuni.mff.been.benchmarkmanager.plugins.TaskGenerator;
import cz.cuni.mff.been.benchmarkmanager.plugins.xampler.XamplerConfigurator.XamplerSuite;
import cz.cuni.mff.been.common.StringUtils;
import cz.cuni.mff.been.common.TaskPathBuilder;
import cz.cuni.mff.been.common.Version;
import cz.cuni.mff.been.common.rsl.AndCondition;
import cz.cuni.mff.been.common.rsl.Condition;
import cz.cuni.mff.been.common.rsl.ContainsCondition;
import cz.cuni.mff.been.common.rsl.EqualsCondition;
import cz.cuni.mff.been.softwarerepository.PackageMetadata;
import cz.cuni.mff.been.softwarerepository.PackageType;
import cz.cuni.mff.been.taskmanager.Dependency;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.data.TaskExclusivity;

public class XamplerTaskGenerator extends TaskGenerator {
	
	/* Constants */
	public static final String XAMPLER_SUITE = "xampler.suite";
	private static final String XAMPLER_SUBSUITE = "xampler.subsuite";
	private static final String XAMPLER_VERSION = "xampler.version";
	private static final String SOFTWARE_VERSION = "software.version";
	private static final String OMNIORB_BRANCH = "software.branch";
	
	public static final String XAMPLER_SERVER = "Xampler Server";
	public  static final String XAMPLER_CLIENT = "Xampler Client";
	public  static final String SERVER_COMPILE = "Server Compile";
	public static final String CLIENT_COMPILE = "Client Compile";
	
	// basic conditions for matching omniorb packages
	private static /*final*/ Condition[] OMNIORB_BASIC_CONDITIONS = null;
	
	// basic conditions for matching xampler packages
	private static /*final*/ Condition[] XAMPLER_BASIC_CONDITIONS = null;
	
	// formatting for cvs-supported timestamps
	private static final SimpleDateFormat CVS_DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	private static final long DETAILED_LOAD_INTERVAL = 500;
	
	private final TaskPathBuilder pathBuilder = new TaskPathBuilder(false); 
	
	private int runCount;
	private int binaryCount;
	private int sampleCount;
	
	private String omniorbVersion;
	private String omniorbBranch;
	
	private String xamplerVersion;
	private String xamplerSuite;
	
	private String suitePath;
	
	private String serverParams;
	private String clientParams;
	private int clientsCount;
	
	/** Host that are available for log collection */
	private HashSet<String> availableLoadHosts = new HashSet<String>();
	private String globalBinaryIdentifier = null;
	
	
	public XamplerTaskGenerator() {
	}
	
	/*
	 * We collect results only from xampler client roles and we will identify them
	 * by numbers 
	 */
	@Override
	protected void registerResults(StandardRole[] roles) throws GeneratorException {
		for (StandardRole br : roles) {
			if (XAMPLER_CLIENT.equals(br.getName())) {
				BenchmarkRole client = (BenchmarkRole)br;
				for (int i=0;  i<br.getCount(); i++) {
					addResultsMapping(XAMPLER_CLIENT + i, client.getUserHosts()[i]);
				}
			}
		}
	}
	
	public void generate() throws GeneratorException {
		
		validateParameters();
		setup();
		
		if (isRegression()) {
			generateRegression();
		} else {
			generateComparison();
		}
	}

	private void setup() throws GeneratorException {
		
		// required property for regression experiment
		if (isRegression() && StringUtils.isEmpty(getExperiment().getProperty("cvs.date"))) {
			throw new GeneratorException("Timestamp revision property 'cvs.date' not set");
		}
		
		// recurrency constants
		runCount = getExperiment().getRunCount();
		binaryCount = getExperiment().getBinaryCount();
		sampleCount = getExperiment().getSamplesCount();
		
		// number of clients to create
		clientsCount = getExperiment().getRole(XAMPLER_CLIENT).getCount();
		
		// versions and suite
		setOmniorbVersion(getExperiment().getProperty(SOFTWARE_VERSION,"4.0.5"));
		// by default we will use 4.0 branch
		setOmniorbBranch(getExperiment().getProperty(OMNIORB_BRANCH,"4.0"));
		setXamplerVersion(getExperiment().getProperty(XAMPLER_VERSION,"1.11"));
	
		xamplerSuite = getExperiment().getProperty(XAMPLER_SUITE,"Ping");
		suitePath = xamplerSuite;
		
		Version version = new Version(getOmniorbVersion());
		if (version.compareTo(new Version("4.1.0")) < 0) {
			suitePath = suitePath + "/" + "omniORB_4.0.X_Linux_GCC";
		} else {
			suitePath = suitePath + "/" + "omniORB_4.1.X_Linux_GCC";
		}
		
		// does the suite use a subsuite?
		if (getExperiment().getProperty(XAMPLER_SUBSUITE) != null) {
			suitePath =  suitePath +  "/" + getExperiment().getProperty(XAMPLER_SUBSUITE);
		}
		
		// runtime parameters - when property is not set, we will pass empty command line
		serverParams = getExperiment().getProperty("xampler.server.params","");
		clientParams = getExperiment().getProperty("xampler.client.params","");
		
		// package matching conditions
		OMNIORB_BASIC_CONDITIONS = new Condition[]{
				new EqualsCondition<String>("name","omniorb"),
				new ContainsCondition<String>("softwarePlatforms", "linux"),
		};
		
		XAMPLER_BASIC_CONDITIONS = new Condition[]{
				new EqualsCondition<String>("name","xampler"),
				new EqualsCondition<Version>("version",new Version(getXamplerVersion())),
				new ContainsCondition<String>("softwarePlatforms", "linux"),
		};
		
	}
	
	
	private void setupLoadHosts() {
		// fill-in unique hostnames for benchmark roles that we will later collect load from while benchmark runs
		for (StandardRole role : getExperiment().getRoles()) {
			if (role.isBenchmarking()) {
				availableLoadHosts.addAll(Arrays.asList(((BenchmarkRole)role).getUserHosts()));
			}
		}
	}
	
	private TaskDescriptor omniorbSource() {
		setActiveRole(SERVER_COMPILE);
		
		TaskDescriptor checkout = createTaskDescriptor("native-cvs-download");

		// create revision timestamp
		String timestamp = getExperiment().getProperty("cvs.date");
		
		Date timeStampDate = new Date(Long.parseLong(timestamp));
		String checkoutDate = CVS_DATE_FORMAT.format(timeStampDate);

		// checkout sources from repository
		checkout.addTaskProperties("repository", getExperiment()
				.getProperty("cvs.repository"));
		checkout.addTaskProperties("module", "omni");
		checkout.addTaskProperties("revision", getExperiment()
				.getProperty("cvs.branch"));
		checkout.addTaskProperties("date", checkoutDate);
		addTask(checkout);

		// pack them to standard tar.gz format
		TaskDescriptor createPackage = createTaskDescriptor("omniorb-source-package");
		createPackage.setHostRuntimesAsTask(checkout.getTaskTid());
		createPackage.addTaskProperties("omniorb.source.root", pathBuilder.addWorking(checkout.getTaskTid()).clean());
		addAfterLast(createPackage);

		// create metadata
		TaskDescriptor metadata = createTaskDescriptor("create-package-metadata");
		metadata.setHostRuntimesAsTask(checkout.getTaskTid());
		metadata.addTaskProperties("package-name", "omniorb");
		metadata.addTaskProperties("package-human-name","omniORB: High performance linux ORB");
		metadata.addTaskProperties("version", timestamp);
		metadata.addTaskProperties("hardware-platforms", "i386");
		metadata.addTaskProperties("software-platforms", "linux win32");
		metadata.addTaskProperties("type", PackageType.SOURCE.toString());
		addTask(metadata);

		// upload to software repository
		TaskDescriptor upload = createTaskDescriptor("localupload");
		upload.setHostRuntimesAsTask(checkout.getTaskTid());
		upload.addDependencyCheckPoint(success(createPackage.getTaskTid()));
		upload.addDependencyCheckPoint(success(metadata.getTaskTid()));

		upload.addTaskProperties("dir", pathBuilder.addWorking(
				createPackage.getTaskTid()).clean());
		upload.addTaskProperties("metadata.file", pathBuilder
				.addWorking(metadata.getTaskTid()).add("metadata.xml")
				.clean());
		addTask(upload);
		
		return upload;
		
	}
	
	private TaskDescriptor omniorbBuild(TaskDescriptor sourceUpload) {
			
			/*
			 * Tasks withing the method must run all on the same host so all must 
			 * "use the same host runtime as download"
			 */
		
			// compiling server
			setActiveRole(SERVER_COMPILE);

			// omniorb source download
			TaskDescriptor download = createTaskDescriptor("local-download");
			download.addTaskPropertyObject("rsl",omniorbRSL(PackageType.SOURCE));
			
			// dependency on possible source package creation
			if (sourceUpload != null) {
				download.addDependencyCheckPoint(success(sourceUpload.getTaskTid()));
			}
			addTask(download);

			// create metadata
			TaskDescriptor metadata = createTaskDescriptor("create-package-metadata");
			metadata.setHostRuntimesAsTask(download.getTaskTid());
			
			metadata.addTaskProperties("package-name", "omniorb");
			metadata.addTaskProperties("package-human-name", "omniORB: High performance linux ORB");
			metadata.addTaskProperties("version", getOmniorbVersion());
			metadata.addTaskProperties("hardware-platforms", "i386");
			metadata.addTaskProperties("software-platforms", "linux win32");
			metadata.addTaskProperties("type", PackageType.BINARY.toString());
			metadata.addTaskProperties("binary-identifier",getOpenBinary().toString());
			
			
			addTask(metadata);


			// omniorb compile
			TaskDescriptor compile = createTaskDescriptor("omniorb-compile-linux");
			compile.addDependencyCheckPoint(success(download.getTaskTid()));
			compile.setHostRuntimesAsTask(download.getTaskTid());
			
			compile.addTaskProperties("source.dir",
					pathBuilder.addWorking(download.getTaskTid()).clean());
			compile.addTaskProperties("omniorb.root", "omniorb");
			addTask(compile);
			
			// omniorb binary package upload 
			TaskDescriptor upload = createTaskDescriptor("localupload");
			upload.setHostRuntimesAsTask(download.getTaskTid());
			upload.addDependencyCheckPoint(success(compile.getTaskTid()));
			upload.addDependencyCheckPoint(success(metadata.getTaskTid()));
		
			// dependency: metadata creation
			upload.addTaskProperties("dir",
					pathBuilder.addWorking(compile.getTaskTid()).clean());
			upload.addTaskProperties("metadata.file",
					pathBuilder.addWorking(metadata.getTaskTid()).add("metadata.xml").clean());
			
			addTask(upload);
			
			return upload;
	}
	
	private TaskDescriptor xamplerBuild(TaskDescriptor omniorbUpload) {
			/*
			 * Tasks withing the method must run all on the same host so all must 
			 * "use the same host runtime as XAMPLER download"
			 */
		
			setActiveRole(SERVER_COMPILE);

			// xampler source download
			TaskDescriptor xamplerDownload = createTaskDescriptor("local-download");
			xamplerDownload.addTaskPropertyObject("rsl", xamplerRSL(PackageType.SOURCE));
			addTask(xamplerDownload);

			TaskDescriptor omniorbDownload = createTaskDescriptor("local-download");
			omniorbDownload.setHostRuntimesAsTask(xamplerDownload.getTaskTid());
			
			// if we compile omniorb, we must wait for binary to be uploaded
			if (omniorbUpload != null) {
				omniorbDownload.addDependencyCheckPoint(success(omniorbUpload.getTaskTid()));
			}
			
			// when we compiled omniorb, we need to use the same binary
			omniorbDownload.addTaskPropertyObject("rsl", omniorbBinaryRSL());
			
			
			addTask(omniorbDownload);
			
			// create metadata
			TaskDescriptor metadata = createTaskDescriptor("create-package-metadata");
			metadata.setHostRuntimesAsTask(xamplerDownload.getTaskTid());
			
			metadata.addTaskProperties("package-name", "xampler");
			metadata.addTaskProperties("package-human-name", "Xampler C/C++ CORBA benchmark");
			metadata.addTaskProperties("version", getXamplerVersion());
			metadata.addTaskProperties("hardware-platforms", "i386");
			metadata.addTaskProperties("software-platforms", "linux win32");
			metadata.addTaskProperties("type", PackageType.BINARY.toString());
			metadata.addTaskProperties("binary-identifier",getOpenBinary().toString());
			addTask(metadata);

			// compile xampler
			TaskDescriptor compile = createTaskDescriptor("xampler-compile-linux");
			compile.setHostRuntimesAsTask(xamplerDownload.getTaskTid());
			
			// dependency: omniorb binary download
			compile.addDependencyCheckPoint(success(omniorbDownload.getTaskTid()));
			compile.addDependencyCheckPoint(success(xamplerDownload.getTaskTid()));
			addTask(compile);
			
			compile.addTaskProperties("xampler.dir",pathBuilder.addWorking(xamplerDownload.getTaskTid()).clean());
			compile.addTaskProperties("omniorb.version",getOmniorbBranch());
			compile.addTaskProperties("omniorb.root",pathBuilder.addWorking(omniorbDownload.getTaskTid()).clean());
		
			// set benchmark constants
			for (String[] constant : XamplerSuite.getCommonConstants()) {
				compile.addTaskProperties(constant[0],
						getExperiment().getProperty(constant[0],constant[1]));
			}
			

			// xampler binary upload
			TaskDescriptor upload = createTaskDescriptor("localupload");
			upload.setHostRuntimesAsTask(xamplerDownload.getTaskTid());
			
			// dependency: metadata & compilation
			upload.addDependencyCheckPoint(success(compile.getTaskTid()));
			upload.addDependencyCheckPoint(success(metadata.getTaskTid()));

			upload.addTaskProperties("dir",
					pathBuilder.addWorking(compile.getTaskTid()).clean());
			upload.addTaskProperties("metadata.file",
					pathBuilder.addWorking(metadata.getTaskTid()).add("metadata.xml").clean());
			addTask(upload);
			
			return upload;
	}
	
	private TaskDescriptor deployOmniORBServer(TaskDescriptor omniorbUpload) {
		setActiveRole(XAMPLER_SERVER);

		TaskDescriptor download = createTaskDescriptor("local-download");
		download.addTaskPropertyObject("rsl", omniorbBinaryRSL());
		
		if (omniorbUpload != null) {
			download.addDependencyCheckPoint(success(omniorbUpload.getTaskTid()));
		}
		
		addTask(download);
		
		return download;
	}
	
	private TaskDescriptor deployOmniorbClient(TaskDescriptor omniorbUpload, int clientIndex) {
			setActiveRole(XAMPLER_CLIENT);
			
			TaskDescriptor download = createTaskDescriptor("local-download", clientIndex);
			download.addTaskPropertyObject("rsl", omniorbBinaryRSL());
			
			if (omniorbUpload != null) { 
				download.addDependencyCheckPoint(
					success(omniorbUpload.getTaskTid()));
			}
			addTask(download);
			
			return download;
	}
	
	private TaskDescriptor deployXamplerServer(TaskDescriptor xamplerUpload) {
		setActiveRole(XAMPLER_SERVER);
		
		TaskDescriptor download = createTaskDescriptor("local-download");
		download.addTaskPropertyObject("rsl", xamplerBinaryRSL());
		
		if (xamplerUpload != null) {
			download.addDependencyCheckPoint(success(xamplerUpload.getTaskTid()));
		}
		
		addTask(download);
		
		return download;
	}
	
	private TaskDescriptor deployXamplerClient(TaskDescriptor xamplerUpload, int clientIndex) {
		setActiveRole(XAMPLER_CLIENT);
		
		TaskDescriptor download = createTaskDescriptor("local-download", clientIndex);
		download.addTaskPropertyObject("rsl", xamplerBinaryRSL());
		
		if (xamplerUpload != null) {
			download.addDependencyCheckPoint(success(xamplerUpload.getTaskTid()));
		}
		
		addTask(download);
		
		return download;
	}

	private TaskDescriptor generateServerExecute(List<TaskDescriptor> resultUploads,TaskDescriptor omniDeploy, TaskDescriptor xamplerDeploy) 
	throws GeneratorException {

		setActiveRole(XAMPLER_SERVER);

		TaskDescriptor serverExecDesc = createTaskDescriptor("xampler-execute");
		
		serverExecDesc.setRestartMax(3);
		serverExecDesc.setTaskExclusive(TaskExclusivity.CONTEXT_EXCLUSIVE);
		
		/*
		 *  Server should not start until previous results upload is finished
		 *  We are using context-exclusive tasks, so we can't block on
		 *  client/server otherwise log-collectors would be starting
		 *  together with new servers/clients
		 */
		for (TaskDescriptor upload : resultUploads) {
			serverExecDesc.addDependencyCheckPoint(finish(upload.getTaskTid()));
		}
		
		/*
		 * Register for load collection if the host is not occupied by different a task
		 */
		String usedHost = ((BenchmarkRole)getExperiment().getRole(XAMPLER_SERVER)).getUserHosts()[0];
		if (availableLoadHosts.contains(usedHost)) {
	 		serverExecDesc.setDetailedLoad(true);
			serverExecDesc.setDetailedLoadInterval(500);
			availableLoadHosts.remove(usedHost);
		}
		
		serverExecDesc.addTaskProperties("omniorb.root",
				pathBuilder.addWorking(omniDeploy.getTaskTid()).clean());					
		serverExecDesc.addTaskProperties("xampler.root",
				pathBuilder.addWorking(xamplerDeploy.getTaskTid()).clean());
		serverExecDesc.addTaskProperties("xampler.role","server");
		serverExecDesc.addTaskProperties("xampler.suite.path", suitePath);
		serverExecDesc.addTaskProperties("xampler.server.params", serverParams);

		addTask(serverExecDesc);
		
		return serverExecDesc;
		
	}
	
	private TaskDescriptor generateClientExecute(int clientIndex, TaskDescriptor serverExec, TaskDescriptor omniorbDeploy, TaskDescriptor xamplerDeploy)  
	throws GeneratorException {
			setActiveRole(XAMPLER_CLIENT);
			TaskDescriptor clientExecDesc = createTaskDescriptor("xampler-execute",clientIndex);
			
			clientExecDesc.addDependencyCheckPoint(success(omniorbDeploy.getTaskTid()));
			clientExecDesc.addDependencyCheckPoint(success(xamplerDeploy.getTaskTid()));
			/*
			 *  This prevents clients from the same context starting while their server 
			 *  has not yet started and all synchronization happens on the server so this must be present
			 */
			clientExecDesc.addDependencyCheckPoint(new Dependency(serverExec.getTaskTid(),"server.started"));
			
			clientExecDesc.setRestartMax(3);
			clientExecDesc.setTaskExclusive(TaskExclusivity.CONTEXT_EXCLUSIVE);
			
			
			/*
			 *  Register for load collection if no other task occupies the machine
			 */
			String usedHost = ((BenchmarkRole)getExperiment().getRole(XAMPLER_CLIENT)).getUserHosts()[clientIndex];
			if (availableLoadHosts.contains(usedHost)) {
				clientExecDesc.setDetailedLoad(true);
				clientExecDesc.setDetailedLoadInterval(DETAILED_LOAD_INTERVAL);
				availableLoadHosts.remove(usedHost);
			}

			clientExecDesc.addTaskProperties("omniorb.root", 
					pathBuilder.addWorking(omniorbDeploy.getTaskTid()).clean());					
			clientExecDesc.addTaskProperties("xampler.root",
					pathBuilder.addWorking(xamplerDeploy.getTaskTid()).clean());
			clientExecDesc.addTaskProperties("xampler.role", "client");
			clientExecDesc.addTaskProperties("xampler.suite.path", suitePath);
			clientExecDesc.addTaskProperties("server.tid", serverExec.getTaskTid());
			clientExecDesc.addTaskProperties("xampler.client.params",clientParams);
			
			addTask(clientExecDesc);
			
			return clientExecDesc;
	}

	public void generateComparison() throws GeneratorException {
		LinkedList<TaskDescriptor> uploads = new LinkedList<TaskDescriptor>();
		LinkedList<TaskDescriptor> checks = new LinkedList<TaskDescriptor>();
		LinkedList<TaskDescriptor> allDeploys = new LinkedList<TaskDescriptor>();
		LinkedList<TaskDescriptor> clientDeploys = new LinkedList<TaskDescriptor>();
		TaskDescriptor omniSrc = null;
		TaskDescriptor omniBin = null;
		TaskDescriptor xamplerBin = null;
		TaskDescriptor omniServDeploy = null;
		TaskDescriptor xamServDeploy = null;
		TaskDescriptor serverExec = null;
		HashSet<String> usedBinaries = new HashSet<String>();
		
		/* 
		 * We will check at first whether we have a matching pair of binaries
		 * NO: If not, we will start compilations
		 * YES: We will use them
		 */
		
		for (int binaryIdx = 0; binaryIdx < binaryCount; binaryIdx++) {
			globalBinaryIdentifier = searchForBinaryPair(usedBinaries);
			createNewBinary();
			
			// we don't have binary and will create new - tag it as used for the next binary iteration
			if (globalBinaryIdentifier == null) {
				if (!isPackageAvailable(omniorbRSL(PackageType.SOURCE))) {
					String currentVersion = String.valueOf(System.currentTimeMillis());
					setOmniorbVersion(currentVersion);
					getExperiment().setProperty("cvs.date", currentVersion);
					omniSrc = omniorbSource();
				}
				usedBinaries.add(getOpenBinary().toString());
				omniBin = omniorbBuild(omniSrc);
				xamplerBin = xamplerBuild(omniBin);
			} else {
				usedBinaries.add(globalBinaryIdentifier);
			}
			
			
			/*
			 * Clear previous deploys
			 */
			allDeploys.clear();
			clientDeploys.clear();
			
			/*
			 * Deploy server
			 */
			allDeploys.add(omniServDeploy = deployOmniORBServer(omniBin));
			allDeploys.add(xamServDeploy = deployXamplerServer(xamplerBin));
			
			/*
			 * Deploys all clients
			 */
			for (int clientIdx=0; clientIdx < clientsCount; clientIdx++) {
				TaskDescriptor omniDeploy = deployOmniorbClient(omniBin,clientIdx);
				TaskDescriptor xamplerDeploy = deployXamplerClient(xamplerBin,clientIdx);
				clientDeploys.add(omniDeploy);
				clientDeploys.add(xamplerDeploy);
			}
			
			/*
			 * Merge deploys so that server exec can synchronize on them
			 */
			allDeploys.addAll(clientDeploys);
			
			for (int i = 0; i < runCount; i++) {
				createNewRun();
				setupLoadHosts();
				/*
				 * Server exec depends on successfull deployment of server & clients
				 * as well as all results uploads to finish
				 * This also causes that the first server executed in binary
				 * synchronized on uploads of the last server executed in the previous binary
				 */
				serverExec = generateServerExecute(uploads,omniServDeploy,xamServDeploy);
				for (TaskDescriptor deploy : allDeploys) {
					serverExec.addDependencyCheckPoint(success(deploy.getTaskTid()));
				}
				/*
				 * Result uploads are not needed anymore
				 */
				uploads.clear();
				
				/*
				 * Client execs - need deployment descriptors (they were stored in a sequence)
				 */
				for (int clientIdx=0; clientIdx < clientsCount; clientIdx++) {
					TaskDescriptor clientExec = 
						generateClientExecute(clientIdx, serverExec, clientDeploys.get(2*clientIdx), clientDeploys.get(2*clientIdx+1));
					uploads.add(generateResultsCollection(clientIdx, clientExec));
				}
				
				/*
				 * Uploads may start only when all checks are finished in current run
				 * 1. Find all checks in current run
				 * 2. Go through uploads and set them dependant on all checks
				 */
				checks.clear();
				for (TaskDescriptor td : getExperiment().getTaskSequence().getTasks(getOpenRun())) {
					if ("check-and-convert-xampler".equals(td.getTaskName())) {
						checks.add(td);
					}
				}
				for (TaskDescriptor upload : uploads) {
					for (TaskDescriptor check : checks) {
						upload.addDependencyCheckPoint(finish(check.getTaskTid()));
					}
				}
				
				/*
				 * End of upload - check dependencies
				 */
				closeRun();
			}
			closeBinary();
		}
	}
	
	public void generateRegression() throws GeneratorException {
		LinkedList<TaskDescriptor> uploads = new LinkedList<TaskDescriptor>();
		LinkedList<TaskDescriptor> checks = new LinkedList<TaskDescriptor>();
		LinkedList<TaskDescriptor> allDeploys = new LinkedList<TaskDescriptor>();
		LinkedList<TaskDescriptor> clientDeploys = new LinkedList<TaskDescriptor>();
		TaskDescriptor omniSrc = null;
		TaskDescriptor omniBin = null;
		TaskDescriptor xamplerBin = null;
		TaskDescriptor omniServDeploy = null;
		TaskDescriptor xamServDeploy = null;
		TaskDescriptor serverExec = null;
		
		/* 
		 * Create new binary of Xampler for:
		 * 	1, every new binary of omniorb
		 * 	2, omniorb binary without existing xampler binary
		 */
		omniSrc = omniorbSource();
		for (int binaryIdx = 0; binaryIdx < binaryCount; binaryIdx++) {
			createNewBinary();
			omniBin = omniorbBuild(omniSrc);
			xamplerBin = xamplerBuild(omniBin);
			
			/*
			 * Clear previous deploys
			 */
			allDeploys.clear();
			clientDeploys.clear();
			
			/*
			 * Deploy server
			 */
			allDeploys.add(omniServDeploy = deployOmniORBServer(omniBin));
			allDeploys.add(xamServDeploy = deployXamplerServer(xamplerBin));
			
			/*
			 * Deploys all clients
			 */
			for (int clientIdx=0; clientIdx < clientsCount; clientIdx++) {
				TaskDescriptor omniDeploy = deployOmniorbClient(omniBin,clientIdx);
				TaskDescriptor xamplerDeploy = deployXamplerClient(xamplerBin,clientIdx);
				clientDeploys.add(omniDeploy);
				clientDeploys.add(xamplerDeploy);
			}
			
			/*
			 * Merge deploys so that server exec can synchronize on them
			 */
			allDeploys.addAll(clientDeploys);
			
			for (int i = 0; i < runCount; i++) {
				createNewRun();
				setupLoadHosts();
				/*
				 * Server exec depends on successfull deployment of server & clients
				 * as well as all results uploads to finish
				 * This also causes that the first server executed in binary
				 * synchronized on uploads of the last server executed in the previous binary
				 */
				serverExec = generateServerExecute(uploads,omniServDeploy,xamServDeploy);
				for (TaskDescriptor deploy : allDeploys) {
					serverExec.addDependencyCheckPoint(success(deploy.getTaskTid()));
				}
				/*
				 * Result uploads are not needed anymore
				 */
				uploads.clear();
				
				/*
				 * Client execs - need deployment descriptors (they were stored in a sequence)
				 */
				for (int clientIdx=0; clientIdx < clientsCount; clientIdx++) {
					TaskDescriptor clientExec = 
						generateClientExecute(clientIdx, serverExec, clientDeploys.get(2*clientIdx), clientDeploys.get(2*clientIdx+1));
					uploads.add(generateResultsCollection(clientIdx, clientExec));
				}
				
				/*
				 * Uploads may start only when all checks are finished in current run
				 * 1. Find all checks in current run
				 * 2. Go through uploads and set them dependant on all checks
				 */
				checks.clear();
				for (TaskDescriptor td : getExperiment().getTaskSequence().getTasks(getOpenRun())) {
					if ("check-and-convert-xampler".equals(td.getTaskName())) {
						checks.add(td);
					}
				}
				for (TaskDescriptor upload : uploads) {
					for (TaskDescriptor check : checks) {
						upload.addDependencyCheckPoint(finish(check.getTaskTid()));
					}
				}
				
				/*
				 * End of upload - check dependencies
				 */
				closeRun();
			}
			closeBinary();
		}
	}
	
	
	
	private TaskDescriptor generateResultsCollection(int clientIndex, TaskDescriptor clientExec) {
		setActiveRole(XAMPLER_CLIENT);

		TaskDescriptor checkAndConvert = createTaskDescriptor("check-and-convert-xampler",clientIndex);

		// dependency: xampler client run
		checkAndConvert.addDependencyCheckPoint(finish(clientExec.getTaskTid()));

		StringBuffer resultEntry = new StringBuffer();
		resultEntry.append(getOpenBinary().toString()).append("\t");
		resultEntry.append(getOpenRun().toString()).append("\t");
		resultEntry.append(pathBuilder.addWorking(clientExec.getTaskTid()).clean()).append("\n");
		
		checkAndConvert.addTaskProperties("suite", xamplerSuite);
		checkAndConvert.addTaskProperties("results.role",XAMPLER_CLIENT + clientIndex);
		checkAndConvert.addTaskProperties("samples.counts",String.valueOf(sampleCount));
		checkAndConvert.addTaskProperties("results.paths",resultEntry.toString());
		// set subsuite property only when exists
		if (StringUtils.isEmpty(getExperiment().getProperty(XAMPLER_SUBSUITE)) == false) {
			checkAndConvert.addTaskProperties("xampler.subsuite", getExperiment().getProperty(XAMPLER_SUBSUITE));
		}
		addTask(checkAndConvert);

		// DEBUG
		/*
		 * try { resultsCheckAndConvertTask.setJavaOptions("-Xdebug
		 * -Xrunjdwp:transport=dt_socket,address=8000,suspend=y,server=y",
		 * true); } catch (Exception e) {
		 *  }
		 */
		// END_DEBUG


		TaskDescriptor uploadTask = createTaskDescriptor("results-collect",clientIndex);
		uploadTask.addDependencyCheckPoint(finish(checkAndConvert.getTaskTid()));

		// DEBUG
		/*
		 * try { uploadTask.setJavaOptions("-Xdebug
		 * -Xrunjdwp:transport=dt_socket,address=8000,suspend=y,server=y",
		 * true); } catch (Exception e) {
		 *  }
		 */
		// END_DEBUG
		// NOTE: in general form, there must be list of all check and
		// convert tasks
		String resultsLocation = pathBuilder.addWorking(checkAndConvert.getTaskTid()).clean();
		uploadTask.addTaskProperties("analysis.id", getExperiment().getAnalysis().toString());
		uploadTask.addTaskProperties("experiment.id", getExperiment().getID().toString());
		uploadTask.addTaskProperties("results.paths", resultsLocation);
		
		addTask(uploadTask);
		
		return uploadTask;
	}	
	
	private void validateParameters() throws GeneratorException {
		for (String property : new String[]{XAMPLER_VERSION, SOFTWARE_VERSION}) {
			if (StringUtils.isEmpty(getExperiment().getProperty(property))) {
				throw new GeneratorException("Required experiment property '" + property + "' not set");
			}
		}
	}
	
	/*
	 * Omniorb package RSL without binary identifier
	 * Should be used when omniorb is not compiled or for source packages 
	 */
	private Condition omniorbRSL(PackageType type) {
		Condition[] conditions = new Condition[OMNIORB_BASIC_CONDITIONS.length+1];
		conditions[0] = new EqualsCondition<PackageType>("type",type);
		System.arraycopy(OMNIORB_BASIC_CONDITIONS,0,conditions,1,OMNIORB_BASIC_CONDITIONS.length);
		Condition version = new EqualsCondition<Version>("version",new Version(getOmniorbVersion()));
		return new AndCondition(new Condition[]{ new AndCondition(conditions),version});
		
	}
	/*
	 * Creates RSL of omniorb binary including binaryIdentifier
	 * Should be used, when package is requested to contain specific binaryIdentifier
	 * The identifier is set from last open binary, or overriden by globalBinaryIdentifier
	 */
	private Condition omniorbBinaryRSL() {
		String ident = globalBinaryIdentifier == null ? getOpenBinary().toString() : globalBinaryIdentifier;
		return new AndCondition(new Condition[]{
							new AndCondition(new Condition[]{omniorbRSL(PackageType.BINARY)}),
							new EqualsCondition<String>("binaryIdentifier",ident)
				});
	}
	
	/*
	 * Xampler package RSL without binary identifier
	 * Should be used when xampler is used without compilation or for source packages
	 */
	private Condition xamplerRSL(PackageType type) {
		Condition[] conditions = new Condition[XAMPLER_BASIC_CONDITIONS.length+1];
		conditions[0] = new EqualsCondition<PackageType>("type",type);
		System.arraycopy(XAMPLER_BASIC_CONDITIONS,0,conditions,1,XAMPLER_BASIC_CONDITIONS.length);
		return new AndCondition(conditions);
	}
	
	/*
	 * Creates RSL of xampler binary including binaryIdentifier
	 * Should be used, when package is requested to contain specific binaryIdentifier
	 * The identifier is set from last open binary, or may be overriden by a global binary identifier
	 */
	private Condition xamplerBinaryRSL() {
		String ident = globalBinaryIdentifier  == null ? getOpenBinary().toString() : globalBinaryIdentifier;
		return new AndCondition(new Condition[]{
							new AndCondition(new Condition[]{xamplerRSL(PackageType.BINARY)}),
							new EqualsCondition<String>("binaryIdentifier",ident)
				});
	}
	
	/*
	 * Searches for omniorb-xampler pair having the same binary identifier
	 * Returns the first pair having common identifier or null when none matches
	 */
	private String searchForBinaryPair(Set<String> used) throws GeneratorException {
		PackageMetadata[] omniorbs = availablePackages(omniorbRSL(PackageType.BINARY));
		PackageMetadata[] xamplers = availablePackages(xamplerRSL(PackageType.BINARY));
		for (PackageMetadata omniorb : omniorbs) {
			for (PackageMetadata xampler : xamplers) {
				if (omniorb.getBinaryIdentifier() != null && 
					omniorb.getBinaryIdentifier().equals(xampler.getBinaryIdentifier()) &&
					!used.contains(omniorb.getBinaryIdentifier())) {
					return omniorb.getBinaryIdentifier();
				}
			}
		}
		
		return null;
	}
	
	private String getOmniorbVersion() {
		return omniorbVersion;
	}
	
	public void setOmniorbVersion(String omniorbVersion) {
		this.omniorbVersion = omniorbVersion;
	}

	private void setXamplerVersion(String xamplerVersion) {
		this.xamplerVersion = xamplerVersion;
	}

	private String getXamplerVersion() {
		return xamplerVersion;
	}
	
	private void setOmniorbBranch(String omniorbBranch) {
		this.omniorbBranch = omniorbBranch;
	}
	
	private String getOmniorbBranch() {
		return omniorbBranch;
	}
	
	
}