package cz.cuni.mff.been.benchmarkmanager.scheduler;

import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * Cron-like scheduler for running regression experiments
 * Listeners are started in separate threads
 * 
 * @author mtomcanyi 
 * @author brepcek
 */
public class Scheduler extends Thread {

	/**
	 *	Class represents scheduler's queue
	 *	Scheduler thread needs to sleep on an object, but also own that object
	 *	Object ownage is acquired by calling synchronized method, in our case
	 *	waitOnMe() methods.
	 *
	 *	The queue elements are stored in order in which they are executed.
	 *	Ordering is done in {@link #add(SchedulerEntry)} method
	 *	
	 *	When only list field was used, we got IllegalMonitorStateException
	 *
	 *	@see Object#notify()
	 *
	 */
	private class SchedulerQueue {
	
		/**
		 * Container for entries
		 */
		private LinkedList< SchedulerEntry > list;

		/**
		 * Creates new SchedulerQueue instance
		 */
		public SchedulerQueue() {
			this.list =  new LinkedList<SchedulerEntry>();
		}
		
		/**
		 * Adds the entry to correct position according to its next
		 * execution time. Uses binary search (O(log(n)).
		 * 
		 * If more entries with the same execution time exists in the queue
		 * their order defined( implies from usage of binary search algorithm) 
		 * 
		 * Notifies thread sleeping on the queue
		 * 
		 * @param entry	entry to add
		 */
		public synchronized void add(SchedulerEntry entry) {
			int result = Collections.binarySearch(list,entry);
			int insertPoint = -result-1;
			
			if (insertPoint == 0) {
				list.add(0,entry);
				notify();
				return;
			}
			
			if (insertPoint > 0) {
				/*
				 * entry not found in the list -> add to correct index
				 * no need to call notify() as we are not changing the first entry
				 * which is being wait for
				 */
				list.add(insertPoint,entry);
				return;
			}
			
			/*
			 * result > 0:
			 * 
			 * NOT FIRST entry with the same value found in the list
			 * such values are processed one-by-one once scheduler wakes up
			 * we add new entry before the found one to avoid adding entry behind
			 * the end of the list
			 * 
			 * No need to call notify() because this entry is not first (result > 0)
			 */  
			list.add(result,entry);
		}
		
		/**
		 * @return first entry in the queue
		 */
		public synchronized SchedulerEntry first() {
			return list.get(0);
		}
		
		/**
		 * Removes the first entry in the queue
		 * Notifies thread sleeping on the queue
		 * @return the removed (first) entry
		 */
		public synchronized SchedulerEntry removeFirst() {
			SchedulerEntry e = list.get(0);
			list.remove(0);
			notify();
			return e;
		}
		
		/**
		 * Removes given entry if exists.
		 * Notifies thread sleeping on the queue, if the first entry is removed
		 */
		public synchronized void remove(SchedulerEntry entry) {
			if (list.isEmpty()) {
				return;
			}
			SchedulerEntry first = first();
			if (list.remove(entry) && first.equals(entry)) {
				notify();
			}
		}
		
		/**
		 * @return true when queue is empty
		 */
		public synchronized boolean isEmpty() {
			return list.isEmpty();
		}
		
		/**
		 * Wait method which allows scheduler to block on the queue
		 * @throws InterruptedException
		 */
		public synchronized void waitOnMe() throws InterruptedException {
			wait();
		}
		
		/**
		 * Wait method with time parameter
		 * @param time	time to sleep
		 * @throws InterruptedException
		 */
		public synchronized void waitOnMe(long time) throws InterruptedException {
			wait(time);
		}
		
		/**
		 * @return all registered scheduler entries
		 */
		public synchronized List<SchedulerEntry> getAllEntries() {
			return new LinkedList<SchedulerEntry>(list);
		}
	}
	
	/**
	 * Thread running entry listener
	 */
	private class EntryThread extends Thread {
		
		private final SchedulerEntry entry;
		
		public EntryThread(SchedulerEntry entry) {
			super("SchedulerThread");
			this.entry = entry;
			start();
		}
		
		@Override
		public void run() {
			entry.startScheduledJob();
		}
	}

	/**
	 * Schedulers queue with registered entries
	 */
	private SchedulerQueue queue;

	/**
	 * Creates scheduler running as daemon thread
	 */
	public Scheduler() {
		this(true);
	}
	
	
	/**
	 * Creates new scheduler, automatically firing its main thread
	 * Deamon flag shoud be set to <code>true</code> except for testing cases
	 * @param isDaemon	starts scheduler as daemon
	 */
	public Scheduler(boolean isDaemon) {
		super();
		this.queue = new SchedulerQueue();
		setDaemon(isDaemon);
		start();
	}
	
	/**
	 * Adds new scheduler entry
	 * @param entry		entry to add
	 */
	public void addEntry(SchedulerEntry entry) {
		queue.add(entry);
	}
	
	/**
	 * Removes entry from scheduler
	 * @param entry 	entry to unschedule
	 */
	public void removeEntry(SchedulerEntry entry) {
		queue.remove(entry);
	}
	
	/**
	 * @return	list of scheduled entries	
	 */
	public List<SchedulerEntry> getAllEntries() {
		return queue.getAllEntries();
	}

	/**
	 * Main thread of scheduler
	 * Method returns when scheduler is interrupted
	 */
	@Override
	public void run() {
		while (true) {
			if (queue.isEmpty()) {
				// nothing to do - sleep until new entry added
				try {
					queue.waitOnMe();
				} catch (InterruptedException e) {
					// interrupted
					return;
				}
			} else {
				long currentTime = System.currentTimeMillis();
				
				try {
					queue.waitOnMe(queue.first().alarmTime - currentTime);
				} catch (InterruptedException e) {
					// interrupted
					return;
				} catch (Exception e) {
					// someone notified() and removed the last entry
					continue;
				}
				
				synchronized (queue) {
					while (!queue.isEmpty()) {
						currentTime = System.currentTimeMillis();
						
						// check whether first entry is ready to run
						if (queue.first().alarmTime - currentTime > 99) {
							break;
						}
						
						/*
						 * Remove first entry, update its time, start new thread
						 * for its listener.
						 * Add this entry to correct place in case it is repeating
						 */
						SchedulerEntry entry = queue.removeFirst();
						entry.updateAlarmTime();
						new EntryThread(entry);
						if (entry.isRepeating()) {
							queue.add(entry);
						}
						
					}
				}
			}
		}
	}

	public static void main(String[] args) throws Exception {
		Scheduler s = new Scheduler(false);
		
		long current = System.currentTimeMillis();
	    System.out.println("Current date is " + new Date(current));

		SchedulerEntry entry1 = new SchedulerEntry(
    			"Fist",
	    		new int[]{51},
	    		new int[]{-1}, 
	    		new int[]{-1}, 
	    		new int[]{-1}, 
	    		new int[]{-1}, 
	    		-1, 
	    		new ScheduledJobListener() {
		      public void runJob(SchedulerEntry entry) {
		        System.out.println(entry.getData() + " (" + new Date() + ")");
		      }
	    });
		
		s.addEntry(entry1);
		
		SchedulerEntry entry2 = new SchedulerEntry(
    			"Fist",
	    		new int[]{52},
	    		new int[]{-1}, 
	    		new int[]{-1}, 
	    		new int[]{-1}, 
	    		new int[]{-1}, 
	    		-1, 
	    		new ScheduledJobListener() {
		      public void runJob(SchedulerEntry entry) {
		        System.out.println(entry.getData() + " (" + new Date() + ")");
		      }
	    });
		
		s.addEntry(entry2);
	    
		
//	    for (int i=0; i<5; i++) {
//	    	String name = "ComplexCron" + i;
//	    	SchedulerEntry entry = new SchedulerEntry(
//	    			name,
//		    		new int[]{28+i,33+i},
//		    		new int[]{-1}, 
//		    		new int[]{-1}, 
//		    		new int[]{-1}, 
//		    		new int[]{-1}, 
//		    		-1, 
//		    		new AlarmListener() {
//		      public void handleAlarm(SchedulerEntry entry) {
//		        System.out.println(entry.getName() + " (" + new Date() + ")");
//		      }
//		    });
//	    	
//	    	s.addEntry(entry);
//	    }

	    System.out.println("Here are the registered alarms: ");
	    System.out.println("----------------------------");
	    for(SchedulerEntry x: s.getAllEntries()) {
	      System.out.println("- " + x);
	    }
	    System.out.println("----------------------------");
	    
	    Thread.sleep(10000);
		System.out.println("Removing entry");
		s.removeEntry(entry1);
	}
}
