/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Branislav Repcek
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */

package cz.cuni.mff.been.hostmanager.value;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import cz.cuni.mff.been.hostmanager.InputParseException;

import cz.cuni.mff.been.hostmanager.database.XMLSerializableInterface;

import cz.cuni.mff.been.hostmanager.util.XMLHelper;

/**
 * Class which represents list of values of the same type. <code>ValueList</code> is limited only 
 * to members of simple types (classes which implement <code>ValueBasicInterface</code> interface).
 *
 * @param <T> Type of the elements of the list. This type has to extend ValueBasicInterface< T >.
 *
 * @author Branislav Repcek
 */
public class ValueList< T extends ValueBasicInterface< T > > 
	implements ValueCompoundInterface< T >, Serializable, XMLSerializableInterface, Iterable< T > {
	
	/**
	 * List of values.
	 */
	private ArrayList< T > value;
	
	/**
	 * Allocate empty <code>ValueList</code> value.
	 */
	public ValueList() {

		value = new ArrayList< T >();
	}
	
	/**
	 * Create new list form array of objects implementing <code>ValueBasicInterface</code>. 
	 * Elements are stored in same order as they appear in <code>newValue</code> array.
	 * 
	 * @param newValue List of objects.
	 */
	public ValueList(T[] newValue) {
	
		value = new ArrayList< T >();

		if (newValue == null) {
			return;
		}
		
		if (newValue.length > 0) {
			for (T x: newValue) {
				value.add(x);
			}
		
			setSubtypeName(newValue[0]);
		}
	}

	/**
	 * Create new list containing all members of given collection.
	 * 
	 * @param values Collection containing values that will be included in the list.
	 */
	public ValueList(Collection< T > values) {
		
		if (values == null) {
			value = new ArrayList< T >();
			return;
		}
		
		value = new ArrayList< T >(values.size());
		
		if (values.size() > 0) {
			
			for (T x: values) {
				value.add(x);
			}
			
			setSubtypeName(values.iterator().next());
		}
	}
	
	/**
	 * Create new ValueList and read data from XML node.
	 * 
	 * @param node Node containing list data.
	 * 
	 * @throws InputParseException If there was an error while parsing XML data.
	 */
	@SuppressWarnings("unchecked")
	public ValueList(Node node) throws InputParseException {
		
		value = new ArrayList();
		
		parseXMLNode(node);
	}
	
	/*
	 * @see java.lang.Object#toString
	 */
	@Override
	public String toString() {
		
		String result = "{";
		
		for (int i = 0; i < length(); i++) {
			
			result += value.get(i).toString();
			if (i < length() - 1) {
				result += ',';
			}
		}
		
		return result + "}";
	}
	
	/**
	 * Get number of elements stored in list.
	 * 
	 * @return Number of elements stored in list.
	 */
	public int length() {
		
		return value.size();
	}

	/**
	 * Get element at given position.
	 * 
	 * @param index Postion of element to get.
	 * @return Element at given postion.
	 * @throws IndexOutOfBoundsException If (index < 0) || (index > length()).
	 */
	public T get(int index) throws IndexOutOfBoundsException {
		
		return value.get(index);
	}
	
	/**
	 * Set element at given postion.
	 * 
	 * @param index Index of element to set.
	 * @param val New value.
	 * @throws IndexOutOfBoundsException If (index < 0) || (index > length()).
	 */
	public void set(int index, T val) throws IndexOutOfBoundsException {
		
		setSubtypeName(val);
		value.set(index, val);
	}
	
	/**
	 * Add element to the end of the list.
	 * 
	 * @param val New element to add to the list.
	 */
	public void add(T val) {

		setSubtypeName(val);
		value.add(val);
	}

	/**
	 * Add all elements from specified list. Duplicate values are not removed. Order of elements 
	 * is not changed.
	 * 
	 * @param list List of values to add to the current list.
	 */
	public void addAll(ValueList< T > list) {
		
		subtypeName = list.subtypeName;

		value.addAll(list.value);
	}
	
	/**
	 * Add all elements from the specified collection to the current list. Duplicate entries are  
	 * allowed. Order of elements is not changed.
	 * 
	 * @param list Collesction of elements to add to current list.
	 */
	public void addAll(Collection< T > list) {
		
		if (list.size() > 0) {
			setSubtypeName(list.iterator().next());
		}
		
		value.addAll(list);
	}
	
	/**
	 * Remove element with given value from the list.
	 * 
	 * @param val Value to remove.
	 * @return <code>true</code> if specified value was present in the list, <code>false</code> otherwise.
	 */
	public boolean remove(T val) {
		
		return value.remove(val);
	}
	
	/**
	 * Remove value at given position in the list.
	 * 
	 * @param index Index of the value to remove.
	 * @return Value which is being removed.
	 * 
	 * @throws IndexOutOfBoundsException If (index < 0) || (index > length()).
	 */
	public T remove(int index) throws IndexOutOfBoundsException {
		
		return value.remove(index);
	}
	
	/**
	 * Test whether list contains any elements.
	 * 
	 * @return <code>true</code> if list is empty, <code>false</code> otherwise.
	 */
	public boolean empty() {
		
		return value.isEmpty();
	}

	/**
	 * Test whether list contains given element.
	 * 
	 * @param vb Element to test.
	 * @return <code>true</code> if element has been found in list, <code>false</code> otherwise.
	 */
	public boolean contains(T vb) {
		
		for (Iterator< T > it = value.iterator(); it.hasNext(); ) {
			
			if (it.next().equals(vb)) {
				return true;
			}
		}
			
		return false;
	}
	
	/*
	 * @see java.lang.Object#equals
	 */
	@Override
	@SuppressWarnings("unchecked")
	public boolean equals(Object o) {
		
		if (o instanceof ValueList) {
			return equals((ValueList) o);
		} else {
			return false;
		}
	}
	
	/**
	 * Compare specified <code>ValueList</code> with this value for equality. Comparison is based only 
	 * on elements contained within respective lists, it does not matter in which order elements are stored.
	 * 
	 * @param vc Value to compare this to.
	 * @return <code>true</code> if values are equal, <code>false</code> otherwise.
	 */
	public boolean equals(ValueList< T > vc) {
		
		if (length() == vc.length()) {

			Iterator< T > it1 = value.iterator();
			Iterator< T > it2 = vc.value.iterator();
			
			for (; it1.hasNext(); ) {
				if (!it1.next().equals(it2.next())) {
					return false;
				}
			}
			
			return true;
		} else {
			return false;
		}
	}

	/*
	 * @see java.lang.Object#hashCode
	 */
	@Override
	public int hashCode() {
		
		return value.hashCode();
	}

	/*
	 * @see cz.cuni.mff.been.hostmanager.database.XMLSerializableInterface#parseXMLNode
	 */
	@SuppressWarnings("unchecked")
	public void parseXMLNode(Node node) throws InputParseException {
		
		if (!node.getNodeName().equals(getXMLNodeName())) {
			throw new InputParseException("Node does not contain list data. Node name is \""
					+ node.getNodeName() + "\".");
		}
		
		subtypeName = XMLHelper.getAttributeValueByName("type", node);

		if (subtypeName.equals("(none)")) {
			
			return;
		}
		
		Class subtypeClass = null;

		try {
			subtypeClass = Class.forName(subtypeName);
		} catch (Exception e) {
			throw new InputParseException("Error creating instance of the list element.", e);
		}
		
		ArrayList< Node > subnodes = null;
		
		try {
			subnodes = XMLHelper.getChildNodesByName(((T) subtypeClass.newInstance()).getXMLNodeName(), node);
		} catch (Exception e) {
			throw new InputParseException("Unable to create instance of the list element.", e);
		}

		for (Node current: subnodes) {
			T vbi = null;
			
			try {
				vbi = (T) subtypeClass.newInstance();
			} catch (Exception e) {
				throw new InputParseException("Error creating list items.", e);
			}
			
			vbi.parseXMLNode(current);
			
			value.add(vbi);
		}
	}
	
	/*
	 * @see cz.cuni.mff.been.hostmanager.database.XMLSerializableInterface#exportAsElement
	 */
	public Element exportAsElement(Document document) {
		
		/* Resulting node
		 * 
		 * <list subtype="<subtype-name>">
		 *    <element-node 0>
		 *          .
		 *          .
		 *          .
		 *    <element-node n-1>
		 * </list>
		 * 
		 * where <subtype-name> is canonical name of the type of elements in the list or (none) if
		 * list  is empty, <element-node 0> to <element-node n-1> are serialized items from the list.
		 * These nodes will not be present if the list is empty.
		 */
		
		Element element = document.createElement(getXMLNodeName());

		element.setAttribute("type", subtypeName == null ? "(none)" : subtypeName);
		
		if (value.size() > 0) {
			
			for (T current: value) {
				element.appendChild(current.exportAsElement(document));
			}
		}

		return element;
	}
	
	/**
	 * Canonical name of the type of the elements in list.
	 */
	private String subtypeName;
	
	/*
	 * Holy cow batman, this is one stupid hack...
	 * 
	 * I was unable to find out how to get canonical name of the type of the elements of the list,
	 * so this method is called everytime something is added to the list and it will get name from
	 * the instance of object being added.
	 */
	private void setSubtypeName(T val) {
		
		if (val != null) {
			subtypeName = val.getClass().getCanonicalName();
		}
	}

	/*
	 * @see cz.cuni.mff.been.hostmanager.database.XMLSerializableInterface#getXMLNodeName
	 */
	public String getXMLNodeName() {
		
		return "list";
	}
	
	/*
	 * @see java.lang.Iterable#iterator()
	 */
	public Iterator< T > iterator() {
		
		return value.iterator();
	}
	
	/**
	 * @return List containing all elements from this list.
	 */
	public List< T > getValue() {
		
		return value;
	}
}
