/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.hostruntime;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Delete;
import cz.cuni.mff.been.hostmanager.IllegalOperationException;
import cz.cuni.mff.been.hostmanager.ValueNotFoundException;
import cz.cuni.mff.been.hostmanager.load.LoadMonitorException;
import cz.cuni.mff.been.hostmanager.load.LoadMonitorImplementation;
import cz.cuni.mff.been.hostmanager.load.LoadMonitorInterface;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryService;
import cz.cuni.mff.been.task.Service;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.taskmanager.HostRuntimesPortInterface;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.TaskManagerInterface;

/**
 * Implementation of Host Runtime.
 * 
 * @author Antonin Tomecek
 * @author David Majda
 */
public class HostRuntimeImplementation extends UnicastRemoteObject
		implements HostRuntimeInterface {
	/** Name of the cache directory. */
	private static final String CACHE_DIR = "cache"; 
	/** Name of the directory with boot packages. */
	private static final String BOOT_PACKAGES_DIR = "boot"; 
		
	/** Name of the directory that contains directories with task data. */
	private static final String TASKS_BASE_DIR = "tasks";

	/** Name of the task directory (task's JAR is extracted there). */
	private static final String TASK_DIR = "task";
	/**
	 * Name of the wokring directory (results of task's work are stored there,
	 * surviving task's death).
	 */
	private static final String WORKING_DIR = "working"; 
	/** Name of the temporary directory (deleted after the task's death). */
	private static final String TEMPORARY_DIR = "temporary"; 
	/** Name of the service directory (package contents are extracted there). */
	private static final String SERVICE_DIR = "service"; 

	/** Name of the directory with Load Monitor native libraries. */
	private static final String NATIVE_LIB_DIR = "native-lib"; 
	/** Name of the directory with load data. */
	private static final String LOAD_DIR = "load"; 

	/** RMI interface to the Task Manager. */
	private TaskManagerInterface taskManager;
	/** RMI interface to the Task Manager's port to the Host Runtime. */
	private HostRuntimesPortInterface hostRuntimesPort;
	/** Package cache manager. */
	private PackageCacheManager packageCacheManager;
	/** List of tasks running in this Host Runtime. */
	private final List<TaskInterface> runningTasks
		= Collections.synchronizedList(new ArrayList<TaskInterface>());
	/** List of contexts to close. */
	private final List<String> closedContexts 
		= Collections.synchronizedList(new LinkedList<String>());
	/**
	 * Number of closed contexts, for which the Host Runtime should keep data on
	 * the disk.
	 */
	private int keptClosedContextCount = TaskManagerInterface.DEFAULT_KEPT_CLOSED_CONTEXT_COUNT;
	/** Load monitor instance. */ 
	private LoadMonitorImplementation loadMonitor;
	
	/** Root directory of this Host Runtime. */
	private String rootDirectory;
	
	/** @return root directory of this Host Runtime */
	public String getRootDirectory() {
		return rootDirectory;
	}
	
	/** @return RMI interface to the Task Manager */
	public TaskManagerInterface getTaskManager() {
		return taskManager;
	}
	
	/** @return RMI interface to the Task Manager's port to the Host Runtime */
	public HostRuntimesPortInterface getHostRuntimesPort() {
		return hostRuntimesPort;
	}
	
	/** @return package cache manager */
	public PackageCacheManager getPackageCacheManager() {
		return packageCacheManager;
	}
	
	/** @return Load Monitor instance */
	public LoadMonitorInterface getLoadMonitor() {
		return loadMonitor;
	}

	private void checkDirectory(String directoryName) throws IOException {
		File directory = new File(directoryName);
		if (!directory.exists()) {
			throw new FileNotFoundException("Error: \"" + directory + "\" does not exist.");
		}
		if (!directory.isDirectory()) {
			throw new FileNotFoundException("Error: \"" + directory + "\" is not directory.");
		}
		if (!directory.canWrite()) {
			throw new IOException("Error: \"" + directory + "\" is not writable.");
		}
	}

	private void deleteContext(String contextID) throws HostRuntimeException {
		try {
			Delete.deleteDirectory(rootDirectory
				+ File.separator + TASKS_BASE_DIR
				+ File.separator + contextID);
		} catch (AntTaskException e) {
			throw new HostRuntimeException(e);
		}

		try {
			loadMonitor.clearDetailedModeData(contextID);
		} catch (IllegalOperationException e) {
			throw new HostRuntimeException(e);
		} catch (RemoteException e) {
			throw new AssertionError("RemoteException should not happen in local call.");
		} catch (ValueNotFoundException e) {
			throw new HostRuntimeException(e);
		}
	}
	
	private void deleteOldClosedContexts() throws HostRuntimeException {
		synchronized (closedContexts) {
			while (closedContexts.size() > keptClosedContextCount) {
				deleteContext(closedContexts.get(0));
				closedContexts.remove(0);
			}
		}
	}
	
	/**
	 * Allocates a new <code>HostRuntimeImplementation</code> object.
	 * 
	 * @param taskManagerHostname host where the Task Manager is running
	 * @param rootDirectory root directory of this Host Runtime
	 * @throws RemoteException if failed to export object
	 * @throws IOException if the root direcotry or one of its required
	 *          subdirectories (<tt>cache</tt>, <tt>boot</tt> and <tt>tasks</tt>)
	 *          does not exist, is not a directory, or is not writable
	 * @throws NotBoundException if the Host Runtime can not connect to the Task
	 *          Manager
	 * @throws LoadMonitorException if the Load Monitor can not be created 
	 */
	public HostRuntimeImplementation(String taskManagerHostname,
			String rootDirectory) throws IOException, LoadMonitorException,
			NotBoundException {
		String cacheDir = rootDirectory + File.separator + CACHE_DIR;
		String bootPackagesDir = rootDirectory + File.separator + BOOT_PACKAGES_DIR;
		String nativeLibDir = rootDirectory + File.separator + NATIVE_LIB_DIR;
		String loadDir = rootDirectory + File.separator + LOAD_DIR;
		String tasksBaseDir = rootDirectory + File.separator + TASKS_BASE_DIR;
		
		checkDirectory(rootDirectory);
		checkDirectory(cacheDir);
		checkDirectory(bootPackagesDir);
		checkDirectory(nativeLibDir);
		checkDirectory(loadDir);
		checkDirectory(tasksBaseDir);
		
		this.rootDirectory = rootDirectory;
		
		packageCacheManager = new PackageCacheManager(cacheDir, bootPackagesDir);
		loadMonitor = new LoadMonitorImplementation(nativeLibDir, loadDir);
		
		Naming.rebind("rmi://localhost:1099" + HostRuntimeInterface.URL, this);

		/* Initialize the Task Manager reference and register the Host Runtime. */
		try {
			hostRuntimesPort = (HostRuntimesPortInterface) Naming.lookup(
				"//" + taskManagerHostname + TaskManagerInterface.URL
			);
		} catch (MalformedURLException e) {
			throw new AssertionError("MalformedURLException could not be thrown.");
		}
		taskManager = hostRuntimesPort.getTaskManager();
		taskManager.registerHostRuntime(InetAddress.getLocalHost().getCanonicalHostName());

		/* If the Host Runtime shuts down, kill all running tasks. */ 
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run() {
				synchronized (runningTasks) {
					for (TaskInterface task : runningTasks) {
						try {
							task.kill();
						} catch (RemoteException e) {
							System.err.println("Error executing remote call from the Host "
								+ "Runtime to the task.");
						}
					}
				}
				
				try {
					loadMonitor.terminate();
				} catch (RemoteException e) {
					throw new AssertionError("Local call should not fail.");
				}

				try {
					taskManager.unregisterHostRuntime(InetAddress.getLocalHost().getCanonicalHostName());
				} catch (RemoteException e) {
					System.err.println("Error executing remote call from the Host "
							+ "Runtime to the Task Manager.");
				} catch (UnknownHostException e) {
					System.err.println(e.getMessage());
				}
			}
		});
	}
	
	/**
	 * Returns directory that contains directories with task data for specified
	 * task (identified by the context and task ID).
	 * 
	 * @param contextID context ID
	 * @param taskID task ID
	 * @return directory that contains directories with task data
	 */
	public String getBaseDirectoryForTask(String contextID, String taskID) {
		return rootDirectory
			+ File.separator + TASKS_BASE_DIR
			+ File.separator + contextID 
			+ File.separator + taskID; 
	}

	/**
	 * Returns task directory (task's JAR is extracted there) for specified
	 * task (identified by the context and task ID).
	 * 
	 * @param contextID context ID
	 * @param taskID task ID
	 * @return task directory (task's JAR is extracted there)
	 */
	public String getTaskDirectoryForTask(String contextID, String taskID) {
		return getBaseDirectoryForTask(contextID, taskID)
			+ File.separator + TASK_DIR; 
	}
	
	/**
	 * Returns wokring directory (results of task's work are stored there,
	 * surviving task's death) for specified task (identified by the context and
	 * task ID).
	 * 
	 * @param contextID context ID
	 * @param taskID task ID
	 * @return wokring directory (results of task's work are stored there,
	 *          surviving task's death)
	 */
	public String getWorkingDirectoryForTask(String contextID, String taskID) {
		return getBaseDirectoryForTask(contextID, taskID)
			+ File.separator + WORKING_DIR; 
	}
	
	/**
	 * Returns temporary directory (deleted after the task's death) for specified
	 * task (identified by the context and task ID).
	 * 
	 * @param contextID context ID
	 * @param taskID task ID
	 * @return temporary directory (deleted after the task's death)
	 */
	public String getTemporaryDirectoryForTask(String contextID, String taskID) {
		return getBaseDirectoryForTask(contextID, taskID)
			+ File.separator + TEMPORARY_DIR; 
	}

	/**
	 * Returns service directory (package contents are extracted there) for
	 * specified task (identified by the context and task ID).
	 * 
	 * @param contextID context ID
	 * @param taskID task ID
	 * @return service directory (package contents are extracted there)
	 */
	public String getServiceDirectoryForTask(String contextID, String taskID) {
		return getBaseDirectoryForTask(contextID, taskID)
			+ File.separator + SERVICE_DIR; 
	}

	/**
	 * Notifies the Host Runtime that the task has finished.
	 * 
	 * @param task task that finished
	 */
	public void notifyTaskFinished(TaskInterface task) {
		runningTasks.remove(task);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#initialize(cz.cuni.mff.been.taskmanager.HostRuntimesPortInterface, long, int)
	 */
	public void initialize(HostRuntimesPortInterface hostRuntimesPort, long maxPackageCacheSize,
			int keptClosedContextCount) throws RemoteException {
		this.hostRuntimesPort = hostRuntimesPort;
		packageCacheManager.setMaxCacheSize(maxPackageCacheSize);
		this.keptClosedContextCount = keptClosedContextCount;
		
		taskManager = hostRuntimesPort.getTaskManager();
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#terminate()
	 */
	public void terminate() throws RemoteException {
		System.exit(0);
		
		/* The dirty work of killing all running tasks will now be done by the
		 * shutdown hook, initialized in the constructor.
		 */
	}

	private boolean isDetailedLoadCollision(TaskDescriptor taskDescriptor) {
		/* If the new task does not specify detailed load flag, no collision can
		 * occur.
		 */
		if (!taskDescriptor.getDetailedLoad()) {
			return false;
		}
		
		/* If there is any other task running with specified load flag, we have a
		 * collision.
		 */  
		try {
			for (TaskInterface task: runningTasks) {
				if (task.isDetailedLoad()) {
					return true;
				}
			}
		} catch (RemoteException e) {
			throw new AssertionError("Remote exception cannot be thrown on local call.");
		}
		
		return false;
	}
	
	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#createTask(cz.cuni.mff.been.taskmanager.TaskDescriptor)
	 */
	public TaskInterface createTask(TaskDescriptor taskDescriptor)
			throws HostRuntimeException, RemoteException {
		/* Check if we are initialized properly. */
		if (taskManager == null) {
			throw new IllegalStateException("Call \"initialize\" method before "
				+ "creating any task.");
		}
		
		/* Get the Software Repository service interface from the Task Manager.
		 * Maybe we should be more clever here and do not try to find it all the
		 * time... 
		 */ 
		SoftwareRepositoryInterface softwareRepository
			= (SoftwareRepositoryInterface) taskManager.serviceFind(
				SoftwareRepositoryService.SERVICE_NAME,
				Service.RMI_MAIN_IFACE
			);
		packageCacheManager.setSoftwareRepository(softwareRepository);
		
		/* Create the task instance. */
		TaskImplementation task;
		try {
			synchronized (runningTasks) {
				boolean measureDetailedLoad = taskDescriptor.getDetailedLoad()
					&& !isDetailedLoadCollision(taskDescriptor);
				task = new TaskImplementation(taskDescriptor, this, measureDetailedLoad);
				runningTasks.add(task);
			}
		} catch (TaskException e) {
			throw new HostRuntimeException(e);
		}
		return task;
	}
		
	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#closeContext(java.lang.String)
	 */
	public void closeContext(String contextID) throws HostRuntimeException, RemoteException {
		/* Check if there is a directory for given context. */
		String contextDir = rootDirectory
			+ File.separator + TASKS_BASE_DIR
			+ File.separator + contextID;
		if (!new File(contextDir).exists()) {
			throw new IllegalArgumentException("No task from context \"" + contextID 
				+ "\" was run on this Host Runtime.");
		}

		/* Check if there is no task running from given context. */
		synchronized (runningTasks) {
			for (TaskInterface task : runningTasks) {
				if (task.getContextID().equals(contextID)) {
					throw new IllegalArgumentException("Task \"" + task.getTaskID()
						+ "\" from context \"" + task.getContextID() + "\" is still running.");
				}
			}
		}
		
		/* Add the context to the queue of closed contexts if it is not already
		 * there. Delete its files if the queue is long enough,  
		 */
		synchronized (closedContexts) {
			for (String closedContextID: closedContexts) {
				if (closedContextID.equals(contextID)) {
					throw new IllegalArgumentException("Context \"" + contextID
						+ "\" was already closed.");
				}
			}
			closedContexts.add(contextID);
		}
		deleteOldClosedContexts();
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#setMaxPackageCacheSize(long)
	 */
	public void setMaxPackageCacheSize(long maxPackageCacheSize) throws RemoteException {
		packageCacheManager.setMaxCacheSize(maxPackageCacheSize);
	}

	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#setKeptClosedContextCount(int)
	 */
	public void setKeptClosedContextCount(int keptClosedContextCount) throws RemoteException {
		this.keptClosedContextCount = keptClosedContextCount;
	}
		
	/**
	 * @see cz.cuni.mff.been.hostruntime.HostRuntimeInterface#getRunningTaskInterfaces()
	 */
	public TaskInterface[] getRunningTaskInterfaces() throws RemoteException {
		synchronized (runningTasks) {
			return runningTasks.toArray(new TaskInterface[runningTasks.size()]);
		}		
	}
}
