/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.hostruntime;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Date;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.softwarerepository.MatchException;
import cz.cuni.mff.been.softwarerepository.PackageMetadata;
import cz.cuni.mff.been.softwarerepository.PackageQueryCallbackInterface;
import cz.cuni.mff.been.softwarerepository.PackageType;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface;

/**
 * Manages the Host Runtime's package cache.
 * 
 * The cache consists of two directories: <em>cache directory</em> and
 * <em>boot packages direcotory</em>. In the constructor, the cache scans the
 * boot packages direcotry and permanenetly adds found packages to the cache.
 * These packages are not counted to the cache size. The cache directory stores
 * packages which were once downloaded from the Software Repository and they are
 * counted to the cache size.
 * 
 * The only operation of the package cache is <code>extractPackage</code>
 * method, which extracts the contents of the package with given filename to the
 * specified directory (possibly downloading the package from the Software
 * Repository and placing it to the cache).
 * 
 * The class maintains invariant that size of all packages in the package cache
 * direcotry is less than <code>maxCacheSize</code>. If the invariant is about
 * to be broken when downloading new package form the Software Repository, it
 * deletes oldest packages until there is enough space  
 * 
 * The class is designed to be thread-safe.
 * 
 * @author David Majda
 */
public class PackageCacheManager {
	/**
	 * Immutable class containing information about one package in the cache.
	 *  
	 * @author David Majda
	 */
	private static class Package {
		/**
		 * Types of packages stored in the cache.
		 * 
		 * @author David Majda
		 */
		public static enum Type {
			/** Boot package. */
			BOOT,
			/** Non-boot package. */
			NON_BOOT
		};
	
		/** Package file name in the cache. */
		private String filename;
		/** Package size in bytes. */
		private long size;
		/** Package download time. */
		private Date downloadTime;
		/**
		 * Package type (boot/non-boot, not the type from the package metadatata).
		 */
		private Type type;
			
		/** @return package file name in the cache */
		public String getFilename() {
			return filename;
		}
		
		/** @return package size in bytes */
		public long getSize() {
			return size;
		}

		/** @return package download time */
		public Date getDownloadTime() {
			return new Date(downloadTime.getTime());
		}

		/**
		 * @return Package type (boot/non-boot, not the type from the package
		 *          metadatata)
		 */
		public Type getType() {
			return type;
		}
				
		/**
		 * Allocates a new <code>Package</code> object.
		 * 
		 * @param file name package filename in the cache
		 * @param size package size in bytes
		 * @param downloadTime package download time
		 * @param type package type (boot/non-boot, not the type from the package
		 *         metadatata)
		 */
		public Package(String filename, long size, Date downloadTime, Type type) {
			this.filename = filename;
			this.size = size;
			this.downloadTime = downloadTime;
			this.type = type;
		}
	}

	/**
	 * Callback interface to the Software Repository, which checks if the package
	 * is task and its file name equals to the specified value.
	 * 
	 * @author David Majda
	 */
	private static class PackageNameQueryCallback
			implements PackageQueryCallbackInterface, Serializable {
		/** Package file name to check. */
		private String packageFilename; 

		/**
		 * Allocates a new <code>PackageNameQueryCallback</code> object.
		 * 
		 * @param packageName package file name to check
		 */
		public PackageNameQueryCallback(String packageFilename) {
			this.packageFilename = packageFilename;
		}

		/**
		 * @see cz.cuni.mff.been.softwarerepository.PackageQueryCallbackInterface#match(cz.cuni.mff.been.softwarerepository.PackageMetadata)
		 */
		public boolean match(PackageMetadata metadata) throws MatchException {
			return metadata.getType().equals(PackageType.TASK) 
				&& metadata.getFilename().equals(packageFilename);
		}
	}
	
	/** Size of the buffer used when extracting the packages. */
	private static final int PACKAGE_EXTRACTION_BUFFER_SIZE = 4096;
	
	/** Cache directory. */
	private String cacheDir;
	/** Directory with boot packages. */
	private String bootPackagesDir;
	/** Limit of the cache size. */
	private long maxCacheSize;
	/** RMI reference to the Software Repository. */
	private SoftwareRepositoryInterface softwareRepository;
	/** List of packages in the cache. */
	private List<Package> packages = new LinkedList<Package>();
	
	/** @return cache directory */
	public String getCacheDir() {
		return cacheDir;
	}

	/** @return directory with boot packages */
	public String getBootPackagesDir() {
		return bootPackagesDir;
	}

	/** @return limit of the cache size */
	public synchronized long getMaxCacheSize() {
		return maxCacheSize;
	}
	
	/**
	 * Sets the limit of the cache size.
	 * 
	 * @param sizeLimit the limit of the cache size to set
	 */
	public synchronized void setMaxCacheSize(long sizeLimit) {
		this.maxCacheSize = sizeLimit;
	}

	/** @return RMI reference to the Software Repository */
	public synchronized SoftwareRepositoryInterface getSoftwareRepository() {
		return softwareRepository;
	}
	
	/**
	 * Sets the RMI reference to the Software Repository.
	 * 
	 * @param softwareRepository the RMI reference to the Software Repository to
	 *         set
	 */
	public synchronized void setSoftwareRepository(SoftwareRepositoryInterface softwareRepository) {
		this.softwareRepository = softwareRepository;
	}

	/**
	 * Finds package in the cache by its name.
	 * 
	 * @param packageFilename package name  
	 * @return object representing found package or <code>null</code> if the
	 *          cache does not contain package with given name
	 */
	private Package findPackage(String packageFilename) {
		for (Package p: packages) {
			if (p.getFilename().equals(cacheDir + File.separator + packageFilename)
					|| p.getFilename().equals(bootPackagesDir + File.separator + packageFilename)) {
				return p;
			}
		}
		return null;
	}

	/**
	 * Finds the oldest non-boot package in the cache.
	 * 
	 * @return object representing the oldest non-boot package in the cache or
	 *          <code>null</code> if there are no non-boot packages in the cache
	 */
	private Package findOldestNonBootPackage() {
		Package result = null;
		for (Package p: packages) {
			if (p.getType().equals(Package.Type.NON_BOOT)
					&& (result == null || p.getDownloadTime().compareTo(result.getDownloadTime()) < 0)) {
				result = p;
			}
		}
		return result;
	}

	/**
	 * Deletes a package from the cache.
	 * 
	 * @param pakkage package to delete
	 * @return <code>true</code> if the package was successfully deleted;
	 *          <code>false</code> otherwise
	 */
	private boolean deletePackage(Package pakkage) {
		if (new File(pakkage.getFilename()).delete()) {
			packages.remove(pakkage);
			return true;
		} else {
			return false;
		}
	}
		
	/**
	 * Deletes the oldest non-boot package in the cache.
	 */
	private void deleteOldestNonBootPackage() {
		deletePackage(findOldestNonBootPackage());
	}
	
	/**
	 * Calculates total size of the cache.
	 * 
	 * @return total size of non-boot packages in the cache in bytes
	 */
	private long calculateCacheSize() {
		long result = 0;
		for (Package p: packages) {
			if (p.type.equals(Package.Type.NON_BOOT)) {
				result += p.getSize();
			}
		}
		return result;
	}

	/**
	 * Makes sure that size of the cache is smaller than the limit, deleting
	 * old non-boot packages if necessary.
	 * 
	 * @param limit size limit
	 */
	private void ensureSize(long limit) {
		while (calculateCacheSize() > limit) {
			deleteOldestNonBootPackage();
		}
	}
	
	/**
	 * Downloads package from the Software Reposiotry and adds it to the cache.
	 * 
	 * @param packageFilename file name of the package to download
	 * @return object representing the downloaded package, or <code>null</code>
	 *          if the package is not found in the Software Repository or the
	 *          Software Repository reference is not set
	 * 
	 * @throws IOException if the download fails
	 */
	private Package tryDownloadPackage(String packageFilename) throws IOException {
		/* No Software Repository reference set? */
		if (softwareRepository == null) {
			return null;
		}
		
		/* Retrieve the package size and ensure we have enough space in the cache. */
		PackageMetadata[] metadata = null;
		try {
			metadata = softwareRepository.queryPackages(
				new PackageNameQueryCallback(packageFilename)
			);
		} catch (MatchException e) {
			assert false: "MatchException should be never thrown here.";
		}
		if (metadata.length == 0) {
			return null;
		}
		long packageSize = metadata[0].getSize();
		ensureSize(maxCacheSize - packageSize);
		
		/* Download the package. */
		ServerSocket serverSocket = new ServerSocket(0); // 0 = use any port
		DownloadHandle handle = softwareRepository.beginPackageDownload(
			packageFilename,
			InetAddress.getLocalHost(),
			serverSocket.getLocalPort()
		);
		byte[] buffer = new byte[SoftwareRepositoryInterface.DOWNLOAD_BUFFER_SIZE];
		int bytesRead;
		Socket socket = serverSocket.accept();
		try {
			InputStream inputStream = new BufferedInputStream(
				socket.getInputStream(),
				SoftwareRepositoryInterface.DOWNLOAD_BUFFER_SIZE
  		);
			OutputStream outputStream = new BufferedOutputStream(
				new FileOutputStream(cacheDir + File.separator + packageFilename),
				SoftwareRepositoryInterface.DOWNLOAD_BUFFER_SIZE
			);
			try {
				while ((bytesRead = inputStream.read(buffer)) != -1) {
					outputStream.write(buffer, 0, bytesRead);     
				}
			}	finally {
				inputStream.close();
				outputStream.close();
			}
		} finally {
			socket.close();
			serverSocket.close();
			softwareRepository.endPackageDownload(handle);
		}
		
		/* Add package to internal data structues. */
		Package result = new Package(
			cacheDir + File.separator + packageFilename,
			packageSize,
			new Date(),
			Package.Type.NON_BOOT
		);
		packages.add(result);
		return result;
	}
	
	/**
	 * Extract contents of the ZIP file to given path.
	 * 
	 * @param zipFilename ZIP file to extract
	 * @param path path to extract the files
	 * @throws IOException if the extraction fails
	 */
	private void extractZipFile(String zipFilename, String path)
			throws IOException {
		ZipFile zipFile = new ZipFile(zipFilename);
		for (Enumeration entries = zipFile.entries(); entries.hasMoreElements(); ) {
			ZipEntry entry = (ZipEntry) entries.nextElement();
			
			if (!entry.isDirectory()) {
				byte[] buffer = new byte[PACKAGE_EXTRACTION_BUFFER_SIZE];
				int bytesRead;
				InputStream inputStream = new BufferedInputStream(
					zipFile.getInputStream(entry),
					PACKAGE_EXTRACTION_BUFFER_SIZE
				);
				OutputStream outputStream = new BufferedOutputStream(
					new FileOutputStream(path + File.separator + entry.getName()),
					PACKAGE_EXTRACTION_BUFFER_SIZE
				);
				try {
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						outputStream.write(buffer, 0, bytesRead);     
					}
				}	finally {
					inputStream.close();
					outputStream.close();
				}
			} else {
				new File(path, entry.getName()).mkdirs();
			}
		}
	}
	
	/**
	 * Extracts the contents of the package with given name to the specified
	 * directory (possibly downloading the package from the Software Repository
	 * and placing it to the cache).
	 * 
	 * Maintains invariant that size of all packages in the package cache
	 * direcotry is less than <code>maxCacheSize</code>. If the invariant is about
	 * to be broken when downloading new package form the Software Repository,
	 * we delete oldest packages until there is enough space.  
	 *
	 * @param packageName name of the package to extract
	 * @param path directory where the package will be extracted
	 * @throws IOException if the download or extraction fails
	 * @throws HostRuntimeException if the package can not be found in the
	 *          Software Repository or the Software Repository reference is not
	 *          set
	 */
	public synchronized void extractPackage(String packageName, String path)
			throws IOException, HostRuntimeException {
		Package pakkage = findPackage(packageName);
		if (pakkage == null) {
			pakkage = tryDownloadPackage(packageName);
		}
		
		if (pakkage != null) {
			extractZipFile(pakkage.getFilename(), path);
		} else {
			throw new HostRuntimeException("Can't find task package: " + packageName);
		}
	}

	/**
	 * Allocates a new <code>PackageCacheManager</code> object.
	 * 
	 * Note that for proper initialization the user must also set the RMI
	 * reference to the Software Repository (using
	 * <code>setSoftwareRepository</code> method) and limit of the cache size
	 * (using <code>setMaxCacheSize</code> method). Those two attributes are not
	 * set in the construcotr, because it is assumed that user (Host Runtime) does
	 * not know them in the time of the construction.
	 * 
	 * @param cacheDir cache directory
	 * @param bootPackagesDir directory with boot packages
	 * @throws IOException if the list of boot packages can't be retrieved
	 */
	public PackageCacheManager(String cacheDir, String bootPackagesDir)
			throws IOException {
		this.cacheDir = cacheDir;
		this.bootPackagesDir = bootPackagesDir;
		
		Date now = new Date();
		File[] files = new File(bootPackagesDir).listFiles();
		if (files == null) {
			throw new IOException("Error getting list of boot packages.");
		}
		
		for (File f: files) {
			packages.add(new Package(
				f.getPath(),
				f.length(),
				now,
				Package.Type.BOOT)
			);
		}
	}
}
