/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.util.List;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.EntityInterface;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;

public class AnalysisImplementation extends  EntityImplementation<AID>
	implements Serializable, EntityInterface<AID>  {
	
	/** Load graphs defined for runs only error message */
	private static final String LOAD_FOR_RUNS_ONLY = "Load graphs defined for runs only.";

	/** Raw data download defined for runs only */ 
	private static final String RAW_DATA_FOR_RUNS_ONLY = "Raw data download defined for runs only.";

	/** Results repository entity manager for analyses */
	private AnalysisEntityManager analysisEntityManager;
	
	/** Construction of basic analysis implementation 
	 * @throws RemoteException */
	public AnalysisImplementation() throws RemoteException {
		super();
		analysisEntityManager = database.getAnalysisEntityManager();
	}
	
	public Entity<AID> getEntity(AID id) throws RemoteException {
		return analysisEntityManager.getAnalysis(id); 
	}

	public void setEntityComment(AID id, String comment) throws RemoteException {
		analysisEntityManager.setAnalysisComment(id, comment);
		
	}

	public List<Entity<AID>> getEntities() throws RemoteException {
		return analysisEntityManager.getAnalyses();
	}

	public EntityInterface getSubentityInterface(AID id) throws RemoteException {
		return new ExperimentImplementation(id);
	}

	public DataFormat[] getExportFormats() throws RemoteException {
		return exporter.getAnalysisExportFormats();
	}

	public DownloadHandle beginExport(AID[] ids, String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {
		return exporter.beginAnalysesExport(ids, formatId, ip, port);
	}
	
	public DataFormat[] getImportFormats() throws RemoteException {
		return importer.getAnalysisImportFormats();
	}

	public UploadHandle beginImport(String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {
		return importer.beginAnalysesImport(formatId, ip, port);
	}

	public void deleteEntity(AID id) throws DeleteLockedException, RemoteException {
		analysisEntityManager.eraseAnalysis(id);
	}

	public int graphCount(GraphType graphType, AID aid) throws RemoteException {
  		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
		return analysisEntityManager.getAnalysisGraphCount(graphType, aid);		
	}
	
	public GraphInfo getGraphInfo(GraphType graphType, AID id, int index) 
		throws RemoteException, ResultsRepositoryException {
		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
		try {
			return analysisEntityManager.getAnalysisGraphInfo(graphType, id, index);
		} catch (IOException e) {
			Logger.logError("I/O error reading graph info for " 
					+ Database.getAnalysisReferenceString(id) + ".");
			return null;
		}
	}

  	public DownloadHandle beginGraphDownload(GraphType graphType, AID id, InetAddress ip, int port,
			int index) throws RemoteException, ResultsRepositoryBusyException, 
			ResultsRepositoryException {

  		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
  		
  		if (graphCount(graphType, id) == 0) {
			throw new IllegalArgumentException("Graph doesn't exist.");
		} else {
			try {
		
			return analysisEntityManager.downloadAnalysisFile(id,  
					analysisEntityManager.getAnalysisGraphPath(graphType, id, index),
					ip.getCanonicalHostName(),
					port, false);
			} catch (IOException e) {
				Logger.logError("I/O error reading graph for " 
						+ Database.getAnalysisReferenceString(id) + ".");
				throw new ResultsRepositoryException(e);
			}
		}		
	}

		public DownloadHandle beginRawDataDownload(AID id, InetAddress ip, int port) throws ResultsRepositoryBusyException, RemoteException {
			throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
		}

		public void endRawDataDownload(DownloadHandle handle) throws RemoteException {
			throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
		}

		public DownloadStatus getRawDataDownloadStatus(DownloadHandle handle) throws RemoteException {
			throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
		}

		public String getContext(AID id) throws RemoteException {
			throw new UnsupportedOperationException("Single context for analysis don't exist.");
		}

		public String[] getTasks(AID id) throws RemoteException {
			throw new UnsupportedOperationException("Task list for whole analysis not supported.");
		}

		public LogRecord[] getLogsForTask(AID id, String taskID) throws RemoteException,
				LogStorageException, IllegalArgumentException, NullPointerException {
			throw new UnsupportedOperationException("Logs for tasks for whole analysis not supported.");
		}
}
