/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Semaphore;

import cz.cuni.mff.been.benchmarkmanager.BenchmarkManagerService;
import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.logging.FilesystemLogStorage;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.task.Task;

/*
 * Note: internal been api uses ResultsRepositoryException, gui uses
 * RuntimeException - unite this
 * 
 * /** Implementation of <code>DatabaseInterface</code> Instance of this class
 * used inside Results Repository This is the core of the whole Results
 * Repository - it allows data storage and retrieval in the database, it
 * performs storage of binary representation of results, simple database
 * querying (simple selects), and network connection synchronization
 * 
 * @author Jakub Lehotsky
 */

public final class Database {

	/** Buffer size for file uploading to database */
	private static final int UPLOAD_BUFFER_SIZE = 4096;

	/** Buffer size for file downloading from database */
	private static final int DOWNLOAD_BUFFER_SIZE = 4096;

	/** Too many connections text message */
	public static final String TOO_MANY_CONNECTIONS_MESSAGE 
		= "Too many connections to Results Repository.";
	
	/** Name of the dir in database to store rscripts */
	public static final String RSCRIPTS_DIRECTORY = "rscripts";

	/** Name of the dir in database to store scripts to be executed on invalidation */ 
	public static final String INVALIDATE_RSCRIPTS_DIRECTORY = "invalidate";
	
	/** Name of the dir in database to store load information */
	public static final String LOAD_STORAGE_DIRECTORY = "load";

	
	/** Extensions of temporary directory for R scripts unpacking */
	private static final String TEMP_DIR_EXT = ".unpack";

	/** Running instance of Benchmark Manager to connect to */
	private static BenchmarkManagerGUIInterface benchmarkManagerInstance;
	
	/** The single existing intance of database */
	private static Database dbInstance = null;
	
	/** Active connections counter */
	private static int activeConnections;

	/** Maximal network connections (uploads + downloads) */
	private static int maxConnections;

	/** Path to the Results Repository path with stored results */
	private static String databasePath;

	/** Internal RID Manager */
	private static RunValidityMapsManager ridManager;

	/** Internal graph manager */
	private static GraphManager graphManager;

	/** Internal statistics manager */
	private static StatisticsManager statisticsManager;

	/** Internal load collecor of database */
	@SuppressWarnings("unused")
	private static LoadCollector loadCollector;

	/** Internal log collector of database */
	private static LogCollector logCollector;

	/** Internal results repository log storage */
	private static FilesystemLogStorage logStorage;

	/**
	 * Connection semaphore - download/upload synchronization in blocking
	 * synchronization mode
	 */
	private static Semaphore connectionSemaphore;

	/** Download statuses. */
	private Map<DownloadHandle, DownloadStatus> downloadStatuses = Collections
			.synchronizedMap(new HashMap<DownloadHandle, DownloadStatus>());

	/** Upload statuses. */
	private Map<UploadHandle, UploadStatus> uploadStatuses = Collections
			.synchronizedMap(new HashMap<UploadHandle, UploadStatus>());

	/** Upload callbacks. */
	private Map<UploadHandle, FileUploadCallbackInterface> uploadCallbacks = Collections
			.synchronizedMap(new HashMap<UploadHandle, FileUploadCallbackInterface>());

	/** Upload callback data. */
	private Map<UploadHandle, Object> uploadCallbackData = Collections
			.synchronizedMap(new HashMap<UploadHandle, Object>());

	/** Upload error messages. */
	private Map<UploadHandle, String[]> uploadErrorMessages = Collections
			.synchronizedMap(new HashMap<UploadHandle, String[]>());

	/** Temporary directory */
	private String tempDirectory;	

	/** Metadata manager - loading and storing */
	private static MetadataManager metaManager;

	/** Role list manager */
	private static RolesManager rolesManager;

	/** Internal Callbacks Manager Factory and holder */
	private static RManager rManager;

	/** Internal Callbacks Manager scripts runner and scheduler */
	private static RScriptsRunner rScriptsRunner;
	
	/** Internal Importer instance */
	@SuppressWarnings("unused")
	private static Importer importer;

	/** Internal Exporter instance */
	@SuppressWarnings("unused")
	private static Exporter exporter;
	
	/** Internal Lock Manager instance */
	@SuppressWarnings("unused")
	private static LockManager lockManager;

	/** Internal Task List manager */
	private TaskListManager taskListManager;

	/** Analyses level of entity manager */
	private AnalysisEntityManager analysisEntityManager;

	/** Experiment level of entity manager */
	private ExperimentEntityManager experimentEntityManager;

	/** Binary level of entity manager */
	private BinaryEntityManager binaryEntityManager;

	/** Run level of entity manager */
	private RunEntityManager runEntityManager;

	
	/**
	 * Constructor - private, void for singleton use
	 * 
	 */
	private Database() {
	}

	/**
	 * Return single instance of Results Repository database
	 * 
	 * @return reference single running instance of Database
	 */
	public static Database getInstance() {
		if (dbInstance == null) {
			dbInstance = new Database();
		}
		return dbInstance;
	}

	/**
	 * Initialize Results Repository database, call to this method intializes all the necessary
	 * components of Results Repository according to their dependencies and pass their references
	 * to each other as needed. It must be called before any other use of Results Repository 
	 * 
	 * @param dataPath
	 *				path to Results Repository database in file system
	 * @param tempDir 
	 * 				path to temporary directory
	 * @param aMaxConnections
	 *				maximal simultaneous connections for Results Repository, 0
	 *				means no limit
	 */
	public void initialize(String dataPath, String tempDir, int aMaxConnections) {
		
		/* Basic setting of database class */
		databasePath = dataPath;
		tempDirectory = tempDir;
		maxConnections = aMaxConnections;
		activeConnections = 0;
		connectionSemaphore = new Semaphore(maxConnections, true);
		
		/* Initialize locks manager */
		lockManager = LockManager.getInstance();
		LockManager.initialize(this);
		
		/* Initialize metadata manager */
		metaManager = MetadataManager.getInstance();
		MetadataManager.initialize(this);
		
		/* Intialize roles manager */
		rolesManager = RolesManager.getInstance();
		RolesManager.initialize(this);
		
		/* Initialization of low-level run validity maps manager */
		ridManager = RunValidityMapsManager.getInstance();
		RunValidityMapsManager.initialize(this, rolesManager);
		
		/* Intializes r packages manager (this is itself boss of few other managers) */
		rManager = RManager.getInstance();
		RManager.initialize(this);
		
		rScriptsRunner = RManager.getRScriptsRunner();
		
		/* Initialization of low-level graphs manager */
		graphManager = GraphManager.getInstance();
		GraphManager.initialize(this);
		
		/* Initialize statistics manager */
		statisticsManager = StatisticsManager.getInstance();
		StatisticsManager.initialize(this);
		
		/* Initialize task-entity mapping manager */
		taskListManager = TaskListManager.getInstance();
		TaskListManager.initialize(this);
		
		/* Intialize reference to benchmark manager */
		initBMReference();
		
		/* Intialize log storage */
		try {
			logStorage = new FilesystemLogStorage(getDatabasePath()
					+ File.separator + "logs");
		} catch (LogStorageException e) {
			Logger.logError(
					"Error intializing results repository log storage: "
							+ e.getMessage() + ".");
		}
		
		/* Intialize log collector */ 
		logCollector = LogCollector.getInstance();
		LogCollector.initialize(this, experimentEntityManager);
		
		/* Initialize high-level analyses manager */
		analysisEntityManager = AnalysisEntityManager.getInstance();
		analysisEntityManager.initialize(this, metaManager, statisticsManager, graphManager,
				rScriptsRunner);

		/* Initialize high-level experiments manager */
		experimentEntityManager = ExperimentEntityManager.getInstance();
		experimentEntityManager.initialize(this, metaManager, statisticsManager, graphManager,
				logCollector, taskListManager, rolesManager, rScriptsRunner, analysisEntityManager);		
		
		/* Initialize high-level binaries manager */
		binaryEntityManager = BinaryEntityManager.getInstance();
		binaryEntityManager.initialize(this, metaManager, statisticsManager, taskListManager, 
				rScriptsRunner, analysisEntityManager, experimentEntityManager);

		/* Initialize high-level runs manager */
		runEntityManager = RunEntityManager.getInstance();
		runEntityManager.initialize(this, statisticsManager, taskListManager, graphManager, 
				ridManager, lockManager, rScriptsRunner, binaryEntityManager, experimentEntityManager);

		/* Initialize load collector */
		loadCollector = LoadCollector.getInstance();
		LoadCollector.initialize(this, binaryEntityManager);

		/* Intialize importer and exporter */
		importer = Importer.getInstance();
		Importer.initialize(this);
		exporter = Exporter.getInstance();
		Exporter.initialize(this);
		
	}
	
	/**
	 * Initializes Benchmark Manager reference (gets its remote interface and store it for use) 
	 */
	private void initBMReference() {
	    
	    Task task = Task.getTaskHandle();
	    
	    if (task != null) {	    
		try {
	            benchmarkManagerInstance = (BenchmarkManagerGUIInterface) task.getTasksPort().serviceFind(
	            	BenchmarkManagerService.SERVICE_NAME, BenchmarkManagerService.RMI_MAIN_IFACE);
                } catch (RemoteException e) {
                    Logger.logError("Unable to get Benchmark Manager RMI reference.");
                    e.printStackTrace();
                }
	    }
	    
	}
	
	/**
	 * @return the logStorage
	 */
	public FilesystemLogStorage getLogStorage() {
		return logStorage;
	}

	/**
	 * Returns internal instance of AnalysisEntityManager
	 * 
	 * @return	The instance
	 */
	public AnalysisEntityManager getAnalysisEntityManager() {
		return analysisEntityManager;
	}

	/**
	 * Returns internal instance of ExperimentEntityManager
	 * 
	 * @return	The instance
	 */
	public ExperimentEntityManager getExperimentEntityManager() {
		return experimentEntityManager;
	}

	/**
	 * Returns internal instance of BinaryEntityManager
	 * 
	 * @return	The instance
	 */
	public BinaryEntityManager getBinaryEntityManager() {
		return binaryEntityManager;
	}

	/**
	 * Returns internal instance of BinaryEntityManager
	 * 
	 * @return	The instance
	 */
	public RunEntityManager getRunEntityManager() {
		return runEntityManager;
	}
	
	/**
	 * Returns R manager instance
	 * 
	 * @return	The instance
	 */
	public RManager getRManager() {
		return rManager;
	}

	/**
	 * Returns run validity map manager 
	 *
	 * @return	The instance
	 */
	public RunValidityMapsManager getRunValidityMapsManager() {
		return ridManager;
	}

	
	/**
	 * Returns lock manager instance  
	 *
	 * @return	The instance
	 */
	public LockManager getLockManager() {
		return lockManager;
	}

	
	/**
	 * Returns benchmark manager Interface
	 * 
	 * @return	Interface of running benchmark manager 
	 */
	public BenchmarkManagerGUIInterface getBenchmarkManagerInstance() {
		
		/*if (!benchmarkManagerInstanceIsStillAlive) */
		initBMReference();
		
		return benchmarkManagerInstance;
	}
	
	/**
	 * Returns temporary directory of Results Repository
	 * 
	 * @return	The directory
	 */
	public String getTempDirectory() {
	    return tempDirectory;
	}

	/**
	 * Shutdown of results repository database. 
	 */
	public void shutdown() {
		rManager.shutdown();
	}

	/**
	 * Returns reference to internal load collector
	 * 
	 * @return Single valid LoadCollector instance
	 */
	public LoadCollector getLoadCollector() {
		return loadCollector;
	}

	/**
	 * Returns reference to internal log collector
	 * 
	 * @return Single valid LogCollector instance
	 */
	public LogCollector getLogCollector() {
		return logCollector;
	}

	/* ************************************************* */
	/* **** ResultsRepositoryImplementation support **** */
	/* ************************************************* */	
	

	/**
	 * Returns path to stored database
	 * 
	 * @return The path
	 */
	public String getDatabasePath() {
		return databasePath;
	}

	/**
	 * Construcucts a path for analysis directory
	 * 
	 * @param aid
	 *            Analysis ID
	 * @return Fully qualified path
	 */
	public String getDirectoryInDatabase(AID aid) {
		return getDatabasePath() + File.separator + aid;
	}

	/**
	 * Construcucts a path for filename to store (or stored) in a given analysis
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param filename
	 *            Filename of the file in database
	 * @return Fully qualified path
	 */
	public String getFileInDatabase(AID aid, String filename) {
		return getDirectoryInDatabase(aid) + File.separator + filename;
	}

	/**
	 * Construcucts a path for experiment directory
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @return Fully qualified path
	 */
	public String getDirectoryInDatabase(AID aid, EID eid) {
		return getDatabasePath() + File.separator + aid + File.separator + eid;
	}

	/**
	 * Construcucts a path for filename to store (or stored) in a given
	 * experiment
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param filename
	 *            Filename of the file in database
	 * @return Fully qualified path
	 */
	public String getFileInDatabase(AID aid, EID eid, String filename) {
		return getDirectoryInDatabase(aid, eid) + File.separator + filename;
	}

	/**
	 * Construcucts a path for binary directory
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @return Fully qualified path
	 */
	public String getDirectoryInDatabase(AID aid, EID eid, BID bid) {
		return getDatabasePath() + File.separator + aid + File.separator + eid
				+ File.separator + bid;
	}

	/**
	 * Construcucts a path for filename to store (or stored) in a given binary
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param filename
	 *            Filename of the file in database
	 * @return Fully qualified path
	 */

	public String getFileInDatabase(AID aid, EID eid, BID bid, String filename) {
		return getDirectoryInDatabase(aid, eid, bid) + File.separator
				+ filename;
	}

	/**
	 * Returns a path where analysis stores its R scripts (no checking whteher
	 * analysis or directory exists)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @return Fully qualified path
	 */
	public String getRScriptsDirectory(AID aid) {
		return getDirectoryInDatabase(aid) + File.separator
				+ RSCRIPTS_DIRECTORY;
	}

	/**
	 * Returns a path where experiment stores its R scripts (no checking whteher
	 * analysis or directory exists)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @return Fully qualified path
	 */

	public String getRScriptsDirectory(AID aid, EID eid) {
		return getDirectoryInDatabase(aid, eid) + File.separator
				+ RSCRIPTS_DIRECTORY;
	}
	
	/**
	 * Returns a path to invalidate R scritps (to be executed after delete) 
	 * directory for an analysis  
	 * 
	 * @param aid
	 * 		Analysis ID
	 * 
	 * @return Fully qualified path
	 */
	public String getInvalidateScriptsDirectory(AID aid) {
		return getDirectoryInDatabase(aid) + File.separator
			+ INVALIDATE_RSCRIPTS_DIRECTORY;		
	}

	/**
	 * Returns a path to invalidate R scritps (to be executed after delete) 
	 * directory for an experiment 
	 * 
	 * @param aid
	 * 		Analysis ID
	 * @param eid
	 * 		Experiment ID
	 * 
	 * @return Fully qualified path
	 */
	public String getInvalidateScriptsDirectory(AID aid, EID eid) {
		return getDirectoryInDatabase(aid, eid) + File.separator
			+ INVALIDATE_RSCRIPTS_DIRECTORY;		
	}
	
	/**
	 * Returns a path to invalidate R scritps (to be executed after delete) 
	 * directory for a binary 
	 * 
	 * @param aid
	 * 		Analysis ID
	 * @param eid
	 * 		Experiment ID
	 * @param bid
	 * 		Binary ID
	 * 
	 * @return Fully qualified path
	 */	
	public String getInvalidateScriptsDirectory(AID aid, EID eid, BID bid) {
		return getDirectoryInDatabase(aid, eid, bid) + File.separator
			+ INVALIDATE_RSCRIPTS_DIRECTORY;		
	}
	
	/**
	 * Returns a path where load information of runs for a given binary is stored
	 * 
	 * @param aid
	 * 		Analysis ID
	 * @param eid
	 * 		Experiment ID
	 * @param bid
	 * 		Binary ID
	 * @return Fully qualified path
	 */
	public String getLoadDirectory(AID aid, EID eid, BID bid) {		
		return getDirectoryInDatabase(aid, eid, bid) + File.separator
			+ LOAD_STORAGE_DIRECTORY;
	}
	

	/**
	 * Constructs short string with analysis ID (useful for logs)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @return String 'analysis <code>aid</code>'
	 */
	public static String getAnalysisReferenceString(AID aid) {
		return "Analysis " + aid;
	}

	/**
	 * Constructs short string with analysis/experiment IDs (useful for logs)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @return String 'experiment <code>eid</code> (analysis <code>aid</code>)'
	 */
	public static String getExperimentReferenceString(AID aid, EID eid) {
		return "Experiment " + eid + " (analysis " + aid + ")";
	}

	/**
	 * Constructs short string with analysis/experiment/binary IDs (useful for
	 * logs)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @return String 'binary <code>bid</code> (analysis <code>aid</code>,
	 *         experiment <code>eid</code>)'
	 */
	public static String getBinaryReferenceString(AID aid, EID eid, BID bid) {
		return "Binary " + bid + " (analysis " + aid + ", experiment " + eid
				+ ")";
	}

	/**
	 * Constructs short string with analysis/experiment/binary IDs (useful for
	 * logs)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param bid
	 *            Binary ID
	 * @param rid
	 *            Run ID
	 * @return String 'run <code>rid</code> (analysis <code>aid</code>,
	 *         experiment <code>eid</code>, binary <code>bid</code>)'
	 */
	public static String getRunReferenceString(AID aid, EID eid, BID bid,
			RID rid) {
		return "run " + rid + " (analysis " + aid + ", experiment " + eid
				+ ", binary " + bid + ")";
	}

	/**
	 * Returns actual active connections (uploads/downloads)
	 * 
	 * @return Active connection count
	 */
	public int getActiveConnections() {
		return activeConnections;
	}

	/**
	 * Returns true, if database is busy with uploading of a file (or files).
	 * 
	 * @return True if busy, False if ready for another upload (download)
	 */
	public boolean isBusy() {
		return (maxConnections > 0)
				&& (getActiveConnections() >= maxConnections);
	}

	/**
	 * Returns true if metadata is non-null, else logs error message about
	 * itemName and entity described in referenceString
	 * 
	 * @param metadata
	 * @param itemName
	 * @param referenceString
	 * @return true iff metadata is non-null
	 */
	public static boolean ensureMetadataNotNull(String metadata,
			String itemName, String referenceString) {
		if (metadata == null) {
			Logger.logWarning(
					"Error reading metadata item '" + itemName + "' - null ("
							+ referenceString + ").");
			return false;
		} else {
			return true;
		}
	}


	/** *********** File ************** */
	/**
	 * Thread used for uploading of files - connects to a given host and port
	 * and stores file read from there in a given filename
	 * 
	 * @author D.Majda, J.Lehotsky
	 */
	private class FileUploader extends Thread {

		/**
		 * How many times we should try to create a temp file. See
		 * <code>createTempFile</code> method for explanation, why we need
		 * this constant.
		 */
		private static final int MAX_CREATE_TEMP_FILE_TRY_COUNT = 1024;

		/** Upload handle. */
		private UploadHandle handle;

		/** Host to upload file from */
		private String host;

		/** Port to download the file from. */
		private int port;

		/** Filename, where to store file in database */
		private String filename;

		/**
		 * Indicates whether we're upload r script and thus registration is
		 * required
		 */
		private boolean rScript;

		/**
		 * Construction of thread
		 * 
		 * @param handle
		 * 
		 * @param filename
		 *            Filename to store file to
		 * @param host
		 *            Host where file will be uploaded
		 * @param port
		 *            Port which this host use for this purpose
		 * @param rScript
		 *            True, if we're uploading R scripts package
		 */
		public FileUploader(UploadHandle handle, String filename, String host,
				int port, boolean rScript) {
			super();
			this.filename = new String(filename);
			this.host = new String(host);
			this.port = port;
			this.handle = handle;
			this.rScript = rScript;
			setStatus(UploadStatus.INITIALIZING);
			setErrorMessages(new String[] {});
		}

		/**
		 * Sets the upload status to given value.
		 * 
		 * @param status
		 *            downloaD status
		 */
		private void setStatus(UploadStatus status) {
			uploadStatuses.put(handle, status);
		}

		/**
		 * Sets the error messages.
		 * 
		 * @param errorMessages
		 *            error messages
		 */
		private void setErrorMessages(String[] errorMessages) {
			uploadErrorMessages.put(handle, errorMessages);
		}

		/**
		 * Performs an upload - connects through socket to given host and port,
		 * downloads files and store them at destination place
		 * 
		 * @param tmpFile
		 * 
		 * @throws UnknownHostException
		 * @throws IOException
		 * @throws InterruptedException
		 */
		public void uploadAFile(File tmpFile) throws UnknownHostException,
				IOException, InterruptedException {
			byte[] buffer = new byte[UPLOAD_BUFFER_SIZE];
			int bytesRead;

			if (maxConnections > 0) {
				activeConnections++;
				connectionSemaphore.acquire();
			}

			File file;
			if (tmpFile == null) {
				file = new File(filename);
			} else {
				file = tmpFile;
			}
			Socket socket = new Socket(host, port);
			try {
				InputStream inputStream = new BufferedInputStream(socket
						.getInputStream(), UPLOAD_BUFFER_SIZE);
				OutputStream outputStream = new BufferedOutputStream(
						new FileOutputStream(file.getPath()),
						UPLOAD_BUFFER_SIZE);

				try {
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						outputStream.write(buffer, 0, bytesRead);
					}
				} finally {
					inputStream.close();
					outputStream.close();
				}

				if (tmpFile == null) {
					// We're not having anything temporary for later validation
					setStatus(UploadStatus.ACCEPTED);
				}

				Task taskHandle = Task.getTaskHandle();
			    	if (taskHandle != null) {
			    	    Task.getTaskHandle().getTasksPort().checkPointReached(
						"upload-finish-" + handle.hashCode(), "1");
			    	} else {
			    	    Logger.logInfo("Unable to reach task's port, checkpoint not set.");
			    	}
				
				Logger.logInfo(
						"Results Repository thread: file stored.");
			} finally {
				socket.close();
				if (maxConnections > 0) {
					connectionSemaphore.release();
					activeConnections--;
				}
			}
		}

		/**
		 * Creates teporary file in the <code>tmpDir</code> directory.
		 * 
		 * @return the temporary file
		 * 
		 * @throws IOException
		 *             if the temporary file can not be created
		 */
		private File createTempFile() throws IOException {
			/*
			 * There is a bug in the Sun's JVM which causes the
			 * File.createTempFile call throw IOException with message "Access
			 * is denied" occasionally on Windows. We work around the bug by
			 * simply retrying the call multiple times. After a while we give
			 * up, to avoid hanging-up.
			 * 
			 * See http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6325169
			 * for more information about the bug.
			 */
			File result = null;
			int tryCount = 0;
			boolean failed;
			IOException thrownException = null;

			do {
				failed = false;
				try {
					result = File.createTempFile("been-results-repository",
							null, new File(Database.getInstance().getTempDirectory()));
				} catch (IOException e) {
					failed = true;
					thrownException = e;
				}
				tryCount++;
			} while (failed && tryCount < MAX_CREATE_TEMP_FILE_TRY_COUNT);

			if (result != null) {
				return result;
			} else {
				throw thrownException;
			}
		}

		/**
		 * Unpacks the contents of package and validates that its contents are valid
		 * R package
		 * 
		 * @param path
		 *            The path to the package to validate
		 * @return The list of error messages while validating package
		 */
		private String[] unpackAndValidateRScript(String path) {
			
			ArrayList<String> errMessages = new ArrayList<String>();
			
			Logger.logDebug(
					"Unpacking " + path + " into " + path + TEMP_DIR_EXT + ".");
			try {
				if (!(new File(path + TEMP_DIR_EXT).mkdir())) {
					throw new IOException("Couldn't create directory - " + path
							+ TEMP_DIR_EXT + ".");
				}
				Zipper.unzipFile(path, path + TEMP_DIR_EXT);
				File[] directoryList;
				directoryList = new File(path + TEMP_DIR_EXT).listFiles();
				for (File file : directoryList) {
					String filename = file.getName();
					if (!file.isFile()) {
						errMessages.add("Package cannot contain directory.");
					}
					if (!filename.equals("description") &&
							!filename.matches(".*\\.r") &&
							!filename.matches(".*\\.hlp")) {
								errMessages.add("Invalid item found in package : \"" + filename  
										+ "\", only 'description', '*.r', '*.hlp' are allowed");
					}
				}
			} catch (IOException e) {
				errMessages.add("I/O error registering package - see Results Repository stderr for stack trace");
				e.printStackTrace();
			}
			return errMessages.toArray(new String[] {});
		}
		
		
		
		/**
		 * Main thread function - just calls file uploading and logs occasional
		 * errors
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {

			setStatus(UploadStatus.UPLOADING);

			try {
				if (rScript) {
					File tempFile = createTempFile();
					File tomoveFile = null;
					try {
						uploadAFile(tempFile);
						String[] errorMessages = unpackAndValidateRScript(tempFile.getPath());
						if (errorMessages.length > 0) {
							throw new RPackageUploadException(
									"Tried to upload invalid package "
										+ "(host = " + host + ", port = " + port + ").",
									errorMessages);
						}

						/*
						 * Try to move the file. If the move fails, bail
						 * everything and delete the temp file. The file name
						 * won't be reused and is lost forever.
						 */
						File packageFile = new File(getDatabasePath()
								+ File.separator + RSCRIPTS_DIRECTORY
								+ File.separator + filename);

						tomoveFile = new File(tempFile.getPath() + TEMP_DIR_EXT);

						if (!tomoveFile.renameTo(packageFile)) {
							throw new IOException("Cant't rename file \""
									+ tomoveFile.getPath() + "\" to \""
									+ packageFile.getPath() + "\".");
						}

						rManager.sourceOnePackageScript("");
						rManager.reloadRFunctionsHelp();

						Logger.logInfo("Successfully registered R package \""
									+ packageFile.getPath() + "\".");
						setStatus(UploadStatus.ACCEPTED);

						/*
						 * After all operations finish (either succesfully or
						 * due to some error), delete the temporary file if it
						 * still exists.
						 */
					} finally {
						eraseDirectoryRecursively(tempFile);
						eraseDirectoryRecursively(tomoveFile);
					}

				} else {
					uploadAFile(null);
				}
			} catch (UnknownHostException e) {
				e.printStackTrace();
				Logger.logError("Upload error: " + e.getMessage() + ".");
				setStatus(UploadStatus.ERROR);
			} catch (RPackageUploadException e) {
				String[] errorMessages = e.getErrorMessages();
				Logger.logError(
						e.getMessage() + " " + Integer.toString(errorMessages.length)
								+ " messages reported:");
				for (String errorMessage : errorMessages) {
					Logger.logError("  " + errorMessage);
				}
				setErrorMessages(errorMessages);
				setStatus(UploadStatus.REJECTED);
			} catch (IOException e) {
				e.printStackTrace();
				Logger.logError("Upload error: " + e.getMessage() + ".");
				setStatus(UploadStatus.ERROR);
			} catch (ResultsRepositoryException e) {
				e.printStackTrace();
				Logger.logError("Package registered - error sourcing into R: " + e.getMessage() + ".");
				setStatus(UploadStatus.ERROR);			
			} catch (InterruptedException e) {
				e.printStackTrace();
				Logger.logError("Upload error: " + e.getMessage() + ".");
				setStatus(UploadStatus.ERROR);
			} finally {
				try {
				    	Task taskHandle = Task.getTaskHandle();
				    	if (taskHandle != null) {
				    	    Task.getTaskHandle().getTasksPort().checkPointReached(
							"upload-finish-" + handle.hashCode(), "1");
				    	} else {
				    	    Logger.logInfo("Unable to reach task's port, checkpoint not set.");
				    	}
				} catch (RemoteException e) {
					Logger.logError(
						"Error comunicating with Host Runtime tasks port - unable to set checkpoint.");
				}
			}
		}
	}

	/**
	 * Thread used for downloading of files - connects to a given host and port
	 * and stores file read from there in a given filename
	 * 
	 * @author D.Majda, J.Lehotsky
	 */
	private class FileDownloader extends Thread {

		/** Upload handle. */
		private DownloadHandle handle;

		/** Host to download file to */
		private String host;

		/** Port to download the file from. */
		private int port;

		/** Filename, where to store file in database */
		private String filename;

		/**
		 * Construction of thread
		 * 
		 * @param handle
		 *            Handle of the file to be downloaded
		 * @param filename
		 *            Filename to store file to
		 * @param host
		 *            Hostname from which uploading sends data
		 * @param port
		 *            Port which this host use for this purpose
		 */
		public FileDownloader(DownloadHandle handle, String filename,
				String host, int port) {
			super();
			this.filename = new String(filename);
			this.host = new String(host);
			this.port = port;
			this.handle = handle;
			setStatus(DownloadStatus.INITIALIZING);

		}

		/**
		 * Sets the download status to given value.
		 * 
		 * @param status
		 *            downloa status
		 */
		private void setStatus(DownloadStatus status) {
			downloadStatuses.put(handle, status);
		}

		/**
		 * Performs a download - sets database status to true (notice, that
		 * database alone doesn't care how many simultaneous transfers run at a
		 * time), connects through socket to given host and port, read file and
		 * upload them
		 * 
		 * @throws UnknownHostException
		 * @throws IOException
		 * @throws InterruptedException
		 */
		public void downloadAFile() throws UnknownHostException, IOException,
				InterruptedException {
			byte[] buffer = new byte[DOWNLOAD_BUFFER_SIZE];
			int bytesRead;

			if (maxConnections > 0) {
				activeConnections++;
				connectionSemaphore.acquire();
			}

			File file = new File(filename);
			Socket socket = new Socket(host, port);
			try {

				InputStream inputStream = new BufferedInputStream(
						new FileInputStream(file.getPath()),
						DOWNLOAD_BUFFER_SIZE);
				OutputStream outputStream = new BufferedOutputStream(socket
						.getOutputStream(), DOWNLOAD_BUFFER_SIZE);

				try {
					while ((bytesRead = inputStream.read(buffer)) != -1) {
						outputStream.write(buffer, 0, bytesRead);
					}
				} finally {
					outputStream.close();
					inputStream.close();
				}
				setStatus(DownloadStatus.SUCCEEDED);
				Logger.logInfo(
						"Results Repository thread: transfer finished, file sent.");

			} finally {

				socket.close();

				if (maxConnections > 0) {
					connectionSemaphore.release();
					activeConnections--;
				}
			}
		}

		/**
		 * Main thread function - just calls file uploading and logs occasional
		 * errors
		 * 
		 * @see java.lang.Runnable#run()
		 */
		public void run() {

			setStatus(DownloadStatus.DOWNLOADING);
			try {
				downloadAFile();
			} catch (UnknownHostException e) {
				e.printStackTrace();
				Logger.logError("Download error: " + e.getMessage() + ".");
				setStatus(DownloadStatus.ERROR);
			} catch (IOException e) {
				e.printStackTrace();
				Logger.logError("Download error: " + e.getMessage() + ".");
				setStatus(DownloadStatus.ERROR);
			} catch (InterruptedException e) {
				e.printStackTrace();
				Logger.logError("Download error: " + e.getMessage() + ".");
				setStatus(DownloadStatus.ERROR);
			}
		}
	}

	/**
	 * General method, wich uploads a file into Results Repository database. The
	 * client must open the socket on some port before calling this function.
	 * Then it calls the <code>uploadFile</code> method.
	 * 
	 * This method work asynchronously and returns immediately. Before return it
	 * starts a new thread that will receive the file contents from the socket
	 * open on given host and port.
	 * 
	 * If there is any error when sending the file contents, the socket is
	 * closed.
	 * 
	 * Client has an opportunity to check (via the
	 * <code>getFileUploadStatus</code> method to check the current status of
	 * upload (eventually can be error)
	 * 
	 * After the upload finishes, client should close the returned handle via
	 * the <code>endFileUpload</code> method.
	 * 
	 * If you need to perform some additional functionality, after file is
	 * uploaded, you should pass it call back function (and its argument
	 * accordingly), otherwise leave them to null. Note: this is used only in
	 * this low-level uploadFile, not in special upload functions and is
	 * reserved for statistical computation and importing.
	 * 
	 * @param filename
	 *            Filename of file to store uploaded data
	 * @param host
	 *            Hostname, from which file is uploaded
	 * @param port
	 *            Open port for network transfer
	 * @param blocking
	 *            Determines whether to use blocking or non-blocking version of
	 *            upload
	 * @param rScript
	 * 			  Tells, whether we're uploading rScript
	 * @param callbackInterface
	 *            callback to run after file is uploaded (can be null)
	 * @param data
	 *            argument of callback (if there is one, can be null)
	 * @return UploadHandle, which could be used to refer to transfer later
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws ResultsRepositoryBusyException
	 */
	public UploadHandle uploadFile(String filename, String host, int port,
			boolean blocking, boolean rScript,
			FileUploadCallbackInterface callbackInterface, Object data)
			throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		// every upload goes through this method

		if (isBusy() && !blocking) {
			throw new ResultsRepositoryBusyException(
					TOO_MANY_CONNECTIONS_MESSAGE);
		}

		UploadHandle handle = UploadHandle.createUploadHandle();

		uploadCallbacks.put(handle, callbackInterface);

		uploadCallbackData.put(handle, data);

		FileUploader uploader = new FileUploader(handle, filename, host, port,
				rScript);
		uploader.start();
		return handle;
	}

	/**
	 * General method, wich downloads a file from Results Repository database.
	 * The client must open the socket on some port before calling this
	 * function. Then it calls the <code>downloadFile</code> method.
	 * 
	 * This method work asynchronously and returns immediately. Before return it
	 * starts a new thread that will receive the file contents from the socket
	 * open on given host and port.
	 * 
	 * If there is any error when sending the file contents, the socket is
	 * closed.
	 * 
	 * Client has an opportunity to check (via the
	 * <code>getFileDownloadStatus</code> method to check the current status
	 * of download (eventually can be error)
	 * 
	 * After the upload finishes, client should close the returned handle via
	 * the <code>endFileDownload</code> method.
	 * 
	 * @param filename
	 *            Filename of file to store uploaded data
	 * @param host
	 *            Hostname, from which file is uploaded
	 * @param port
	 *            Open port for network transfer
	 * @param blocking
	 *            Determines, whether to use blocking download
	 * @return DownloadHandle, which could be used to refer transfer later
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws ResultsRepositoryBusyException
	 */
	public DownloadHandle downloadFile(String filename, String host, int port,
			boolean blocking) throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		// every download goes through this method

		if (isBusy() && !blocking) {
			throw new ResultsRepositoryBusyException(
					TOO_MANY_CONNECTIONS_MESSAGE);
		}

		DownloadHandle handle = DownloadHandle.createDownloadHandle();
		FileDownloader downloader = new FileDownloader(handle, filename, host,
				port);
		downloader.start();
		return handle;
	}

	/**
	 * Returns the status of the file upload. Result is one of the values listed
	 * in the <code>FileTransferStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>UPLOADING</code> mean that
	 * Results Repository still works on the file upload. Values
	 * <code>ACCEPTED</code> and <code>ERROR</code> mean that Results
	 * Repository has finished its work (either successfully or unsuccessfully)
	 * and the download handle is ready to be closed by the
	 * <code>endFileUpload</code> method.
	 * 
	 * @param handle
	 *            file upload handle
	 * @return upload status
	 */
	public UploadStatus getFileUploadStatus(UploadHandle handle) {

		UploadStatus value = uploadStatuses.get(handle);

		if (value != null) {
			return value;
		} else {
			throw new IllegalArgumentException("Invalid file upload handle.");
		}
	}

	/**
	 * Returns the status of the file download. Result is one of the values
	 * listed in the <code>FileTransferStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>DOWNLOADING</code> mean
	 * that Software Repository still works on the file download. Values
	 * <code>SUCCEEDED</code> and <code>ERROR</code> mean that Results
	 * Repository has finished its work (either successfully or unsuccessfully)
	 * and the download handle is ready to be closed by the
	 * <code>endFileDownload</code> method.
	 * 
	 * @param handle
	 *            file download handle
	 * @return download status
	 */
	public DownloadStatus getFileDownloadStatus(DownloadHandle handle) {

		DownloadStatus value = downloadStatuses.get(handle);

		if (value != null) {
			return value;
		} else {
			throw new IllegalArgumentException("Invalid file download handle.");
		}
	}

	/**
	 * Closes the upload handle. Client should call this method after the upload
	 * status returned by <code>getFileUploadStatus</code> has value
	 * <code>ACCEPTED</code>, or <code>ERROR</code>.
	 * 
	 * If the client doesn't close the upload handle, Results Repository will
	 * leak a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle.
	 * 
	 * @param handle
	 *            upload handle
	 */
	public void endFileUpload(UploadHandle handle) {

		Object callback;
		Object callbackData;

		callback = uploadCallbacks.get(handle);
		callbackData = uploadCallbackData.get(handle);

		if (callback != null) {
			
			RScriptsRunner rScriptsRunner = RManager.getRScriptsRunner();
			
			rScriptsRunner.enqueueCallbackExecution(
					((FileUploadCallbackInterface) callback).getAid(),
					((FileUploadCallbackInterface) callback).getEid(),
					((FileUploadCallbackInterface) callback).getBid(),
					((FileUploadCallbackInterface) callback).getRid(),
					(FileUploadCallbackInterface) callback,
					callbackData);
					
		} else {
			Logger.logInfo(
			"Upload thread: callback function is not mapped for upload.");
		}

		uploadStatuses.remove(handle);
		uploadErrorMessages.remove(handle);
		uploadCallbacks.remove(handle);
		uploadCallbackData.remove(handle);
	}

	/**
	 * Closes the download handle. Client should call this method after the
	 * downlaod status returned by <code>getFileDownloadStatus</code> has
	 * value <code>SUCCEEDED</code> or <code>ERROR</code>.
	 * 
	 * If the client doesn't close the download handle, Software Repository will
	 * leak a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the download handle.
	 * 
	 * @param handle
	 *            download handle
	 */
	public void endFileDownload(DownloadHandle handle) {
		downloadStatuses.remove(handle);
	}

	/**
	 * Helper method for recursive directory erasing
	 * 
	 * @param dir
	 *            Directory to delete
	 * @return True iff successful
	 */
	public static boolean eraseDirectoryRecursively(File dir) {
		if (dir == null) {
			return false;
		}
		if (dir.isDirectory()) {
			String[] children = dir.list();
			for (int i = 0; i < children.length; i++) {
				boolean success = eraseDirectoryRecursively(new File(dir,
						children[i]));
				if (!success) {
					return false;
				}
			}
		}

		return dir.delete();
	}
			
	/**
	 * In the case, we were uploading a R package for registration and it was rejected,
	 * this method returns the list of error messages describing the reasons, why it was
	 * rejected.
	 * 
	 * @param handle	Upload handle of the R package transfer
	 * 
	 * @return	The String array with messages
	 */
	public String[] getRejectedUploadErrorMessages(UploadHandle handle) {
		UploadStatus uploadStatus = uploadStatuses.get(handle);
		if (uploadStatus == null) {
			throw new IllegalArgumentException("Invalid handle.");
		}
		if (uploadStatus != UploadStatus.REJECTED) {
			throw new IllegalStateException("R Package not rejected.");
		}
		return uploadErrorMessages.get(handle);
	}

}
