/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.UnknownHostException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import cz.cuni.mff.been.benchmarkmanager.EntityStatus;
import cz.cuni.mff.been.benchmarkmanager.ExperimentEntity;
import cz.cuni.mff.been.benchmarkmanager.gui.BenchmarkManagerGUIInterface;
import cz.cuni.mff.been.common.BeenUtils;
import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.Pair;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;
import cz.cuni.mff.been.resultsrepository.gui.Statistics;


/**
 * Simple class for performing of logs collection from Task Manager and storing it
 * in database
 * 
 * @author Jakub Lehotsky
 */
public final class ExperimentEntityManager {

	/** Unable to contact Benchmark manager to get deletion permit error */ 
	private static final String BM_CONTACT_DELETE_PERMIT_ERROR 
		= "Unable to contact benchmark manager to get experiment deletion permit.";

	private static final String COLUMNS_FILE = "columns";

	private static final Object ASCII_TAB = "\t";

	/** Single instance of load collector */
	private static ExperimentEntityManager experimentEntityManager = null;
	
	/** Databse that is used for results in Results Repository */
	private Database database;

	/** Metadata manager - loading and storing */
	private static MetadataManager metaManager;
	
	/** Internal statistics manager */
	private StatisticsManager statisticsManager;
	
	/** Internal graph manager */
	private GraphManager graphManager;

	/** Internal analysis entity manager */
	private LogCollector logCollector;
		
	/** Internal task list manager */
	private TaskListManager taskListManager;
	
	/** Internal roles manager */
	private RolesManager rolesManager;
	
	/** Internal scripts runner reference */
	private RScriptsRunner rScriptsRunner;

	/** Internal analysis entity manager */
	private AnalysisEntityManager analysisEntityManager;

	/**
	 *  Construction, notify load collector about Results Repository database when constructing
	 */
	private ExperimentEntityManager() {
		database = Database.getInstance();
	}
	
	/**
	 * Returns the single instance of load collector
	 * 
	 * @return	Reference to LoadCollector instance 
	 */
	public static ExperimentEntityManager getInstance() {
		if (experimentEntityManager == null) {
			return new ExperimentEntityManager();
		} else {
			return experimentEntityManager;
		}
	}	
	
	/**
	 * Use to initialize analysis entity manager 
	 * 
	 * @param database			Instance of Results Repository database
	 * @param metaManager		Instance of Results Repository metadata manager
	 * @param statisticsManager	Instance of Results Repository statistics manager
	 * @param graphManager		Instance of Results Repository graph manager
	 * @param logCollector		Instance of Results Repository log collector 
	 * @param taskListManager 	Instance of Results Repository task list manager
	 * @param rolesManager		Instance of Results Repository roles manager
	 * @param scriptsRunner 
	 * @param analysisEntityManager	Instance of Results Repository analysis entity manager

	 */
	public void initialize(Database database, MetadataManager metaManager, 
			StatisticsManager statisticsManager, GraphManager graphManager,
			LogCollector logCollector, TaskListManager taskListManager,
			RolesManager rolesManager,
			RScriptsRunner scriptsRunner, AnalysisEntityManager analysisEntityManager) {
		
		this.database = database;
		ExperimentEntityManager.metaManager = metaManager;
		this.statisticsManager = statisticsManager;
		this.graphManager = graphManager;
		this.logCollector = logCollector;
		this.taskListManager = taskListManager;
		this.rolesManager = rolesManager;
		this.rScriptsRunner = scriptsRunner;
		
		this.analysisEntityManager = analysisEntityManager;		
	}


        /**
         * Adds new experiment into results repository, so it can be filled with entities and results,
         * it must not exist before and containing analysis must be already created
         * 
         * @param description Description of analysis - all metadata for creation
         * 
         * @throws ResultsRepositoryException in case experiment already exists or some I/O error occurs
         */

	public void addExperiment(ExperimentEntity description)
	throws ResultsRepositoryException {

		AID analysisId = description.getAnalysis();
		analysisEntityManager.ensureValidAid(analysisId);
		createExperiment(analysisId, description.getId());

		MetadataRef analysisMetadata = new MetadataRef(new Metadata());
		MetadataRef experimentMetadata = new MetadataRef(new Metadata());
		String metaItem;
		Integer completeExperiments;
		Integer validExperiments;

		experimentMetadata.metadata.addItem("name", description.getName());
		experimentMetadata.metadata.addItem("comment", description
				.getDescription());
		experimentMetadata.metadata
				.addItem("created on", new Date().toString());
		experimentMetadata.metadata.addItem("plugin id", description
				.getPluginID().toString());
		experimentMetadata.metadata.addItem("plugin name", description
				.getPlugin());
		experimentMetadata.metadata.addItem("binaries", "0");
		experimentMetadata.metadata.addItem("complete binaries", "0");
		experimentMetadata.metadata.addItem("expected binaries", String
				.valueOf(description.getBinaryCount()));
		experimentMetadata.metadata.addItem("valid binaries", "0");
		experimentMetadata.metadata.addItem("expected runs per binary", String
				.valueOf(description.getRunCount()));
		experimentMetadata.metadata.addItem("samples per run count", String
				.valueOf(description.getSampleCount()));
		experimentMetadata.metadata.addItem("complete", "false");
		experimentMetadata.metadata.addItem("valid", "true");
		
		String contextName = "system";
		try {
		    BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();
		    if (bmInstance != null) {
		    	contextName = bmInstance.getAnalysisInterface()
		    		.getSubentityInterface(analysisId).getContext(description.getId());
		    } else {
		    	Logger.logError("Error connecting to Benchmark Manager!");
		    }
		} catch (RemoteException e) {
		    throw new ResultsRepositoryException(
			    "Remote exception when contacting Benchmark Manager.", e); 
		}
		
		experimentMetadata.metadata.addItem("context name", contextName);
		
		/*
		 * Store 1:1 role-host mapping information for experiment
		 */
		List<Pair<String,String>> mapping = description.getResultsMapping();
		RoleItem[] roleItems = new RoleItem[mapping.size()];
		for (int i = 0; i < roleItems.length; i++) {
			roleItems[i] = new RoleItem(i, 
					mapping.get(i).getFirst(), 
					mapping.get(i).getSecond());
		}
		rolesManager.makeExperimentTable(analysisId, description.getId(), roleItems);
		
		ensureSuccessStoreExperimentMetadata(experimentMetadata.metadata,
				analysisId, description.getId());
		
		/* Now update experiments count in analysis metadata */
		AnalysisEntityManager.ensureSuccessLoadAnalysisMetadata(analysisMetadata, analysisId);
		
		if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
				.getItemByKey("experiments"), "experiments",
				Database.getAnalysisReferenceString(analysisId))) {
			completeExperiments = new Integer(metaItem);
		} else {
			completeExperiments = new Integer(0);
		}
		
		if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
				.getItemByKey("valid experiments"), "valid experiments",
				Database.getAnalysisReferenceString(analysisId))) {
			validExperiments = new Integer(metaItem);
		} else {
			validExperiments = new Integer(0);
		}
		
		completeExperiments++;
		analysisMetadata.metadata.setItemByKey("experiments",
				completeExperiments.toString());
		
		validExperiments++;
		analysisMetadata.metadata.setItemByKey("valid experiments",
				completeExperiments.toString());
		
		if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(analysisMetadata.metadata,
				analysisId)) {
			return;
		}
			
		logCollector.registerContextForLogging(contextName);
		
		String[] taskNames = null; 
		
		try {
		    BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();

		    if (bmInstance == null) {
		    	Logger.logError("Error contacting Benchmark Manager.");
		    } else {
		    	taskNames = BeenUtils.extractTaskIDs(database.getBenchmarkManagerInstance().getAnalysisInterface()
		    		.getSubentityInterface(analysisId).getAllTasks(description.getId()));
		    
		    	if (taskNames == null) {
		    		Logger.logError("Task names list of experiment is null.");
		    		return;
		    	}
			        
		    	synchronized (LockManager.taskListLock) {
		    		
		    		taskListManager.loadExperimentTaskList(analysisId, description.getId());
		    		taskListManager.addNewExperimentTasks(taskNames);
		    		taskListManager.storeExperimentTaskList(analysisId, description.getId());
		    		
		    	}
		    }

		} catch (RemoteException e) {
			Logger.logError("Error contacting Benchmark Manager.");
		    e.printStackTrace();
		} catch (IllegalArgumentException e) {
			Logger.logError("Entities integrity error while creating experiment.");
			e.printStackTrace();
		}
		        
	
	}

        /**
         * Allows adding new custom experiment metadata. 
         * 
         * @param analysisId	Analysis Id
         * @param experimentId	Experiment Id
         * @param key			Key of metadata item
         * @param value			Value of metadata item
         * @throws ResultsRepositoryException	In case you attempt to overwrite existing metadata item
         */

	public void addExperimentMetadata(AID analysisId, EID experimentId,
		String key, String value) throws ResultsRepositoryException {
	
		ensureValidEid(analysisId, experimentId);
		MetadataRef experimentMetadata = new MetadataRef(new Metadata());
		ensureSuccessLoadExperimentMetadata(experimentMetadata, analysisId,
			experimentId);
	
		if (experimentMetadata.metadata.getItemByKey(key) != null) {
			throw new ResultsRepositoryException(
				"Trying to overwrite analysis metadata - "
						+ Database.getExperimentReferenceString(analysisId,
								experimentId) + ".");
		}
	
		experimentMetadata.metadata.addItem(key, value);
	
		ensureSuccessStoreExperimentMetadata(experimentMetadata.metadata,
				analysisId, experimentId);	
	}


	/* ************ Experiment ************** */

	/**
	 * Initializes new experiment (creates it in a database - for a given
	 * analysis)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID to be created
	 * @throws ResultsRepositoryException
	 */
	public void createExperiment(AID aid, EID eid)
			throws ResultsRepositoryException {

		if (isValidEid(aid, eid)) {
			throw new ResultsRepositoryException(new IllegalArgumentException(
					Database.getExperimentReferenceString(aid, eid) + " already exists."));
		}

		File experimentDirectory = new File(database.getDirectoryInDatabase(aid, eid));
		if (!experimentDirectory.mkdir()) {
			throw new ResultsRepositoryException(
					"Error creating experiment directory.");
		}
		
		File experimentScriptsDirectory = new File(database.getRScriptsDirectory(aid,
				eid));
		if (!experimentScriptsDirectory.mkdir()) {
			throw new ResultsRepositoryException(
					"Error creating experiment scritps directory.");

		}
		
		File experimentInvalidateScriptsDirectory 
			= new File(database.getInvalidateScriptsDirectory(aid, eid));
		if (!experimentInvalidateScriptsDirectory.mkdir()) {
			throw new ResultsRepositoryException(
					"Error creating experiment invalidate scripts directory.");
		}

	}
	
	/**
	 * Asks benchmark manager, whether experiment is finished
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * 
	 * @return	True if experiment is finished
	 */
	public boolean experimentFinished(AID aid, EID eid) {
		
		EntityStatus experimentStatus = null;
		
		try {
			
		    BenchmarkManagerGUIInterface bmInstance = database.getBenchmarkManagerInstance();
		    if (bmInstance != null) {
		    	experimentStatus = bmInstance.getAnalysisInterface()
		    		.getSubentityInterface(aid).getEntity(eid).getStatus();
		    	Logger.logDebug(Database.getExperimentReferenceString(aid, eid) + " status is " + experimentStatus);
		    } else {
		    	Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
		    	return false;
		    }
		    
		} catch (RemoteException e) {
			
		    Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
		    return false;
		    
		} catch (IllegalArgumentException e) {
			
			Logger.logInfo("Test \"experiment finished\": experiment no longer exists in Benchmark Manager.");
			return true;
		}
		
		if (experimentStatus == null) {
			
			Logger.logError(BM_CONTACT_DELETE_PERMIT_ERROR);
			return false;
			
		}
		
		return experimentStatus.equals(EntityStatus.FINISHED); 
	}

	
    /**
     * Returns the list of experiment in a given analysis stored in Results Repository
     * 
     * @param aid	Analysis Id 
     * 
     * @return The list
     */    
	@SuppressWarnings("unchecked")
	public List<Entity<EID>> getExperimentsInAnalysis(AID aid) {

		analysisEntityManager.ensureValidAid(aid);

		File[] directoryList;
		ArrayList<Entity<EID>> result = new ArrayList<Entity<EID>>();

		File path = new File(database.getDirectoryInDatabase(aid));
		if ((directoryList = path.listFiles()) == null) {
			throw new RuntimeException(new ResultsRepositoryException(
					"Error reading experiments directory ("
							+ Database.getAnalysisReferenceString(aid) + ")."));
		}

		MetadataRef metadataFile = new MetadataRef(new Metadata());

		for (int i = 0; i < directoryList.length; i++) {
			if (directoryList[i].isDirectory()) {

				EID eid;
				try {
					eid = new EID(new Long(directoryList[i].getName())
							.longValue());
				} catch (NumberFormatException e) {
					Logger.logTrace(
							"Encountered non-EID item in analysis directory: "
									+ directoryList[i].getName() + ".");
					continue;
				}

				ensureSuccessLoadExperimentMetadata(metadataFile, aid, eid);

				Statistics[] stat = statisticsManager.loadExperimentStatistics(
						aid, eid);

				/*Logger.logDebug(
						"Reading " + getExperimentReferenceString(aid, eid));*/

				String metaItem;
				boolean complete;
				
				if (Database.ensureMetadataNotNull(metaItem = metadataFile.metadata
						.getItemByKey("complete"), "complete",
						Database.getExperimentReferenceString(aid, eid))) {
					complete = new Boolean(metaItem);
				} else {
					/* invalid metadata shouldn't prevent deleting of entity */
					complete = true; 
				}
				
				result.add(new Entity<EID>(eid, metadataFile.metadata
						.getMetadata(), stat, 
						(rScriptsRunner.isDeleteExperimentAllowed(aid, eid) || 
								experimentFinished(aid, eid)) 
								&& (complete || experimentFinished(aid, eid))));
			}
		}

		Collections.sort((List<Entity<EID>>) result);

		return result;
	}



	/**
	 * Checks whether EID conforms to some existing experiment in results
	 * repository
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @return True iff analysis does exist in results repository
	 */
	public boolean isValidEid(AID aid, EID eid) {
		File experimentDir = new File(database.getDirectoryInDatabase(aid, eid));

		return experimentDir.exists() && experimentDir.isDirectory()
				&& experimentDir.canRead();
	}

        /**
         * Ensures that given EID belongs to a valid experiment in a given analysis in results repository,
         * otherwise throws an IllegalArgumentException
         * 
         * @param aid
	 *		Analysis ID
         * @param eid
         * 		Experiment Id          
         */
	public void ensureValidEid(AID aid, EID eid) {
		if (!isValidEid(aid, eid)) {
			throw new IllegalArgumentException(Database.getExperimentReferenceString(
					aid, eid) + " doesn't exist.");
		}
	}
	
        /**
         * Returns the flag, whether experiment is deltable. Experiment is deletable if it contains no
         * incomplete binaries and no script is running for any binary. Or it has "finished" 
         * state in Benchmark manager. 
         * 
         * Synchronization note: experiment may come non-deletable between this call and call to eraseExperiment()
         * 
         * @param aid	Analysis Id
         * @param eid	Experiment Id 
         * @return	True if deletable
         */	
	private boolean isDeletable(AID aid, EID eid) {

		MetadataRef metadataFile = new MetadataRef(new Metadata());

		ensureSuccessLoadExperimentMetadata(metadataFile, aid, eid);

		String metaItem;
		boolean complete;
		
		if (Database.ensureMetadataNotNull(metaItem = metadataFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getExperimentReferenceString(aid, eid))) {
			complete = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			complete = true; 
		}
		
		return complete;

	}


        /**
         * Returns the Entity&lt;EID&gt; object representing experiment for use in GUI
         * 
         * @param aid	Analysis Id
         * @param eid	Experiment Id 
         * 
         * @return	The experiment entity
         */
    
	public Entity<EID> getExperiment(AID aid, EID eid) {

		ensureValidEid(aid, eid);

		Entity<EID> result;
		MetadataRef metadataFile = new MetadataRef(new Metadata());

		ensureSuccessLoadExperimentMetadata(metadataFile, aid, eid);

		Statistics[] stat = statisticsManager.loadExperimentStatistics(aid, eid);

		String metaItem;
		boolean complete;
		
		if (Database.ensureMetadataNotNull(metaItem = metadataFile.metadata
				.getItemByKey("complete"), "complete",
				Database.getExperimentReferenceString(aid, eid))) {
			complete = new Boolean(metaItem);
		} else {
			/* invalid metadata shouldn't prevent deleting of entity */
			complete = true; 
		}
		
		result = new Entity<EID>(eid, metadataFile.metadata.getMetadata(), stat, 
				rScriptsRunner.isDeleteExperimentAllowed(aid, eid) && 
				(complete || experimentFinished(aid, eid)));

		return result;
	}

        /**
         * Tries to load experiment metadata and notify, whether it was successful, eventually
         * log error
         * 
         * @param experimentMetadata	Reference to metadata object, where they will be stored
         * @param aid		Analysis Id
         * @param eid		Experiment Id
         * 
         * @return	True if metadata was loaded successfuly
         */
	public static boolean ensureSuccessLoadExperimentMetadata(
			MetadataRef experimentMetadata, AID aid, EID eid) {
		try {
			experimentMetadata.metadata = metaManager.loadExperimentMetadata(
					aid, eid);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(
					Metadata.IO_ERROR_METADATA_READING_MESSAGE
							+ Database.getExperimentReferenceString(aid, eid));
			return false;
		}
		return true;
	}

        /**
         * Tries to store experiment metadata and notify, whether it was successful, eventually
         * log error
         * 
         * @param experimentMetadata	Reference to metadata object, where they will be stored
         * @param aid	Analysis Id
         * @param eid	Experiment Id
         * @return	True if metadata was loaded successfuly
         */
	public static boolean ensureSuccessStoreExperimentMetadata(
			Metadata experimentMetadata, AID aid, EID eid) {
		try {
			metaManager.storeExperimentMetadata(aid, eid, experimentMetadata);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(
					Metadata.IO_ERROR_METADATA_STORING_MESSAGE
							+ Database.getExperimentReferenceString(aid, eid));
			return false;
		}
		return true;
	}

        /**
         * Sets the comment metadata of an experiment
         * 
         * @param aid	Analysis Id
         * @param eid	Experiment Id 
         * @param comment	Comment to set
         */
	public void setExperimentComment(AID aid, EID eid, String comment) {

		ensureValidEid(aid, eid);
		metaManager.setExperimentMetadataItem(aid, eid, "comment", comment);
	}

	/**
	 * Deletes given experiment (results, logs, load info, metadata, additional
	 * info, etc.)
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * 
	 * @return True iff successful
	 * 
	 * @throws DeleteLockedException	If experiment is locked for deleting 
	 */
	public boolean eraseExperiment(AID aid, EID eid) 
		throws DeleteLockedException {

		ensureValidEid(aid, eid);
		
		synchronized (LockManager.eraseLock) {
		
			/* Note: for further improvement of synchronization, callbacks should be locked too 
			 * (because they handle metadata of existing entities), now it cannot happen. For 
			 * additional measurements, it must be introduced */
			
			if (!(rScriptsRunner.isDeleteExperimentAllowed(aid, eid) && 
					(isDeletable(aid, eid) || experimentFinished(aid, eid))
					)) {
				Logger.logWarning("Tried to delete locked " 
						+ Database.getExperimentReferenceString(aid, eid) + ".");
				throw new DeleteLockedException("Experiment is being processed, unable to delete.");
			}
	
			MetadataRef analysisMetadata = new MetadataRef(new Metadata());
			MetadataRef experimentMetadata = new MetadataRef(new Metadata());
			Integer experiments;
			Integer completeExperiments;
			Integer validExperiments;
			String metaItem;
			Boolean completeFlag;
			boolean valid;
	
			/* Update experiments count in analysis metadata */
			if (!(AnalysisEntityManager.ensureSuccessLoadAnalysisMetadata(analysisMetadata, aid))
					|| !(ensureSuccessLoadExperimentMetadata(experimentMetadata,
							aid, eid))) {
				return false;
			}
	
			if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
					.getItemByKey("complete experiments"), "complete experiments",
					Database.getAnalysisReferenceString(aid))) {
				completeExperiments = new Integer(metaItem);
			} else {
				completeExperiments = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
					.getItemByKey("valid experiments"), "valid experiments",
					Database.getAnalysisReferenceString(aid))) {
				validExperiments = new Integer(metaItem);
			} else {
				validExperiments = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = analysisMetadata.metadata
					.getItemByKey("experiments"), "experiments",
					Database.getAnalysisReferenceString(aid))) {
				experiments = new Integer(metaItem);
			} else {
				experiments = new Integer(0);
			}
	
			if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
					.getItemByKey("complete"), "complete",
					Database.getAnalysisReferenceString(aid))) {
				completeFlag = new Boolean(metaItem);
			} else {
				completeFlag = false;
			}
	
			if (Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata
					.getItemByKey("valid"), "valid", Database.getExperimentReferenceString(
					aid, eid))) {
				valid = metaItem.equals("true");
			} else {
				valid = false;
			}
	
			experiments--;
			analysisMetadata.metadata.setItemByKey("experiments", experiments
					.toString());
	
			if (completeFlag) {
				completeExperiments--;
				analysisMetadata.metadata.setItemByKey("complete experiments",
						completeExperiments.toString());
			}
	
			if (valid && experiments > 0) {
				validExperiments--;
				analysisMetadata.metadata.setItemByKey("valid experiments",
						validExperiments.toString());
	
				if (validExperiments == 0) {
					analysisMetadata.metadata.setItemByKey("valid", "false");
					analysisMetadata.metadata
							.setItemByKey("valid experiments", "0");
				}
			}
	
			if (experiments == 0) {
				analysisMetadata.metadata.setItemByKey("valid", "true");
			}
	
			if (!AnalysisEntityManager.ensureSuccessStoreAnalysisMetadata(analysisMetadata.metadata, aid)) {
				return false;
			}
	
			File file = new File(database.getDirectoryInDatabase(aid, eid));
			// Note: if this fails, metadata might quite well be inconsistent
			
			boolean result = Database.eraseDirectoryRecursively(file); 		 

			/* Now execute invalidation callbacks */
			try {
				
				File scriptsFile = new File(
		        		database.getInvalidateScriptsDirectory(aid) + File.separator +
		        		RManager.INVALIDATE_SCRIPT_FILENAME);
				
				if (scriptsFile.exists()) {
		        		
			        BufferedReader scriptsReader = new BufferedReader(new FileReader(scriptsFile));
			        		        
			        String script;
			        
			        while ((script = scriptsReader.readLine()) != null) {
			        	
			        	if (!script.trim().equals("")) {
			        		rScriptsRunner.enqueueScriptExecution(aid, eid, script);
			        	}
			        }
			        
			        scriptsReader.close();
				}
			        
		    } catch (IOException e) {
		    	Logger.logError("I/O error accessing invalidation script files.");
		    }
			
			return result;
		}
	}

	/**
	 * Uploads experiment file (through TCP, etc...)
	 * 
	 * @see cz.cuni.mff.been.resultsrepository.Database#uploadFile
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param name
	 *            Name of file to store
	 * @param host
	 *            Hostname to connect to for data retrieving
	 * @param port
	 *            Port used in connection with host
	 * @param blocking
	 *            Determines, whether to use blocking transfer
	 * @return Handle of network connection
	 * 
	 * @throws UnknownHostException
	 *             In the case host could not be found
	 * @throws IOException
	 *             In the case of some IO error
	 * @throws ResultsRepositoryBusyException
	 *             In the case of non-blocking upload when Results Repository is
	 *             busy with (down/)uploading files
	 */
	public UploadHandle uploadExperimentFile(AID aid, EID eid, String name,
			String host, int port, boolean blocking)
			throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		ensureValidEid(aid, eid);
		return database.uploadFile(database.getFileInDatabase(aid, eid, name), host, port,
				blocking, false, null, null);
	}

	/**
	 * Download experiment file data from database
	 * 
	 * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile
	 * 
	 * @param aid
	 *            Analysis ID
	 * @param eid
	 *            Experiment ID
	 * @param name
	 *            Name of file to download
	 * @param host
	 *            Hostname to connect to for data retrieving
	 * @param port
	 *            Port used in connection with host
	 * @param blocking
	 *            Determines, whether to use blocking transfer
	 * @return Handle of network connection
	 * 
	 * @throws UnknownHostException
	 *             In the case host could not be found
	 * @throws IOException
	 *             In the case of some IO error
	 * @throws ResultsRepositoryBusyException
	 *             In the case of non-blocking upload when Results Repository is
	 *             busy with (down/)uploading files
	 */
	public DownloadHandle downloadExperimentFile(AID aid, EID eid, String name,
			String host, int port, boolean blocking)
			throws UnknownHostException, IOException,
			ResultsRepositoryBusyException {

		ensureValidEid(aid, eid);

		if (database.isBusy() && !blocking) {
			throw new ResultsRepositoryBusyException(
					Database.TOO_MANY_CONNECTIONS_MESSAGE);
		}
		return database.downloadFile(database.getFileInDatabase(aid, eid, name), host, port,
				blocking);
	}
	
	/**
	 * Returns the context of a given experiment, in case it is already registered (unless an error
	 * occurs in BM, it should be registered upon creation)
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @return		Context Id
	 */
	public String getExperimentContext(AID aid, EID eid) {
		
		MetadataRef experimentMetadata = new MetadataRef(new Metadata());
		
		if (!ensureSuccessLoadExperimentMetadata(experimentMetadata, 
			    aid, eid)) {
			Logger.logError("Error reading metadata - \"context name \" not found for " +
					Database.getExperimentReferenceString(aid, eid));
		}
		    
		String metaItem;
		    
		Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata.getItemByKey(
				"context name"), "context name", Database.getExperimentReferenceString(aid, eid));
		
		return metaItem;
	}
	
	
	/**
	 * Returns the experiment graph count of a given graph type
	 * 
	 * @param graphType	Load or Results graph type
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id 
	 * @return	The count of graphs
	 */
	public int getExperimentGraphCount(GraphType graphType, AID aid, EID eid) {
		ensureValidEid(aid, eid);
		return graphManager.getExperimentGraphCount(graphType, aid, eid);
	}

	/**
	 * Returns the experiment graph info for a given graph. GraphInfo object returned contains 
	 * width, height and title of the graph
	 * 
	 * @param graphType	Load or Results graph type
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id 
	 * @param index	Index of graph - can be one of 0..getExperimentGraphCount()-1
	 * @return	The graph info
	 * 
	 * @throws IOException	In case of error of getting metadata
	 */
	public GraphInfo getExperimentGraphInfo(GraphType graphType, AID aid,
			EID eid, int index) throws IOException {

		ensureValidEid(aid, eid);
		if (!graphManager.validExperimentGraph(graphType, aid, eid, index)) {
			throw new IllegalArgumentException(GraphManager.GRAPH_DOESNT_EXIST_MESSAGE);
		}
		return graphManager.getExperimentGraphInfo(graphType, aid, eid, index);
	}

	/**
	 * Returns the complete path for a given experiment graph
	 * 
	 * @param graphType
	 * 		Load or Results graph type
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id 
	 * @param index
	 * 		Index of graph - can be one of 0..getExperimentGraphCount()-1
	 * @return
	 * 		The path to the file with graph image on disk
	 * @throws IOException
	 * 		if this info couldn't be retrieved because of I/O problems
	 * @throws IllegalArgumentException 
	 * 		if graph doesn't exist
	 */
	public String getExperimentGraphPath(GraphType graphType, AID aid, EID eid,
			int index) throws IOException {
		ensureValidEid(aid, eid);
		if (!graphManager.validExperimentGraph(graphType, aid, eid, index)) {
			throw new IllegalArgumentException(GraphManager.GRAPH_DOESNT_EXIST_MESSAGE);
		}
		return graphManager.getExperimentGraphPath(graphType, aid, eid, index);
	}

	/**
	 * Reports column names for experiment
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param columnNames	<code>ArrayList&lt;String&gt;</code> containing list of column names 
	 */
	public void reportExperimentColumnNames(AID aid, EID eid, ArrayList<String> columnNames) {
		
		ensureValidEid(aid, eid);
		
		String columnsFilename = database.getFileInDatabase(aid, eid, COLUMNS_FILE);
		
		if (new File(columnsFilename).exists()) {
			return;
		}
		
		int i;		
		PrintWriter outputWriter;
		
		try {
		
			outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(columnsFilename)));
		
			for (i = 0; i < columnNames.size(); i++) {
				
				if (i > 0) {
					outputWriter.print(ASCII_TAB);
				}
				
				outputWriter.print(columnNames.get(i));
				
			}
			outputWriter.println();
		
			outputWriter.close();
			
		} catch (IOException e) {
			Logger.logError("Error writing columns file.");
		}
		
	}
    	
}
