/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.rmi.RemoteException;
import java.util.List;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;
import cz.cuni.mff.been.resultsrepository.gui.Entity;
import cz.cuni.mff.been.resultsrepository.gui.EntityInterface;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;

public class ExperimentImplementation extends EntityImplementation<EID> 
	implements Serializable, EntityInterface<EID>  {
	private AID aid;

	/** Load graphs defined for runs only error message */
	private static final String LOAD_FOR_RUNS_ONLY 
		= "Load graphs are defined for runs only.";
	
	/** Raw data download for runs only error message */
	private static final String RAW_DATA_FOR_RUNS_ONLY 
		= "Raw data download is avaliable for runs only.";
	
	/** Results repository entity manager for analyses */
	private ExperimentEntityManager experimentEntityManager;

	/** Construction of basic Experiment implementation 
	 * @param aid	AnalysisID
	 * @throws RemoteException */
	public ExperimentImplementation(AID aid) throws RemoteException {
		
		super();
		
		AnalysisEntityManager.getInstance().ensureValidAid(aid);
		
		this.aid = aid;
		
		experimentEntityManager = database.getExperimentEntityManager();
	}	

	public Entity<EID> getEntity(EID id) throws RemoteException {
		return experimentEntityManager.getExperiment(aid, id); 
	}

	public void setEntityComment(EID id, String comment) throws RemoteException {
		experimentEntityManager.setExperimentComment(aid, id, comment);		
	}

	public List<Entity<EID>> getEntities() throws RemoteException {
		return experimentEntityManager.getExperimentsInAnalysis(aid);
	}

	public EntityInterface getSubentityInterface(EID id) throws RemoteException {
		return new BinaryImplementation(aid, id);
	}

	public DataFormat[] getExportFormats() throws RemoteException {
		return exporter.getExperimentExportFormats();
	}

	public DownloadHandle beginExport(EID[] ids, String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {
		return exporter.beginExperimentsExport(aid, ids, formatId, ip, port);
	}

	public DataFormat[] getImportFormats() throws RemoteException {
		return importer.getExperimentImportFormats();
	}

	public UploadHandle beginImport(String formatId, InetAddress ip, int port) 
		throws RemoteException, ResultsRepositoryBusyException {		
		return importer.beginExperimentsImport(formatId, ip, port);
	}

	public void deleteEntity(EID id) throws DeleteLockedException, RemoteException {	
		experimentEntityManager.eraseExperiment(aid, id);
	}
	
	public int graphCount(GraphType graphType, EID id) throws RemoteException {

  		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
		
  		return experimentEntityManager.getExperimentGraphCount(graphType, aid, id);
	}

	public GraphInfo getGraphInfo(GraphType graphType, EID id, int index) 
		throws RemoteException, ResultsRepositoryException {
		
  		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}

  		try {
			return experimentEntityManager.getExperimentGraphInfo(graphType, aid, id, index);
		} catch (IOException e) {
			Logger.logError("I/O error reading graph info for " 
					+ Database.getExperimentReferenceString(aid, id) + ".");
			return null;
		}
	}

	public DownloadHandle beginGraphDownload(GraphType graphType, EID id, InetAddress ip, int port,
			int index) throws RemoteException, ResultsRepositoryBusyException {

  		if (graphType == GraphType.LOAD) {
  			throw new UnsupportedOperationException(LOAD_FOR_RUNS_ONLY);
  		}
		
		if (graphCount(graphType, id) == 0) {
			throw new IllegalArgumentException("Graph doesn't exist.");
		} else {
			try {		
				return experimentEntityManager.downloadExperimentFile(aid, id,  
						experimentEntityManager.getExperimentGraphPath(graphType, aid, id, index),
							ip.getCanonicalHostName(), port, false);
			} catch (IOException e) {
				Logger.logError("I/O error reading graph for " 
						+ Database.getExperimentReferenceString(aid, id) + ".");
				return null;
			}
		}
	}

	public DownloadHandle beginRawDataDownload(EID id, InetAddress ip, int port) 
		throws ResultsRepositoryBusyException, RemoteException {
		throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
	}

	public void endRawDataDownload(DownloadHandle handle) throws RemoteException {
		throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
	}

	public DownloadStatus getRawDataDownloadStatus(DownloadHandle handle) throws RemoteException {
		throw new UnsupportedOperationException(RAW_DATA_FOR_RUNS_ONLY);
	}
	
	public String getContext(EID id) throws RemoteException {
		return experimentEntityManager.getExperimentContext(aid, id);
	}

	public String[] getTasks(EID id) throws RemoteException {		
		TaskListManager taskListManagerInstance = TaskListManager.getInstance();
		
		synchronized (LockManager.taskListLock) {
			taskListManagerInstance.loadExperimentTaskList(aid, id);
			return taskListManagerInstance.getExperimentTaskList();
		}
	}

	public LogRecord[] getLogsForTask(EID id, String taskID) throws RemoteException,
			LogStorageException, IllegalArgumentException, NullPointerException {
	    
	    String contextId = experimentEntityManager.getExperimentContext(aid, id);
	    
	    if (!database.getLogStorage().isTaskRegistered(contextId, taskID)) {
		return new LogRecord[] {};
	    }
	    
	    return database.getLogStorage().getLogsForTask(contextId, taskID);
	}
}
