/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;


import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.util.zip.ZipOutputStream;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID; 

/**
 * Temporary class for example purposes
 * 
 * @author Jakub Lehotsky
 *
 */
public class ExportBEEN implements ExportInterface {
	
	/** Database instance to use */
	private static Database database;
	
	/** Export metadata filename */
	private static final String EXPORT_METADATA_FILENAME = "export.meta";
	
	/** Temporary zipfile filename */
	private static final String TEMPORARY_ZIP_FILENAME = "temporary.zip";
	
	/**
	 * Method for setting up Results Repository database instance
	 * 
	 *  @param aDatabase Database to set up
	 */
	public void setDatabase(Database aDatabase) {
		database = aDatabase;
	}
		
	/**
	 * Check if file exists and delete it eventually
	 * 
	 * @param filename	Filename of file to delete
	 */
	void deleteIfExists(String filename) {
		File file = new File(filename);
		
		if (file.exists()) {
			file.delete();
		}
	}
	
	/*
	 * @see cz.cuni.mff.been.resultsrepository.ExportInterface#beginAnalysesExport
	 */
	public DownloadHandle beginAnalysesExport(AID[] aids, InetAddress ip, int port)
			throws ResultsRepositoryBusyException {

		for (int i = 0; i < aids.length; i++) {
			database.getAnalysisEntityManager().ensureValidAid(aids[i]);
		}
		
		String tempDir = database.getTempDirectory();
		
		
		deleteIfExists(tempDir + File.separator + TEMPORARY_ZIP_FILENAME);		
		deleteIfExists(tempDir + File.separator + EXPORT_METADATA_FILENAME);
		
		File tempZip = new File(tempDir + File.separator + TEMPORARY_ZIP_FILENAME);
				
		try {
			FileOutputStream f = new FileOutputStream(tempZip);
			ZipOutputStream zos = new ZipOutputStream(f);
			BufferedOutputStream out =
			   new BufferedOutputStream(zos);
			for (int i = 0; i < aids.length; i++) {
				Zipper.zipDirectoryRecursively(zos, out, 
						new File(database.getDirectoryInDatabase(aids[i])), aids[i].toString());				
			}
			
			Metadata metaFile = new Metadata();
			metaFile.addItem("objects", "analyses");
			metaFile.addItem("count", new Integer(aids.length).toString());
			metaFile.store(tempDir + File.separator + EXPORT_METADATA_FILENAME);
			Zipper.zipFile(zos, out, new File(tempDir 
					+ File.separator + EXPORT_METADATA_FILENAME), "");
			
			out.close();
			  
		} catch (IOException e) {
			Logger.logError("I/O error exporting analyses.");
			e.printStackTrace();
			return null;
		}
		
		try {
			return (database.downloadFile(
					tempZip.getAbsolutePath(), ip.getCanonicalHostName(), port, false));
		} catch (IOException e) {
			Logger.logError("I/O error exporting analyses.");
			e.printStackTrace();
			return null;
		}	
	}
	
	/*
	 * @see cz.cuni.mff.been.resultsrepository.ExportInterface#beginExperimentsExport
	 */
	public DownloadHandle beginExperimentsExport(AID aid, EID[] eids, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {

		for (int i = 0; i < eids.length; i++) {
			database.getExperimentEntityManager().ensureValidEid(aid, eids[i]);			
		}
		
		String tempDir = database.getTempDirectory();
		
		File tempZip = new File(tempDir + File.separator + TEMPORARY_ZIP_FILENAME);		
		if (tempZip.exists()) {
			tempZip.delete();
		}
		
		try {
			FileOutputStream f = new FileOutputStream(tempZip);
			ZipOutputStream zos = new ZipOutputStream(f);
			BufferedOutputStream out =
			   new BufferedOutputStream(zos);			
			for (int i = 0; i < eids.length; i++) {				
				Zipper.zipDirectoryRecursively(zos, out, 
						new File(database.getDirectoryInDatabase(aid, eids[i])), eids[i].toString());				
			}
			
			Metadata metaFile = new Metadata();
			metaFile.addItem("objects", "experiments");
			metaFile.addItem("count", new Integer(eids.length).toString());
			metaFile.store(tempDir + File.separator + EXPORT_METADATA_FILENAME);
			Zipper.zipFile(zos, out, new File(tempDir 
					+ File.separator + EXPORT_METADATA_FILENAME), "");
			/*Zipper.zipFile(zos, out, new File(database.getFileInDatabase(aid, 
					"analsysis-export.meta")),	"");*/
			
			out.close();
			  
		} catch (IOException e) {
			Logger.logError("I/O error exporting experiments.");
			e.printStackTrace();
			return null;
		}
		
		try {
			return (database.downloadFile(
					tempZip.getAbsolutePath(), ip.getCanonicalHostName(), port, false));
		} catch (IOException e) {
			Logger.logError("I/O error exporting analyses.");
			e.printStackTrace();
			return null;
		}
	}

	/*
	 * @see cz.cuni.mff.been.resultsrepository.ExportInterface#beginBinariesExport
	 */
	public DownloadHandle beginBinariesExport(AID aid, EID eid, BID[] bids, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {

		for (int i = 0; i < bids.length; i++) {
			database.getBinaryEntityManager().ensureValidBid(aid, eid, bids[i]);			
		}
		
		String tempDir = database.getTempDirectory();
		
		File tempZip = new File(tempDir + File.separator + TEMPORARY_ZIP_FILENAME);		
		if (tempZip.exists()) {
			tempZip.delete();
		}
		
		try {
			FileOutputStream f = new FileOutputStream(tempZip);
			ZipOutputStream zos = new ZipOutputStream(f);
			BufferedOutputStream out =
			   new BufferedOutputStream(zos);
			
			for (int i = 0; i < bids.length; i++) {
				Zipper.zipDirectoryRecursively(zos, out, 
						new File(database.getDirectoryInDatabase(aid, eid, bids[i])), 
								bids[i].toString());				
			}
			
			Metadata metaFile = new Metadata();
			metaFile.addItem("objects", "binaries");
			metaFile.addItem("count", new Integer(bids.length).toString());
			metaFile.store(tempDir + File.separator + EXPORT_METADATA_FILENAME);
			Zipper.zipFile(zos, out, new File(tempDir 
					+ File.separator + EXPORT_METADATA_FILENAME), "");
			/*Zipper.zipFile(zos, out, new File(database.getFileInDatabase(aid,
					"analysis-Database.ANALYSIS_METADATA_FILENAME)), "");			
			Zipper.zipFile(zos, out, new File(database.getFileInDatabase(aid, eid, 
					Database.EXPERIMENT_METADATA_FILENAME)), "");*/
			
			out.close();
			  
		} catch (IOException e) {
			
			Logger.logError("I/O error exporting binaries.");
			e.printStackTrace();
			return null;
		}
		
		try {
			return (database.downloadFile(
					tempZip.getAbsolutePath(), ip.getCanonicalHostName(), port, false));
		} catch (IOException e) {
			
			Logger.logError("I/O error exporting binaries.");
			e.printStackTrace();
			return null;
		}		
	}

	/*
	 * @see cz.cuni.mff.been.resultsrepository.ExportInterface#beginRunsExport
	 */
	public DownloadHandle beginRunsExport(AID aid, EID eid, BID bid, RID[] runIndexes, 
			InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {

		for (int i = 0; i < runIndexes.length; i++) {
			database.getRunEntityManager().ensureValidRid(aid, eid, bid, runIndexes[i]);
		}
		
		String tempDir = database.getTempDirectory();
		
		File tempZip = new File(tempDir + File.separator + TEMPORARY_ZIP_FILENAME);		
		if (tempZip.exists()) {
			tempZip.delete();
		}
		
		try {
			FileOutputStream f = new FileOutputStream(tempZip);
			ZipOutputStream zos = new ZipOutputStream(f);
			BufferedOutputStream out =
			   new BufferedOutputStream(zos);
			
			for (int i = 0; i < runIndexes.length; i++) {
			    String[] runPaths = database.getRunEntityManager().getRunPaths(aid, eid, bid, runIndexes[i]);
			    for (String runPath : runPaths) {
				File file = new File(database.getFileInDatabase(aid, eid, bid, runPath));
				Zipper.zipFile(zos, out, file, "");
			    }
					
			    File file = new File(database.getFileInDatabase(aid, eid, bid, runIndexes[i] + ".meta"));
			    if (file.exists()) {
				Zipper.zipFile(zos, out, file, "");
			    }
			}
			
			Metadata metaFile = new Metadata();
			metaFile.addItem("objects", "runs");
			metaFile.addItem("count", new Integer(runIndexes.length).toString());
			metaFile.store(tempDir + File.separator + EXPORT_METADATA_FILENAME);
			Zipper.zipFile(zos, out, 
					new File(tempDir + File.separator + EXPORT_METADATA_FILENAME), "");
			/*Zipper.zipFile(zos, out, new File(database.getFileInDatabase(aid, 
					Database.ANALYSIS_METADATA_FILENAME)), "");			
			Zipper.zipFile(zos, out, new File(database.getFileInDatabase(aid, eid,
					Database.EXPERIMENT_METADATA_FILENAME)), "");
			Zipper.zipFile(zos, out, new File(database.getFileInDatabase(aid, eid, bid, 					 
					Database.BINARY_METADATA_FILENAME)), "");*/
			
			out.close();
			  
		} catch (IOException e) {
			
			Logger.logError("I/O error exporting runs.");
			e.printStackTrace();
			return null;
		}
		
		try {
			return (database.downloadFile(
					tempZip.getAbsolutePath(), ip.getCanonicalHostName(), port, false));
		} catch (IOException e) {
			Logger.logError("I/O error exporting runs.");
			e.printStackTrace();
			return null;
		}
	}
}
