/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.net.InetAddress;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;

/**
 * Export interface. If you want to make new exporting format, you have to implement this interface 
 * and register in the Exporter class (for now, change the hardcoded table in Exporter.java)
 * 
 * NOTE: arguments (arrays with entities) are checked for existence and for duplicates, before 
 * passed to these methods 
 *
 * @author Jakub Lehotsky
 */
public interface ExportInterface {
	
	/**
	 * Notice given export implementation with Results Repository database, exporter calls this 
	 * prior to begin{Whatever}Export to notice about database instance to use.
	 * @param aDatabase	Database instance
	 */
	void setDatabase(Database aDatabase);

	/**
	 * Begins analyses export, you should implement the sending of single export package to 
	 * InetAddress ip and port. Otherwise this methods works like <code>downloadFile</code> from 
	 * Database.java, using <code>getAnalysesExportStatus</code> and 
	 * <code>endAnalysesExportStatus</code> accordingly.
	 * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile 
	 * 
	 * @param aids	Array of analysis IDs for export 
	 * @param ip	ip address to send exported package to
	 * @param port	Port to use for network communication
	 * @return		Download handle to use for checking status and finishing export
	 * @throws ResultsRepositoryBusyException
	 */
	DownloadHandle beginAnalysesExport(AID[] aids, InetAddress ip, int port)
			throws ResultsRepositoryBusyException;
	
	/**
	 * Begins experiments export, you should implement the sending of single export package to 
	 * InetAddress ip and port. Otherwise this methods works like <code>downloadFile</code> from 
	 * Database.java, using <code>getExperimentsExportStatus</code> and
	 * <code>endExperimentsExportStatus</code> accordingly.
	 * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile 
	 * 
	 * @param aid	Analysis ID (with experiments)
	 * @param eids	Array of experiment IDs for export 
	 * @param ip	ip address to send exported package to
	 * @param port	Port to use for network communication
	 * @return		Download handle to use for checking status and finishing export
	 * @throws ResultsRepositoryBusyException
	 */
	DownloadHandle beginExperimentsExport(AID aid, EID[] eids, InetAddress ip, int port)
			throws ResultsRepositoryBusyException; 
	
	/**
	 * Begins binaries export, you should implement the sending of single export package to 
	 * InetAddress ip and port. Otherwise this methods works like <code>downloadFile</code> from 
	 * Database.java, using <code>getBinariesExportStatus</code> and 
	 * <code>endBinariesExportStatus</code> accordingly.
	 * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile 
	 * 
	 * @param aid	Analysis ID (with binaries)
	 * @param eid	Experiment ID (with binaries)
	 * @param bids	Array of binary IDs for export 
	 * @param ip	ip address to send exported package to
	 * @param port	Port to use for network communication
	 * @return		Download handle to use for checking status and finishing export
	 * @throws ResultsRepositoryBusyException
	 */
	DownloadHandle beginBinariesExport(AID aid, EID eid, BID[] bids, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException;
	
	/**
	 * Begins runs export, you should implement the sending of single export package to 
	 * InetAddress ip and port. Otherwise this methods works like <code>downloadFile</code> from 
	 * Database.java, using <code>getRunsExportStatus</code> and <code>endRunsExportStatus</code> 
	 * accordingly.
	 * @see cz.cuni.mff.been.resultsrepository.Database#downloadFile 
	 * 
	 * @param aid	Analysis ID (with runs)
	 * @param eid	Experiment ID (with runs)
	 * @param bid	Binary ID (with runs)
	 * @param runIndexes	Array of run indexes for export 
	 * @param ip	ip address to send exported package to
	 * @param port	Port to use for network communication
	 * @return		Download handle to use for checking status and finishing export
	 * @throws ResultsRepositoryBusyException
	 */
	DownloadHandle beginRunsExport(AID aid, EID eid, BID bid, RID[] runIndexes, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException;

}
