/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.net.InetAddress;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.hostmanager.InvalidArgumentException;
import cz.cuni.mff.been.resultsrepository.gui.DataFormat;

/**
 * This class read the raw data (binary, filtered) from database, converts them to desired
 * format and send them out to the user
 * 
 * Exporter and Importer form the main part of user interface for working with data
 * 
 * @author Jakub Lehotsky
 *
 */
public final class Exporter {
	
	/** Single instance of exporter */
	private static Exporter exporterInstance = null;
		
	/** Instance of Results Repository database to use */
	private static Database database;
		
	/** Table of used data format for export - maybe will be loaded from file in future */
	private static DataFormat[] exportFormats = new DataFormat[] { 
		new DataFormat("application/been", "BEEN archive database format")};
		
	/**
	 * Constructor - initializes exporter and let it know about Results Repository databse
	 */
	private Exporter() {		
	}
	
	/**
	 * Returns the single instance of exporter
	 *  
	 * @return	Reference to Exporter instance 
	 */
	public static Exporter getInstance() {
		if (exporterInstance == null) {
			exporterInstance = new Exporter();
		}
		return exporterInstance;
	}
	
	/** 
	 * Intialization of exporter. Must be called prior to use
	 * 
	 * @param database	Instance of Results Repository database  
	 */
	public static void initialize(Database database) {
		Exporter.database = database;
	}
	
	// Temporary:
	private DataFormat[] getExportFormats() {			
		return exportFormats;
	}
	
	/**
	 * Returns the class with ExportInterface determined by formatId 
	 * @param formatId	String identification (mime type), see DataFormat class
	 * @return	class instance implementing ExportInterface
	 */
	private ExportInterface getExportInterface(String formatId) {
					
		if (formatId.compareTo(exportFormats[0].getId()) == 0) {
			return new ExportBEEN();
		}
		
		throw new InvalidArgumentException("Unknow export format: " + formatId + "."); 				
	}

	/* ********* ANALYSIS ********* */ 
	
	
	public DataFormat[] getAnalysisExportFormats() {
		// Return hardcoded list of available export formats
		return getExportFormats();
	}
	
	
	public DownloadHandle beginAnalysesExport(AID[] aids, String formatId, InetAddress ip, int port)
			throws ResultsRepositoryBusyException {
		
		ExportInterface exportInterface;
		
		for (int i = 0; i < aids.length; i++) {
			database.getAnalysisEntityManager().ensureValidAid(aids[i]);
			for (int j = i + 1; j < aids.length; j++) {
				if ((aids[i]).equals(aids[j])) {
					throw new IllegalArgumentException(Database.getAnalysisReferenceString(aids[i])
							+ " duplicated in analysis list.");
				}
			}
			
		}
		
		exportInterface = getExportInterface(formatId);		
		
		exportInterface.setDatabase(database);
		return exportInterface.beginAnalysesExport(aids, ip, port);

	}
	  	
	/* ********* EXPERIMENT ********* */ 
	
	public DataFormat[] getExperimentExportFormats() {		
		return getExportFormats();
	}
	
	
	public DownloadHandle beginExperimentsExport(AID aid, EID[] eids, String formatId, 
			InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {		
		
		ExportInterface exportInterface;
		
		for (int i = 0; i < eids.length; i++) {
			database.getExperimentEntityManager().ensureValidEid(aid, eids[i]);			
			for (int j = i + 1; j < eids.length; j++) {
				if ((eids[i]).equals(eids[j])) {
					throw new IllegalArgumentException(Database.getExperimentReferenceString(aid, 
							eids[i]) + " duplicated in experiment list.");
				}
			}
		}
		
		exportInterface = getExportInterface(formatId);
		
		exportInterface.setDatabase(database);
		return exportInterface.beginExperimentsExport(aid, eids, ip, port);
	}	
	
	/* ********* BINARY ********* */ 
	
	
	public DataFormat[] getBinaryExportFormats() {
		// Return hardcoded list
		return getExportFormats();
	}
	
	
	public DownloadHandle beginBinariesExport(AID aid, EID eid, BID[] bids, String formatId, 
			InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {
		
		ExportInterface exportInterface;
		
		for (int i = 0; i < bids.length; i++) {
			database.getBinaryEntityManager().ensureValidBid(aid, eid, bids[i]);
			for (int j = i + 1; j < bids.length; j++) {
				if ((bids[i]).equals(bids[j])) {
					throw new IllegalArgumentException(Database.getBinaryReferenceString(aid, eid, 
							bids[i]) + " duplicated in binary list.");
				}
			}
		}
		
		exportInterface = getExportInterface(formatId);				
		
		exportInterface.setDatabase(database);
		return exportInterface.beginBinariesExport(aid, eid, bids, ip, port);
	}
	  	
	
	/* ********* RUN ********* */ 
	
	
	public DataFormat[] getRunExportFormats() {
		// Return hardcoded list
		return getExportFormats();
	}
	
	
	public DownloadHandle beginRunsExport(AID aid, EID eid, BID bid, RID[] runIndexes, 
			String formatId, InetAddress ip, int port) 
			throws ResultsRepositoryBusyException {
		
		ExportInterface exportInterface;
		
		if (runIndexes == null) {
			throw new IllegalArgumentException("Runs argument inavlid - null.");
		}
		
		for (int i = 0; i < runIndexes.length; i++) {
			database.getRunEntityManager().ensureValidRid(aid, eid, bid, runIndexes[i]);			
			for (int j = i + 1; j < runIndexes.length; j++) {
				if ((runIndexes[i] == runIndexes[j])) {
					throw new IllegalArgumentException(Database.getRunReferenceString(aid, eid, bid,
							runIndexes[i]) + " duplicated in run list.");
				}
			}
		}

		exportInterface = getExportInterface(formatId);		
		
		exportInterface.setDatabase(database);
		return exportInterface.beginRunsExport(aid, eid, bid, runIndexes, ip, port);
	}
	
	/* ** common ** */	
	public DownloadStatus getExportStatus(DownloadHandle handle) {
		return database.getFileDownloadStatus(handle);
	}
	  	
	public void endExport(DownloadHandle handle) {
		database.endFileDownload(handle);
	}
}
