/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.resultsrepository.gui.GraphInfo;
import cz.cuni.mff.been.resultsrepository.gui.GraphType;

/**
 * Class implementing manipulation with graphs for entities. Implemented solely for caching
 * of reading metadata about garphs
 * 
 * NOTE: all supressed warning for unused variables are just pending TODOS (for synchronization
 * with RR-side GUI Graph caching 
 * 
 * @author Jakub Lehotsky
 */
public class GraphManager {
	
	/** Graph doesn't exist error message */
	public static final String GRAPH_DOESNT_EXIST_MESSAGE 
		= "Graph doesn't exist.";
	/** Load graph doesn't exist error message */
	public static final String LOAD_GRAPH_DOESNT_EXIST_MESSAGE 
		= "Load graph doesn't exist.";

	/** Results graphs table filename */
	public static final String RESULTS_GRAPH_TABLE = "results.graphs";
	/** Load graphs table filename */
	public static final String LOAD_GRAPH_TABLE = "load.graphs";
	
	
	/** Reference to results repository database */
	private static Database database = null;
	
	/** The single existing intance of RID Manager */	
	private static GraphManager gMInstance = null;

	/** AID of analysis graph table in memory */
	@SuppressWarnings("unused")
	private static AID analTableAID = null;
	
	/** Type of analysis graph table in memory */
	@SuppressWarnings("unused")
	private static GraphType analType = GraphType.RESULTS; // correctly - nondefined
		
	/** AID of experiment graph table in memory */
	@SuppressWarnings("unused")
	private static AID expTableAID = null;
	
	/** EID of experiment graph table in memory */
	@SuppressWarnings("unused")
	private static EID expTableEID = null;
	
	/** Type of analysis graph table in memory */
	@SuppressWarnings("unused")
	private static GraphType expType = GraphType.RESULTS; // correctly - nondefined
	
	/** AID of run load graph table in memory */
	@SuppressWarnings("unused")
	private static AID runTableAID = null;
	
	/** EID of run load graph table in memory */
	@SuppressWarnings("unused")
	private static EID runTableEID = null;
	
	/** BID of run load graph table in memory */
	@SuppressWarnings("unused")
	private static BID runTableBID = null;
	
	/** RID of run load graph table in memory */
	@SuppressWarnings("unused")
	private static RID runTableRID = null;	
	
	@SuppressWarnings("unused")
	private static GraphType runType = GraphType.LOAD; // correctly - nondefined
	
	/** Analysis graph table data */	
	private NumberedTable analGraphTable;
	
	/** Experiment graph table data */	
	private NumberedTable expGraphTable;
	
	/** Run load graph table data */	
	private NumberedTable runGraphTable;
	
	/** Ascii of tab character */
	protected static final String ASCII_TAB = "\t";
	
	
	/**
	 * Constructor - private, void for singleton use 
	 */
	private GraphManager() {
	}
	
	/**
	 * Return single instance of Graph manager 
	 * 
	 * @return	reference single running instance of Graph manager
	 */
	public static GraphManager getInstance() {
		if (gMInstance == null) {
			gMInstance = new GraphManager();
		}
		return gMInstance;
	}
	
	/**
	 * Initialize RID Manager before its first use 
	 * 
	 * @param database	Instance of database we use
	 */
	public static void initialize(Database database) {
		GraphManager.database = database;
	}
	
	/* *************************** ANALYSES ******************************** */

	/**
	 * Loads analysis graphs table of a given analysis into memory
	 * 
	 * @param graphType	
	 * 		Results or Load graph type
	 * @param aid
	 * 		Analysis Id
	 */
	private void loadAnalysisGraphTable(GraphType graphType, AID aid) {
		
		/* Test, if we haven't validity map in the memory */
		// note: uncomment this to allow caching, 
		// This caching must be very well synchronized with deleting and R recounting !!!
		// If not possible will be removed
		/*if ((analTableAID != null) && (analType != null) &&
		 *  (aid.equals(analTableAID)) && (graphType.equals(analType)))
			return;*/
		
		/* Load it */
		try {
			analGraphTable = new NumberedTable(database.getFileInDatabase(aid, 
					((graphType == GraphType.RESULTS) 
					? RESULTS_GRAPH_TABLE 
					: LOAD_GRAPH_TABLE)));
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
					+ Database.getAnalysisReferenceString(aid));
			
			analGraphTable = new NumberedTable();
			analTableAID = new AID(-1);
			return;
		}
		
		/* Remember actual graph table position, we are working with */
		analTableAID = aid;
		analType = graphType;
	
	}		
	
	/**
	 * Validates that given graph really exists
	 * 
	 * @param graphType
	 * 		Results or Load graph
	 * @param aid
	 * 		Analysis Id
	 * @param index
	 * 		Index of the graph
	 * 
	 * @return	True if graph is valid and file with its image exists
	 */
	public boolean validAnalysisGraph(GraphType graphType, AID aid, int index) {
		
		String line;
		loadAnalysisGraphTable(graphType, aid);
		line = analGraphTable.getItemByKey(index);
		if ((line == null) || (line.compareTo("") == 0)) {
				return false;
		}
		
		return true;
	}

	/**
	 * Returns analysis graph count
	 * 
	 * @param graphType
	 * 		Results of Load graph
	 * @param aid
	 * 		Analysis Id
	 * @return	The count of graphs of a given type for a given analysis
	 */
	public int getAnalysisGraphCount(GraphType graphType, AID aid) {
		
		loadAnalysisGraphTable(graphType, aid);
		//Logger.logDebug("Analysis graph count = " + analGraphTable.getItemCount());
		return analGraphTable.getItemCount();
	}

	/**
	 * Returns analysis graph info (height, width, title)
	 * 
	 * @param graphType
	 * 		Results or Load graph type
	 * @param aid
	 * 		Analysis Id
	 * @param index
	 * 		Index of graph
	 * @return	GraphInfo descrining graph
	 * @throws IOException	in case metadata couldn't be read
	 */
	public GraphInfo getAnalysisGraphInfo(GraphType graphType, AID aid, int index) 
		throws IOException {
		
		String[] lineTokens;
		String line;
		loadAnalysisGraphTable(graphType, aid);
		
		line = analGraphTable.getItemByKey(index);
		if (line == null) {
			throw new IllegalArgumentException("Invalid graph index " 
					+ Database.getAnalysisReferenceString(aid) + ", graph no." + index + ".");
		}
		lineTokens = line.split(ASCII_TAB);
		
		if (lineTokens.length != 4) {
			Logger.logError("Line in graph table corrupted! "
					+ Database.getAnalysisReferenceString(aid) + ", graph. no " + index + ".");
			throw new IOException("Invalid analysis graphs, file format, " 
					+ Database.getAnalysisReferenceString(aid));			
		}

		try {
			return new GraphInfo(Integer.valueOf(lineTokens[1]), Integer.valueOf(lineTokens[2]), 
					lineTokens[0]);
		} catch (NumberFormatException e) {
			throw new IOException("Invalid analysis graphs, file format, " 
					+ Database.getAnalysisReferenceString(aid) + ".");
		}
		
	}
	
	/**
	 * Returns full path to a given graph
	 * 
	 * @param graphType
	 * 		Results or Load graph
	 * @param aid
	 * 		Analysis Id
	 * @param index
	 * 		Index of graph
	 * @return	The path to a graph
	 * 
	 * @throws IOException in case of metadata reading I/O error 
	 */
	public String getAnalysisGraphPath(GraphType graphType, AID aid, int index) 
		throws IOException {
		
		String[] lineTokens;
		String line;
		
		Logger.logDebug("getAnalysisGraphPath called");
		
		loadAnalysisGraphTable(graphType, aid);
		
		line = analGraphTable.getItemByKey(index);
		if (line == null) {
			throw new IllegalArgumentException("Invalid graph index " 
					+ Database.getAnalysisReferenceString(aid) + ", graph no." + index + ".");
		}
		lineTokens = line.split(ASCII_TAB);
				
		if (lineTokens.length != 4) {
			Logger.logError("Line in graph table corrupted! "
					+ Database.getAnalysisReferenceString(aid) + ", graph. no " + index + ".");
			throw new IOException("Invalid analysis graphs, file format, " 
					+ Database.getAnalysisReferenceString(aid) + ".");
		}
		

		//Logger.logDebug("Graph filename for analysis is: " + lineTokens[3]);
		
		return lineTokens[3];
	}

	/* *************************** EXPERIMENTS ******************************** */

	/**
	 * Loads experiment graphs table of a given experiment into memory
	 * 
	 * @param graphType	
	 * 		Results or Load graph type
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 */
	private void loadExperimentGraphTable(GraphType graphType, AID aid, EID eid) {
		
		/* Test, if we haven't validity map in the memory */
		// note: uncomment to allow caching
		// This caching must be very well synchronized!!!
		// This caching must be very well synchronized with deleting and R recounting !!!
		// If not possible will be removed		
		/*if ((expTableAID != null) && (expTableEID != null) && (expType != null) &&
		 * (aid.equals(expTableAID)) && (eid.equals(expTableEID)) && (graphType.equals(expType)))
			return;*/
		
		/* Load it */
		try {
			expGraphTable = new NumberedTable(database.getFileInDatabase(aid, eid,
					((graphType == GraphType.RESULTS) 
					? RESULTS_GRAPH_TABLE 
					: LOAD_GRAPH_TABLE)));
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
					+ Database.getAnalysisReferenceString(aid));
			
			expGraphTable = new NumberedTable();
			expTableAID = new AID(-1);
			expTableEID = new EID(-1);
			return;
		}
		
		/* Remember actual graph table position, we are working with */
		expTableAID = aid;
		expTableEID = eid;
		expType = graphType;
	
	}

	/**
	 * Validates that given graph really exists
	 * 
	 * @param graphType
	 * 		Results or Load graph
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @param index
	 * 		Index of the graph
	 * 
	 * @return	True if graph is valid and file with its image exists
	 */	
	public boolean validExperimentGraph(GraphType graphType, AID aid, EID eid, int index) {

		String line;
		loadExperimentGraphTable(graphType, aid, eid);
		line = expGraphTable.getItemByKey(index);
		if ((line == null) || (line.compareTo("") == 0)) {
			return false;
		}
	
		return true;
	}


	/**
	 * Returns experiment graph count
	 * 
	 * @param graphType
	 * 		Results of Load graph
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @return	The count of graphs of a given type for a given experiment
	 */
	public int getExperimentGraphCount(GraphType graphType, AID aid, EID eid) {
		
		loadExperimentGraphTable(graphType, aid, eid);
		return expGraphTable.getItemCount();
	}

	/**
	 * Returns experiment graph info (height, width, title)
	 * 
	 * @param graphType
	 * 		Results or Load graph type
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @param index
	 * 		Index of graph
	 * @return	GraphInfo descrining graph
	 * @throws IOException	in case metadata couldn't be read
	 */	
	public GraphInfo getExperimentGraphInfo(GraphType graphType, AID aid, EID eid, int index) 
		throws IOException {
		
		String[] lineTokens;
		String line;
		loadExperimentGraphTable(graphType, aid, eid);
		
		line = expGraphTable.getItemByKey(index); // ???
		if (line == null) {
			throw new IllegalArgumentException("Invalid graph index " 
					+ Database.getExperimentReferenceString(aid, eid) + ", graph no." 
					+ index + ".");
		}
		lineTokens = line.split(ASCII_TAB);
		
		if (lineTokens.length != 4) {
			Logger.logError("Line in graph table corrupted! "
					+ Database.getExperimentReferenceString(aid, eid) + ", graph. no " 
					+ index + ".");
			throw new IOException("Invalid experiment graphs file format, " 
					+ Database.getExperimentReferenceString(aid, eid) 
					+ ".");			
		}

		try {
			return new GraphInfo(Integer.valueOf(lineTokens[1]), Integer.valueOf(lineTokens[2]), 
					lineTokens[0]);			
		} catch (NumberFormatException e) {
			throw new IOException("Invalid experiment graphs, file format, " 
					+ Database.getExperimentReferenceString(aid, eid) + ".");
		}
	}

	/**
	 * Returns full path to a given graph
	 * 
	 * @param graphType
	 * 		Results or Load graph
	 * @param aid
	 * 		Analysis Id
	 * @param eid 
	 * 		Experiment Id
	 * @param index
	 * 		Index of graph
	 * @return	The path to a graph
	 * 
	 * @throws IOException in case of metadata reading I/O error 
	 */
	public String getExperimentGraphPath(GraphType graphType, AID aid, EID eid, int index) 
		throws IOException {
		
		String[] lineTokens;
		String line;
		loadExperimentGraphTable(graphType, aid, eid);
		
		line = expGraphTable.getItemByKey(index);
		if (line == null) {
			throw new IllegalArgumentException("Invalid graph index " 
					+ Database.getExperimentReferenceString(aid, eid) + ", graph no." + index + ".");
		}
		lineTokens = line.split(ASCII_TAB);
		
		if (lineTokens.length != 4) {
			Logger.logError("Line in graph table corrupted! "
					+ Database.getExperimentReferenceString(aid, eid) + ", graph. no " + index + ".");
			throw new IOException("Invalid experiment graphs file format, " 
					+ Database.getExperimentReferenceString(aid, eid) + ".");			
		}
		
		return lineTokens[3];		
	}
	
	
	/* *************************** RUN ******************************** */

	/**
	 * Loads run load graphs table of a given run into memory
	 * 
	 * @param graphType	
	 * 		Results or Load graph type
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @param bid
	 * 		Binary Id
	 * @param rid
	 * 		Run Id
	 */
	private void loadRunLoadGraphTable(AID aid, EID eid, BID bid, RID rid) {
		
		/* Test, if we haven't validity map in the memory */
		// note: uncommend this to allow caching 
		// This caching must be very well synchronized with deleting and R recounting !!!
		// If not possible will be removed
		/*if ((analTableAID != null) && (analTableEID != null) && (analTableBID != null)
		 * && (aid == analTableAID) && (eid == analTableEID) && (aid == analTableBID) 
		 * &&  (rid == analTableRID) && (graphType == analType))
			return;*/
		
		/* Load it */
		try {
			String runLoadGraphPath = 
				RunValidityMapsManager.getInstance().getRunLoadGraphsPath(aid, eid, bid, rid) 
					+ File.separator + LOAD_GRAPH_TABLE;
			Logger.logDebug("Loading load graph table from : " + runLoadGraphPath);
			
			if (runLoadGraphPath == null) { 
				runGraphTable = null;
			} else {
				runGraphTable = new NumberedTable(runLoadGraphPath);
			}
			
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
					+ Database.getAnalysisReferenceString(aid));
			
			runGraphTable = new NumberedTable();
			runTableAID = new AID(-1);
			runTableEID = new EID(-1);
			runTableBID = new BID(-1);
			runTableRID = new RID(-1);
		}
				
		runTableAID = aid;
		runTableEID = eid;
		runTableBID = bid;
		runTableRID = rid;
		runType = GraphType.LOAD; // not quite clean code
	}

	/**
	 * Validates that given graph really exists
	 * 
	 * 		Results or Load graph
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @param bid
	 * 		Binary Id
	 * @param rid
	 * 		Run Id
	 * @param index
	 * 		Index of the graph
	 * 
	 * @return	True if graph is valid and file with its image exists
	 */	
	public boolean validRunLoadGraph(AID aid, EID eid, BID bid, RID rid, int index) {

		String line;
		loadRunLoadGraphTable(aid, eid, bid, rid);
		
		if (runGraphTable == null) { 
			return false;
		}
		
		line = runGraphTable.getItemByKey(index);
		if ((line == null) || (line.compareTo("") == 0)) {
			return false;
		}
	
		return true;
	}

	/**
	 * Returns run load graph count
	 * 
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @param bid
	 * 		Binary Id
	 * @param rid
	 * 		Run Id
	 * 
	 * @return	The count of graphs of a given type for a given run
	 */
	public int getRunLoadGraphCount(AID aid, EID eid, BID bid, RID rid) {
		
		loadRunLoadGraphTable(aid, eid, bid, rid);
		
		if (runGraphTable == null) {
			return 0;
		}
		
		return runGraphTable.getItemCount();
	}

	/**
	 * Returns run load graph info (height, width, title)
	 * 
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Experiment Id
	 * @param bid
	 * 		Binary Id
	 * @param rid
	 * 		Run Id
	 * @param index
	 * 		Index of graph
	 * 
	 * @return	GraphInfo descrining graph
	 * 
	 * @throws IOException	in case metadata couldn't be read
	 */	
	public GraphInfo getRunLoadGraphInfo(AID aid, EID eid, BID bid, RID rid, int index) 
		throws IOException {
		
		String[] lineTokens;
		String line;
		loadRunLoadGraphTable(aid, eid, bid, rid);
		
		if (runGraphTable == null) { 
			throw new IllegalArgumentException("Load graph #" + index + " doesn't exist for " 
					+ Database.getRunReferenceString(aid, eid, bid, rid) + ".");
		}
		
		line = runGraphTable.getItemByKey(index);
		if (line == null) {
			throw new IllegalArgumentException("Invalid graph index " 
					+ Database.getRunReferenceString(aid, eid, bid, rid) + ", graph no." + index + ".");
		}
		lineTokens = line.split(ASCII_TAB);
		
		if (lineTokens.length != 4) {
			Logger.logError("Line in graph table corrupted! "
					+ Database.getRunReferenceString(aid, eid, bid, rid) + ", graph. no " + index + ".");
			throw new IOException("Invalid runs load graph file format, " 
					+ Database.getRunReferenceString(aid, eid, bid, rid) + ".");
		}
		
		try {
			return new GraphInfo(Integer.valueOf(lineTokens[1]), Integer.valueOf(lineTokens[2]), 
					lineTokens[0]);
		} catch (NumberFormatException e) {
			throw new IOException("Invalid load runs graphs, file format, " 
					+ Database.getRunReferenceString(aid, eid, bid, rid));
		}
	}
	
	/**
	 * Returns full path to a given load graph
	 * 
	 * @param aid
	 * 		Analysis Id
	 * @param eid
	 * 		Analysis Id
	 * @param bid
	 * 		Analysis Id
	 * @param rid
	 * 		Analysis Id
	 * 
	 * @param index
	 * 		Index of graph
	 * @return	The path to a graph
	 * 
	 * @throws IOException in case of metadata reading I/O error 
	 */
	public String getRunLoadGraphPath(AID aid, EID eid, BID bid, RID rid, int index) 
		throws IOException {
		
		String[] lineTokens;
		String line;
		loadRunLoadGraphTable(aid, eid, bid, rid);
		
		line = runGraphTable.getItemByKey(index);
		if (line == null) {
			throw new IllegalArgumentException("Invalid graph index " 
					+ Database.getRunReferenceString(aid, eid, bid, rid) + ", graph no." + index + ".");
		}
		lineTokens = line.split(ASCII_TAB);
		
		if (lineTokens.length != 4) {
			Logger.logError("Line in graph table corrupted! "
					+ Database.getRunReferenceString(aid, eid, bid, rid) + ", graph. no " + index + ".");
			throw new IOException("Invalid run graphs file format, " 
					+ Database.getRunReferenceString(aid, eid, bid, rid));			
		}
		
		return lineTokens[3];
	}
}
