/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.File;
import java.io.IOException;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;


/**
 * Class that is managing arbitrary locking in all database on three entity levels - analyses,
 * experiments and binaries. For each lock, you just support a name. 
 
 * 
 * @author Jakub Lehotsky
 */
public final class LockManager {

    	/** Directory name for storing locks */ 
	private static final String LOCK_DIR = "locks";

	/** Single instance of lock manager */
	private static LockManager lockManagerInstance = null;
		
	/** The path to root of the database */
	private static String databasePath;

	/** Object to refer as erase lock */
	public static Object eraseLock;

	/** Object to refer as task list lock */
	public static Object taskListLock;

	/** Object to refer as validity maps lock */
	public static Object validityMapLock;

	/** Object to refer as table with column names lock */ 
	public static Object columnsLock;

	
	/**
	 *  Private singleton constructor
	 */
	private LockManager() {
	}
	
	/**
	 * Returns the single instance of lock manager
	 * 
	 * @return	Reference to LockManager instance 
	 */
	public static LockManager getInstance() {
		if (lockManagerInstance == null) {
			return new LockManager();
		} else {
			return lockManagerInstance;
		}
	}
	
	/**
	 * Intialize lock manager, must be called before first use
	 * @param database Database instance to work with
	 */
	public static void initialize(Database database) {
	    LockManager.databasePath = database.getDatabasePath();
	    eraseLock = new Object();
	    taskListLock = new Object();
	    validityMapLock = new Object();
	    columnsLock = new Object();
	}	

	/**
	 * If lock directory doesn't exist, it creates new one
	 * 
	 * @param lockDirPath	Lock directory
	 */
	private void ensureDirExists(String lockDirPath) {
		
		String dirPath = databasePath + File.separator + lockDirPath;
		File lockDir = new File(dirPath);
		
	    if (!lockDir.exists()) {
	    	lockDir.mkdir();
	    }
	    
	}
	
	/**
	 * Tries to acquire a lock determined be path (lock dir must exist),
	 * returns true if successful (lock is acquired), otherwise returns 
	 * false (already locked, IO error, etc.)
	 * 
	 * @param localPath	Path to lock
	 * @param lockName	The name of the lock
	 * 
	 * @return	True if lock acquired, false if not
	 */
	private boolean getLock(String localPath, String lockName) {

		Logger.logTrace("setting lock - " + localPath + "/" + lockName);
		
	    synchronized (this) {
	    	try {
	    		
	    		String lockPath = databasePath + File.separator + localPath;
	    		ensureDirExists(localPath);
	    		
	    		File lockFile = new File(lockPath + File.separator + lockName);
	    		
	    		return lockFile.createNewFile();
	    		
	    	} catch (IOException e) {
	    		
	    		Logger.logWarning("I/O error accessing lock: " 
	    				+ localPath + File.separator + lockName);
	    		
	    		return false;	        
	    	}
	    }
	}
		
	/**
	 * Clears the given lock. It is cleared in any case, owners of locks
	 * are not tracked!
	 * 
	 * @param localPath	Path to lock
	 * 
	 * @return	True if lock is cleared, false if it failed (IO error, etc.)
	 */
	private boolean clearLock(String localPath) {
		
		Logger.logTrace("clearing lock - " + localPath);
		
		String lockPath = databasePath + File.separator + localPath;		
	    File lockFile = new File(lockPath);
	    
	    synchronized (this) {
	    	return lockFile.delete();
	    }
	 	    
	}

	/**
	 * Tests whether some lock exists.
	 * 
	 * @param localPath	Path to lock
	 * 
	 * @return	True if lock exists, false if not
	 */
	private boolean isLocked(String localPath) {
		
		Logger.logTrace("checking lock - " + localPath);
		
	    String lockPath = databasePath + File.separator + localPath;
	    File lockFile = new File(lockPath);
	    
	    synchronized (this) {
		return lockFile.exists() && lockFile.isFile()
			&& lockFile.canRead();
	    }
	}

	/**
	 * Tries to acquire an analysis lock determined by the lock name.
	 * It returns true if successful (lock is acquired), otherwise returns 
	 * false (already locked, IO error, etc.)
	 * 
	 * @param aid	Analysis Id
	 * @param name	Name of the lock to ask for
	 * 
	 * @return	True if lock acquired, false if not
	 */
	public boolean getLock(AID aid, String name) {
	    return getLock(aid + File.separator + File.separator 
		    + LOCK_DIR + File.separator, name);
	}
	
	/**
	 * Tries to acquire an experiment lock determined by the lock name.
	 * It returns true if successful (lock is acquired), otherwise returns 
	 * false (already locked, IO error, etc.)
	 * 
	 * @param aid	Analysis Id
	 * @param eid 	Experiment Id
	 * @param name	Name of the lock to ask for
	 * 
	 * @return	True if lock acquired, false if not
	 */
	public boolean getLock(AID aid, EID eid, String name) {
	    return getLock(aid + File.separator + LOCK_DIR + File.separator, name);
	}

	/**
	 * Tries to acquire a binary lock determined by the lock name.
	 * It returns true if successful (lock is acquired), otherwise returns 
	 * false (already locked, IO error, etc.)
	 * 
	 * @param aid	Analysis Id
	 * @param eid 	Experiment Id
	 * @param bid 	Binary Id
	 * @param name	Name of the lock to ask for
	 * 
	 * @return	True if lock acquired, false if not
	 */
	public boolean getLock(AID aid, EID eid, BID bid, String name) {
	    return getLock(aid + File.separator + eid + File.separator + bid 
		    + File.separator + LOCK_DIR, name);
	}

	/**
	 * Tests whether some analysis lock exists.
	 * 
	 * @param aid 	Analysis ID
	 * @param name	The name of the lock
	 * 
	 * @return	True if lock exists, false if not
	 */
	public boolean isLocked(AID aid, String name) {
	    
	    String lockDirPath = aid + File.separator + LOCK_DIR;
	    ensureDirExists(lockDirPath);
	    
	    return isLocked(lockDirPath + File.separator + name);	    
	}

	/**
	 * Tests whether some experiment lock exists.
	 *
	 * @param aid 	Analysis ID
	 * @param eid 	Experiment ID 
	 * @param name	The name of the lock
	 * 
	 * @return	True if lock exists, false if not
	 */
	public boolean isLocked(AID aid, EID eid, String name) {
	    
	    String lockDirPath = aid + File.separator + eid + File.separator + LOCK_DIR;
	    ensureDirExists(lockDirPath);
	    
	    return isLocked(lockDirPath + File.separator + name);	    
	}

	/**
	 * Tests whether some binary lock exists.
	 *
	 * @param aid 	Analysis ID
	 * @param eid 	Experiment ID
	 * @param bid	Binary ID 
	 * @param name	The name of the lock
	 * 
	 * @return	True if lock exists, false if not
	 */
	public boolean isLocked(AID aid, EID eid, BID bid, String name) {
	    
	    String lockDirPath = aid + File.separator + eid 
	    	+ File.separator + bid + File.separator + LOCK_DIR;
	    
	    ensureDirExists(lockDirPath);
	    
	    return isLocked(lockDirPath + File.separator + name);	    
	}
	
	/**
	 * Tests whether there's any lock beginning with name
	 *
	 * @param aid 	Analysis ID
	 * @param eid 	Experiment ID
	 * @param bid	Binary ID 
	 * @param name	The name of the lock
	 * 
	 * @return	True if any lock exists, false if none
	 */
	public boolean isAnyLock(AID aid, EID eid, BID bid, String name) {
		
		Logger.logTrace("checking locks: " + name + "* in " 
				+ Database.getBinaryReferenceString(aid, eid, bid));
		
	    synchronized (this) {
	    	
	    	String lockDirPath = aid + File.separator + eid 
	    		+ File.separator + bid + File.separator + LOCK_DIR;
	    
	    	ensureDirExists(lockDirPath);
	    	
	    	File locksPathFile = new File(databasePath + File.separator + lockDirPath);
	                
	    	File[] lockList = locksPathFile.listFiles();
		    	
	    	for (File lockFile : lockList) {
	    		if (lockFile.getName().matches(name + ".*") && lockFile.exists() 
	    				&& lockFile.isFile() && lockFile.canRead()) {
	    			return true;
	    	    }
	    	}
	    	    
	    	return false;
	    }
	}

	/**
	 * Clears the given lock for analysis. It is cleared in any case, 
	 * owners of locks are not tracked.
	 * 
	 * @param aid	Analysis ID
	 * @param name	The name of the lock to clear
	 * 
	 * @return	True if lock is cleared, false if it failed (IO error, etc.)
	 */
	public boolean clearLock(AID aid, String name) {
	    
	    String lockDirPath = aid + File.separator + LOCK_DIR;
	    
	    return clearLock(lockDirPath + File.separator + name);	    
	}

	/**
	 * Clears the given lock for experiment. It is cleared in any case, 
	 * owners of locks are not tracked.
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param name	The name of the lock to clear
	 * 
	 * @return	True if lock is cleared, false if it failed (IO error, etc.)
	 */
	public boolean clearLock(AID aid, EID eid, String name) {
	
	    String lockDirPath = aid + File.separator + eid
	    	+ File.separator + LOCK_DIR;
	    
	    return clearLock(lockDirPath + File.separator + name);	    
	}

	/**
	 * Clears the given lock for binary. It is cleared in any case, 
	 * owners of locks are not tracked.
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 * @param name	The name of the lock to clear
	 * 
	 * @return	True if lock is cleared, false if it failed (IO error, etc.)
	 */
	public boolean clearLock(AID aid, EID eid, BID bid, String name) {
	    
	    String lockDirPath = aid + File.separator + eid
	    	+ File.separator + bid + File.separator + LOCK_DIR
	    	;
	    return clearLock(lockDirPath + File.separator + name);	    
	} 
		
}
