/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.RemoteException;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.hostruntime.TasksPortInterface;
import cz.cuni.mff.been.logging.FilesystemLogStorage;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.task.Task;

/**
 * Simple class for performing of logs collection from Task Manager and storing it
 * in database
 * 
 * @author Jakub Lehotsky
 */
public final class LogCollector {

	/** Single instance of load collector */
	private static LogCollector logCollectorInstance = null;
	
	/** Database that is used for results in Results Repository */
	private static Database database;
	
	/**
	 *  Construction, notify load collector about Results Repository database when constructing
	 */
	private LogCollector() {
	}
	
	/**
	 * Returns the single instance of load collector
	 * 
	 * @return	Reference to LoadCollector instance 
	 */
	public static LogCollector getInstance() {
		if (logCollectorInstance == null) {
			return new LogCollector();
		} else {
			return logCollectorInstance;
		}
	}
	
	/** 
	 * Must be called prior to use
	 * 	 
	 * @param database	An instance of Results Repository database 
	 * @param experimentEntityManager 	An instance of Results Repository experiment entity manager
	 */
	public static void initialize(Database database, ExperimentEntityManager experimentEntityManager) {
		LogCollector.database = database;
	}

	/**
	 * This method causes log upload to start. It is syncrhonnous and blocking to ensure that all
	 * logs are uploaded, before work continues on the other side (so that they are not cleaned up)
	 *  
	 * @param aid 			Analysis Id
	 * @param eid 			Experiment Id
	 * @param contextName	Context of task to upload logs from
	 * @param taskName		The name of the task to upload logs from
	 * 
	 * @throws ResultsRepositoryException
	 */
	public void performLogUpload(AID aid, EID eid, String contextName, String taskName)
		throws ResultsRepositoryException {

	    TaskListManager taskListManager;
	    taskListManager = TaskListManager.getInstance();
		
	    TasksPortInterface tasksPort = Task.getTaskHandle().getTasksPort();
		
		FilesystemLogStorage logStorage;
		
		logStorage = database.getLogStorage();
		try {
			if (!logStorage.isTaskRegistered(contextName, taskName)) {
				logStorage.addTask(contextName, taskName);			
			} else {
			    // Logs are already uploaded 
			    return;
			}
			logStorage.setTaskHostname(contextName, taskName, 
				InetAddress.getLocalHost().getCanonicalHostName());
		} catch (LogStorageException e) {
			throw new ResultsRepositoryException("Error getting hostname when trying to upload logs.", e);
		} catch (UnknownHostException e) {
			throw new ResultsRepositoryException("Error getting hostname when trying to upload logs.", e);
		}

		try {
			LogRecord[] logRecords = tasksPort.getLogsForTask(contextName, taskName);
			for (LogRecord logRecord : logRecords) {
				logStorage.log(logRecord.getContext(),
					logRecord.getTaskID(),
					logRecord.getTimestamp(),
					logRecord.getLevel(), 
					logRecord.getMessage());
			}
		} catch (LogStorageException e) {
			throw new ResultsRepositoryException("Error uploading logs.", e);
		} catch (RemoteException e) {
			throw new ResultsRepositoryException("RMI error contacting task manager when uploading logs.", e);
		}
		
		synchronized (LockManager.taskListLock) {
			
			taskListManager.loadExperimentTaskList(aid, eid);
			taskListManager.setExperimentTaskUploadedFlag(taskName);
			taskListManager.storeExperimentTaskList(aid, eid);
			
		}
		
	}
	
	/**
	 * Returns the context of a given experiment, in case it is already registered (unless an error
	 * occurs in BM, it should be registered upon creation)
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @return		Context Id
	 */
	public String getExperimentContext(AID aid, EID eid) {
		
		MetadataRef experimentMetadata = new MetadataRef(new Metadata());
		
		if (!ExperimentEntityManager.ensureSuccessLoadExperimentMetadata(experimentMetadata, 
			    aid, eid)) {
			Logger.logError("Error reading metadata - \"context name \" not found for " +
					Database.getExperimentReferenceString(aid, eid));
		}
		    
		String metaItem;
		    
		Database.ensureMetadataNotNull(metaItem = experimentMetadata.metadata.getItemByKey(
				"context name"), "context name", Database.getExperimentReferenceString(aid, eid));
		
		return metaItem;
	}


	/**
	 * Finishes logs upload. This method should be called on the end of the experiments, as it
	 * reads through the list of tasks and uploads logs of all tasks, which weren't uploaded yet.
	 * 
	 * @param analysisId	Analysis Id
	 * @param experimentId	Experiment Id
	 * @throws ResultsRepositoryException
	 */
	public void finishLogUpload(AID analysisId, EID experimentId) 
		throws ResultsRepositoryException {
		
		Logger.logInfo("Collecting logs at finish of experiment");
		String contextName = getExperimentContext(analysisId, experimentId);
	    
		TaskListManager taskListManager = TaskListManager.getInstance();
		
		String[] tasks; 
		
		synchronized (taskListManager) {
			tasks = taskListManager.getExperimentUnuploadedTaskList();
		}
	    
	    for (String taskId  : tasks) {
	    		    	
	    	performLogUpload(analysisId, experimentId, contextName, taskId);
	    	
	    }
	}

	/**
	 * Registers context for logging, must be done prior to start of any logging for a context into
	 * Results Repository
	 * 
	 * @param contextName		The name of the context to register
	 * @throws ResultsRepositoryException
	 */
	public void registerContextForLogging(String contextName) 
		throws ResultsRepositoryException {
		try {
			if (!Database.getInstance().getLogStorage().isContextRegistered(contextName)) {
				Database.getInstance().getLogStorage().addContext(contextName);
			}
		} catch (LogStorageException e) {
			throw new ResultsRepositoryException(e);
		}
		
	}
}
