/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import cz.cuni.mff.been.resultsrepository.gui.MetadataItem;


/**
 * Implements management and loading and storing of metadata to the file
 * 
 * @author Jakub Lehotsky
 */
public class Metadata {
		
	/** I/O error in metadata reading message */
	public static final String IO_ERROR_METADATA_READING_MESSAGE = "I/O error reading metadata of ";
	/** I/O error in metadata reading message */
	public static final String IO_ERROR_METADATA_STORING_MESSAGE = "I/O error storing metadata of ";
	
	/** ASCII of delim. character (TAB) */
	private static final String ASCII_TAB = "\t";	
	
	/** Actual metadata */
	private ArrayList<MetadataItem> metadata;
	
	/** Last item counter */
	private int lastItem;
	
	/**
	 * Constructor - creates metadata, void object 
	 * (meant for creating new metadata items, initializing them and storing into file)
	 * 
	 */
	public Metadata() {
		metadata = new ArrayList<MetadataItem>();
		this.lastItem = 0;
	}
	
	/**
	 * Returns metadata we store
	 * 
	 * @return the metadata
	 */
	
	public MetadataItem[] getMetadata() {
		return metadata.toArray(new MetadataItem[] {});
	}
	
	/** 
	 * Constructor - initializes metadata in memory and read their contents from the file.
	 * 
	 * @param filename	Name of file to read metadata from
	 * @throws IOException 
	 */
	public Metadata(String filename) throws IOException {
				
		String[] lineTokens;
		String line;
		BufferedReader inputReader;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {			
			metadata = new ArrayList<MetadataItem>();
			return;
		}
				
		metadata = new ArrayList<MetadataItem>();
			
		while ((line = inputReader.readLine()) != null) {								
			lineTokens = line.split(ASCII_TAB);
			try {
				if (lineTokens.length == 1) { 
					addItem(lineTokens[0], "");
				} else {
					String finalLine = "";
					for (int i = 1; i < lineTokens.length; i++) {
						finalLine += lineTokens[i];
						if (i < lineTokens.length - 1) {
							finalLine += ASCII_TAB;
						}
					}
					addItem(lineTokens[0], finalLine);
				}
				
			} catch (IndexOutOfBoundsException e) {
				throw new IOException("Invalid format in metadata file " + filename);
			}
		}
		this.uncodeComment();
				
		inputReader.close();
	}

	
	/**
	 * Stores metadata we have in our memory to the file.
	 * 
	 * @param filename	Where to store metadata.
	 * @throws IOException
	 */
	public void store(String filename) throws IOException {
		
		int i;		
		PrintWriter outputWriter;
		
		this.codeComment();
		
		outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
		
		for (i = 0; i < metadata.size(); i++) {
			outputWriter.println(getItemKey(i) + ASCII_TAB + getItemValue(i));		
		}
		
		outputWriter.close();
	}
	
	/**
	 * Returns the count of metadata items currently loaded in memory
	 * 
	 * @return	The count of items
	 */
	public int getItemCount() {
		return metadata.size();
	}
	
	/**
	 * Adding key-value pair to metadata - note: this can be used only with void metadata
	 * created by first constructor (Metadata(int count))
	 * 
	 * @param key	Key to added
	 * @param value	Associated value to add
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public void addItem(String key, String value) {		
		metadata.add(new MetadataItem(key, value));
		lastItem++;
	}
	
	/**
	 * Returns the key of the item with index.
	 * 
	 * @param index	Index of they key to give.
	 * @return	String with key
	 */
	public String getItemKey(int index) {
		return metadata.get(index).getKey();
	}
	
	/**
	 * Returns the value of the item with index.
	 * 
	 * @param index	Index of the value to give.
	 * @return	String with value.
	 */
	public String getItemValue(int index) {
		return metadata.get(index).getValue();
	}
	

	/**
	 * Returns the value of the item, with given key. 
	 * 
	 * @param key	Key to ask for.
	 * @return		String with associated value, or null if not found. 
	 */
	public String getItemByKey(String key) {
		
		int i = 0;
		
		try {		
			while (getItemKey(i).compareTo(key) != 0) {
				i++;
				if (i == lastItem) {
					return null;
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
		return getItemValue(i);		
	}
	
	/**
	 * Sets the value of the item, with the given key. 
	 * 
	 * @param key	Key
	 * @param value	New Value
	 * 
	 * @return True iff succesful (false if key doesn't exist)
	 * 
	 */
	public boolean setItemByKey(String key, String value) {
		int i = 0;
		
		while (getItemKey(i).compareTo(key) != 0) {
			i++;
			if (i == lastItem) {
				return false; 
			}
		}
		
		try {
			metadata.set(i, new MetadataItem(key, value));
		} catch (IndexOutOfBoundsException e) {
			return false;
		}
				
		return true; 		
	}
	
	/**
	 * Encodes the comment metadata value. This must be called prior
	 * to metadata storing. It allows storing comment with newlines in it.
	 */
	private void codeComment() {
		
		String comment = getItemByKey("comment");
		if (comment == null) {
			return;
		}
			
		comment = comment.replaceAll("\r", "");
		comment = comment.replaceAll("\n", "\t");
		
		setItemByKey("comment", comment);
		
	}
	
	/**
	 * Decodes the comment metadata value. This must be called after
	 * metadata loading. It allows storing comment with newlines in it.
	 */
	private void uncodeComment() {
		
		String comment = getItemByKey("comment");
		if (comment == null) {
			return;
		}
		
		comment = comment.replaceAll("\t", "\n");
		
		setItemByKey("comment", comment);
		
	}
	
}
