/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.EID;

/**
 * This class handles storage of all possible callbacks managers - for analyses and for experiments.
 * Its single and through static access, you can transparently get given callback manager for any 
 * experiment or analysis (it is automatically created, if it doesn't yet exist)  
 * 
 * @author Jakub Lehotsky
 */
public final class RCallbacksManagerFactory {

    /** Single instance of callbacks manager factory */
	private static RCallbacksManagerFactory rcmfInstance = null; 
	
	/**
	 * Inner class to be used as a key for hashmap allowing acces to callback managers.
	 * IDPair denotes analysis, if eid is null and aid shows to a given analysis.
	 * IDPair denotes experiment, if aid and eid point to given analysis/experiment combination
	 * 
	 * @author Jakub Lehotsky
	 */
	class IDPair {

	    /** Analysis Id */
	    private AID aid;
	    
	    /** Experiment Id */
	    private EID eid;
	    
	    /** @return Analysis Id */
	    public AID getAid() {
                return aid;
            }

	    /** @return Experiment Id */
	    public EID getEid() {
                return eid;
	    }
	    
	    /** 
	     * Constructs AID-EID pair
	     *  
	     * @param aid	Analysis Id 
	     * @param eid 	Experiment Id
	     */
	    public IDPair(AID aid, EID eid) {
	        super();
	        this.aid = aid;
	        this.eid = eid;
            }
	    
	    public boolean equals(Object o) {
	    	
	    	Class objectClass = o.getClass();
			Class thisClass = getClass();
			if (!objectClass.isInstance(this) || !thisClass.isInstance(o)) {
				return false;
			}
			if (!aid.equals(((IDPair) o).aid)) {
				return false;
			}
			if ((eid == null) && ((IDPair) o).eid == null) {
				return true;
			}
			if ((eid == null) || ((IDPair) o).eid == null) {
				return false;
			}
			return	eid.equals(((IDPair) o).eid);
	    }	    
	    
	    public int hashCode() {
	    	
	    	if (aid == null) {
		    	return 0;
			}
			if (eid == null) { 
				return aid.hashCode();
			} else {
				return eid.hashCode();
			}
	    }
	}

	
	/** Download statuses. */
	private static Map<IDPair, RCallbacksManager> callbacksManagers = Collections
			.synchronizedMap(new HashMap<IDPair, RCallbacksManager>());
	
	/**
	 * Constructor - private, void for singleton use 
	 */
	private RCallbacksManagerFactory() {		
	}
	
	/**
	 * Return single instance of R Callbacks Manager Factory 
	 * 
	 * @return	reference single running instance of R Callbacks Manager Factory
	 */
	public static RCallbacksManagerFactory getInstance() {
		if (rcmfInstance == null) {
			rcmfInstance = new RCallbacksManagerFactory();
		}
		return rcmfInstance;
	}
	
	/**
	 * Intializes R callbacks manager - call before first usage
	 */
	public void initialize() {
		// NOTE: now void - this method exist to follow standard architecture of Results Repository
	}
	
	/**
	 * Returns callbacks manager for a given analysis. If it doesn't exist, it is created 
	 * 
	 * @param aid	Analysis Id
	 * @return		RCallbacksManager instance tied with given analysis.
	 */
	public static RCallbacksManager getRCallbacksManager(AID aid) {
		RCallbacksManager rcManager = callbacksManagers.get(
			RCallbacksManagerFactory.getInstance().new IDPair(aid, null));
		if (rcManager == null) {
		    	Logger.logTrace("Creating new analysis callback manager for " 
		    		+ Database.getAnalysisReferenceString(aid) + ".");
			rcManager = new RCallbacksManager(aid, null, true);
			callbacksManagers.put(RCallbacksManagerFactory.getInstance().new IDPair(aid, null), 
				rcManager);
		}
		return rcManager;
	}

	/**
	 * Returns callbacks manager for a given experiment. If it doesn't exist, it is created 
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @return		RCallbacksManager instance tied with given experiment.
	 */
	public static RCallbacksManager getRCallbacksManager(AID aid, EID eid) {
		RCallbacksManager rcManager = callbacksManagers.get(
			RCallbacksManagerFactory.getInstance().new IDPair(aid, eid));
		if (rcManager == null) {
		    	Logger.logTrace("Creating new experiment callback manager for " 
		    		+ Database.getExperimentReferenceString(aid, eid) + ".");
			rcManager = new RCallbacksManager(aid, eid, false);
			callbacksManagers.put(RCallbacksManagerFactory.getInstance().new IDPair(aid, eid), 
				rcManager);
			
		}
		return rcManager;
	}	
}
