/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */

package cz.cuni.mff.been.resultsrepository;

import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.Semaphore;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;


/**
 * The class managing execution of R scripts. It sets up and tears down the background thread
 * and allows access to background queue of R scripts, which is used by background thread to
 * schedule and execute R scripts.
 * 
 * @author Jakub Lehotsky
 */

public class RScriptsRunner {

	/** The single existing intance of RID Manager */	
	private static RScriptsRunner rsRInstance = null;

	/** Queue of registered scripts to run */
	private Queue<RREvent> eventQueue;
	
	/** Actually running event */
	private static RREvent actualEvent;
	
	/** Background thread reference */
	private static RBackgroundThread rBackgroundThread;
	
	/** Sempahore for classical consument/producer synchronization of scripts execution */
	static volatile Semaphore scriptsSemaphore;

	/** The status of background thread */
	private volatile boolean runBackground;
	
	
	/**
	 * Constructor - private, void for singleton use 
	 */
	private RScriptsRunner() {
	}
	
	/**
	 * Return single instance of R Scripts help manager 
	 * 
	 * @return	reference single running instance of Graph manager
	 */
	public static RScriptsRunner getInstance() {
		if (rsRInstance == null) {
			rsRInstance = new RScriptsRunner();
		}
		return rsRInstance;
	}

	/**
	 * Initialization of scripts runner
	 */
	public void initialize() {
		eventQueue = new LinkedList<RREvent>();
	}
	
	/**
	 * Sets the state of background thread running
	 * 
	 * @param runBackground	the state
	 */
	private void setRunBackroundThread(boolean runBackground) {
	    this.runBackground = runBackground;
	}
	
	/**
	 * Returns the state of background thread runnign
	 * @return	The state
	 */
	public boolean getRunBackroundThread() {
	    return runBackground;
	}
	
	/**
	 * Start R scripts execution background thread
	 * 
	 * @return	True iff succesful
	 */
	public boolean startBackgroundThread() {
	    if (runBackground) {
	    	return false; // background thread already running
	    }
	    scriptsSemaphore = new Semaphore(0, true);
	    RBackgroundThread rBackgroundThread = new RBackgroundThread();
	    setRunBackroundThread(true);	    
	    rBackgroundThread.start();	
	    
	    return true;
	}
	
	/**
	 * Stops R scripts execution background thread and wait for thread to finish
	 *
	 */
	public void stopBackgroundThread() {
	    if (!runBackground) {
	    	Logger.logWarning("Tried to stop R background thread but thread not running.");
	    }
	  
	    Logger.logDebug("Background thread == null : " + (rBackgroundThread == null));
	    
	    setRunBackroundThread(false);	    
	}
	
	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid Experiment ID
	 * @param	functionToRun	the string containing function to be executed in R
	 */
	void enqueueScriptExecution(AID aid, EID eid, String functionToRun) {
		synchronized (eventQueue) {
			eventQueue.add(new RREvent(aid, eid, null, null, true, functionToRun, null, null));
		}
		Logger.logTrace("Calling semaphore release.");		
		scriptsSemaphore.release();
	}

	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid Experiment ID
	 * @param	bid	Binary ID
	 * @param	functionToRun	the string containing function to be executed in R
	 */
	void enqueueScriptExecution(AID aid, EID eid, BID bid, String functionToRun) {
		synchronized (eventQueue) {
			eventQueue.add(new RREvent(aid, eid, bid, null, true, functionToRun, null, null));
		}
		Logger.logTrace("Calling semaphore release.");		
		scriptsSemaphore.release();
	}

	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid Experiment ID
	 * @param	bid Binary ID
	 * @param	functionToRun	the string containing function to be executed in R
	 */
	void enqueueScriptExecution(AID aid, EID eid, BID bid, RID rid, String functionToRun) {
		synchronized (eventQueue) {
	    	eventQueue.add(new RREvent(aid, eid, bid, rid, true, functionToRun, null, null));
		}
		Logger.logTrace("Calling semaphore release.");		
		scriptsSemaphore.release();	
	}
	
	/**
	 * Adds new function execution into the background queue of R scripts execution
	 * 
	 * @param 	aid	Analysis ID
	 * @param	eid	Experiment ID
	 * @param	bid	Binary ID
	 * @param 	callback	upload callback object
	 * @param 	callbackData	it's data 
	 *  
	 */
	void enqueueCallbackExecution(AID aid, EID eid, BID bid, RID rid, 
			FileUploadCallbackInterface callback, Object callbackData) {
		synchronized (eventQueue) {
	    	eventQueue.add(new RREvent(aid, eid, bid, rid, false, null, callback, callbackData));
		}
		Logger.logTrace("Calling semaphore release.");		
		scriptsSemaphore.release();	
	}

	/**
	 * Returns the next function on the top of the queue
	 * 
	 * @return The script to be executed next
	 */
	public RREvent getNextEventFromQueue() {
		synchronized (eventQueue) {			
			actualEvent = eventQueue.poll();			
			return actualEvent;
		}
    }
	
	/**
	 * Clears the actual running event from memory
	 */
	public void clearActualEventRecord() {
		synchronized (eventQueue) {
			actualEvent = null;
		}
	}

	/**
	 * Returns events count
	 * 
	 * @return	The count
	 */
	public int getEventCount() {
		synchronized (eventQueue) {
			return eventQueue.size();
		}
	}

	/**
	 * Test whether run event is in queue
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @param rrEvent	Event
	 *
	 * @return	True, if event is in queue
	 */
	private boolean testRunInQueue(AID aid, EID eid, BID bid, RID rid, RREvent rrEvent) {
		
		boolean result = true;
		
		/* Test for script execution events */
		if (rrEvent.getIsScript()) {
			
			if (rrEvent.getRid() == null && rrEvent.getBid() == null 
					&& rrEvent.getEid() != null && rrEvent.getEid().equals(eid)
					&& rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
				
				/* There is script registered for containing experiment */
				result = false;
			}
			
			if (rrEvent.getRid() == null 
					&& rrEvent.getBid() != null && rrEvent.getBid().equals(bid)
					&& rrEvent.getEid() != null && rrEvent.getEid().equals(eid)
					&& rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
				
				/* There is script registered for containing binary */
				result = false;
				
			}
			
			if (rrEvent.getRid() != null && rrEvent.getRid().equals(rid)
					&& rrEvent.getBid() != null && rrEvent.getBid().equals(bid)
					&& rrEvent.getEid() != null && rrEvent.getEid().equals(eid)
					&& rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
				
				/* There is script registered for a given run */
				result = false;
				
			}	
			
		} else {
			
			/* Test for callback execution events */
			if (aid.equals(rrEvent.getAid()) ||
					eid.equals(rrEvent.getEid()) ||
					bid.equals(rrEvent.getBid()) ||
					rid.equals(rrEvent.getRid())) {
				
				/* There is callback registered for a given run */
				result = false;
			}
		}
		
		return result;
	}
	
	/**
	 * Checks event queue, whether it contains registered event that would block erasing
	 * of a run
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * 
	 * @return	True, if any entity is in queue
	 */
	public boolean isDeleteRunAllowed(AID aid, EID eid, BID bid, RID rid) {
		synchronized (eventQueue) {
			
			boolean result = true;
			
			/* Test each event in the current queue */
			for (RREvent rrEvent : eventQueue) {
				result = result && testRunInQueue(aid, eid, bid, rid, rrEvent);
			}
			
			if (actualEvent != null) {
				result = result && testRunInQueue(aid, eid, bid, rid, actualEvent);
			}
			
			return result;
		}
	}
	
	/**
	 * Test whether binary event is in queue
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rrEvent	Event
	 *
	 * @return	True, if event is in queue
	 */
	private boolean testBinaryInQueue(AID aid, EID eid, BID bid, RREvent rrEvent) {
		
		boolean result = true;
		
		/* Test for script execution events */
		if (rrEvent.getIsScript()) {
			
			if (rrEvent.getRid() == null && rrEvent.getBid() == null 
					&& rrEvent.getEid() != null && rrEvent.getEid().equals(eid)
					&& rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
				
				/* There is script registered for containing experiment */
				result = false;
			}
			
			if (rrEvent.getBid() != null && rrEvent.getBid().equals(bid)
					&& rrEvent.getEid() != null && rrEvent.getEid().equals(eid)
					&& rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
				
				/* There is script registered for a given binary or child run*/
				result = false;
			}
			
		} else {
			
			/* Test for callback execution events */
			if (aid.equals(rrEvent.getAid()) ||
					eid.equals(rrEvent.getEid()) ||
					bid.equals(rrEvent.getBid())) {
				
				/* There is callback registered for any run from this binary */
				result = false;
			}
		}
		
		return result;
	}
	
	/**
	 * Checks event queue, whether it contains registered event that would block erasing
	 * of a binary
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * 
	 * @return	True, if any entity is in queue
	 */
	public boolean isDeleteBinaryAllowed(AID aid, EID eid, BID bid) {
		synchronized (eventQueue) {
			
			boolean result = true;
						
			/* Test each event in the current queue */
			for (RREvent rrEvent : eventQueue) {
				result = result && testBinaryInQueue(aid, eid, bid, rrEvent);				
			}
			
			if (actualEvent != null) {
				result = result && testBinaryInQueue(aid, eid, bid, actualEvent);
			}
			
			return result;
		}
	}

	/**
	 * Test whether experiment event is in queue
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param rrEvent	Event
	 *
	 * @return	True, if event is in queue
	 */
	private boolean testExperimentInQueue(AID aid, EID eid, RREvent rrEvent) {
		
		boolean result = true;
		
		/* Test for script execution events */
		if (rrEvent.getIsScript()) {
			
			if (rrEvent.getEid() != null && rrEvent.getEid().equals(eid)
					&& rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
				
				/* There is script registered for a given experiemnt, 
				 * child binary or child run*/
				result = false;
			}
			
		} else {
			
			/* Test for callback execution events */
			if (aid.equals(rrEvent.getAid()) ||
					eid.equals(rrEvent.getEid()))  {
				
				/* There is callback registered for any run from this experiment */
				result = false;
			}
		}
		
		return result;
	}

	
	/**
	 * Checks event queue, whether it contains registered event that would block erasing
	 * of a experiment
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * 
	 * @return	True, if any entity is in queue
	 */
	public boolean isDeleteExperimentAllowed(AID aid, EID eid) {
		synchronized (eventQueue) {
			
			boolean result = true;
			
			/* Test each event in the current queue */
			for (RREvent rrEvent : eventQueue) {
				result = result && testExperimentInQueue(aid, eid, rrEvent);
			}
			
			if (actualEvent != null)
				result = result && testExperimentInQueue(aid, eid, actualEvent);
			
			return result;
		}
	}

	/**
	 * Test whether analysis event is in queue
	 * 
	 * @param aid	Analysis Id
	 * @param rrEvent	Event
	 *
	 * @return	True, if event is in queue
	 */
	private boolean testAnalysisInQueue(AID aid, RREvent rrEvent) {
	
		if (rrEvent.getAid() != null && rrEvent.getAid().equals(aid)) {
			
			/* There is a script or callback registered for entity 
			 * in a given analysis */
			return false;
		} else {
			return true;
		}
	}
	
	/**
	 * Checks event queue, whether it contains registered event that would block erasing
	 * of an analysis
	 * 
	 * @param aid	Analysis Id
	 * 
	 * @return	True, if any entity is in queue
	 */
	public boolean isDeleteAnalysisAllowed(AID aid) {
		synchronized (eventQueue) {
			
			boolean result = true;
			
			/* Test each event in the current queue */
			for (RREvent rrEvent : eventQueue) {
				result = result && testAnalysisInQueue(aid, rrEvent);												
			}
			
			if (actualEvent != null) {
				result = result && testAnalysisInQueue(aid, actualEvent);
			}
			
			return result;
		}
	}
	
}
