/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.IOException;
import java.net.UnknownHostException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.ArrayList;

import cz.cuni.mff.been.benchmarkmanager.AnalysisEntity;
import cz.cuni.mff.been.benchmarkmanager.BinaryEntity;
import cz.cuni.mff.been.benchmarkmanager.ExperimentEntity;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;

/**
 * Main interface for communication with other parts of BEEN (RMI interface)
 * (see also <code>ResultsRepositoryUserInterface</code> and <code>DatabaseInterface</code>)
 * 
 * @author Jakub Lehotsky
 */

public interface ResultsRepositoryInterface extends Remote {
		
	/**
	 * RMI name
	 */
	String RMI_NAME = "main";
	
	/**
	 * Notice Results Repository about new analysis being created in Benchmark Manager - 
	 * initializes all metadata, etc. This call creates analysis in Results Repository database.
	 * 
	 * @param description	All info about analysis being created
	 * @throws ResultsRepositoryException	In case something went wrong (analysis already exists,
	 * 										 i/o error)
	 * @throws RemoteException				In case of network connection malfunction
	 */
	void addAnalysis(AnalysisEntity description) throws 
			ResultsRepositoryException, RemoteException;

	/**
	 * Notice Results Repository about new experiment being created in Benchmark Manager - 
	 * initializes all metadata, etc. This call creates experiment in Results Repository database.
	 * 
	 * @param description	All info about experiment being created
	 * @throws ResultsRepositoryException	In case something went wrong (experiment already exists, 
	 * 										or analysis doesn't exist, io error)
	 * @throws RemoteException				In case of network connection malfunction
	 */
	void addExperiment(ExperimentEntity description) throws 
			ResultsRepositoryException, RemoteException;

	/**
	 * Notice Results Repository about new binary being created in Benchmark Manager - initializes
	 * all metadata, etc. This call creates binary in Results Repository database.
	 * 
	 * @param description	All info about experiment being created
	 * @throws ResultsRepositoryException	In case something went wrong (binary already exists, or
	 * 										analysis/experiment doesn't exist, io error)
	 * @throws RemoteException				In case of network connection malfunction
	 */	
	void addBinary(BinaryEntity description) 
		throws ResultsRepositoryException, RemoteException;


	/**
	 * Used to start upload of binary file - for specific run results upload 
	 * The client must open the socket on some port before calling this function. 
	 * Then it calls the <code>uploadFile</code> method.
	 * 
	 * This method work asynchronously and returns immediately.
	 * Before return it starts a new thread that will receive the file contents
	 * from the socket open on given host and port.
	 * 
	 * If there is any error when sending the file contents, the socket
	 * is closed. 
	 * 
	 * Client has an opportunity to check (via the <code>getFileUploadStatus</code> 
	 * method to check the current status of upload (eventually can be error)
	 *  
	 * After the upload finishes, client should close the returned handle
	 * via the <code>endFileUpload</code> method.  
	 *
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 * @param rid	Run ID for upload
	 * @param roleName The name of the role
	 * @param host	Hostname, from which file is uploaded
	 * @param port	Open port for network transfer
	 * @param runValid	Determines, whether run is valid or crashed 
	 * @param blocking 	Determines, whether transfer will be blockin or non-blocking
	 * @param tableIndex	Index of the currently uploaded table
	 * @param totalTables	Total number of tables to be uploaded for a given run
	 * 
	 * @return 	UploadHandle, which you can use later to refer to transfer
	 * 
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws RemoteException 
	 * @throws ResultsRepositoryBusyException 
	 */
	UploadHandle uploadRunResults(AID aid, EID eid, BID bid, RID rid, String roleName, 
			String host, int port, boolean runValid, boolean blocking, int tableIndex, 
			int totalTables) 	
			throws UnknownHostException, IOException, RemoteException, ResultsRepositoryBusyException;
	
	/**
	 * Initializes results repository load upload for a load of a given run. It must be called
	 * prior to any load information upload  
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * 
	 * @throws RemoteException
	 */
	void initLoadUpload(AID aid, EID eid, BID bid, RID rid) throws RemoteException;
		
	/**
	 * Notify results repository, that load upload is done. If valid parameter is
	 * set to false, it cleans stored load information if there are any partial uploads, etc.
	 * This must be called at finish in order for Results Repository to mark load data as valid
	 * and use them for load graphs drawing.
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id
	 * @param valid Validity of uploaded load information
	 * 
	 * @throws RemoteException
	 */
	void closeLoadUpload(AID aid, EID eid, BID bid, RID rid, boolean valid) throws RemoteException;
	
	/**
	 * Used to start upload of load information file 
	 * The client must open the socket on some port before calling this function. 
	 * Then it calls the <code>uploadFile</code> method.
	 * 
	 * This method work asynchronously and returns immediately.
	 * Before return it starts a new thread that will receive the file contents
	 * from the socket open on given host and port.
	 * 
	 * If there is any error when sending the file contents, the socket
	 * is closed. 
	 * 
	 * Client has an opportunity to check (via the <code>getFileUploadStatus</code> 
	 * method to check the current status of upload (eventually can be error)
	 *  
	 * After the upload finishes, client should close the returned handle
	 * via the <code>endFileUpload</code> method.  
	 *
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 * @param rid	Run ID for upload
	 * @param filename	Filename
	 * @param host	Hostname, from which file is uploaded
	 * @param port	Open port for network transfer
	 * @param blocking 	Determines, whether transfer will be blockin or non-blocking
	 * 
	 * @return 	UploadHandle, which you can use later to refer to transfer
	 * 
	 * @throws UnknownHostException
	 * @throws IOException
	 * @throws RemoteException 
	 * @throws ResultsRepositoryBusyException 
	 */
	UploadHandle uploadLoadInfo(AID aid, EID eid, BID bid, RID rid,  
			String filename, String host, int port, boolean blocking) 	
			throws UnknownHostException, IOException, RemoteException, ResultsRepositoryBusyException;	

	
	//	 Note: following methods has meaning only for non-blocking upload (default mode)
	
	/**
	 * Returns the status of the file upload. Result is one
	 * of the values listed in the <code>FileTransferStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>UPLOADING</code> mean that
	 * Results Repository still works on the file upload. Values
	 * <code>ACCEPTED</code> and <code>ERROR</code> mean that Results Repository
	 * has finished its work (either successfully or unsuccessfully) and the 
	 * download handle is ready to be closed by the <code>endFileUpload</code> method.   
	 * 
	 * @param handle file upload handle
	 * @return upload status
	 * @throws RemoteException when something in RMI goes bad
	 */
	UploadStatus getFileUploadStatus(UploadHandle handle) throws RemoteException;
	
	/**
	 * Closes the upload handle. Client should call this method after the upload
	 * status returned by <code>getFileUploadStatus</code> has value
	 * <code>ACCEPTED</code>, or <code>ERROR</code>.
	 * 
	 * This method is crucial, so that synchronization and upload callbacks work correctly
	 * 
	 * If the client doesn't close the upload handle, Results Repository will leak
	 * a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle. 
	 * @param aid Analysis Id
	 * @param eid Experimet Id
	 * @param bid Binary Id
	 * @param rid Run Id
	 * @param host Hostname from where results are uploaded
	 * 
	 * @param handle upload handle
	 * @throws RemoteException when something in RMI goes bad
	 */
	public void endRunUpload(AID aid, EID eid, BID bid, RID rid,
			String host, UploadHandle handle) throws RemoteException;; 
	
	/**
	 * Closes the upload handle after uploading the run. Client should call 
	 * this method after the upload status returned by <code>getFileUploadStatus</code> 
	 * has value <code>ACCEPTED</code>, or <code>ERROR</code>. 
	 * 
	 * If the client doesn't close the upload handle, Results Repository will leak
	 * a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle. 
	 * 
	 * @param handle upload handle
	 * @throws RemoteException when something in RMI goes bad
	 */
	void endFileUpload(UploadHandle handle) throws RemoteException;
	
	/**
	 * Request upload of logs from a given context and task into results repository. This tasks
	 * checks, whether 
	 * 
	 * @param aid		AnalysisId
	 * @param eid		ExperimentId 
	 * @param contextName	The name of task's context
	 * @param taskName		The name of task to upload logs from
	 * @throws ResultsRepositoryException 
	 * @throws RemoteException 
	 */
	void performLogUpload(AID aid, EID eid, String contextName, String taskName) 
		throws ResultsRepositoryException, RemoteException;
	
	/**
	 * Finishes upload of all the logs, which where registered for a given experiment, but still not
	 * uploaded
	 * 
	 * @param analysisId 	AID of analysis in account
	 * @param experimentId 	EID of experiment in account
	 * @throws ResultsRepositoryException 
	 * @throws RemoteException 
	 */
	void finishLogUpload(AID analysisId, EID experimentId) 
		throws ResultsRepositoryException, RemoteException;

	/**
	 * Sets callback R scripts for given experiment.
	 * 
	 * @param aid analysis identifier
	 * @param eid experiment identifier
	 * @param callbackRScripts object containing texts of the scripts
	 * @throws IllegalArgumentException if the analysis or experiment does not
	 *          exist
	 * @throws RemoteException when something in RMI goes bad
	 */
	void setExperimentCallbackRScripts(AID aid, EID eid,
		CallbackRScripts callbackRScripts) throws RemoteException;

	/**
	 * Reports column names for experiment
	 * 
	 * @param analysisId	Analysis Id
	 * @param experimentId	Experiment Id
	 * @param columnNames	<code>ArrayList&lt;String&gt;</code> containing list of column names 
	 * @throws RemoteException when something in RMI goes bad 
	 */
	void reportExperimentColumnNames(AID analysisId, EID experimentId, ArrayList<String> columnNames)
		throws RemoteException;
			
}
