/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;


/**
 * Implements management and loading and storing of roles and hostname mapping table
 * 
 * @author Jakub Lehotsky
 */
public class RoleList {
		
	/** I/O error in validity map reading message */
	public static final String IO_ERROR_LIST_READING_MESSAGE = "I/O error reading role list of ";
	/** I/O error in validity map reading message */
	public static final String IO_ERROR_LIST_STORING_MESSAGE = "I/O error storing role list of ";
	
	/** ASCII of delim. character (TAB) */
	private static final String ASCII_TAB = "\t";	
	
	/** Actual metadata */
	private ArrayList<RoleItem> roleTable;
	
	/** Last item counter */
	private int lastItem;		
	
	/**
	 * Constructor - creates numbered table, void object 
	 * (meant for creating new validity map tables items, initializing them and storing into file)
	 * 
	 */
	public RoleList() {
		roleTable = new ArrayList<RoleItem>();
		this.lastItem = 0;
	}
	
	/**
	 * Returns the roles and host names mapping table we store
	 * 
	 * @return the metadata
	 */
	
	public RoleItem[] getRolesTable() {
		return roleTable.toArray(new RoleItem[] {});
	}	
	
	/** 
	 * Constructor - initializes numbered map in memory and read its contents from the file.
	 * 
	 * @param filename	Name of file to read validity table from
	 * @throws IOException 
	 */
	public RoleList(String filename) throws IOException {
				
		String[] lineTokens;
		String line;
		BufferedReader inputReader;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {			
			roleTable = new ArrayList<RoleItem>();
			return;
		}
				
		roleTable = new ArrayList<RoleItem>();
		
		while ((line = inputReader.readLine()) != null) {								
			lineTokens = line.split(ASCII_TAB);
			try {
				if (lineTokens.length == 3) { 					
					addItem(new Integer(lineTokens[0]), lineTokens[1], lineTokens[2]);					
				} else {
					throw new IOException("Invalid format in roles table, file " + filename + ".");
				}
			} catch (IndexOutOfBoundsException e) {
				throw new IOException("Invalid format in table, file " + filename + ".");
			}
		}
				
		inputReader.close();
	}
	
	/**
	 * Stores numbered list we have in our memory to the file.
	 * 
	 * @param filename	Where to store numbered list.
	 * @throws IOException
	 */
	public void store(String filename) throws IOException {
		
		int i;		
		PrintWriter outputWriter;
		
		outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(filename)));
		
		for (i = 0; i < roleTable.size(); i++) {
			outputWriter.println(getItemKey(i) + ASCII_TAB 
					+ getItemRoleName(i) + ASCII_TAB + getItemHostName(i));		
		}
		
		outputWriter.close();
	}
	
	/**
	 * Returns the count of numbered items currently loaded in memory
	 * 
	 * @return	The count of items
	 */
	public int getItemCount() {
		return roleTable.size();
	}
	
	/**
	 * Adding item to role list map
	 * 
	 * @param index	Key of new item
	 * @param roleName	Name of the role
	 * @param hostName	Assocciated hostname
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public void addItem(long index, String roleName, String hostName) {		
		roleTable.add(new RoleItem(index, roleName, hostName));
		lastItem++;
	}
	
	/**
	 * Adding item to role list map using a RoleItem object
	 * 
	 * @param item	Item to add
	 *  
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public void addItem(RoleItem item) {		
		roleTable.add(item);
		lastItem++;
	}
	
	/**
	 * Returns one item of roles table identified by index
	 * 
	 * @param index	Index of the item to get
	 * 
	 * @return	The item
	 */
	RoleItem getItem(int index) {
		return roleTable.get(index);
	}
	
	/**
	 * Deleting item by key
	 * 
	 * @param key	Key of item to be deleted
	 */
	public void deleteItemByKey(long key) {
		
		int i = 0;		
		try {		
			while (getItemKey(i) != key) {
				i++;
				if (i == lastItem) {
					return; // not found
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return; //not found
		}
		
		roleTable.remove(i);
	}
	
	/**
	 * Returns the key of the item with index.
	 * 
	 * @param index	Index of they key to give.
	 * @return	int - the key
	 */
	public long getItemKey(int index) {
		return roleTable.get(index).getKey();
	}
	
	/**
	 * Returns the role name of the item with index.
	 * 
	 * @param index	Index of the value to give.
	 * @return	String with value.
	 */
	public String getItemRoleName(int index) {
		return roleTable.get(index).getRoleName();
	}
	
	/**
	 * Returns the role name of the item with index.
	 * 
	 * @param index	Index of the value to give.
	 * @return	String with value.
	 */
	public String getItemHostName(int index) {
		return roleTable.get(index).getHostName();
	}

	/**
	 * Returns the index the item, with given key. (can be used by <code>getItemRoleName()</code>
	 * or <code>getItemHostName</code> 
	 * 
	 * @param key	Key to ask for.
	 * @return		Index of associated item in the table, -1 if it doesn't exist 
	 */
	public int getItemIndex(long key) {
		
		if (roleTable == null) {
			Logger.logError("While trying to read long " + key + ", encountered " 
					+ "null!");
			assert false : "Table unitialized!";
			return -1; 			
		}
		
		int i = 0;
		
		try {		
			while (getItemKey(i) != key) {
				i++;
				if (i == lastItem) {
					return -1;
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return -1;
		}
		
		return i;
	}
	
	/**
	 * Returns the index the item, with given hostName. (can be used by <code>getItemRoleName()</code>
	 * or <code>getItemHostName</code> 
	 * 
	 * @param hostName	Hostname by which to search for item in the table
	 * @return		Index of associated item in the table, -1 if it doesn't exist 
	 */
	public int getItemIndexByHostname(String hostName) {
		
		if (roleTable == null) {
			Logger.logError("While trying to search for " + hostName + "in metadata, encountered " 
					+ "null roles table!");
			assert false : "Table unitialized!";
			return -1; 			
		}
		
		int i = 0;
		
		try {		
			while (!getItemHostName(i).equals(hostName)) {
				i++;
				if (i == lastItem) {
					return -1;
				}
			}
		} catch (IndexOutOfBoundsException e) {
			return -1;
		}
		
		return i;
	}
	
	/**
	 * Sets the value of the table item, with the given long key. 
	 * 
	 * @param key	Key
	 * @param roleName	The name of role
	 * @param hostName 	The name of the host
	 * 
	 * @return True iff succesful (false if key doesn't exist)
	 */
	public boolean setItemByKey(long key, String roleName, String hostName) {
		int i = 0;
		
		while (getItemKey(i) != key) {
			i++;
			if (i == lastItem) {
				return false; 
			}
		}
		
		try {
			roleTable.set(i, new RoleItem(key, roleName, hostName));
		} catch (IndexOutOfBoundsException e) {
			return false;
		}
				
		return true; 		
	}
	
	
}
