/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.resultsrepository.gui.Statistics;

/**
 * Class implementing manipulation with multiple statistics tables for entities.
 * 
 * @author Jakub Lehotsky
 */
public final class StatisticsManager {
	
	/** Filename with stored statistics tables references */
	public static final String STATISTICS_TABLE_FILE = "statistics.tables";
	/** Filename with stored statistics tables references */
	public static final String STATISTICS_RUN_HEADER_FILE = "runstatistics-head";
	
	/** Reference to results repository database */
	private static Database database = null;
	
	/** The single existing intance of RID Manager */	
	private static StatisticsManager sMInstance = null;
	
	/** Ascii of tab character */
	protected static final String ASCII_TAB = "\t";
	
	/** Stastitic file with dimensions */
	private static final String STATISTICS_DIM_FILE = ".dim";
	/** Stastitic file with table label */
	private static final String STATISTICS_LABEL_FILE = ".label";
	/** Stastitic file with column names */
	private static final String STATISTICS_COLUMNS_FILE = ".col";
	/** Stastitic file with row names */
	private static final String STATISTICS_ROWS_FILE = ".row";
	/** Stastitic file with values of statistics */
	private static final String STATISTICS_VALUE_FILE = ".value";	
	
	/**
	 * Constructor - private, void for singleton use 
	 */
	private StatisticsManager() {
	}
	
	/**
	 * Return single instance of Graph manager 
	 * 
	 * @return	reference single running instance of Graph manager
	 */
	public static StatisticsManager getInstance() {
		if (sMInstance == null) {
			sMInstance = new StatisticsManager();
		}
		return sMInstance;
	}
	
	/**
	 * Initialize Statistics Manager before its first use 
	 * 
	 * @param database	Instance of database we use
	 */
	public static void initialize(Database database) {
		StatisticsManager.database = database;
	}

	/**
	 * Reads one line from reader and splits it by tabs 
	 * 
	 * @param inputReader	Reader to read line from
	 * @param filename	Filename to be reported if error occurs
	 * 
	 * @return	Array of Strings containing tokens, that were separated by tabs
	 */
	private String[] getParsedLine(BufferedReader inputReader, String filename) {
		
		String line;
		try {
			line = inputReader.readLine();
			if (line == null) {
				return null;
			}
		} catch (IOException e) {
			Logger.logError("I/O error reading statistics from " + filename + ".");
			return null;
		}
		
		return line.split(ASCII_TAB);				
	}
	
	/**
	 * Loads statistics table into memory
	 * 
	 * @param tableName	The name of the statistics table to read from
	 * 
	 * @return	Statistics object containing loaded statistics
	 */
	private Statistics loadStatisticsTable(String tableName) {
		BufferedReader inputReader;
		int rows;
		int columns;
		String label;
		String[] rowNames;
		String[] columnNames;
		Double[][] valuesTable;
		boolean[] columnHighlights;
		String filename;
		
		String[] lineTokens;
		
		Logger.logTrace("Reading statistics from " + tableName  + ".");
		
		/* ************************/
		/* First, read dimensions */		
		filename = tableName + STATISTICS_DIM_FILE;
		// Note: dimensions file is determinative for existence of statistics
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logTrace("No statistics (.dim file not present): " + filename + ".");
			return null;
		}
		
		lineTokens = getParsedLine(inputReader, filename);
		
		if (lineTokens.length < 2) {
			Logger.logError("Invalid statistics format - file: " 
					+ filename + ", line 1.");
			return null;
		}	
				
		try {
			rows = new Integer(lineTokens[0]);
			columns = new Integer(lineTokens[1]);
		} catch (NumberFormatException e) {
			Logger.logError("Invalid statistics format - file: " + filename  + ".");
			return null;
		}
		
		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename  + ".");
			return null;
		}
		
		/* **************************/
		/* Now, read label of table */		
		filename = tableName + STATISTICS_LABEL_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing statistics table label file: " + filename  + ".");
			return null;
		}
		
		lineTokens = getParsedLine(inputReader, filename);
		label = lineTokens[0];
		
		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename  + ".");
			return null;
		}
		
		
		/* *******************/
		/* Read column names */		
		
		filename = tableName + STATISTICS_COLUMNS_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing column names statistics file: " + filename + ".");
			return null;
		}
		
		columnNames = getParsedLine(inputReader, filename);
		
		if (columnNames.length != columns) {
			Logger.logError("Invalid statistics format, column names count doesn't " 
					+ "fit - file: " + filename  + ".");
			return null;
		}
		
		columnHighlights = new boolean[columns];
		for (int i = 0; i < columns; i++) {
			columnHighlights[i] = false;
		}
		
		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename  + ".");
			return null;
		}
		
		
		/* ****************/
		/* Read row names */		
		
		filename = tableName + STATISTICS_ROWS_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing row names statistics file: " + filename  + ".");
			return null;
		}						
		
		rowNames = getParsedLine(inputReader, filename);
		
		if (rowNames.length != rows) {
			Logger.logError("Invalid statistics format, row names count doesn't " 
					+ "fit - file: " + filename  + ".");
			return null;
		}
		
		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename  + ".");
			return null;
		}
		
		
		/* *******************/
		/* Read values table */		
		
		filename = tableName + STATISTICS_VALUE_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing file with statistics values: " + filename  + ".");
			return null;
		}
		
		valuesTable = new Double[rows][];
		
		for (int row = 0; row < rows; row++) {
			
			valuesTable[row] = new Double[columns];
	
			lineTokens = getParsedLine(inputReader, filename);
			
			if (lineTokens.length != columns) {
				Logger.logError("Error reading statistics table, row " + row 
						+ " - column count doesn't fit."  + ".");
				return null;
			}
					
			for (int col = 0; col < columns; col++) {
				try {			
					valuesTable[row][col] = new Double(lineTokens[col]);			
				} catch (NumberFormatException e) {
					valuesTable[row][col] = null;					
				} catch (Exception e) {
					Logger.logError("Error reading statistics table, row " + row 
							+ " exception: " + e.getMessage()  + ".");
					return null;			
				}
			}							
		}
		

		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename  + ".");
			return null;
		}

		Logger.logTrace("Statistics successfully read.");
		
		return new Statistics(label, columnNames, columnHighlights, rowNames, valuesTable);
		
	}

	/**
	 * Load all statistics given a statistics table index directory for given entity
	 * 
	 * @param entityDirectory	Directory which holds stastistics
	 * 
	 * @return	Array of all counted statistics
	 */

	private Statistics[] loadStatistics(String entityDirectory) {
		
		Statistics[] returnValue;
		NumberedTable table;		
	
		try {
			table = new NumberedTable(entityDirectory + File.separator + STATISTICS_TABLE_FILE);
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
					+ entityDirectory);			
			return new Statistics[] { null };
		}
		
		returnValue = new Statistics[table.getItemCount()];
		
		for (int i = 0; i < table.getItemCount(); i++) {
			returnValue[i] = loadStatisticsTable(entityDirectory + File.separator + table.getItemValue(i));
		}
		
		return returnValue;		
	}	

	/**
	 * Loads analysis statistics
	 * 
	 * @param aid	Analysis Id
	 * 
	 * @return	Statistics array
	 */
	public Statistics[] loadAnalysisStatistics(AID aid) {
		return loadStatistics(database.getDirectoryInDatabase(aid));
	}

	/**
	 * Loads experiment statistics
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * 
	 * @return	Statistics array
	 */
	public Statistics[] loadExperimentStatistics(AID aid, EID eid) {
		return loadStatistics(database.getDirectoryInDatabase(aid, eid));
	}

	/**
	 * Loads binary statistics
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id 
	 * 
	 * @return	Statistics array
	 */
	public Statistics[] loadBinaryStatistics(AID aid, EID eid, BID bid) {
		return loadStatistics(database.getDirectoryInDatabase(aid, eid, bid));
	}
	
	/**
	 * Loads run statistics
	 * 
	 * @param aid	Analysis Id
	 * @param eid	Experiment Id
	 * @param bid	Binary Id
	 * @param rid	Run Id 
	 * 
	 * @return	Statistics array
	 */	
	public Statistics[] loadRunStatistics(AID aid, EID eid, BID bid, RID rid) {
	
		Statistics returnValue;
		BufferedReader inputReader;
		int rows;
		int columns;
		String label;
		String[] rowNames;
		String[] columnNames;
		Double[][] valuesTable;
		boolean[] columnHighlights;

		String[] lineTokens;
		String tableName;
		String filename;
					
		tableName = database.getDirectoryInDatabase(aid, eid, bid) + File.separator 
			+ STATISTICS_RUN_HEADER_FILE;

		
		/* ************************/
		/* First, read dimensions */		
		filename = tableName + STATISTICS_DIM_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logTrace("No statistics (.dim file not present): " + filename + ".");
			return new Statistics[] { null }; 
		}
			
		lineTokens = getParsedLine(inputReader, filename);
			
		if (lineTokens.length < 2) {
			Logger.logError("Invalid statistics format - file: " 
					+ filename + ", line 1.");
			return new Statistics[] { null };
		}
			
		try {
			rows = new Integer(lineTokens[0]);
			columns = new Integer(lineTokens[1]);
		} catch (NumberFormatException e) {
			Logger.logError("Invalid statistics format - file: " + filename  + ".");
			return new Statistics[] { null };
		}
		
		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename + ".");
			return null;
		}
		
		/* **************************/
		/* Now, read label of table */		
		filename = tableName + STATISTICS_LABEL_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing statistics table label file: " + filename + ".");
			return new Statistics[] { null }; 
		}		

		lineTokens = getParsedLine(inputReader, filename);
		label = lineTokens[0];

		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename + ".");
			return null;
		}


		/* *******************/
		/* Read column names */		

		filename = tableName + STATISTICS_COLUMNS_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing column names statistics file: " + filename + ".");
			return null; 
		}

		columnNames = getParsedLine(inputReader, filename);

		if (columnNames.length != columns) {
			Logger.logError("Invalid statistics format, column names count doesn't " 
					+ "fit - file: " + filename + ".");
			return new Statistics[] { null };
		}

		columnHighlights = new boolean[columns];
		for (int i = 0; i < columns; i++) {
			columnHighlights[i] = false;
		}
		

		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics file " + filename + ".");
			return null;
		}
		
		
		/* ****************/
		/* Read row names */		
		
		filename = tableName + STATISTICS_ROWS_FILE;
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {					
			Logger.logError("Missing row names statistics file: " + filename + ".");
			return null; 
		}
		
		rowNames = getParsedLine(inputReader, filename);
		
		if (rowNames.length != rows) {
			Logger.logError("Invalid statistics format, row names count doesn't " 
					+ "fit - file: " + filename + ".");
			return new Statistics[] { null };
		}

		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Error closing statistics run header file " + filename + ".");
			return new Statistics[] { null };
		}
				
		valuesTable = new Double[rows][];
		
		filename = database.getFileInDatabase(aid, eid, bid, 
				RunValidityMapsManager.getInstance().getRunStatisticsPath(aid, eid, bid, rid));
		
		try {
			inputReader = new BufferedReader(new FileReader(filename));
		} catch (FileNotFoundException e) {								
			Logger.logError("File with run's statistics not found:" + filename);
			return new Statistics[] { null };
		}		
		
		for (int row = 0; row < rows; row++) {
			
			valuesTable[row] = new Double[columns];			
			
			lineTokens = getParsedLine(inputReader, filename);
			
			if (lineTokens.length != columns) {
				Logger.logError("Error reading statistics table, run " + rid 
						+ ",row " + row	+ " - column count doesn't fit.");
				return new Statistics[] { null };
			}
					
			for (int col = 0; col < columns; col++) {
				try {			
					valuesTable[row][col] = new Double(lineTokens[col]);			
				} catch (Exception e) {
					Logger.logError("Error reading statistics rable, run " + rid
							+ ", row " + row +	" exception: " + e.getMessage()  + ".");
					return new Statistics[] { null };			
				}
			}							
		}

		try {
			inputReader.close();
		} catch (IOException e) {
			e.printStackTrace();
			Logger.logError("Missing file with statistics values: " + filename + ".");
			return new Statistics[] { null };
		}

		returnValue = new Statistics(label, columnNames, columnHighlights, 
				rowNames, valuesTable);

		return new Statistics[] { returnValue };
	}	
}
