/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.common.id.RID;


/**
 * This class handles all the tasks information stored in entities. On experiment level, info
 * about whether logs are already uploaded is present, on binary level, only list is stored, on 
 * runs level, tables per run are stored.
 * 
 * Purpose is to allow logs upload and their displaying through GUI 
 * 
 * @author Jakub Lehotsky
 */
public final class TaskListManager {

	/** Extensions of temporary directory for R scripts unpacking */
	private static final String TASK_LIST_FILENAME = "tasks";

	/** Extensions of temporary directory for R scripts unpacking */
	private static final String RUNSTASK_LIST_FILENAME = "runs-tasks";

	
	/** ASCII of TAB character */
	private static final String ASCII_TAB = "\t";
    
	/** Single instance of load collector */
	private static TaskListManager taskListManagerInstance = null;
	
	/** Databse that is used for results in Results Repository */
	private static Database database;
	
	/** Experiment task list */
	private static NumberedTable experimentTaskList;
	
	/** Binary task list */
	private static ArrayList<String> binaryTaskList;
	
	/** Runs task list */
	private static NumberedTable runsTaskList;

	/**
	 *  Construction
	 */
	private TaskListManager() {
	}
	
	public static void initialize(Database database) {
		TaskListManager.database = database;
	}

	
	/**
	 * Returns the single instance of load collector
	 * 
	 * @return	Reference to LoadCollector instance 
	 */
	public static TaskListManager getInstance() {
	    
		if (taskListManagerInstance == null) {
			return new TaskListManager();
		} else {
			return taskListManagerInstance;
		}
	}

	/* ****************** */
	/* *** EXPERIMENT *** */
	/* ****************** */
	
	/**
	 * Loads experiment task list into memory, this should be performed every time before another
	 * operation is done upon experiment task list. In experiment task list, also information about
	 * logs uploaded are stored (they are not stored with binaries and runs)
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 */
	void loadExperimentTaskList(AID aid, EID eid) {
	
	    try {
	    	experimentTaskList = new NumberedTable(database.getFileInDatabase(aid, eid, TASK_LIST_FILENAME));
	    } catch (IOException e) {
	    	e.printStackTrace();
	    	Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
			+ Database.getExperimentReferenceString(aid, eid));
		
	    	experimentTaskList = new NumberedTable();
	    	return;	
	    } 
	}
	
	/**
	 * Stores experiment task list in memory into disk including "logs uploaded" flag 
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 */
	void storeExperimentTaskList(AID aid, EID eid) {
	    try {
	        	experimentTaskList.store(database.getFileInDatabase(aid, eid, TASK_LIST_FILENAME));
            } catch (IOException e) {	 
            	e.printStackTrace();
            	Logger.logError(NumberedTable.IO_ERROR_LIST_STORING_MESSAGE 
            			+ Database.getExperimentReferenceString(aid, eid));	        
            }
	}

	/**
	 * Adds new experiment tasks into list (loadExperimentTaskList should be called before!)
	 * These tasks are selected as with logs not yet uploaded.  
	 * 
	 * @param taskIds
	 */
	void addNewExperimentTasks(String[] taskIds) {
	    for (String taskId : taskIds ) {
	    	if (experimentTaskList.getItemByKey(Integer.valueOf(taskId)) == null)
	    		experimentTaskList.addItem(Integer.valueOf(taskId), "0");
	    }
	}
	
	/**
	 * Sets the flag "logs are uploaded" for a giben task in experiment (this is stored 
	 * accross whole experiment only). This is changed in memory only, so be shure to have
	 * <code>loadExperimentTaskList()</code> before and call <code>storeExperimentTaskList()</code> 
	 * to persist information. 
	 * 
	 * @param taskId	The name of task, to set "logs are uploaded" flag 
	 */
	void setExperimentTaskUploadedFlag(String taskId) {
		Logger.logDebug("taskId to store logs for = " + taskId);
	    if (!experimentTaskList.setItemByKey(Long.valueOf(taskId), "1")) {
	    	Logger.logError("Error setting 'logs uploaded' flag for task " + taskId);
	    }
	}
	
	/**
	 * Returns the list of all tasks registered with experiment, which don't have their logs 
	 * uploaded (i.e. have not "logs uploaded" flag set)
	 * 
	 * @return	The list of tasks which don't have logs uploaded
	 */
	String[] getExperimentUnuploadedTaskList() {
	    ArrayList<String> unuploaded = new ArrayList<String> ();
	    
	    for (NumberedItem numberedItem : experimentTaskList.getNumberedTable() ) {
		if (numberedItem.getValue().equals("0")) {
		    unuploaded.add(Long.toString(numberedItem.getKey()));
		}
	    }	
	    
	    return unuploaded.toArray(new String[] {});
	}
	
	/**
	 * Returns list of all tasks contained in experiment (both which uploaded their logs, and 
	 * which didn't)
	 * 
	 * @return	The list of task names in experiment
	 */
	String[] getExperimentTaskList() {
	    ArrayList<String> taskArrayList = new ArrayList<String> ();
	    
	    for (NumberedItem numberedItem : experimentTaskList.getNumberedTable() ) {
	    	taskArrayList.add(Long.toString(numberedItem.getKey()));	    	
	    }	
	    
	    return taskArrayList.toArray(new String[] {});
	}

	/* ************** */
	/* *** BINARY *** */
	/* ************** */

	
	/**
	 * Load binary task list into memory. This is simpler than experiment, it doesn't contain
	 * information about logs presence for a given task, so just list of tasks is stored. 
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 */
	public void loadBinaryTaskList(AID aid, EID eid, BID bid) {
		String[] lineTokens;
		String line;
		BufferedReader inputReader;
		
		try {
			inputReader = new BufferedReader(new FileReader(
					database.getFileInDatabase(aid, eid, bid, TASK_LIST_FILENAME)));
		} catch (FileNotFoundException e) {			
			binaryTaskList = new ArrayList<String>();
			return;
		}
				
		binaryTaskList = new ArrayList<String>();
		
		try {
			/* Read only first line, others are invalid */
			if ((line = inputReader.readLine()) != null) {								
				lineTokens = line.split(ASCII_TAB);
				try {
					for (String lineToken : lineTokens) {
						binaryTaskList.add(lineToken);
					}
				} catch (IndexOutOfBoundsException e) {
					throw new IOException("Invalid format in task list, file " + 
							database.getFileInDatabase(aid, eid, bid, TASK_LIST_FILENAME) + ".");
				}
			}
	    } catch (IOException e) {
	    	e.printStackTrace();
	    	Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
	    			+ Database.getExperimentReferenceString(aid, eid));
		
	    	experimentTaskList = new NumberedTable();
	    	return;	
	    } finally { 
	    	try {
	    		inputReader.close();
	    	} catch (IOException e) {
	    		e.printStackTrace();
	    		Logger.logError("Error closing the file " +
	    				database.getFileInDatabase(aid, eid, bid, TASK_LIST_FILENAME));
	    	}	    	
	    }
		
	}
	
	/**
	 * Stores task list of a given binary to the disk
	 *  
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 */
	public void storeBinaryTaskList(AID aid, EID eid, BID bid) {
			
		int i;		
		PrintWriter outputWriter = null;
		
		try {
			outputWriter = new PrintWriter(new BufferedWriter(new FileWriter(
					database.getFileInDatabase(aid, eid, bid, TASK_LIST_FILENAME))));			
			
			if (binaryTaskList.size() > 0) {
				outputWriter.print(binaryTaskList.get(0));
				for (i = 1; i < binaryTaskList.size(); i++) {
					outputWriter.print( ASCII_TAB + binaryTaskList.get(i));		
				}
			}
			
			outputWriter.println();
		} catch (IOException e) {
	    	e.printStackTrace();
	    	Logger.logError(NumberedTable.IO_ERROR_LIST_STORING_MESSAGE 
	    			+ Database.getBinaryReferenceString(aid, eid, bid));
		
	    	return;	
		} finally {
			outputWriter.close();
		}
		
	}

	/**
	 * Adds list of tasks to existing binary task list. It is done in memory, so be shure to have 
	 * actual data in memory loaded with <code>loadBinaryTaskList()</code>  and then to store it 
	 * to disk with <code>storeBinaryTaskList()</code>
	 * 
	 * @param taskIds
	 */
	void addNewBinaryTasks(String[] taskIds) {
	    for (String taskId : taskIds ) {
	    	if (!binaryTaskList.contains(taskId)) {
	    		binaryTaskList.add(taskId);
	    	}
	    }
	}	
	
	/**
	 * Returnns binary tasks list stored in memory.
	 * 
	 * @return	The list of task names for a given binary
	 */
	public String[] getBinaryTaskList() {
		return binaryTaskList.toArray(new String[] {});
	}
	
	/* ************** */
	/* *** BINARY *** */
	/* ************** */
		
	/**
	 * Loads runs task list into memory. This is loaded for whole binary at once. When some run
	 * is deleted, item is still kept. This little DB-leak allows for more simplicity of managing 
	 * code. 
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 */
	public void loadRunsTaskList(AID aid, EID eid, BID bid) {
	    try {
	    	runsTaskList = new NumberedTable(
	    			database.getFileInDatabase(aid, eid, bid, RUNSTASK_LIST_FILENAME));
	    } catch (IOException e) {
	    	e.printStackTrace();
	    	Logger.logError(NumberedTable.IO_ERROR_LIST_READING_MESSAGE 
			+ Database.getExperimentReferenceString(aid, eid));
		
	    	runsTaskList = new NumberedTable();
	    	return;	
	    } 
	}
	
	/**
	 * Adds task list for a given run. Noticed that for a run, just a single list is provided and
	 * it is not changed any more in future. For experiment and binaries, it can be added to allow
	 * additional measurements mechanism. 
	 * 
	 * @param rid		Run ID
	 * @param taskIds	List of tasks that belong to RID
	 */
	void addNewRunsTasks(RID rid, String[] taskIds) {
		
		String toStore;
		String taskList;
		
		taskList = runsTaskList.getItemByKey(rid.hashCode());
		
		if (taskList != null &&	!taskList.equals("")) {
			return;			
		}
		
		Logger.logTrace("Task count for run " + rid + " is " + taskIds.length);
		Logger.logTrace("Task list : " + taskIds);
		
		if (taskIds.length > 0) {
			
			toStore = taskIds[0];
			
			for (int i = 1; i < taskIds.length; i++) {
				toStore += ASCII_TAB + taskIds[i];
			}  		
			
		} else {
			toStore = "";
		}
		
		runsTaskList.addItem((Long) rid.value(), toStore);
		
	}	
	
	
	/**
	 * Stores task list of runs of given binary to disk
	 * 
	 * @param aid	Analysis ID
	 * @param eid	Experiment ID
	 * @param bid	Binary ID
	 */
	void storeRunsTaskList(AID aid, EID eid, BID bid) {
		
		try {
        	runsTaskList.store(
        			database.getFileInDatabase(aid, eid, bid, RUNSTASK_LIST_FILENAME));
        } catch (IOException e) {	 
        	e.printStackTrace();
        	Logger.logError(NumberedTable.IO_ERROR_LIST_STORING_MESSAGE 
        			+ Database.getExperimentReferenceString(aid, eid));
        	
        }
	}

	/**
	 * Returns task list of a given run. This is read from memory. So, if you want to read from the
	 * disk, be shure to call loadRunsTaskList before.
	 * 
	 * @param rid	Run ID, for which we want task list
	 * @return		List of tasks
	 */
	public String[] getRunTaskList(RID rid) {

		String toParse = runsTaskList.getItemByKey((Long) rid.value());
		if (toParse == null) {
			return new String[] {};
		}
		
		String[] returnValue = toParse.split(ASCII_TAB);
				
		if ((returnValue.length == 1) && returnValue[0].equals("")) {
			return new String[] {};
		} else {
			return returnValue;
		}
	}
}
