/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;

/**
 * This class conatin helper functions for zipping and unzipping useful work
 * 
 * @author Jakub Lehotsky      
 */ 

public class Zipper {
	
	private Zipper() {		
	}
	
	/* this is small support function - useful also for external use (see relativePath note above) */
	private static void putDirectoryZipEntry(ZipOutputStream zos, String dirName) throws IOException {		
		ZipEntry entry = new ZipEntry(dirName + File.separator);
		entry.setMethod(ZipEntry.STORED);
		entry.setSize(0);
		entry.setCompressedSize(0);
		entry.setCrc(0);
		zos.putNextEntry(entry);
		zos.closeEntry();		
	}

	/**
	 * Zips one file
	 * 
	 * Please do this before callbefore call:
	 * 		
	 *	BufferedOutputStream bufOutputStream = new BufferedOutputStream(zipOutputStream)
	 * 
	 *   this is because I need access to both streams (no way to get inner stream from 
	 *   filter stream in code) and pass out and zipOutputStream as parameters of this method
	 * 
	 * @param fileToZip	file to zip
	 * @param relativePath	path in resulting zip file, it will consist of 
	 *      (warning: you should manually add ZipEntries for all subdirectories, if you want to 
	 *      use this - use with care)   
	 * @param zipOutputStream	see above
	 * @param bufOutputStream	see above
	 *	
	 * @return	True iff succesful
	 * @throws IOException
	 */
	public static boolean zipFile(ZipOutputStream zipOutputStream, 
		BufferedOutputStream bufOutputStream, File fileToZip, String relativePath) throws IOException { 
	    		 
	    if (fileToZip.isFile()) {	           	 
	    	BufferedInputStream in =
	    		new BufferedInputStream(new FileInputStream(fileToZip));
	          	  	           	  
	    	zipOutputStream.putNextEntry(new ZipEntry(
	    			((relativePath.length() == 0) ? "" : (relativePath + File.separator)) 
	    			+ fileToZip.getName()));
	    	byte[] buffer = new byte[4096];
	    	int bytesRead;
	    	while ((bytesRead = (in.read(buffer))) != -1) {
	    		bufOutputStream.write(buffer, 0, bytesRead);
	    	}
	    	in.close();
	    	bufOutputStream.flush(); // important!!! must do this before we make new ZipEntry
	    }	            
	    return true;
	}
	
	/**
	 * Zips directory into file
	 * 
	 * Please do this before callbefore call:
	 * 		
	 *	BufferedOutputStream bufOutputStream = new BufferedOutputStream(zipOutputStream)
	 * 
	 *   this is because I need access to both streams (no way to get inner stream from 
	 *   filter stream in code) and pass out and zipOutputStream as parameters of this method
	 * 
	 * @param fileToZip	directory (must be directory!) to compress
	 * @param relativePath	path in resulting zip file, it will consist of 
	 *      (warning: you should manually add ZipEntries for all subdirectories, if you want to 
	 *      use this - use with care)   
	 * @param zipOutputStream	see above
	 * @param bufOutputStream	see above
	 *	
	 * @return	True iff succesful
	 * @throws IOException
	 */	
	public static boolean zipDirectoryRecursively(ZipOutputStream zipOutputStream, 
			BufferedOutputStream bufOutputStream, File fileToZip, String relativePath) throws IOException { 
	    
		if (fileToZip.isDirectory()) {
	    	if (relativePath.compareTo("") != 0) {
	    		putDirectoryZipEntry(zipOutputStream, relativePath);	
	    	}
	        String[] children = fileToZip.list();
	        for (int i = 0; i < children.length; i++) {
	        	File dirItem = new File(fileToZip.getAbsolutePath() + File.separator + children[i]); 
	        	if (dirItem.isDirectory()) {
	        		zipDirectoryRecursively(zipOutputStream, bufOutputStream, 
	        				dirItem, ((relativePath.length() == 0) ? ""  
	        						: (relativePath + File.separator))
	        						+ dirItem.getName());
	        			
	            } else if (dirItem.isFile()) {
	            	BufferedInputStream in =
	        			new BufferedInputStream(new FileInputStream(dirItem));
	            	  
	        		zipOutputStream.putNextEntry(new ZipEntry(
	        				((relativePath.length() == 0) ? "" : (relativePath + File.separator))
	        				+ dirItem.getName()));
	        		byte[] buffer = new byte[4096];
	        		int bytesRead;
	        		while ((bytesRead = (in.read(buffer))) != -1) {
	        			bufOutputStream.write(buffer, 0, bytesRead);
	        		}
	        		in.close();
	        		bufOutputStream.flush(); // important!!! must do this before we make new ZipEntry
	            }	            
	        }
	    }	    
	    return true;
	}
	
	/**
	 * Helper method - copies whole input stream (while not EOF) into output stream
	 * 
	 * @param in	Input Stream
	 * @param out	Output Stream
	 * @throws IOException
	 */
	private static final void copyInputStream(InputStream in, OutputStream out)
		throws IOException {
		byte[] buffer = new byte[1024];
		int len;

		while ((len = in.read(buffer)) >= 0) {
			out.write(buffer, 0, len);
		}

		in.close();
		out.close();
	}

	/**
	 * Unzips file into a given directory
	 * 
	 * @param inputFile	File to be unzipped
	 * @param outputDirectory
	 * 			Directory, where file will be unzipped
	 * @throws IOException
	 */
	 public static void unzipFile(String inputFile, String outputDirectory) 
	 	throws IOException {
		 
	     Enumeration entries;
	     ZipFile zipFile;	     
	     
	     zipFile = new ZipFile(inputFile);
			
		entries = zipFile.entries();
			
		while (entries.hasMoreElements()) {
				
			ZipEntry entry = (ZipEntry) entries.nextElement();
			
			if (entry.isDirectory()) {
				/* Assume directories are stored parents first then children - 
				 * we're unpacking non-directories packages anyway */
			//System.err.println("Extracting directory: " + entry.getName());
				(new File(entry.getName())).mkdir();
					continue;
			}
			
			//System.err.println("Extracting file: " + entry.getName());
				copyInputStream(zipFile.getInputStream(entry),
					new BufferedOutputStream(new FileOutputStream(outputDirectory 
						+ File.separator + entry.getName())));
			}

			zipFile.close();
	 }

}
