/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository.gui;

import java.net.InetAddress;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.logging.LogRecord;
import cz.cuni.mff.been.logging.LogStorageException;
import cz.cuni.mff.been.resultsrepository.DeleteLockedException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryBusyException;
import cz.cuni.mff.been.resultsrepository.ResultsRepositoryException;

/**
 * Interface encapsulating operations with given entity.
 * 
 * To avoid <code>ResultsRepositoryInterface</code> clutter and code
 * duplication, each Results Repository entity (analysis, experiment, binary and
 * run) has its own interface. This interface excapsulates all operations which
 * can be done with entities. The <code>ResultsRepositoryInterface</code> has
 * only one method, which gets the interface for analyses (top-level entity).
 * 
 * The interface is typed by the identifier class of the entity which it
 * encapsulates.
 * 
 * If the concrete implementation of this interface doesn't support some
 * particular method, it should throw <code>UnsupportedOperationException</code>.
 * methods which can throw this exceptions are marked as such using
 * <code>throws</code> statement and in JavaDoc. Described behavior is
 * instance of the Composite design pattern.
 * 
 * Note the interface encapsulates operation with all entities of given type,
 * not just one. This is to allow more flexibility and is (maybe?) a bit nicer
 * as many methods aren't parametrized by concrete entity.
 * 
 * Entities are sent to the web interface using <code>Entity&lt;ID&ht;</code>
 * class, which is only a lightweight container, not an remote interface. This
 * is to avoid frequent RMI calls, which has probably better performance
 * characteristics (not tested, though).
 * 
 * @author David Majda
 * 
 * @param <ID>
 *            identifier of encapsulated entity
 * 
 */
public interface EntityInterface<ID extends cz.cuni.mff.been.common.id.ID>
		extends Remote {

	/**
	 * Returns object with information about given entity.
	 * 
	 * @param id
	 *            entity identifier
	 * @return object with information about given entity
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	Entity<ID> getEntity(ID id) throws RemoteException;

	/**
	 * Sets entity comment.
	 * 
	 * @param id
	 *            entity identifier
	 * @param comment
	 *            entity comment
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	void setEntityComment(ID id, String comment) throws RemoteException;

	/**
	 * Returns list of all entities represented by this interface.
	 * 
	 * @return list of all entities represented by this interface
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	List<Entity<ID>> getEntities() throws RemoteException;

	/**
	 * Returns interface to given subentity.
	 * 
	 * @param id
	 *            subentity identifier
	 * @return interface to given subentity
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	EntityInterface getSubentityInterface(ID id) throws RemoteException;

	/**
	 * Returns an array of export formats supported by this entity type.
	 * 
	 * @return array of export formats supported by this entity type
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	DataFormat[] getExportFormats() throws RemoteException;

	/**
	 * Export entities form the Results Repository. The client must open the
	 * socket on some port before calling this function. Then it calls the
	 * <code>beginExport</code> function.
	 * 
	 * This function works asynchronously and returns immediately. Before return
	 * it starts a new thread that will send the exported entities to the socket
	 * open on given host and port.
	 * 
	 * If there is any error when sending the entities' contents, the socket is
	 * closed. It's client's responsibility to check (via the
	 * <code>getExportStatus</code> method whether the data arrived complete
	 * and undamaged.
	 * 
	 * After the export finishes, client should close the returned handle via
	 * the <code>endExport</code> method.
	 * 
	 * @param ids
	 *            entity identifiers of the entities to export
	 * @param formatId
	 *            export format identifier
	 * @param ip
	 *            IP address of the machine waiting for receiving the data (not
	 *            necessairly the caller itself)
	 * @param port
	 *            port open for receiving the data
	 * @return entity export handle
	 * @throws IllegalArgumentException
	 *             if some of the the entities identified by the entity
	 *             identifiers doesn't exist
	 * @throws ResultsRepositoryBusyException
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	DownloadHandle beginExport(ID[] ids, String formatId, InetAddress ip,
			int port) throws ResultsRepositoryBusyException, RemoteException;

	/**
	 * Returns the status of the entity export. Result is one of the values
	 * listed in the <code>FileTransferStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>DOWNLOADING</code> mean
	 * that Results Repository still works on the entity export. Values
	 * <code>SUCCEEDED</code> and <code>ERROR</code> mean that Results
	 * Repository has finished its work (either successfully or unsuccessfully)
	 * and the export handle is ready to be closed by the <code>endExport</code>
	 * method.
	 * 
	 * @param handle
	 *            entity export handle
	 * @return export status
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	DownloadStatus getExportStatus(DownloadHandle handle)
			throws RemoteException;

	/**
	 * Closes the export handle. Client should call this method after the export
	 * status returned by <code>getExportStatus</code> has value
	 * <code>SUCCEEDED</code> or <code>ERROR</code>.
	 * 
	 * @param handle
	 *            entity export handle
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	void endExport(DownloadHandle handle) throws RemoteException;

	/**
	 * Returns an array of import formats supported by this entity type.
	 * 
	 * @return array of export formats supported by this entity type
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	DataFormat[] getImportFormats() throws RemoteException;

	/**
	 * Imports entities to the Results Repository. The client must open the
	 * socket on some port before calling this function. Then it calls the
	 * <code>beginImport</code> function.
	 * 
	 * This function work asynchronously and returns immediately. Before return
	 * it starts a new thread that will receive the imported entities from the
	 * socket open on given host and port.
	 * 
	 * If there is any error when sending the imported entities, the socket is
	 * closed.
	 * 
	 * Calling this function doesn't guarantee that imported entities will be
	 * added to the Results Repository - if they arrive damaged or some error
	 * during processing occurs, Results Repository rejects the data. Client has
	 * an opportunity to check (via the <code>getImportStatus</code> method
	 * whether the data arrived complete and undamaged.
	 * 
	 * After the upload finishes, client should close the returned handle via
	 * the <code>endImport</code> method.
	 * 
	 * @param formatId
	 *            import format identifier
	 * @param ip
	 *            IP address of the machine prepared for sending the packcage
	 *            (not necessairly the caller itself)
	 * @param port
	 *            port open for sending the package contents
	 * @return entity import handle
	 * @throws ResultsRepositoryBusyException
	 *             if the Results Repository is currently busy - caller should
	 *             retry after a while or abort the import
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	UploadHandle beginImport(String formatId, InetAddress ip, int port)
			throws ResultsRepositoryBusyException, RemoteException;

	/**
	 * Returns the status of the entity import. Result is one of the values
	 * listed in the <code>UploadStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>UPLOADING</code> mean that
	 * Results Repository still works on the entity upload. Values
	 * <code>ACCEPTED</code>, <code>REJECTED</code> and <code>ERROR</code>
	 * mean that Results Repository has finished its work (either successfully
	 * or unsuccessfully) and the import handle is ready to be closed by the
	 * <code>endImport</code> method.
	 * 
	 * @param handle
	 *            entity import handle
	 * @return upload status
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	UploadStatus getImportStatus(UploadHandle handle) throws RemoteException;

	/**
	 * Closes the import handle. Client should call this method after the import
	 * status returned by <code>getImportStatus</code> has value
	 * <code>ACCEPTED</code>, <code>REJECTED</code> or <code>ERROR</code>.
	 * 
	 * @param handle
	 *            import handle
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	void endImport(UploadHandle handle) throws RemoteException;

	/**
	 * Deletes an entity.
	 * 
	 * @param id
	 *            entity identifier of the entity to delete
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 * @throws DeleteLockedException
	 *             if entity is locked (scripts are running in it, results are
	 *             uploading, etc.)
	 */
	void deleteEntity(ID id) throws DeleteLockedException, RemoteException;

	/**
	 * Returns the graph count for given entity.
	 * 
	 * @param id
	 *            entity identifier
	 * @param graphType
	 *            Determines, whether we ask for results graphs or logs graphs
	 * @return graph count for given entity
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 */
	int graphCount(GraphType graphType, ID id) throws RemoteException;

	/**
	 * Returns information about given graph.
	 * 
	 * @param id
	 *            entity identifier
	 * @param graphType
	 *            Determines, whether we ask for results graphs or logs graphs
	 * @param index
	 *            index of the graph whose title is retrieved
	 * @return graph title
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 * @throws ResultsRepositoryException
	 *             in case of Results Repository error
	 */
	GraphInfo getGraphInfo(GraphType graphType, ID id, int index)
			throws RemoteException, ResultsRepositoryException;

	/**
	 * Downloads a graph form the Results Repository. The client must open the
	 * socket on some port before calling this function. Then it calls the
	 * <code>beginGraphDownload</code> function.
	 * 
	 * This function works asynchronously and returns immediately. Before return
	 * it starts a new thread that will send the graph to the socket open on
	 * given host and port.
	 * 
	 * If there is any error when sending the graph, the socket is closed. It's
	 * client's responsibility to check (via the
	 * <code>getGraphDownloadStatus</code> method whether the data arrived
	 * complete and undamaged.
	 * 
	 * After the download finishes, client should close the returned handle via
	 * the <code>endGraphDownload</code> method.
	 * 
	 * @param graphType
	 *            Determines, whether we ask for results graphs or logs graphs
	 * @param id
	 *            entity identifier of the entity whose graph should be
	 *            downloaded
	 * @param ip
	 *            IP address of the machine waiting for receiving the data (not
	 *            necessairly the caller itself)
	 * @param port
	 *            port open for receiving the data
	 * @param index
	 *            index of the downladed graph
	 * @return graph download handle
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws IndexOutOfBoundsException
	 *             if the graph index is invalid
	 * @throws ResultsRepositoryBusyException
	 *             if the Results Repository is currently busy - caller should
	 *             retry after a while or abort the import
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 * @throws ResultsRepositoryException
	 *             in case of Results Repository error
	 */
	DownloadHandle beginGraphDownload(GraphType graphType, ID id,
			InetAddress ip, int port, int index)
			throws ResultsRepositoryBusyException, RemoteException,
			ResultsRepositoryException;

	/**
	 * Returns the status of the graph download. Result is one of the values
	 * listed in the <code>FileTransferStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>DOWNLOADING</code> mean
	 * that Results Repository still works on the graph download. Values
	 * <code>SUCCEEDED</code> and <code>ERROR</code> mean that Results
	 * Repository has finished its work (either successfully or unsuccessfully)
	 * and the download handle is ready to be closed by the
	 * <code>endGraphDownload</code> method.
	 * 
	 * @param handle
	 *            graph download handle
	 * @return download status
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	DownloadStatus getGraphDownloadStatus(DownloadHandle handle)
			throws RemoteException;

	/**
	 * Closes the download handle. Client should call this method after the
	 * download status returned by <code>getGraphDownloadStatus</code> has
	 * value <code>SUCCEEDED</code> or <code>ERROR</code>.
	 * 
	 * @param handle
	 *            entity export handle
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	void endGraphDownload(DownloadHandle handle) throws RemoteException;

	/**
	 * Downloads a raw data form the Results Repository. The client must open
	 * the socket on some port before calling this function. Then it calls the
	 * <code>beginRawDataDownload</code> function.
	 * 
	 * This function works asynchronously and returns immediately. Before return
	 * it starts a new thread that will send the raw data to the socket open on
	 * given host and port.
	 * 
	 * If there is any error when sending the raw data, the socket is closed.
	 * It's client's responsibility to check (via the
	 * <code>getRawDataDownloadStatus</code> method whether the data arrived
	 * complete and undamaged.
	 * 
	 * After the download finishes, client should close the returned handle via
	 * the <code>endRawDataDownload</code> method.
	 * 
	 * @param id
	 *            entity identifier of the entity whose raw data should be
	 *            downloaded
	 * @param ip
	 *            IP address of the machine waiting for receiving the data (not
	 *            necessairly the caller itself)
	 * @param port
	 *            port open for receiving the data
	 * @return raw data download handle
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws IllegalArgumentException
	 *             if the entity identified by the entity identifier doesn't
	 *             exist
	 * @throws ResultsRepositoryBusyException
	 *             if the Results Repository is currently busy - caller should
	 *             retry after a while or abort the download
	 * @throws RemoteException
	 *             if something in RMI goes bad
	 * @throws ResultsRepositoryException
	 */
	DownloadHandle beginRawDataDownload(ID id, InetAddress ip, int port)
			throws ResultsRepositoryBusyException, RemoteException,
			ResultsRepositoryException;

	/**
	 * Returns the status of the raw data download. Result is one of the values
	 * listed in the <code>FileTransferStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>DOWNLOADING</code> mean
	 * that Results Repository still works on the raw data download. Values
	 * <code>SUCCEEDED</code> and <code>ERROR</code> mean that Results
	 * Repository has finished its work (either successfully or unsuccessfully)
	 * and the download handle is ready to be closed by the
	 * <code>endRawDataDownload</code> method.
	 * 
	 * @param handle
	 *            raw data download handle
	 * @return download status
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	DownloadStatus getRawDataDownloadStatus(DownloadHandle handle)
			throws RemoteException;

	/**
	 * Closes the download handle. Client should call this method after the
	 * download status returned by <code>getRawDataDownloadStatus</code> has
	 * value <code>SUCCEEDED</code> or <code>ERROR</code>.
	 * 
	 * @param handle
	 *            entity export handle
	 * @throws UnsupportedOperationException
	 *             if this operation isn't supported by encapsulated entity type
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	void endRawDataDownload(DownloadHandle handle) throws RemoteException;

	/**
	 * Return context in which tasks used to generate data for given entity were
	 * run.
	 * 
	 * @param id
	 *            entity identifier
	 * @return context in which entity's tasks were run
	 * @throws RemoteException
	 *             when something in RMI goes bad
	 */
	String getContext(ID id) throws RemoteException;

	/**
	 * Return identifiers of tasks which were run in order to generate data for
	 * given entity.
	 * 
	 * @param id
	 *            entity identifier
	 * @return identifiers of tasks
	 * @throws RemoteException
	 */
	String[] getTasks(ID id) throws RemoteException;

	/**
	 * Gets all log messages produced by a task, which was run in order to
	 * generate data for given entity.
	 * 
	 * @param id
	 *            entity identifier; cannot be null
	 * @param taskID
	 *            task's ID; cannot be null or an empty string
	 * @return logs of the task
	 * @throws RemoteException
	 *             if something failed during the execution of the remote method
	 *             call
	 * @throws LogStorageException
	 *             if an error occured while retrieving the logs
	 * @throws IllegalArgumentException
	 *             if the the context name or task's ID are empty strings or if
	 *             the task wasn't run
	 * @throws NullPointerException
	 *             if any argument is <code>null</code>
	 */
	LogRecord[] getLogsForTask(ID id, String taskID) throws RemoteException,
			LogStorageException, IllegalArgumentException, NullPointerException;
}
