/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.resultsrepository.gui;

import java.net.InetAddress;
import java.rmi.Remote;
import java.rmi.RemoteException;

import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.id.AID;

/**
 * RMI interface to the Results Repository, intended for the GUI (web interface).
 *  
 * @author David Majda
 */
public interface ResultsRepositoryInterface extends Remote {
	/** RMI Name for RMI registration */
	String RMI_NAME = "gui";

	/** Size of buffer used when exporting results in the GUI. */
	int EXPORT_BUFFER_SIZE = 4096; 
	/** Size of buffer used when importing results in the GUI. */
	int IMPORT_BUFFER_SIZE = 4096;
	/** Size of buffer used when downloading graphs for the GUI. */
	int GRAPH_BUFFER_SIZE = 4096;
	/** Size of buffer used when downloading raw data in the GUI. */
	int RAW_DATA_BUFFER_SIZE = 4096;
	/** Size of buffer used when uploading R script packages in the GUI. */
	int UPLOAD_BUFFER_SIZE = 4096; 

	/** MIME type of graphs sent to the web interface. */
	String GRAPH_MIME_TYPE = "image/png";
		
	/** MIME type of thw raw data sent to the web interface. */
	String RAW_DATA_MIME_TYPE = "application/zip";

	/**
	 * Returns the interface for manipulation with analyses.
	 * 
	 * @return interface for manipulation with analyses
	 * @throws RemoteException when something in RMI goes bad
	 */
	EntityInterface<AID> getAnalysisInterface() throws RemoteException;

	/**
	 * Uploads a R script package to the ResultsRepository. The client must open
	 * the socket on some port before calling this function. Then it calls the
	 * <code>beginRScriptPackageUpload</code> function.
	 * 
	 * This function work asynchronously and returns immediately. Before return it
	 * starts a new thread that will receive the package contents from the socket
	 * open on given host and port.
	 * 
	 * If there is any error when sending the package contents, the socket
	 * is closed. 
	 * 
	 * Calling this function doesn't guarantee that this package will be added
	 * to the Results Repository - if it arrives damaged, it is invalid,
	 * or some error during processing occurs, Results Repository rejects
	 * the package. Client has an opportunity to check (via the
	 * <code>getRScriptPackageUploadStatus</code> method whether the package
	 * arrived complete and undamaged.
	 *  
	 * After the upload finishes, client should close the returned handle
	 * via the <code>endRScriptPackageUpload</code> method.  
	 *
	 * @param name R script package name (an unique identifier of the package
	 *         in the Results Repository) 
	 * @param ip IP address of the machine prepared for sending the packcage
	 *         (not necessairly the caller itself)
	 * @param port port open for sending the package contents
	 * @return Handle to use when checking status of the upload
	 * @throws IllegalArgumentException if the package with given name is already
	 *          present in Results Repository
	 * @throws RemoteException when something in RMI goes bad
	 */
	UploadHandle beginRScriptPackageUpload(String name, InetAddress ip, int port)
		throws RemoteException;
	
	/**
	 * Returns the status of the R script package upload. Result is one
	 * of the values listed in the <code>UploadStatus</code> enumeration.
	 * 
	 * Values <code>INITIALIZING</code> and <code>UPLOADING</code> mean that
	 * Results Repository still works on the package upload. Values
	 * <code>ACCEPTED</code>, <code>REJECTED</code> and <code>ERROR</code>
	 * mean that Results Repository has finished its work (either successfully
	 * or unsuccessfully) and the download handle is ready to be closed by the
	 * <code>endRScriptsPackageUpload</code> method.   
	 * 
	 * @param handle package upload handle
	 * @return upload status
	 * @throws RemoteException when something in RMI goes bad
	 */
	UploadStatus getRScriptPackageUploadStatus(UploadHandle handle)
		throws RemoteException;
	
	/**
	 * Returns the error messages associated with rejected R scripts package
	 * upload. These messages contain human-readable messages describing the
	 * reason of the rejection and are suitable for displaying to the user.
	 * 
	 * @param handle package upload handle
	 * @return error messages associated with rejected package upload
	 * @throws IllegalStateException if the package upload status is different
	 *          from <code>REJECTED</code>
	 * @throws RemoteException when something in RMI goes bad
	 */
	String[] getRejectedRScriptPackageUploadErrorMessages(UploadHandle handle)
		throws RemoteException;

	/**
	 * Closes the upload handle. Client should call this method after the upload
	 * status returned by <code>getRScriptPackageUploadStatus</code> has value
	 * <code>ACCEPTED</code>, <code>REJECTED</code> or <code>ERROR</code>.
	 * 
	 * If the client doesn't close the upload handle, Results Repository will leak
	 * a little amount of memory. However, no other resources than memory
	 * (such as open files, locks...) are associated with the upload handle. 
	 * 
	 * @param handle upload handle
	 * @throws RemoteException when something in RMI goes bad
	 */
	void endRScriptPackageUpload(UploadHandle handle) throws RemoteException;
	
	/**
	 * Deletes the R script package form the Results Repository.
	 * 
	 * @param name name of the deleted R scriptspackage
	 * @return <code>true</code> if the deletion was successfull;
	 *          <code>false</code> otherwise
	 * @throws RemoteException when something in RMI goes bad
	 */
	boolean deleteRScriptPackage(String name) throws RemoteException;
	
	/**
	 * Lists all R script packages installed in the Results Repository.
	 * 
	 * @return array of <code>RScriptPackages</code> objects describing the
	 *          R script packages installed in the Results Repository, sorted
	 *          alphabetically by package name
	 * @throws RemoteException when something in RMI goes bad
	 */
	RScriptPackage[] getRScriptPackages() throws RemoteException;
	
	/**
	 * Retrieves callback R scripts for given analysis.
	 * 
	 * @param aid analysis identifier
	 * @return object containing texts of the scripts
	 * @throws IllegalArgumentException if the analysis does not exist
	 * @throws RemoteException when something in RMI goes bad
	 */
	CallbackRScripts getAnalysisCallbackRScripts(AID aid) throws RemoteException;

	/**
	 * Sets callback R scripts for given analysis.
	 * 
	 * @param aid analysis identifier
	 * @param callbackRScripts object containing texts of the scripts
	 * @throws IllegalArgumentException if the analysis does not exist
	 * @throws RemoteException when something in RMI goes bad
	 */
	void setAnalysisCallbackRScripts(AID aid, CallbackRScripts callbackRScripts)
		throws RemoteException;

	/**
	 * Retrieves help for all R functions.
	 * 
	 * @return array of <code>RFunctionHelp</code> objects containing help for
	 *          the functions; sorted alphabetically by function name
	 * @throws RemoteException when something in RMI goes bad
	 */
	RFunctionHelp[] getRFunctionsHelp() throws RemoteException;
	
 	/**
	 * Informs whether R scripts execution is enabled. It can be turned off
	 * in the case something went wrong during the initialization of Results
	 * Repository (R not installed, RNetCDF or SJava libraries not installed, etc.)
	 *
	 * @return True if R scripts execution is enabled
	 *
	 * @throws RemoteException when something in RMI goes bad
	 */
	boolean rScriptingEnabled() throws RemoteException;
}
