/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.softwarerepository;

import java.io.Serializable;

/**
 * Enum representing the package type (source, binary, task or data).
 * 
 * @author David Majda
 */
public enum PackageType implements Serializable {
	/** Source package type. */
	SOURCE,
	/** Binary package type. */
	BINARY,
	/** Task package type. */
	TASK,
	/** Data package type. */
	DATA;

	/** All enum values. */
	private static final PackageType[] VALUES = {
		SOURCE,
		BINARY,
		TASK,
		DATA
	};
	
	/** String representation of the package types. */
	private static final String[] STRINGS = {
		"source",
		"binary",
		"task",
		"data"
	};
	
	/** Suffixes of the package types. */
	private static final String[] SUFFIXES = {
		"src",
		"bin",
		"task",
		"data"
	};
	
	/**
	 * Returns a string representation of the object.
	 * 
	 * @return a string representation of the object
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return STRINGS[ordinal()];
	}
	
	/**
	 * Return a <code>PackageType</code> object corresponding to given string. The
	 * passed string is not the name of the enum constant, but rather its
	 * <code>toString</code> value.
	 * 
	 * This method should be really named <code>valueOf</code>, but Java is not
	 * flexible enough to not allow it, as it generates implicit
	 * <code>valueOf</code> method for each enum class. 
	 * 
	 * @param name string representing a <code>PackageType</code> enum constant
	 * @return <code>PackageType</code> object corresponding to given string
	 * @throws NullPointerException if <code>name</code> is <code>null</code>
	 * @throws IllegalArgumentException if no enum constant corresponding to
	 *          <code>name</code> exists
	 */
	public static PackageType realValueOf(String name) {
    if (name == null) {
      throw new NullPointerException("Name is null");
    }
		for (int i = 0; i < VALUES.length; i++) {
			if (name.equals(STRINGS[i])) {
				return VALUES[i];
			}
		}
		throw new IllegalArgumentException("No enum value named \"" + name
			+ "\" exists.");
	}
	
	/**
	 * Returns a package suffix for this package type.
	 * 
	 * @return packge suffix
	 */
	public String getSuffix() {
		return SUFFIXES[ordinal()];
	}
}
