/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task;

import java.io.Serializable;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import cz.cuni.mff.been.hostruntime.TasksPortInterface;
import cz.cuni.mff.been.logging.HostRuntimeAppender;
import cz.cuni.mff.been.logging.LogLevel;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;


/**
 * Class representing a Task, which is the entity executed by BEEN
 * 
 * @author Michal Tomcanyi
 */
public abstract class Task {
	
	/**
	 * Default log level.
	 */
	public static final LogLevel DEFAULT_LOG_LEVEL = LogLevel.INFO;
	
    // ----------------------- CONSTANTS -------------------------------
	/** Property name: directory with task package's content */
	public static final String PROP_DIR_TASK = "hostruntime.directory.task";
	/** Property name: working directory of a task, it IS NOT deleted immediately after task's end*/
	public static final String PROP_DIR_WORK = "hostruntime.directory.working";
	/** Property name: temporary directory of a task, it IS deleted immediately after task's end */
    public static final String PROP_DIR_TEMP = "hostruntime.directory.temporary";
    /** Logging format */
    public static final String STDOUT_LOG_FORMAT = "%d %p [%c{1}]  %m";
    /** Task exit code: success */
    public static final int EXIT_CODE_SUCCESS = 0;
    /** Task exit code: error */
    public static final int EXIT_CODE_ERROR = 1;
    
	/** Name of the checkpoint used to signal task start. */
	public static final String CHECKPOINT_NAME_STARTED = "task started";
	/** Name of the checkpoint used to signal task finish. */
	public static final String CHECKPOINT_NAME_FINISHED = "task finished";
    
    // ------------------------ ATTRIBUTES -------------------------------
    /** Static reference to the current running task.  You can use it e.g. for logging. */
    private static Task taskHandle = null;
	/** Task monitoring and logging facility */
	private TasksPortInterface tasksPort = null;
	/** Task descriptor of this task. */
	private TaskDescriptor taskDescriptor = null;
	/** Directory specified by Host Runtime which contains the task package's content */
	private String taskDirectory = System.getProperty(Task.PROP_DIR_TASK);
	/** Working directory specified by Host Runtime for this task, it IS NOT deleted
	 * immediately after task's end
	 */
	private String workingDirectory = System.getProperty(Task.PROP_DIR_WORK);
	/** Temporary directory for this task specified by Host Runtime, it IS deleted
	 * immediately after task's end
	 */
	private String tempDirectory = System.getProperty(Task.PROP_DIR_TEMP);
	
	/** Task properties that can be used instead of command line arguments */
	private Properties taskProperties;
	/** Task properties with objects as values. */
	private Map<String, Serializable> taskPropertyObjects;
	
	private Logger logger = Logger.getLogger("task." + this.getClass().getCanonicalName());

	protected Task() throws TaskInitializationException {
		taskHandle = this;
	
		// obtain monitor for the task
		try {
			tasksPort = (TasksPortInterface) Naming.lookup(System.getProperty("hostruntime.tasksport.uri"));
			taskProperties = tasksPort.getTaskProperties();
			taskPropertyObjects = tasksPort.getTaskPropertyObjects();
			taskDescriptor = tasksPort.getTaskDescriptor();
		} catch (Exception e) {
			throw new TaskInitializationException(e);
		}
			
		// check if directories were correctly set
		if (tempDirectory == null) {
			throw new TaskInitializationException("Temporary directory not set (check property " + Task.PROP_DIR_TEMP + ")");
		}

		if (workingDirectory == null) {
			throw new TaskInitializationException("Working directory not set (check property " + Task.PROP_DIR_WORK + ")");
		}
		if (taskDirectory == null) {
			throw new TaskInitializationException("Task directory not set (check property " + Task.PROP_DIR_TASK + ")");
		}
		
		// set-up logging
		Logger taskRootLogger = Logger.getLogger("task");
		// set additivity to false, we don't want to log to stdout (the root logger has an appender
		// logging to stdout, which was assigned to it by the BeenLogger class)
		taskRootLogger.setAdditivity(false);
		taskRootLogger.addAppender(new HostRuntimeAppender(this));
		setLogLevel(DEFAULT_LOG_LEVEL);
		
		for (Object key : taskProperties.keySet()) {
			logDebug("Task property " + (String) key + "=" + taskProperties.getProperty((String) key)); 
		}
	}
	
	/**
	 * Return a reference to the task which is running this code
	 * @return task, null if the current code isn't executed in a task
	 */
	public static Task getTaskHandle() {
		return taskHandle;
	}
	
	/**
	 * Returns the tasksPort
	 * @return the tasksPort
	 */
	public TasksPortInterface getTasksPort() {
		return tasksPort;
	}
	
	
	/**
	 * Returns the directory  where task can store its results (permanently)
	 * @return directory name
	 */
	public String getWorkingDirectory() {
		return workingDirectory;
	}
	/**
	 * Returns the temporary working directory for the task
	 * @return directory name
	 */
	public String getTempDirectory() {
		return tempDirectory;
	}
	
	/**
	 * Returns the directroy with the task package's content
	 * @return directroy name
	 */
	public String getTaskDirectory() {
		return taskDirectory;
	}

	/**
	 * Sets the log level for logging to stdout
	 * @param level 
	 */
	public void setLogLevel(LogLevel level) {
		logger.setLevel(level.toLog4jLevel());
	}
	
	/**
	 * @return log level for logging to stdout
	 */
	public LogLevel getLogLevel() {
		return LogLevel.getInstance(logger.getLevel());
	}

	/**
	 * Logs a message on the DEBUG debug level
	 * @param message
	 */
	public void logDebug(String message) {
		logger.debug(message);
	}
	
	/**
	 * Logs a message on the ERROR debug level
	 * @param message
	 */
	public void logError(String message) {
		logger.error(message);
	}

	/**
	 * Logs a message on the FATAL debug level
	 * @param message
	 */
	public void logFatal(String message) {
		logger.fatal(message);
	}

	/**
	 * Logs a message on the INFO debug level
	 * @param message
	 */
	public void logInfo(String message) {
		logger.info(message);
	}
	
	/**
	 * Logs a message on the TRACE debug level
	 * @param message
	 */
	public void logTrace(String message) {
		logger.trace(message);
	}

	/**
	 * Logs a message on the WARN debug level
	 * @param message
	 */
	public void logWarning(String message) {
		logger.warn(message);
	}

	/**
	 * Signals that a checkpoint was reached by the task.
	 * 
	 * @param name name of the checkpoint
	 * @param value value of the checkpoint
	 * @throws TaskException if anything goes wrong.
	 */
	public void checkPointReached(String name, Serializable value) throws TaskException {
		try {
			tasksPort.checkPointReached(name, value);
		} catch (RemoteException e) {
			throw new TaskException(e);
		}
	}
	
	/**
	 * Waits until a task reaches a checkpoint of specified type and returns its
	 * value.
	 *        
	 * @param contextID context ID, if null then the context of the calling task
	 * will be used.
	 * @param taskID task ID
	 * @param name checkpoint name
	 * @param timeout number of miliseconds to wait for a checkpoint;
	 *         <code>0</code> means that the call will not block and returns
	 *         immediately
	 * @return value of the checkpoint.
	 * @throws TaskException if anything goes wrong.
	 */ 
	public Serializable checkPointWait(String contextID, 
			String taskID, 
			String name, 
			long timeout)
	throws TaskException {
		try {
			return tasksPort.checkPointWait(contextID, taskID, name, timeout);
		} catch (RemoteException e) {
			throw new TaskException(e);
		}
	}
	
	/**
	 * Returns the value of a task's property
	 * 
	 * @param name name of the task's property
	 * @return value of the task's property or <code>null</code>
	 * if the specified property is not found
	 */
	public String getTaskProperty(String name) {
		return taskProperties.getProperty(name);
	}
	
	/**
	 * Returns the value of a task property that can have boolean
	 * values, e.g. "yes", "true" etc.
	 * @param name name of the task property.
	 * @return true if the value of the task property is was "yes" or "true".
	 */
	public boolean getBooleanTaskProperty(String name) {
		String value = getTaskProperty(name);
		return (!(value == null) 
				&& (
						(value.equals("yes"))
						|| (value.equals("true"))
					)
				);
	}
	
	/**
	 * 
	 * @param name name of the task property.
	 * @return the object value of the task property.
	 */
	public Serializable getTaskPropertyObject(String name) {
		return taskPropertyObjects.get(name);
	}

	/**
	 * Exits the task which succesfully finished.
	 *
	 */
	public static void exitSuccess() {
		System.exit(EXIT_CODE_SUCCESS);
	}
	
	/**
	 * Exits the task that finished with an error.
	 *
	 */
	public static void exitError() {
		System.exit(EXIT_CODE_ERROR);
	}

	/**
	 * @return the task descriptor of this task.
	 */
	public TaskDescriptor getTaskDescriptor() {
		return this.taskDescriptor;
	}
	
	/**
	 * 
	 * @return true if the task is running in Windows.
	 */
	public boolean isRunningInWindows() {
		return System.getProperty("os.name").toLowerCase().indexOf("windows") != -1;
	}
	
	/**
	 * 
	 * @return true if the task is running in Linux.
	 */
	public boolean isRunningInLinux() {
		return System.getProperty("os.name").toLowerCase().indexOf("linux") != -1;
	}
}
