/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.download;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import cz.cuni.mff.been.common.DownloadHandle;
import cz.cuni.mff.been.common.DownloadStatus;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Chmod;
import cz.cuni.mff.been.common.anttasks.Move;
import cz.cuni.mff.been.common.anttasks.Unzip;
import cz.cuni.mff.been.common.rsl.Condition;
import cz.cuni.mff.been.softwarerepository.MatchException;
import cz.cuni.mff.been.softwarerepository.PackageMetadata;
import cz.cuni.mff.been.softwarerepository.RSLPackageQueryCallback;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * Downloads a package from the Software Repository.
 * 
 * @author Michal Tomcanyi
 */
public class LocalDownload extends Job {

	private static final String RSL_PROPERTY = "rsl";
	
	// ------------- CONSTANTS ---------------
	private static final long serialVersionUID = -8627630020974278481L;

	// ------------- ATTRIBUTES --------------
	private SoftwareRepositoryInterface swRepo;

	private ServerSocketChannel serverSocketChan;

	private FileChannel fileChannel;

	private String outputFileName;

	private File outputFile;

	private DownloadThread downloadThread;

	/**
	 * 
	 * Allocates a new <code>LocalDownload</code> object.
	 * 
	 * @throws TaskInitializationException
	 */
	public LocalDownload() throws TaskInitializationException {
		super();
	}

	/**
	 * Locates and queries Software Repository, sets up socket and prepares
	 * download thread
	 * 
	 * @throws IOException
	 * @throws NotBoundException
	 */
	private void setup() 
	throws IOException, NotBoundException, MatchException, TaskInitializationException  {
		swRepo = (SoftwareRepositoryInterface) getTasksPort().serviceFind(
				"softwarerepository", "main");
		if (swRepo == null) {
			throw new TaskInitializationException("Software repository not available");
		}
		
		Condition rslQuery = (Condition) getTaskPropertyObject(RSL_PROPERTY);
		logInfo("RSL query for the downloaded package: " + rslQuery);
		
		RSLPackageQueryCallback rslQueryIface = new RSLPackageQueryCallback(rslQuery);
		PackageMetadata[] pkgMeta = null;
		pkgMeta = swRepo.queryPackages(rslQueryIface);

		// no match - nothing to do
		if (pkgMeta.length == 0) {
			throw new MatchException("No matching package found for given criteria");
		}

		outputFileName = pkgMeta[0].getFilename();

		// setup socket
		serverSocketChan = ServerSocketChannel.open();
		ServerSocket serverSocket = serverSocketChan.socket();
		serverSocket.bind(null);

		outputFile = new File(getTempDirectory(), outputFileName);
		fileChannel = new FileOutputStream(outputFile).getChannel();
		downloadThread = new DownloadThread(serverSocketChan, fileChannel);
	}

	private void download() throws TaskException, IOException {
		// start download thread and give it time to initialize
		downloadThread.start();
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			// ignore
		}

		logInfo("Beginning package download");
		logDebug("Downloading file: " + outputFileName);

		// inform repository to start downloading
		DownloadHandle downHandle = swRepo.beginPackageDownload(outputFileName,
				InetAddress.getLocalHost(), serverSocketChan.socket()
						.getLocalPort());

		// monitor status
		DownloadStatus status = DownloadStatus.ERROR;
		while (true) {
			status = swRepo.getPackageDownloadStatus(downHandle);
			if (status == DownloadStatus.DOWNLOADING
					|| status == DownloadStatus.INITIALIZING) {
				// still working
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					// nothing bad happened
					Thread.currentThread().interrupt();
				}
			} else {
				// finished
				break;
			}
		}

		// evaluate final download status
		if (status == DownloadStatus.ERROR) {
			// finished with error - kill downloader, delete file
			if (downloadThread.isAlive()) {
				downloadThread.interrupt();
			}
			fileChannel.close();
			new File(outputFileName).delete();
			throw new TaskException("Package download failed");
		} else {
			// repository reported finished work - check local downloading
			// thread
			if (downloadThread.isAlive()) {
				int i = 0;
				for (i = 0; i < 10; i++) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						// nothing bad heppened
						Thread.currentThread().interrupt();
					}
					if (!downloadThread.isAlive()) {
						fileChannel.close();
						break;
					}
				}
				if (i >= 10) {
					// download thread is out of time
					downloadThread.interrupt();
					fileChannel.close();
					new File(outputFileName).delete();
					throw new TaskException(
							"Package download failed (but SoftwareRepository reports OK!)");
				}
			}
		}

		if (downloadThread.getStoredException() != null) {
			throw new TaskException("Download failed", downloadThread
					.getStoredException());
		}
		downloadThread = null;
		logInfo("Download finished");
	}

	private void finishDownload() {
		swRepo = null;
		try {
			if (serverSocketChan != null) {
				serverSocketChan.close();
			}
			serverSocketChan = null;
		} catch (IOException e) {
			logWarning("Socket could not be closed");
		}
	}

	private void unpack() throws AntTaskException {
		String archiveName = outputFile.getAbsolutePath();
		logDebug("Unpacking archive: " + archiveName);

		Unzip.unzip(archiveName, getTempDirectory());

		String sourceDir = getTempDirectory() + File.separatorChar + "files";
		Move.filesetMove(sourceDir, "**", getWorkingDirectory());
		
		Chmod.recursiveDirectoryChmod(getWorkingDirectory(), "u+rwx");
	}

	private void finish() throws RemoteException {
		getTasksPort().checkPointReached("finish", "1");
		logInfo("Finished sucesfully");
	}

	@Override
	protected void checkRequiredProperties() throws TaskException {
	}
	
	@Override
	protected void run() throws TaskException {
		try {
			if (getTaskPropertyObject(RSL_PROPERTY) == null) {
				throw new TaskException(RSL_PROPERTY + " task property not set");
			}
			
			setup();
			download();
			unpack();
			finish();
		} catch (Exception e) {
			throw new TaskException("Download failed", e);
		} finally {
			finishDownload();
		}
	}
}
