/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.edit.regexsubstitute;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * Subtitutes all matches of a regular expression in a file  with a string.
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #FILE}: <br>
 * 		- file to edit <br>
 * 		- mandatory <br>
 * <li>{@value #REGEX}: <br>
 * 		- regular expression <br>
 * 		- mandatory <br>
 * <li>{@value #SUBSTITUTION}: <br>
 * 		- substitution of the regular expression matches <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class RegexSubstitute extends Job {
	//--------------CONSTANTS---------------
	/** Task property name for the path to the edited file */
	public static final String FILE = "file";
	/** Task property  name for the matched regular expression */
	public static final String REGEX = "regex";
	/** Task property  name for the substitution */
	public static final String SUBSTITUTION = "substitution";
	
	//--------------ATTRIBUTES--------------
	/** The edited file */
	private File file;
	/** The matched regular expression */
	private String regex;
	/** Substitution of the matched string */
	private String substitution;
	
	/**
	 * 
	 * Allocates a new <code>RegexSubstitute</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public RegexSubstitute() throws TaskInitializationException {
		super();
	}
	
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		String path = getTaskProperty(FILE);
		file = new File(path);
		logInfo("File: " + path);
		
		regex = getTaskProperty(REGEX);
		logInfo("Regular expression: " + regex);
		
		substitution = getTaskProperty(SUBSTITUTION);
		logInfo("Substitution: " + substitution);

		StringBuffer contents = new StringBuffer();
		
		/* Read the original contents of the file */
		BufferedReader input = null;
		try {
			String line = null;
			input = new BufferedReader(new FileReader(file));
			
			while ((line = input.readLine()) != null) {
				contents.append(line + "\n");
			}

			if (input != null) {
				input.close();
			}
		} catch (Exception e) {
			throw new TaskException("Cannot read file: " + path, e);
		}
		
		/* Create the new contents of the file, i.e. do the substitution */
		String newContents = null;
		try {
			newContents = (contents.toString()).replaceAll(regex, substitution);
		} catch (Exception e) {
			throw new TaskException("Cannot do substitution", e);
		}
		
		/* Overwrite the file with new contents */
		BufferedWriter output = null;
		try {
			output = new BufferedWriter(new FileWriter(file));
			output.write(newContents.toString());

			if (output != null) {
				output.close();
			}
		} catch (Exception e) {
			throw new TaskException("Cannot save modified file: " + path, e);
		}
		
		checkPointReached("finish",	null);
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{FILE,REGEX,SUBSTITUTION});
	}
}
