/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jboss.configure;

import java.io.File;

import cz.cuni.mff.been.common.RegexSubstitute;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Configures various settings of JBoss. It must be run this task before starting
 * JBoss. It edits configuration files of the default server. It supports MySQL 
 * datasource configuration only, and it copies the MySQL JDBC connector to
 * the JBoss deployment directory.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JBOSS_ROOT}: <br>
 * 		- path to the JBoss installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #JBOSS_JNP_PORT}: <br>
 * 		- port the the JNP service <br>
 * <li>{@value #JBOSS_WEBSERVICE_PORT}: <br>
 * 		- port the the web service service <br>
 * <li>{@value #JAWS_DATASOURCE}: <br>
 * 		- datasource name for JAWS <br>
 * <li>{@value #JAWS_TYPEMAPPING}: <br>
 * 		- datasource type mapping for JAWS <br>
 * <li>{@value #CMP_JDBC_DATASOURCE}: <br>
 * 		- datasource name for CMP JDBC <br>
 * <li>{@value #CMP_JDBC_TYPEMAPPING}: <br>
 * 		- datasource type mapping for CMP JDBC <br>
 * <li>{@value #NAMING_CALL_BY_VALUE}: <br>
 * 		- value of the CallByValue option of the JBoss' naming service <br>
 * <li>{@value #USE_JBOSS_WEB_LOADER}: <br>
 * 		- value of the UseJBossWebLoader attribute of the JBoss' naming service <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JBossConfigure extends Job {
	/** Task property name for the parh to the JBoss installation directory. */
	public static final String JBOSS_ROOT = "jboss.root";
	/** Task property name for the port of the JNP service. */
	public static final String JBOSS_JNP_PORT = "jboss.jnp.port";
	/** Task property name for the port of the webservice. */
	public static final String JBOSS_WEBSERVICE_PORT = "jboss.webservice.port";
	/** Task property name for the datasource name for JAWS. */
	public static final String JAWS_DATASOURCE = "jaws.datasource";
	/** Task property name for the type mapping for JAWS. */
	public static final String JAWS_TYPEMAPPING = "jaws.typemapping";
	/** Task property name for the datasource name for JDBC. */
	public static final String CMP_JDBC_DATASOURCE = "cmp-jdbc.datasouce";
	/** Task property name for the type mapping for JAWS. */
	public static final String CMP_JDBC_TYPEMAPPING = "cmp-jdbc.typemapping";
	/** Task property name for the CallByValue option of JBoss' naming service. */
	public static final String NAMING_CALL_BY_VALUE = "naming.call-by-value";
	/** 
	 * Task property name for the UseJBossWebLoader attribute in the 
	 * "deploy/jbossweb-tomcat55.sar/META-INF/jboss-service.xml"
	 * file.
	 */
	public static final String USE_JBOSS_WEB_LOADER = "use.jboss.web.loader";

	/** Path to the installation directory of JBoss. */
	private String jbossRoot;
	
	/**
	 * 
	 * Allocates a new <code>JBossConfigure</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JBossConfigure() throws TaskInitializationException {
		super();
	}


	@Override
	protected void run() throws TaskException {
		
		jbossRoot = getTaskProperty(JBOSS_ROOT);
		logInfo("JBoss root: " + jbossRoot);

		
		String fs = File.separator;
		
		try {
			String jawsDatasource = getTaskProperty(JAWS_DATASOURCE);
			if (jawsDatasource != null) {
				String path = jbossRoot + fs + "server" + fs + "default" 
				+ fs + "conf" + fs + "standardjaws.xml";
				logInfo("Setting jaws datasource to \"" + jawsDatasource + "\" in file " 
						+ path);
				RegexSubstitute.substitute(path, 
						"<datasource>java:/DefaultDS</datasource>", 
						"<datasource>" + jawsDatasource + "</datasource>");
			}
			
			String jawsTypeMapping = getTaskProperty(JAWS_TYPEMAPPING);
			if (jawsTypeMapping != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "conf" + fs + "standardjaws.xml";
				logInfo("Setting jaws type mapping tp \"" + jawsTypeMapping + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"<type-mapping>Hypersonic SQL</type-mapping>", 
						"<type-mapping>" + jawsTypeMapping + "</type-mapping>");
			}
			
			String cmpJdbcDatasource = getTaskProperty(CMP_JDBC_DATASOURCE);
			if (cmpJdbcDatasource != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "conf" + fs + "standardjbosscmp-jdbc.xml";
				logInfo("Setting jdbc datasource to \"" + cmpJdbcDatasource + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"<datasource>java:/DefaultDS</datasource>", 
						"<datasource>" + cmpJdbcDatasource + "</datasource>");
			}
			
			String cmpJdbcTypeMapping = getTaskProperty(CMP_JDBC_TYPEMAPPING);
			if (cmpJdbcTypeMapping != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "conf" + fs + "standardjbosscmp-jdbc.xml";
				logInfo("Setting jdbc type mapping to \"" + cmpJdbcTypeMapping + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"<!-- optional since 4.0 <datasource-mapping>Hypersonic SQL</datasource-mapping> -->", 
						"<datasource-mapping>" + cmpJdbcTypeMapping + "</datasource-mapping>");
			}
			
			String jnpPort = getTaskProperty(JBOSS_JNP_PORT);
			if (jnpPort != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "conf" + fs + "jboss-service.xml";
				logInfo("Setting JNP port to \"" + jnpPort + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"1099", 
						jnpPort);
			}

			String webServicePort = getTaskProperty(JBOSS_WEBSERVICE_PORT);
			if (webServicePort != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "conf" + fs + "jboss-service.xml";
				logInfo("Setting webservice port to \"" + webServicePort + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"8083", 
						webServicePort);
			}

			String callByValue = getTaskProperty(NAMING_CALL_BY_VALUE);
			if (callByValue != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "conf" + fs + "jboss-service.xml";
				logInfo("Setting Naming CallByValue attribute to \"" + callByValue + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"<attribute name=\"CallByValue\">false</attribute>", 
						"<attribute name=\"CallByValue\">" + callByValue + "</attribute>");
			}
			
			String useJBossWebLoader = getTaskProperty(USE_JBOSS_WEB_LOADER);
			if (useJBossWebLoader != null) {
				String path = jbossRoot + fs + "server" + fs + "default"
				+ fs + "deploy" + fs + "jbossweb-tomcat55.sar" + fs + "META-INF" + fs
				+ "jboss-service.xml";
				logInfo("Setting UseJBossWebLoader attribute to \"" + useJBossWebLoader + "\" in file "
						+ path);
				RegexSubstitute.substitute(path, 
						"<attribute name=\"UseJBossWebLoader\">false</attribute>", 
						"<attribute name=\"UseJBossWebLoader\">" + useJBossWebLoader
						+ "</attribute>");
			}

		} catch (Exception e) {
			throw new TaskException("Cannot edit configuration file", e);
		}

		logInfo("Copying the MySQL JDBC connector");
		try {
			Copy.copyToDir(getTaskDirectory() + fs + "mysql-connector.jar", 
					jbossRoot + fs + "server" + fs + "default"
					+ fs + "lib");
		} catch (AntTaskException e) {
			throw new TaskException("Cannot copy the mysql jdbc connector "
					+ getTaskDirectory() + fs + "mysql-connector.jar"
					+ " to the directory " 
					+ jbossRoot + fs + "server" + fs + "default"
					+ fs + "lib",
					e);
		}
	}

	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(JBOSS_ROOT);
	}
	
}
