/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jonas.build;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Compiles Jonas from it's source. It requires a Tomcat binary for the 
 * compilation. </p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JONAS_DIR}: <br>
 * 		- path to the Jonas source code <br>
 * 		- mandatory <br>
 * <li>{@value #TOMCAT_DIR}: <br>
 * 		- path to the Tomcat installation directory <br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JonasBuild extends Job {
	/** Task property name for the Jonas installation directory. */
	public static final String JONAS_DIR = "jonas.dir";
	/** Task property name for the Tomcat installation directory. */
	public static final String TOMCAT_DIR = "tomcat.dir";
	
	/** Jonas installation directory. */
	private String jonasDir;
	/** Tomcat installation directory */
	private String tomcatDir;
	
	/**
	 * 
	 * Allocates a new <code>JonasBuild</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JonasBuild() throws TaskInitializationException {
		super();
	}

	@Override
	protected void run() throws TaskException {
		jonasDir = getTaskProperty(JONAS_DIR);
		logInfo("Jonas directory: " + jonasDir);
		
		tomcatDir = getTaskProperty(TOMCAT_DIR);
		logInfo("Tomcat directory: " + tomcatDir);

		// create the command to launch Ant
		ArrayList<String> params = new ArrayList<String>();
		
		if (isRunningInWindows()) {
			params.add("cmd");
			params.add("/C");
			params.add("call");
		}

		params.add("ant");
		
		// add the task directory to Ant's library search path so that
		// bcel.jar is added to the classpath for Jonas compilation
		params.add("-lib");
		params.add(getTaskDirectory());
		
		params.add("-Dcatalina.home\"=\"" + tomcatDir);
		params.add("-Djonas.tag\"=\"jonas");
		String[] cmdArray = params.toArray(new String[params.size()]);
		
		try {
			logInfo("Starting Ant");
			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));
			
			ProcessBuilder builder = new ProcessBuilder(cmdArray);
			Map<String, String> env = builder.environment();
			builder.directory(new File(jonasDir));
			Process p = builder.start();
			p.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new TaskException("Build failed");
			}

		} catch (TaskException e) {
			throw e;
		} catch (Exception e) {
			throw new TaskException("Cannot launch Ant", e);
		}
		
		logInfo("Build succesfull");
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{JONAS_DIR,TOMCAT_DIR});
	}
}
