/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jonas.configure;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import cz.cuni.mff.been.common.RegexSubstitute;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.common.anttasks.Delete;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Configures Jonas. It must be run before Jonas starts.
 * Supports the MySQL datasource only.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JONAS_ROOT}: <br>
 * 		- path to the Jonas installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #JRMP_PORT}: <br>
 * 		- port number of the Jonas' JRMP service <br>
 * <li>{@value #MYSQL_DATASOURCE_NAME}: <br>
 * 		- datasource name in the MySQL.properties file <br>
 * <li>{@value #MYSQL_DATASOURCE_USER}: <br>
 * 		- user name in the MySQL.properties file <br>
 * <li>{@value #MYSQL_DATASOURCE_URL}: <br>
 * 		- URL in the MySQL.properties file <br>
 * <li>{@value #MYSQL_MIN_CONNECTION_POOL}: <br>
 * 		- minimum connection pool size in the MySQL.properties file <br>
 * <li>{@value #MYSQL_MAX_CONNECTION_POOL}: <br>
 * 		- maximum connection pool size in the MySQL.properties file <br>
 * <li>{@value #MYSQL_MAX_WAITTIME}: <br>
 * 		- maximum wait time for threads that didn't fit into the connection pool.
 * 		It's set in the MySQL.properties file. <br>
 * <li>{@value #HTTP_PORT}: <br>
 * 		- port for the Jonas' webserver <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JonasConfigure extends Job {
	/** Task property name for the path to the root directory of Jonas. */
	public static final String JONAS_ROOT = "jonas.root";
	/**
	 * Task property name for the port number of Jonas JRMP service. It's not
	 * mandatory to set this property.
	 */
	public static final String JRMP_PORT = "jrmp.port";
	/**
	 * Task property name for the datasource name of the MySQL datasource.
	 */
	public static final String MYSQL_DATASOURCE_NAME = "mysql.datasource.name";
	/**
	 * Task property name for the datasource url of the MySQL datasource.
	 */
	public static final String MYSQL_DATASOURCE_URL = "mysql.datasource.url";
	/**
	 * Task property name for the datasource user name of the MySQL datasource.
	 */
	public static final String MYSQL_DATASOURCE_USER = "mysql.datasource.user";
	/**
	 * Task property name for the minimum connection pool size of the MySQL datasource.
	 */
	public static final String MYSQL_MIN_CONNECTION_POOL = "mysql.minconnpool";
	/**
	 * Task property name for the maximum connection pool size of the MySQL datasource.
	 */
	public static final String MYSQL_MAX_CONNECTION_POOL = "mysql.maxconnpool";
	/**
	 * Task property name for the max wait time for connections which
	 * didn't fit in the connection pool the MySQL datasource.
	 */
	public static final String MYSQL_MAX_WAITTIME = "mysql.maxwaittime";
	/**
	 * Task property name for the HTTP listening port.
	 */
	public static final String HTTP_PORT = "http.port";

	
	/** Property name in jonas.properties name for datasource */
	private static final String DATASOURCES_PROPERTY = "jonas.service.dbm.datasources";
	/** Property name in mysql.properties name for datasource name*/
	private static final String DATASOURCE_NAME_PROPERTY = "datasource.name";
	/** Property name in mysql.properties name for datasource url*/
	private static final String DATASOURCE_URL_PROPERTY = "datasource.url";
	/** Property name in mysql.properties name for maximum connection pool size*/
	private static final String MAX_CONNPOOL_PROPERTY = "jdbc.maxconpool";
	/** Property name in mysql.properties name for minimum connection pool size*/
	private static final String MIN_CONNPOOL_PROPERTY = "jdbc.minconpool";
	/** 
	 * Property name in mysql.properties name for maximum wait time for connections
	 * that didn't fit in the connection pool.
	 */
	private static final String MAX_CONN_WAIT_TIME = "jdbc.maxwaittime";
	/** Property name in mysql.properties name for datasource user name*/
	private static final String DATASOURCE_USERNAME_PROPERTY = "datasource.username";
	/** Property name in carol.properties name for RMI JRMP URL */
	private static final String JRMP_URL_PROPERTY = "carol.jrmp.url";


	/** Root directory of Jonas. */
	private String jonasRoot;
	/** Port number for the Jonas' JRNP service. */
	private int jrmpPort = 1097;

	/**
	 * 
	 * Allocates a new <code>JonasConfigure</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JonasConfigure() throws TaskInitializationException {
		super();
	}

	@Override
	protected void run() throws TaskException {
		jonasRoot = getTaskProperty(JONAS_ROOT);
		logInfo("Jonas root: " + jonasRoot);

		if (getTaskProperty(JRMP_PORT) != null) {
			jrmpPort = Integer.valueOf(getTaskProperty(JRMP_PORT));
			logInfo("JRMP port: " + jrmpPort);
		}

		/*
		 * Delete webapps/autoload/ctxroot.war , which causes that Jonas doesn't
		 * start
		 */
		try {
			Delete.deleteFile(jonasRoot + File.separator + "webapps"
					+ File.separator + "autoload" + File.separator
					+ "ctxroot.war");
		} catch (AntTaskException e) {
			throw new TaskException("Couldn't delete the " + jonasRoot 
					+ "webapps/autoload/ctxroot.war file", e);
		}

		logInfo("webapps/autoload/ctxroot.war file deleted");

		/*
		 * Copy the jdbc connector.
		 */

		try {
			String srcFile = getTaskDirectory() + File.separator + "mysql-connector.jar";
			String destDir = jonasRoot + File.separator + "lib" + File.separator + "ext";
			Copy.copyToDir(srcFile, destDir);
		} catch (AntTaskException e) {
			throw new TaskException("Couldn't copy the jdbc connector to the lib/ext directory", e);
		}

		logInfo("Jdbc connector copied to lib/ext directory");

		/*
		 * Edit configuration property files.
		 */

		Properties jonasProperties = new Properties();

		// load the jonas.properties file
		String path = jonasRoot + File.separator + "conf" + File.separator
		+ "jonas.properties";
		File jonasPropertiesFile = new File(path);
		try {
			FileInputStream is = new FileInputStream(jonasPropertiesFile);
			jonasProperties.load(is);
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while reading file: " + path, e);
		}

		// set the properties
		jonasProperties.setProperty(DATASOURCES_PROPERTY, "MySQL");

		// save to the jonas.properties file
		try {
			FileOutputStream os = new FileOutputStream(jonasPropertiesFile);
			jonasProperties.store(os, "This file is generated by BEEN");
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while writing to file: " + path, e);
		}

		logInfo("Datasource set to MySQL in file: " + path);

		Properties carolProperties = new Properties();

		// load the carol.properties file
		path = jonasRoot + File.separator + "conf" + File.separator
		+ "carol.properties";
		File carolPropertiesFile = new File(path);
		try {
			FileInputStream is = new FileInputStream(carolPropertiesFile);
			carolProperties.load(is);
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while reading file: " + path, e);
		}

		// set the properties
		carolProperties.setProperty(JRMP_URL_PROPERTY, "rmi://localhost:" + jrmpPort);

		String domainPath = jonasRoot + File.separator + "conf" + File.separator
		+ "domain.xml";
		try {
			RegexSubstitute.substitute(domainPath, "1099", String.valueOf(jrmpPort));
		} catch (Exception e) {
			throw new TaskException("Cannot edit the file: " + domainPath, e);
		} 

		// save to the carol.properties file
		try {
			FileOutputStream os = new FileOutputStream(carolPropertiesFile);
			carolProperties.store(os, "This file is generated by BEEN");
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while writing to file: " + path, e);
		}

		logInfo("JRMP port: " + jrmpPort);

		Properties mysqlProperties = new Properties();

		// load the MySQL.properties file
		path = jonasRoot + File.separator + "conf" + File.separator + "MySQL.properties";
		File mysqlPropertiesFile = new File(path);
		try {
			FileInputStream is = new FileInputStream(mysqlPropertiesFile);
			mysqlProperties.load(is);
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while reading file: " + path, e);
		}

		// set the properties
		String datasourceName = getTaskProperty(MYSQL_DATASOURCE_NAME);
		if (datasourceName != null) {
			logInfo("MySQL datasource name: " + datasourceName);
			mysqlProperties.setProperty(DATASOURCE_NAME_PROPERTY, datasourceName);
		}
		String datasourceUrl = getTaskProperty(MYSQL_DATASOURCE_URL);
		if (datasourceUrl != null) {
			logInfo("MySQL datasource URL: " + datasourceUrl);
			mysqlProperties.setProperty(DATASOURCE_URL_PROPERTY, datasourceUrl);
		}
		String datasourceUsername = getTaskProperty(MYSQL_DATASOURCE_USER);
		if (datasourceUsername != null) {
			logInfo("MySQL datasource user name: " + datasourceUsername);
			mysqlProperties.setProperty(DATASOURCE_USERNAME_PROPERTY, datasourceUsername);
		}
		String maxConnPool = getTaskProperty(MYSQL_MAX_CONNECTION_POOL);
		if (maxConnPool != null) {
			logInfo("MySQL maximum connection pool size: " + maxConnPool);
			mysqlProperties.setProperty(MAX_CONNPOOL_PROPERTY, maxConnPool);
		}
		String minConnPool = getTaskProperty(MYSQL_MIN_CONNECTION_POOL);
		if (minConnPool != null) {
			logInfo("MySQL minimum connection pool size: " + minConnPool);
			mysqlProperties.setProperty(MIN_CONNPOOL_PROPERTY, minConnPool);
		}
		String maxWaitTime = getTaskProperty(MYSQL_MAX_WAITTIME);
		if (maxWaitTime != null) {
			logInfo("MySQL maximum connection wait time: " + maxWaitTime);
			mysqlProperties.setProperty(MAX_CONN_WAIT_TIME, maxWaitTime);
		}

		// save to the MySQL.properties file
		try {
			FileOutputStream os = new FileOutputStream(mysqlPropertiesFile);
			mysqlProperties.store(os, "This file is generated by BEEN");
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while writing to file: " + path, e);
		}

		logInfo("conf/MySQL.properties file modified");
		
		String httpPort = getTaskProperty(HTTP_PORT);
		if (httpPort != null) {
			logInfo("HTTP port: " + httpPort);
			String configPath = jonasRoot + File.separator + "conf" + File.separator
			+ "server.xml";
			try {
				RegexSubstitute.substitute(configPath, 
						"<Connector port=\"9000\"", 
						"<Connector port=\"" + httpPort + "\"");
			} catch (Exception e) {
				throw new TaskException("Cannot set the HTTP port in the file: " + configPath, e);
			}
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(JONAS_ROOT);
	}
}
