/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.jonas.shutdown;

import java.io.File;
import java.util.ArrayList;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Stops Jonas.</p> 
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #JONAS_DIR}: <br>
 * 		- path to the Jonas installation directory <br>
 * 		- mandatory <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class JonasShutdown extends Job {
	//------------------------CONSTANTS--------------------
	/** Task property name for Jonas installation directory. */
	public static final String JONAS_DIR = "jonas.dir";
	
	//-----------------------ATTRIBUTES--------------------
	/** Path to directory where Jonas is installed */
	private String jonasDir = null;

	/**
	 * Allocates a new <code>JonasShutdown</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public JonasShutdown() throws TaskInitializationException {
		super();
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		jonasDir = getTaskProperty(JONAS_DIR);
		logInfo("Jonas directory: " + jonasDir);
		
		/*
		 * Shutdown Jonas
		 */
		
		String fSep = File.separator;
		ArrayList<String> cmd = new ArrayList<String>(); 
		Process jonasProcess = null;

		if (isRunningInLinux()) {
			String jonasPath = jonasDir + fSep + "bin" + fSep + "unix";
			cmd.add(jonasPath + fSep + "jonas");
			cmd.add("stop");
			cmd.add("-fg");
		}
		if (isRunningInWindows()) {
			cmd.add("cmd");
			cmd.add("/C");
			cmd.add("call");
			String jonasPath = jonasDir + fSep + "bin" + fSep + "nt";
			cmd.add(jonasPath + fSep + "jonas.bat");
			cmd.add("stop");
			cmd.add("-fg");
		}

		String[] cmdArray = (String[]) cmd.toArray(new String[cmd.size()]);
		
		try {
			logInfo("Shutting down Jonas");
			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));
			jonasProcess = Runtime.getRuntime().exec(cmdArray, null, new File(jonasDir));
			jonasProcess.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(jonasProcess.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(jonasProcess.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();

			// wait untill Jonas finishes
			jonasProcess.waitFor();
			logInfo("Jonas shut down");
		} catch (Exception e) {
			throw new TaskException("Cannot shutdown Jonas", e);
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(JONAS_DIR);
	}
}
