/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.mysql.run;

import java.io.File;
import java.util.ArrayList;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Starts the MySQL database server.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #CONNECTIONS_MAX}: <br>
 * 		- maximum number of concurrent connections to MySQL <br>
 * </ul>
 * 
 * <b>Checkpoints:</b><br>
 * <ul>
 * <li>{@value #CHECKPOINT_RUNNING}: <br>
 * 		- set when MySQL has succefully started<br>
 * </ul>
 *  
 * @author Jaroslav Urban
 */
public class MySQLRun extends Job {
	//---------------- CONSTANTS ---------------
	
	/** Task property name for the root directory of the MySQL installation */
	public static final String MYSQL_ROOT = "mysql.root";
	/** Task property name for the maximum number of open connections */
	public static final String CONNECTIONS_MAX = "connections.max";
	/** Name of the checkpoint which is set when MySQL is succesfully started.*/
	public static final String CHECKPOINT_RUNNING = "running";

	
	/** Wait time for 1 MySQL startup check */
	private static final int MYSQL_STARTUP_CHECK_TIME = 10000;
	/** Number of MySQL startup checks */
	private static final int MYSQL_STARTUP_CHECK_COUNT = 12;
	
	//---------------- ATTRIBUTES --------------
	
	/** Root directory of the MySQL installation */
	private String mysqlRoot;
	
	/**
	 * Allocates a new <code>MySQLRun</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public MySQLRun() throws TaskInitializationException {
		super();
	}
	
	@Override
	protected void run() throws TaskException {
		mysqlRoot = getTaskProperty(MYSQL_ROOT);
		logInfo("MySQL installation root directory: " + mysqlRoot);
		
		String maxConnections = null;
		if ((maxConnections = getTaskProperty(CONNECTIONS_MAX)) != null) {
			logInfo("Maximum number of connections: " + maxConnections);
		}

		
		// run the MySQL server
		Process mysqlProcess = null;
		String fSep = File.separator;
		
		ArrayList<String> cmd = new ArrayList<String>();
		cmd.add(mysqlRoot + fSep + "bin" + fSep + "mysqld");
		cmd.add("--basedir=" + mysqlRoot);

		if (maxConnections != null) {
			cmd.add("--set-variable=max_connections=" + maxConnections);
		}
		String[] cmdArray = (String[]) cmd.toArray(new String[cmd.size()]);

		logDebug("Executing : " + ArrayUtils.join(" ", cmdArray));
		
		try {
			logInfo("Starting MySQL");
			mysqlProcess = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
			mysqlProcess.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(mysqlProcess.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(mysqlProcess.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			
			// run a check for a succesfull MySQL start
			
			boolean started = false;
			try {
				
				for (int i = 0; i < MYSQL_STARTUP_CHECK_COUNT; i++) {
					try {
						// wait a bit, give some time to mysql to start
						Thread.sleep(MYSQL_STARTUP_CHECK_TIME);
					} catch (InterruptedException e) {
						// nothing bad happened
						Thread.currentThread().interrupt();
					}
					
					// check succesfull start
					
					logInfo("Checking if MySQL started succesfully");
					String[] checkCmdArray = { mysqlRoot + fSep + "bin" + fSep + "mysqladmin" ,
					"ping"};
					logDebug("Executing : " + checkCmdArray[0] + " " + checkCmdArray[1]);
					Process checkProcess = Runtime.getRuntime().exec(checkCmdArray, null, new File(mysqlRoot));

					/* Read in the output from the subprocess */
					OutputReader stdoutReader2 = 
						new OutputReader(checkProcess.getInputStream(), OutputType.STDOUT);
					OutputReader stderrReader2 = 
						new OutputReader(checkProcess.getErrorStream(), OutputType.STDERR);
					stdoutReader2.start();
					stderrReader2.start();
					
					if (checkProcess.waitFor() == 0) {
						started = true;
						break;
					}

				}
				
			} catch (Exception e) {
				throw new TaskException("Cannot run MySQL startup test", e);
			}
			
			if (!started) {
				throw new TaskException("MySQL not started");
			}
			
			logInfo("MySQL started");
			checkPointReached(CHECKPOINT_RUNNING, null);
			// wait untill MySQL finishes
			mysqlProcess.waitFor();
			logInfo("MySQL finished");
		} catch (Exception e) {
			throw new TaskException("Cannot run MySQL", e);
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperty(MYSQL_ROOT);
	}
}
