/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.mysql.runcommand;

import java.io.File;
import java.util.ArrayList;

import cz.cuni.mff.been.common.ArrayUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Runs a command on the MySQL server (e.g. an SQL statement).</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #MYSQL_ROOT}: <br>
 * 		- path to the MySQL installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #USER_NAME}: <br>
 * 		- user name under which will the command be called <br>
 * 		- mandatory <br>
 * <li>{@value #COMMAND}: <br>
 * 		- MySQL command that will be executed <br>
 * 		- mandatory <br>
 * <li>{@value #DATABASE_NAME}: <br>
 * 		- name of the database which is the context of the command <br>
 * </ul>
 * 
 */
public class MySQLRunCommand extends Job {
	
	/** Task property name for the root directory of the MySQL installation. */
	public static final String MYSQL_ROOT = "mysql.root";

	/** Task property name for the user name of the user which will run the command. */
	public static final String USER_NAME = "user.name";

	/** Task property name for the database name. */
	public static final String DATABASE_NAME = "database.name";
	
	/** Task property name for the command that will be executed. */
	public static final String COMMAND = "command";

	/** Too directory of the MySQL installation. */
	private String mysqlRoot;
	/** Name of the user that will execute the command. */
	private String userName;
	/** Name of the database for the command. */
	private String databaseName;
	/** Command that will be executed. */
	private String command;
	
	/**
	 * Allocates a new <code>MySQLRunCommand</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public MySQLRunCommand() throws TaskInitializationException{
		super();
	}

	private void runCommand() throws TaskException {
		String fSep = File.separator;
		
		try {
			ArrayList< String> commands = new ArrayList<String>();
			commands.add(mysqlRoot + fSep + "bin" + fSep + "mysql");
			commands.add("-u");
			commands.add(userName);
			if (databaseName != null) {
				commands.add(databaseName);
			}
			commands.add("-e");
			commands.add(command);

			String[] cmdArray = (String[]) commands.toArray(new String[commands.size()]);
			logDebug("Executing: " + ArrayUtils.join(" ", cmdArray));

			Process p = Runtime.getRuntime().exec(cmdArray, null, new File(mysqlRoot));
			p.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new TaskException("Execution of MySQL command unsuccessfull");
			}
		} catch (TaskException e) {
			throw e;
		} catch (Exception e) {
			throw new TaskException("Cannot execute MySQL commandr",
					e);
		}
	}

	@Override
	protected void checkRequiredProperties() throws TaskException {
	}

	@Override
	protected void run() throws TaskException {
		if ((mysqlRoot = getTaskProperty(MYSQL_ROOT)) == null ) {
			throw new TaskException(MYSQL_ROOT + " task property not set");
		}
		logInfo("MySQL root: " + mysqlRoot);

		if ((userName = getTaskProperty(USER_NAME)) == null ) {
			throw new TaskException(USER_NAME + " task property not set");
		}
		logInfo("User name: " + userName);
		
		if ((databaseName = getTaskProperty(DATABASE_NAME)) != null ) {
			logInfo("Database name: " + databaseName);
		}
		
		if ((command = getTaskProperty(COMMAND)) == null ) {
			throw new TaskException(COMMAND + " task property not set");
		}
		logInfo("Command: " + command);
		
		runCommand();
		
		logInfo("Command executed succesfully");
	}
}
