/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jakub Lehotsky
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.results.rubis;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.List;

import cz.cuni.mff.been.common.id.BID;
import cz.cuni.mff.been.common.id.RID;
import cz.cuni.mff.been.task.TaskInitializationException;
import cz.cuni.mff.been.task.results.CheckAndConvertTask;

/**
 * Task to check and convert Rubis results
 * 
 * @author Jakub Lehotsky
 *
 */
public class CheckAndConvertRubisTask extends CheckAndConvertTask {
	
	/** The only rubis role identification */
	public static final String ROLE_CLIENT = "client";
	
	/** Task name */
	public static final String TASK_NAME = "check-and-convert-rubis";

	/** Count of columns for each rubis run */
	private static final int TABLE_COUNT = 28;
	
	/** Granularity of results */
	private static final int GRANULARITY = 1000;
	
	/** The names of the tables of Rubis results */
	private static final String[] TABLE_NAMES = { "Home", "Register", "RegisterUser", "Browse",
	"BrowseCategories",	"SearchItemsInCategory", "BrowseRegions", "BrowseCategoriesInRegion",
	"SearchItemsInRegion", "ViewItem", "ViewUserInfo", "ViewBidHistory", "BuyNowAuth",
	"BuyNow", "StoreBuyBow", "PutBidAuth", "PutBid", "StoreBid", "PutCommentAuth", "PutComment",
	"StoreComment", "Sell", "SelectCategoryToSellItem", "SellItemForm", "RegisterItem",
	"AboutMe (auth form)", "AboutMe", "Back probability", "End of Session" };

	private static final long INVALID_VALUE = -1;


	/**
	 * Allocates a new <code>RubisResultsCollectionTask</code> object. It is here
	 * only to shut up the compiler.
	 * 
	 * @throws TaskInitializationException if something fails during the task
	 *          creation
	 **/
	public CheckAndConvertRubisTask() throws TaskInitializationException {
		super();
	}	
				
	/* ********* Results checking and converting functions (the core of the task) ********* */	
	
	/**
	 * Converts ascii data format of one run into netcdf format, 
	 * making it ready to upload to Results Repository
	 * 
	 * @param bid	Binary Id 
	 * @param rid	Run Id
	 * @param sourceName	Source filename (ASCII)
	 * @param destinatioName	Destination filename (NetCDF)
	 * @param resultsRole	The results role of the machine in benchmark 
	 * @param suiteName 	Identification of used suite
	 * @param sampleCount 	Count of samples for each table 
	 * @return 	True if sucesful
	 * @throws IOException
	 */
	public boolean checkAndConvertOneRun(BID bid, RID rid, String sourceName, String destinatioName, 
			String resultsRole, String suiteName, long sampleCount) throws IOException {
				
		PrintWriter resultsWriter;		
		String line;
		int counter;
	
		BufferedReader sourceReader = null;
		Long value = 0L;

		counter = 0;
		

		
		for (int tableIndex = 0; tableIndex < TABLE_COUNT; tableIndex++) {
			
			resultsWriter = new PrintWriter(new BufferedWriter(new FileWriter(destinatioName + "-" + tableIndex)));
			
			try {
				sourceReader = new BufferedReader(new FileReader(new File(sourceName + File.separator 
						+ "state-" + tableIndex + "-times.txt")));
			} catch (IOException e) {
				logError("Error opening results file : " + sourceName + File.separator
						+ "state-" + tableIndex + "-times.txt");
				notifyTableCreation(tableIndex, TABLE_COUNT , bid, rid, false);
				return false;
			}		

			try {
				
				while ((line = sourceReader.readLine()) != null) {
					
					try {
						value = Long.valueOf(line) / GRANULARITY;
						resultsWriter.println(value);
					} catch (NumberFormatException e) {													
						value = Long.valueOf(INVALID_VALUE);
					} 										
				}

			} catch (IOException e) {
				
				e.printStackTrace();
				logError("I/O error converting " + sourceName + " - table number " + tableIndex + ".");
				notifyTableCreation(tableIndex, TABLE_COUNT , bid, rid, false);
				return false; // Run is corrupted
				
			} finally {				
				resultsWriter.close();				
			}
			
			PrintWriter metaWriter = new PrintWriter(new BufferedWriter(new FileWriter(destinatioName + "-" 
					+ tableIndex + ".meta")));

			metaWriter.println(suiteName);
			metaWriter.println(TABLE_NAMES[tableIndex]);
			metaWriter.close();
		
			notifyTableCreation(tableIndex, TABLE_COUNT , bid, rid, true);
			
		}

		return true;
	}
	
	public List<String> getCrashList(String srcName) {
		return Arrays.asList(new String[] { null });		
	}
	
}
