/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.rubis.deployer;

import java.io.File;
import java.io.IOException;
import java.util.regex.PatternSyntaxException;

import cz.cuni.mff.been.common.RegexSubstitute;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Deploys Rubis to an EJB server, i.e. copies the right files to
 * right places. It does NOT start the EJB server. </p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #RUBIS_ROOT}: <br>
 * 		- path to the Rubis source code <br>
 * 		- mandatory <br>
 * <li>{@value #EJB_SERVER}: <br>
 * 		- which EJB server will be used by Rubis. Valid values are 
 * 		{@value #EJB_SERVER_JBOSS} and {@value #EJB_SERVER_JONAS}<br>
 * 		- mandatory <br>
 * <li>{@value #JBOSS_ROOT}: <br>
 * 		- path to the JBoss installation directory <br>
 * 		- mandatory if the {@value #EJB_SERVER} task property is set to
 * 		{@value #EJB_SERVER_JBOSS}<br>
 * <li>{@value #JONAS_ROOT}: <br>
 * 		- path to the Jonas installation directory <br>
 * 		- mandatory if the {@value #EJB_SERVER} task property is set to
 * 		{@value #EJB_SERVER_JONAS}<br>
 * <li>{@value #RUBIS_BENCHMARK}: <br>
 * 		- specifies which one of the several Rubis benchmark suites will be used.
 * 		Valid values are 
 * 		{@value #RUBIS_BENCHMARK_EJB_CMP2}, 
 * 		{@value #RUBIS_BENCHMARK_EJB_ENTITYBEAN_ID},
 * 		{@value #RUBIS_BENCHMARK_EJB_ENTITYBEAN_ID_BMP},
 * 		{@value #RUBIS_BENCHMARK_EJB_LOCAL_REMOTE},
 * 		{@value #RUBIS_BENCHMARK_EJB_MDB},
 * 		{@value #RUBIS_BENCHMARK_EJB_SESSION_FACADE} and
 * 		{@value #RUBIS_BENCHMARK_EJB_SESSIONBEAN}. <br>
 * 		- mandatory <br>
 * <li>{@value #DATABASE_HOSTNAME}: <br>
 * 		- hostname of the database which will be used by Rubis <br>
 * 		- mandatory if the {@value #EJB_SERVER} task property is set to
 * 		{@value #EJB_SERVER_JBOSS} because it has to be set in a datasource
 * 		descriptor that is deployed with Rubis.<br>
 * </ul>

 * 
 * @author Jaroslav Urban
 */
public class RubisDeployer extends Job {
	//------------ CONSTANTS ---------------
	
	/** Task property name for the root directory of the Rubis binary. */
	public static final String RUBIS_ROOT = "rubis.root";

	/** Task property name for the name of a Rubis benchmark that will
	 * be deployed. Valid values are in the RUBIS_BENCHMARK_* constants. 
	 */
	public static final String RUBIS_BENCHMARK = "rubis.benchmark";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the EJB_CMP2.0 benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_CMP2 = "EJB_CMP2.0";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the EJB_EntityBean_id benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_ENTITYBEAN_ID = "EJB_EntityBean_id";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the EJB_EntityBean_id_BMP benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_ENTITYBEAN_ID_BMP = "EJB_EntityBean_id_BMP";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the EJB_SessionBean benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_SESSIONBEAN = "EJB_SessionBean";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the EJB_Session_facade benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_SESSION_FACADE = "EJB_Session_facade";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the EJB_local_remote benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_LOCAL_REMOTE = "EJB_local_remote";

	/** Valid value of the 
	 * {@value #RUBIS_BENCHMARK}
	 * task property that specifies that the MDB benchmark should be used. 
	 */
	public static final String RUBIS_BENCHMARK_EJB_MDB = "MDB";

	/** Task property name for the name of the EJB server. Valid values are
	 * <code>jboss</code> and <code>jonas</code>.
	 */
	public static final String EJB_SERVER = "ejb.server";

	/** Task property name for the hostname of the database server. */
	public static final String DATABASE_HOSTNAME = "database.hostname";

	/** Valid value of the 
	 * {@value #EJB_SERVER}
	 * task property for JBoss. 
	 */
	public static final String EJB_SERVER_JBOSS = "jboss";

	/** Valid value of the 
	 * {@value #EJB_SERVER}
	 * task property for Jonas. 
	 */
	public static final String EJB_SERVER_JONAS = "jonas";

	/** Task property name for the root directory of JBOSS. */
	public static final String JBOSS_ROOT = "jboss.root";
	/** Task property name for the root directory of Jonas. */
	public static final String JONAS_ROOT = "jonas.root";
	
	/** Name of the Rubis war file. */
	private static final String WAR_FILE = "ejb_rubis_web.war";
	/** Name of the rubis jar file. */
	private static final String RUBIS_JAR_FILE = "rubis.jar";

	// ------------ ATTRIBUTES -------------
	
	/** Path to the root directory of the Rubis binary. */
	private String rubisRoot;

	/**
	 * Which of the Rubis' benchmarks should be used.
	 */
	private String rubisBenchmark;

	/**
	 * Supported EJB servers for Rubis deployment.
	 * 
	 * @author Jaroslav Urban
	 */
	private enum EJBServer {
		/** JBoss application server */
		JBoss,
		/** Jonas application server */
		Jonas
	}

	/** Which EJB server will be used. */
	private EJBServer ejbServer;
	
	/**
	 * Path to JBoss.
	 */
	private String jbossRoot;
	
	/**
	 * Path to Jonas.
	 */
	private String jonasRoot;

	/**
	 * Allocates a new <code>RubisDeployer</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public RubisDeployer() throws TaskInitializationException {
		super();
	}

	/**
	 * Gets the values of task properties.
	 * 
	 * @throws TaskException if some mandatory task property is missing or if 
	 * a task property has invalid value.
	 */
	private void init() throws TaskException {
		rubisRoot = getTaskProperty(RUBIS_ROOT);
		logInfo("Rubis root: " + rubisRoot);
		
		rubisBenchmark = getTaskProperty(RUBIS_BENCHMARK);
		if ((!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_CMP2))
				&& (!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_ENTITYBEAN_ID))
				&& (!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_ENTITYBEAN_ID_BMP))
				&& (!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_LOCAL_REMOTE))
				&& (!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_MDB)) 
				&& (!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_SESSION_FACADE))
				&& (!rubisBenchmark.equals(RUBIS_BENCHMARK_EJB_SESSIONBEAN))
				) {
			throw new TaskException("Unsupported value of the " 
					+ RUBIS_BENCHMARK + " task property: " + rubisBenchmark);
		}
		logInfo("Rubis benchmark: " + rubisBenchmark);
		
		String server = getTaskProperty(EJB_SERVER);
		
		if (server.equals(EJB_SERVER_JBOSS)) {
			ejbServer = EJBServer.JBoss;
			logInfo("EJB server: JBoss");
			
			if ((jbossRoot = getTaskProperty(JBOSS_ROOT)) == null) {
				throw new TaskException(JBOSS_ROOT + " task property not set");
			}
			logInfo("JBoss root: " + jbossRoot);
		} else if (server.equals(EJB_SERVER_JONAS)) {
			ejbServer = EJBServer.Jonas;
			logInfo("EJB server: Jonas");
			
			if ((jonasRoot = getTaskProperty(JONAS_ROOT)) == null) {
				throw new TaskException(JONAS_ROOT + " task property not set");
			}
			logInfo("Jonas root: " + jonasRoot);
		} else {
			throw new TaskException("Unsupported EJB server: " + server);
		}
	}
	
	@Override
	protected void run() throws TaskException {
		init();
		
		// path to the rubis.war file
		String warPath = rubisRoot + File.separator + rubisBenchmark + File.separator
			+ WAR_FILE;
		String rubisJarPath = rubisRoot + File.separator + rubisBenchmark + File.separator
		+ RUBIS_JAR_FILE;

		try {
			if (ejbServer == EJBServer.Jonas) {
				String webappsDir = jonasRoot + File.separator + "webapps"
				+ File.separator + "autoload";
				Copy.copyToDir(warPath, webappsDir);

				String ejbJarDir = jonasRoot + File.separator + "ejbjars"
				+ File.separator + "autoload";
				Copy.copyToDir(rubisJarPath, ejbJarDir);
			}
			if (ejbServer == EJBServer.JBoss) {
				String targetDir = jbossRoot + File.separator + "server" 
				+ File.separator + "default" + File.separator + "deploy";

				Copy.copyToDir(warPath, targetDir);
				Copy.copyToDir(rubisJarPath, targetDir);
				
				// copy and configure the datasource
				String databaseHostname = null;
				if ((databaseHostname = getTaskProperty(DATABASE_HOSTNAME)) == null) {
					throw new TaskException(DATABASE_HOSTNAME + " task property not set");
				}
				Copy.copyToDir(getTaskDirectory() + File.separator + "mysql-ds.xml", 
						targetDir);
				RegexSubstitute.substitute(targetDir + File.separator + "mysql-ds.xml",
						"jdbc:mysql:///rubis",
						"jdbc:mysql://" + databaseHostname + "/rubis");
			}
		} catch (AntTaskException e) {
			throw new TaskException("Cannot copy file", e);
		} catch (IOException e) {
			throw new TaskException("Cannot edit datasource descriptor", e);
		} catch (PatternSyntaxException e) {
			// should never happen
			assert false : "Bad regexp";
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{RUBIS_ROOT, RUBIS_BENCHMARK, EJB_SERVER});
	}
}
