/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.rubis.run;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;
import cz.cuni.mff.been.taskmanager.CheckPoint;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;
import cz.cuni.mff.been.taskmanager.TaskManagerInterface;

/**
 * <p>Runs the Rubis client emulator. This task can work in 2 roles: main and 
 * slave. When all slave clients are ready to start, the main client starts
 * them by setting a checkpoint.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #RUBIS_ROOT}: <br>
 * 		- path to the Rubis installation directory <br>
 * 		- mandatory <br>
 * <li>{@value #EJB_SERVER_HOSTNAME}: <br>
 * 		- hostname of the EJB server<br>
 * 		- mandatory <br>
 * <li>{@value #DATABASE_SERVER_HOSTNAME}: <br>
 * 		- hostname of the database server<br>
 * 		- mandatory <br>
 * <li>{@value #RUN_INDEX}: <br>
 * 		- index of the benchmarking run for this execution of the client
 * 		emulator<br>
 * 		- mandatory <br>
 * <li>{@value #MAIN_CLIENT}: <br>
 * 		- task ID of the main client. If this task property is set, then this
 * 		client will be in the slave role.<br>
 * 		- mandatory if the {@value #CLIENTS} task property is not set<br>
 * <li>{@value #CLIENTS}: <br>
 * 		- String array of task IDs of the slave clients. If this task property is 
 * 		set, this client will be in the main role.<br>
 * 		- mandatory if the {@value #MAIN_CLIENT} task property is not set<br>
 * <li>{@value #TRANSITION_TABLE}: <br>
 * 		- name of the transition table which will be used<br>
 * <li>{@value #TRANSITIONS_MAX}: <br>
 * 		- maximum number of transitions during one client session<br>
 * <li>{@value #UP_RAMP_TIME}: <br>
 * 		- length of the up ramp, in milliseconds<br>
 * <li>{@value #UP_RAMP_SLOWDOWN_FACTOR}: <br>
 * 		- slowdown factor used during the up ramp<br>
 * <li>{@value #SESSION_TIME}: <br>
 * 		- length of the benchmarking session, in milliseconds<br>
 * <li>{@value #DOWN_RAMP_TIME}: <br>
 * 		- length of the down ramp, in milliseconds<br>
 * <li>{@value #DOWN_RAMP_SLOWDOWN_FACTOR}: <br>
 * 		- slowdown factor used during the down ramp<br>
 * <li>{@value #CLIENT_COUNT}: <br>
 * 		- number of concurrently emulated clients<br>
 * <li>{@value #DEBUG_LEVEL}: <br>
 * 		- debug level of the client emulator, valid values are 0, 1, 2 or 3<br>
 * </ul>
 * 
 * <b>Checkpoints:</b><br>
 * <ul>
 * <li>{@value #READY_CHECKPOINT}: <br>
 * 		- client in slave role sets this checkpoint to indicate readiness to start
 * 		the client emulator. The main client waits for every slave client to
 * 		set this checkpoint and then it sets the {@value #START_CHECKPOINT}
 * 		checkpoint.<br>
 * <li>{@value #START_CHECKPOINT}: <br>
 * 		- task in master role sets this checkpoint to tell all slave clients to
 * 		start the client emulator. Every slave client sets the 
 * 		{@value #READY_CHECKPOINT} when it's ready to start the client emulator,
 * 		and then waits for the main client to set this checkpoint. <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class RubisRun extends Job {
	//------------ CONSTANTS ---------------
	
	/** Task property name for an array of task IDs of ther Rubis clients. */
	public static final String CLIENTS = "clients";
	/** Task property name for the task ID of the main client. */
	public static final String MAIN_CLIENT = "main.client";
	
	/** Task property name for the hostname of the EJB server. */
	public static final String EJB_SERVER_HOSTNAME = "ejb.hostname";
	/** Task property name for the hostname of the database server. */
	public static final String DATABASE_SERVER_HOSTNAME = "database.hostname";
	/** Task property name for the rubis installation directory. */
	public static final String RUBIS_ROOT = "rubis.root";
	
	/** Task property name for the used transition table. */
	public static final String TRANSITION_TABLE = "transition.table";
	/** Task property name for the maximum number of transitions. */
	public static final String TRANSITIONS_MAX = "transitions.max";
	/** Task property name for up ramp time, in milliseconds. */
	public static final String UP_RAMP_TIME = "upramp.time";
	/** Task property name for up ramp slowdown factor. */
	public static final String UP_RAMP_SLOWDOWN_FACTOR = "upramp.slowdown.factor";
	/** Task property name for session time, in milliseconds. */
	public static final String SESSION_TIME = "session.time";
	/** Task property name for down ramp time, in milliseconds. */
	public static final String DOWN_RAMP_TIME = "downramp.time";
	/** Task property name for down ramp slowdown factor. */
	public static final String DOWN_RAMP_SLOWDOWN_FACTOR = "downramp.slowdown.factor";
	/** Task property name for the client count. */
	public static final String CLIENT_COUNT = "client.count";
	/** Task property name for the debug level. */
	public static final String DEBUG_LEVEL = "debug.level";
	/** Task property name for the index of the current run. */
	public static final String RUN_INDEX = "run.index";

	/** Property name in the rubis.properties file for the hostname of the EJB server. */
	private static final String EJB_SERVER_HOSTNAME_PROPERTY = 
		"ejb_server";
	/** Property name in the rubis.properties file for the hostname of the HTTP server. */
	private static final String HTTPD_HOSTNAME_PROPERTY = 
		"httpd_hostname";
	/** Property name in the rubis.properties file for the HTTPD port. */
	private static final String HTTPD_PORT =
		"httpd_port";
	/** Property name in the rubis.properties file for the hostname of the database server. */
	private static final String DATABASE_SERVER_HOSTNAME_PROPERTY = 
		"database_server";
	/** Property name in the rubis.properties file for the used transition table. */
	private static final String TRANSITION_TABLE_PROPERTY = 
		"workload_transition_table";
	/** Property name in the rubis.properties file for the maximum number of transitions. */
	private static final String TRANSITIONS_MAX_PROPERTY = 
		"workload_maximum_number_of_transitions";
	/** Property name in the rubis.properties file for up ramp time, in milliseconds. */
	private static final String UP_RAMP_TIME_PROPERTY = 
		"workload_up_ramp_time_in_ms";
	/** Property name in the rubis.properties file for up ramp slowdown factor. */
	private static final String UP_RAMP_SLOWDOWN_FACTOR_PROPERTY = 
		"workload_up_ramp_slowdown_factor";
	/** Property name in the rubis.properties file for session time, in milliseconds. */
	private static final String SESSION_TIME_PROPERTY = 
		"workload_session_run_time_in_ms";
	/** Property name in the rubis.properties file for down ramp time, in milliseconds. */
	private static final String DOWN_RAMP_TIME_PROPERTY = 
		"workload_down_ramp_time_in_ms";
	/** Property name in the rubis.properties file for down ramp slowdown factor. */
	private static final String DOWN_RAMP_SLOWDOWN_FACTOR_PROPERTY = 
		"workload_down_ramp_slowdown_factor";
	/** Property name in the rubis.properties file for database regions file. */
	private static final String DATABASE_REGIONS_FILE_PROPERTY = 
		"database_regions_file";
	/** Property name in the rubis.properties file for database categories file. */
	private static final String DATABASE_CATEGORIES_FILE_PROPERTY = 
		"database_categories_file";
	/** Property name in the rubis.properties file for the number of users. */
	private static final String DATABASE_NUMBER_OF_USERS_PROPERTY = 
		"database_number_of_users";
	/** Property name in the rubis.properties file for the debug level. */
	private static final String DEBUG_LEVEL_PROPERTY =
		"monitoring_debug_level";
	/** Property name in the rubis.properties file for the client count. */
	private static final String CLIENT_COUNT_PROPERTY =
		"workload_number_of_clients_per_node";
	
	/** 
	 * Name of a checkpoint set by a slave client, which indicates that it's 
	 * ready to start.
	 */
	public static final String READY_CHECKPOINT = "ready";
	/** 
	 * Name of a checkpoint set by the main client, which indicates that slave
	 * clients can start.
	 */
	public static final String START_CHECKPOINT = "start";
	
	//------------ ATTRIBUTES -------------
	
	/** Hostname of the EJB server */
	private String ejbHostname;
	/** Hostname of the database server */
	private String databaseHostname;
	/** Rubis installation directory */
	private String rubisRoot;
	/** Transition table which will be used */
	private String transitionTable = "default_transitions.txt";
	/** Maximum number of transitions */
	private String maxTransitions;
	/** Up Ramp time in milliseconds */
	private String upRampTime;
	/** Up Ramp slowdown factor */
	private String upRampSlowdownFactor;
	/** Session run time in milliseconds */
	private String sessionTime;
	/** Down Ramp time in milliseconds */
	private String downRampTime;
	/** Down Ramp slowdown factor */
	private String downRampSlowdownFactor;
	/** Index of the current run */
	private String runIndex;
	/** Client count. */
	private String clientCount;
	/** Debug level */
	private String debugLevel;
	
	private String[] clients;
	private String mainClient;
	
	/**
	 * @throws TaskInitializationException
	 */
	public RubisRun() throws TaskInitializationException {
		super();
	}

	/**
	 * Gets the values of task properties.
	 * 
	 * @throws TaskException if a mandatory task property is missing.
	 */
	private void init() throws TaskException {
		
		if ((clients = (String[]) getTaskPropertyObject(CLIENTS)) != null) {
			logInfo("Main client");

			for (String taskId : clients) {
				logDebug("Slave client task ID: " + taskId);
			}
		} else if ((mainClient = getTaskProperty(MAIN_CLIENT)) != null) {
			logInfo("Slave client");
			logDebug("Main client task ID:" + mainClient);
		} else {
			throw new TaskException("Must be the main client or a slave client");
		}
		
		runIndex = getTaskProperty(RUN_INDEX);
		logInfo("Run index: " + runIndex);
		
		ejbHostname = getTaskProperty(EJB_SERVER_HOSTNAME);
		logInfo("EJB server hostname: " + ejbHostname);
		
		databaseHostname = getTaskProperty(DATABASE_SERVER_HOSTNAME);
		logInfo("Database server hostname: " + databaseHostname);
		
		rubisRoot = getTaskProperty(RUBIS_ROOT);
		logInfo("Rubis installation directory: " + rubisRoot);
		
		String newTransitionTable = getTaskProperty(TRANSITION_TABLE);
		if (newTransitionTable != null) {
			transitionTable = newTransitionTable;
			logInfo("Transition table: " + transitionTable);
		}
		
		maxTransitions = getTaskProperty(TRANSITIONS_MAX);
		if (maxTransitions != null) {
			logInfo("Maximum number of transitions: " + maxTransitions);
		}
		
		upRampTime = getTaskProperty(UP_RAMP_TIME);
		if (upRampTime != null) {
			logInfo("Up Ramp time: " + upRampTime);
		}
		
		upRampSlowdownFactor = getTaskProperty(UP_RAMP_SLOWDOWN_FACTOR);
		if (upRampSlowdownFactor != null) {
			logInfo("Up Ramp slowdown factor: " + upRampSlowdownFactor);
		}
		
		sessionTime = getTaskProperty(SESSION_TIME);
		if (sessionTime != null) {
			logInfo("Session time: " + sessionTime);
		}
		
		downRampTime = getTaskProperty(DOWN_RAMP_TIME);
		if (downRampTime != null) {
			logInfo("Down Ramp time: " + downRampTime);
		}
		
		downRampSlowdownFactor = getTaskProperty(DOWN_RAMP_SLOWDOWN_FACTOR);
		if (downRampSlowdownFactor != null) {
			logInfo("Down Ramp slowdown factor: " + downRampSlowdownFactor);
		}
		
		clientCount = getTaskProperty(CLIENT_COUNT);
		if (clientCount != null) {
			logInfo("Client count: " + clientCount);
		}
		
		debugLevel = getTaskProperty(DEBUG_LEVEL);
		if (debugLevel != null) {
			logInfo("Debug level: " + debugLevel);
		}
	}

	/**
	 * Synchronizes the start of all clients.
	 * 
	 * @throws TaskException
	 */
	private void synchronize() throws TaskException {
		if (clients != null) {
			// this is the main client, synchronize the start of all clients
			for (String  taskId : clients) {
				checkPointWait(null, taskId, READY_CHECKPOINT, 
						TaskManagerInterface.INFINITE_TIME);
			}
			checkPointReached(START_CHECKPOINT, null);
		} else {
			// this is a slave client, synchronize with the main client
			checkPointReached(READY_CHECKPOINT, null);
			checkPointWait(null, mainClient, START_CHECKPOINT, 
					TaskManagerInterface.INFINITE_TIME);
		}
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		init();
		synchronize();
		
		/*
		 * set values in the rubis.properties file
		 */ 
		
		Properties rubisProperties = new Properties();
		
		// load the rubis.properties file
		String path = rubisRoot + File.separator + "Client" + 
			File.separator + "rubis.properties";
		File rubisPropertiesFile = new File(path);
		try {
			FileInputStream is = new FileInputStream(rubisPropertiesFile);
			rubisProperties.load(is);
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while reading file: " + path, e);
		}

		// set the properties
		rubisProperties.setProperty(EJB_SERVER_HOSTNAME_PROPERTY, ejbHostname);
		rubisProperties.setProperty(HTTPD_HOSTNAME_PROPERTY, ejbHostname);
		rubisProperties.setProperty(DATABASE_SERVER_HOSTNAME_PROPERTY, databaseHostname);

		if (transitionTable != null) {
			String tablePath = rubisRoot + File.separator + "workload" + File.separator +
				transitionTable;
			rubisProperties.setProperty(TRANSITION_TABLE_PROPERTY, tablePath);
		}
		if (maxTransitions != null) {
			rubisProperties.setProperty(TRANSITIONS_MAX_PROPERTY, maxTransitions);
		}
		if (upRampTime != null) {
			rubisProperties.setProperty(UP_RAMP_TIME_PROPERTY, upRampTime);
		}
		if (upRampSlowdownFactor != null) {
			rubisProperties.setProperty(UP_RAMP_SLOWDOWN_FACTOR_PROPERTY, 
					upRampSlowdownFactor);
		}
		if (sessionTime != null) {
			rubisProperties.setProperty(SESSION_TIME_PROPERTY, sessionTime);
		}
		if (downRampTime != null) {
			rubisProperties.setProperty(DOWN_RAMP_TIME_PROPERTY, downRampTime);
		}
		if (downRampSlowdownFactor != null) {
			rubisProperties.setProperty(DOWN_RAMP_SLOWDOWN_FACTOR_PROPERTY, 
					downRampSlowdownFactor);
		}
		if (debugLevel != null) {
			rubisProperties.setProperty(DEBUG_LEVEL_PROPERTY, 
					debugLevel);
		}
		if (clientCount != null) {
			rubisProperties.setProperty(CLIENT_COUNT_PROPERTY, 
					clientCount);
		}
		
		// Set the paths to regions and categories files. These paths
		// aren't user settable, but must be modified according to
		// the path to the Rubis installation directory
		
		String databaseDirPath = rubisRoot + File.separator + "database";
		rubisProperties.setProperty(DATABASE_CATEGORIES_FILE_PROPERTY, 
				databaseDirPath + File.separator + "ebay_simple_categories.txt");
		rubisProperties.setProperty(DATABASE_REGIONS_FILE_PROPERTY, 
				databaseDirPath + File.separator + "ebay_regions.txt");

		// set the number of users to 1000000, small numbers cause errors
		rubisProperties.setProperty(DATABASE_NUMBER_OF_USERS_PROPERTY, "1000000");
		
		// set the debug level to 5
		rubisProperties.setProperty(DEBUG_LEVEL_PROPERTY, debugLevel);
		
		// set the port of the web server to 8080
		rubisProperties.setProperty(HTTPD_PORT, "8080");
		
		// save to the rubis.properties file
		try {
			FileOutputStream os = new FileOutputStream(rubisPropertiesFile);
			rubisProperties.store(os, "This file is generated by BEEN");
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + path, e);
		} catch (IOException e) {
			throw new TaskException("Error occured while writing to file: " + path, e);
		}
		
		logInfo("rubis.properties file modified");
		
		/*
		 * Run Rubis
		 */
	
		// classpath for Rubis. The first path element points to the 
		// "Client" directory which contains the rubis.properties file.
		// The second path element points to the directory with 
		// the classes.
		String classpath = rubisRoot + File.separator + "Client" +
				File.pathSeparator +
				rubisRoot + File.separator + "Client" + File.separator +
				"build" + File.separator + "RUBiS-1.0";
		String libraryPath = System.getProperty("java.library.path") + File.pathSeparator +
			rubisRoot + File.separator + "Client";
		
		try {
			String[] cmdArray = { "java", "-classpath", classpath, 
					"-Djava.library.path=" + libraryPath,
					"edu.rice.rubis.client.ClientEmulatorBeen",
					runIndex};

			logDebug("Executing: " + cmdArray[0] + " " + cmdArray[1] + 
					" " + cmdArray[2] +
					" " + cmdArray[3] +
					" " + cmdArray[4] +
					" " + cmdArray[5]);

			logInfo("Starting Rubis");
			Process p = Runtime.getRuntime().exec(cmdArray, null, new File(rubisRoot));
			p.getOutputStream().close();
			
			/* Read in the output from the subprocess */
			OutputReader stdoutReader = 
				new OutputReader(p.getInputStream(), OutputType.STDOUT);
			OutputReader stderrReader = 
				new OutputReader(p.getErrorStream(), OutputType.STDERR);
			stdoutReader.start();
			stderrReader.start();
			// wait untill the script finishes
			if (p.waitFor() != 0) {
				throw new Exception("Rubis exited with an error, see the standard" +
						"output of this task");
			}
		} catch (Exception e) {
			throw new TaskException("Cannot run Rubis", e);
		}
	
		logInfo("Rubis finished succesfully");
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{
				RUN_INDEX,
				EJB_SERVER_HOSTNAME,
				DATABASE_SERVER_HOSTNAME,
				RUBIS_ROOT
		});
	}
}
