/*
 * Created on Nov 7, 2006
 *
 * BEEN
 * @author Jaroslav Urban
 */
package cz.cuni.mff.been.task.test.testworker;

import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;
import cz.cuni.mff.been.taskmanager.TaskManagerInterface;

/**
 * <p>This task can be set to do various example work. It's used for
 * testing purposes. You have to choose which actions should it perform.
 * It's designed to cooperate with other running <tt>TestWorker</tt> tasks.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #DO_CHECKPOINT_VALUE}: <br>
 * 		- if set to "true" or "yes", it retrieves a value of a checkpoint. 
 * 		The value must be of the String[2]
 * 		type. The name of the checkpoint must be specified in the 
 * 		{@value #CHECKPOINT_NAME} task property. Task ID of a task that sets the 
 * 		checkpoint must be specified in the {@value #CHECKPOINT_TASK}
 * 		task property.<br>
 * <li>{@value #DO_CHECKPOINT_SET}: <br>
 * 		- if set to "true" or "yes", it sets a checkpoint with a value. 
 * 		The value will be {"BEEN", "DSRG"}. 
 * 		The name of the checkpoint must be specified in the 
 * 		{@value #CHECKPOINT_NAME} task property.<br>
 * <li>{@value #DO_CHECKPOINT_BLOCK}: <br>
 * 		- if set to "true" or "yes", it retrieves a value of a checkpoint in a 
 * 		similar way as when the {@value #DO_CHECKPOINT_VALUE} task property is 
 * 		set to "true" or "yes", but the query for the checkpoint value is 
 * 		blocking. The {@value #CHECKPOINT_NAME} and {@value #CHECKPOINT_TASK}
 * 		task properties must be also set.<br>
 * <li>{@value #DO_PROPERTY_VALUE}: <br>
 * 		- if set to "true" or "yes", it retrieves the value of a task property. 
 * 		The value must be of the String[2] type. Name of the task property 
 * 		must be specified in the {@value #PROPERTY_NAME} task property.<br>
 * <li>{@value #DO_WAIT}: <br>
 * 		- if set to "true" or "yes", the task sleeps for the ammount of seconds 
 * 		set in the {@value #WAIT_TIME} task property.<br>
 * <li>{@value #WAIT_TIME}: <br>
 * 		- number of seconds to sleep. <br>
 * 		- mandatory if {@value #DO_WAIT} is "true" or "yes".  
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class TestWorker extends Job {
	/** 
	 * Task property name for a property indicating that getting a checkpoint
	 * value should be done.
	 */
	public static final String DO_CHECKPOINT_VALUE = "do.checkpoint.value";
	/** 
	 * Task property name for a property indicating that setting a checkpoint
	 * value should be done.
	 */
	public static final String DO_CHECKPOINT_SET = "do.checkpoint.set";
	/** 
	 * Task property name for a property indicating that a blocking wait for 
	 * a checkpoint value should be done.
	 */
	public static final String DO_CHECKPOINT_BLOCK = "do.checkpoint.block";
	/**
	 * Name of the checkpoint that should be set/queried.
	 */
	public static final String CHECKPOINT_NAME = "checkpoint.name";
	/**
	 * Task ID of the task that set a checkpoint that will be queried.
	 */
	public static final String CHECKPOINT_TASK = "checkpoint.task";
	/** 
	 * Task property name for a property indicating that getting a task property
	 * value should be done.
	 */
	public static final String DO_PROPERTY_VALUE = "do.property.value";
	/**
	 * Name of the task property which value should be gotten.
	 */
	public static final String PROPERTY_NAME = "property.name";
	/** 
	 * Task property name for a property indicating that sleeping should be done.
	 */
	public static final String DO_WAIT = "do.wait";
	/**
	 * Name of the task property for the sleep time (in seconds).
	 */
	public static final String WAIT_TIME = "wait.time";
	
	/**
	 * 
	 * Allocates a new <code>TestWorker</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public TestWorker() throws TaskInitializationException {
		super();
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#checkRequiredProperties()
	 */
	@Override
	protected void checkRequiredProperties() throws TaskException {
	}


	/**
	 * Gets the value of a checkpoint.
	 * 
	 * @throws TaskException
	 */
	private void doCheckpointValue() throws TaskException {
		if (!getBooleanTaskProperty(DO_CHECKPOINT_VALUE)) {
			return;
		}
		logInfo("Getting checkpoint value");

		String name = getTaskProperty(CHECKPOINT_NAME);
		if (name == null) {
			throw new TaskException(CHECKPOINT_NAME + " task property not set");
		}
		logInfo("Checkpoint name: " + name);
		
		String taskID = getTaskProperty(CHECKPOINT_TASK);
		if (taskID == null) {
			throw new TaskException(CHECKPOINT_TASK + " task property not set");
		}
		logInfo("Task ID: " + taskID);
		
		String[] value = (String[]) checkPointWait(null, taskID, name, 0);
		logInfo("Checkpoint value: [" + value[0] + ", " + value[1] + "]");
	}

	/**
	 * Sets the value of a checkpoint.
	 * 
	 * @throws TaskException
	 */
	private void doCheckpointSet() throws TaskException {
		if (!getBooleanTaskProperty(DO_CHECKPOINT_SET)) {
			return;
		}
		logInfo("Setting a checkpoint");
		
		String name = getTaskProperty(CHECKPOINT_NAME);
		if (name == null) {
			throw new TaskException(CHECKPOINT_NAME + " task property not set");
		}
		logInfo("Checkpoint name: " + name);
		
		String[] value = {"BEEN", "DSRG"};
		checkPointReached(name, value);
		logInfo("Set checkpoint with value: [" + value[0] + ", " + value[1] + "]");
	}

	/**
	 * Does a blocking wait for a checkpoint.
	 * 
	 * @throws TaskException
	 */
	private void doCheckpointBlock() throws TaskException {
		if (!getBooleanTaskProperty(DO_CHECKPOINT_BLOCK)) {
			return;
		}
		
		logInfo("Blocking on waiting for checkpoint value");

		String name = getTaskProperty(CHECKPOINT_NAME);
		if (name == null) {
			throw new TaskException(CHECKPOINT_NAME + " task property not set");
		}
		logInfo("Checkpoint name: " + name);
		
		String taskID = getTaskProperty(CHECKPOINT_TASK);
		if (taskID == null) {
			throw new TaskException(CHECKPOINT_TASK + " task property not set");
		}
		logInfo("Task ID: " + taskID);

		String[] value = (String[]) checkPointWait(null, taskID, name, 
			TaskManagerInterface.INFINITE_TIME);
		logInfo("Checkpoint value: [" + value[0] + ", " + value[1] + "]");
	}
	
	/**
	 * Gets the value of a task property.
	 * 
	 * @throws TaskException
	 */
	private void doPropertyValue() throws TaskException {
		if (!getBooleanTaskProperty(DO_PROPERTY_VALUE)) {
			return;
		}
		logInfo("Getting task property object value");
		
		String name = getTaskProperty(PROPERTY_NAME);
		if (name == null) {
			throw new TaskException(PROPERTY_NAME + " task property not set");
		}
		logInfo("Task property name: " + name);

		
		String[] value = (String[]) getTaskPropertyObject(name);
		logInfo("Property value: [" + value[0] + ", " + value[1] + "]");
	}

	/**
	 * Sleeps for some time.
	 * 
	 * @throws TaskException
	 */
	private void doWait() throws TaskException {
		if (!getBooleanTaskProperty(DO_WAIT)) {
			return;
		}
		
		String time = getTaskProperty(WAIT_TIME);
		if (time == null) {
			throw new TaskException(WAIT_TIME + " task property not set");
		}
		logInfo("Waiting for " + time + " seconds");
		
		try {
			Thread.sleep(Long.valueOf(time) * 1000);
		}
		catch (InterruptedException e) {
			logWarning("Sleep interrupted");
			Thread.currentThread().interrupt();
		}
		
		logInfo("Woke up");
	}

	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		doCheckpointSet();
		doCheckpointValue();
		doCheckpointBlock();
		doPropertyValue();
		doWait();
	}
}
