/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.upload.localupload;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.rmi.RemoteException;

import cz.cuni.mff.been.common.UploadHandle;
import cz.cuni.mff.been.common.UploadStatus;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Copy;
import cz.cuni.mff.been.common.anttasks.Zip;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryInterface;
import cz.cuni.mff.been.softwarerepository.SoftwareRepositoryService;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.Service;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Creates a BEEN package from a directory, and uploads it to the Software
 * Repository.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #DIRECTORY}: <br>
 * 		- path to the directory whose contents will be packed in a BEEN
 * 		package <br>
 * 		- mandatory <br>
 * <li>{@value #METADATA_FILE}: <br>
 * 		- path to the metadata file describing the BEEN package <br>
 * 		- mandatory <br>
 * <li>{@value #USE_CONFIG}: <br>
 * 		- if set to "true" or "yes", a "config.xml" file describing a BEEN task
 * 		will be a part of the BEEN package. The path to the "config.xml" file
 * 		must be set in the {@value #CONFIG_FILE} task property. <br>
 * <li>{@value #CONFIG_FILE}: <br>
 * 		- path to the "config.xml" file for a task package <br>
 * 		- mandatory if {@value #USE_CONFIG} is "true" or "yes" <br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class LocalUpload extends Job {
	/** Task property name for the directory that will be packaged **/
	public static final String DIRECTORY = "dir";
	/** 
	 * Task property name for the location of the metadata.xml file that will be
	 * put into the package. Don't set this parameter
	 * if you want to get the location from some task's
	 * checkpoint.
	 */
	public static final String METADATA_FILE = "metadata.file";
	/**
	 * Task property name, set to "yes" of you want to package config.xml
	 * in the package
	 */
	public static final String USE_CONFIG = "use.config";
	/**
	 * Task property name for the location of the config.xml file that will be
	 * put into the package. Don't set this parameter
	 * if you want to get the location from some task's
	 * checkpoint.
	 */
	public static final String CONFIG_FILE = "config.file";
	
	
	/** Name of the temporary directory with the package contents */
	private static final String PKG_DIR = "package";
	/** Name of the temporary file which will be the uploaded package */
	private static final String PKG_FILE = "package.bpk";
	/** Time in milliseconds given to the upload thread to initialize */
	private static final long UPLOAD_THREAD_INIT_TIME = 3000;
	/** Interval in milliseconds for checking if the upload is finished */
	private static final long UPLOAD_WAIT_TIME = 5000;
	/** 
	 * Time in milliseconds given to the upload thread to finish 
	 * after the package was succesfully uploaded 
	 */
	private static final long UPLOAD_THREAD_FINISH_TIME = 20000;
	
	
	
	/** directory to package **/
	private String packageDirectory;
	/** location of the metadata file **/
	private String metadataPath;
	/** true when config.xml should be put into the package **/
	private boolean useConfig = false;
	/** location of the config.xml file **/
	private String configPath;
	/** reference to the Software Repository **/
	private SoftwareRepositoryInterface swRepo;
	
	
	/**
	 * Allocates a new <code>LocalUpload</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public LocalUpload() throws TaskInitializationException {
		super();
	}
	
	private void init() throws TaskException {
		/*
		 * get the path to the packaged directory
		 */
		packageDirectory = getTaskProperty(DIRECTORY);
		logInfo("Packaged directory: " + packageDirectory);

		// test the packaged directory
		File origDir = new File(packageDirectory);
		if (!origDir.exists()) {
			throw new TaskException("Directory not found " + packageDirectory);
		}
		if (!origDir.isDirectory()) {
			throw new TaskException("Not a directory: " + packageDirectory);
		}

		/*
		 * get the path to the metadata.xml file
		 */
		metadataPath = getTaskProperty(METADATA_FILE);
		logInfo("Metadata file: " + metadataPath);

		// test the metadata file
		File metaFile = new File(metadataPath);
		if (!metaFile.exists()) {
			throw new TaskException(metadataPath + "does not exist");
		}
		if (!metaFile.isFile()) {
			throw new TaskException("Not a file: " + metadataPath);
		}

		
		/*
		 * find out whether the config.xml file should be packaged
		 */
		String useCfg = null;
		if ((useCfg = getTaskProperty(USE_CONFIG)) != null) {
			if (useCfg.equals("yes")) {
				useConfig = true;
				logInfo("Will package a config.xml file");
			} else {
				logInfo("Will not package a config.xml file");
			}
		}
		
		/*
		 * get the path to the config.xml file
		 */
		if (useConfig) {
			// config.xml must be packaged, get the path to it
			
			configPath = getTaskProperty(CONFIG_FILE);
			if (configPath == null) {
				throw new TaskException(CONFIG_FILE + "property not set");
			}
			logInfo("Config file: " + configPath);

			// test the config file
			File config = new File(configPath);
			if (!config.exists()) {
				throw new TaskException(configPath + " does not exist");
			}
			if (!config.isFile()) {
				throw new TaskException("Not a file: " + configPath);
			}
			if (!config.canRead()) {
				throw new TaskException("Cannot read: " + configPath);
			}

		}
	}

	@Override
	protected void run() throws TaskException {
		init();
		logInfo("Creating package");
		String packagePath = createPackage();
		logInfo("Package succesfully created");
		logInfo("Uploading package");
		uploadPackage(packagePath);
		logInfo("Package succesfully uploaded");
		checkPointReached("finish", null);
	}

	/**
	 * Creates the package file
	 *
	 */
	private String createPackage() throws TaskException {
		String fs = File.separator;
		
		// create a directory with the package's contents
		String pkgDirPath = getTempDirectory() + fs + PKG_DIR;
		File pkgDir = new File(pkgDirPath);
		if (!pkgDir.mkdir()) {
			throw new TaskException("Cannot create temporary package directory: " + pkgDirPath);
		}
		
		// must create "files" subdirectroy because packages have their files
		// in a "files" subdirectory
		String pkgFilesDirPath = pkgDirPath + fs + "files";
		File pkgFilesDir = new File(pkgFilesDirPath);
		if (!pkgFilesDir.mkdir()) {
			throw new TaskException("Cannot create package files subdirectory: " + pkgFilesDirPath);
		}


		/*
		 * Copy the directory which we want to package to a temporary directory
		 * that contains the package content's
		 */
		try {

			Copy.directoryCopy(packageDirectory, pkgFilesDirPath);
		} catch (AntTaskException e) {
			throw new TaskException("Cannot copy directory " + packageDirectory + " to directory "
					+ pkgFilesDirPath, e);
		}


		/*
		 * copy package metadata
		 */ 

		try {
			Copy.copy(metadataPath, pkgDirPath + File.separator + "metadata.xml");
		} catch (AntTaskException e) {
			throw new TaskException("Cannot copy file " + metadataPath + " to directory "
					+ pkgDirPath, e);
		}
		
		/*
		 * copy config.xml
		 */
		if (useConfig) {
			try {
				Copy.copy(configPath, pkgDirPath + File.separator + "config.xml");
			} catch (AntTaskException e) {
				throw new TaskException("Cannot copy file " + configPath + " to directory "
						+ pkgDirPath, e);
			}
		}
		/*
		 * create the package
		 */
		String packagePath = getTempDirectory() + fs + PKG_FILE;
		try {
			Zip.zipDirectory(pkgDirPath, packagePath);
		} catch (AntTaskException e) {
			throw new TaskException("Cannot zip directory " + pkgDirPath, e);
		}
		
		return packagePath;
	}
	

	private void uploadPackage(String path) throws TaskException {
		swRepo = null;
		try {
			//swRepo = (SoftwareRepositoryInterface)Naming.lookup(getTasksPort().getLookupService().lookup(BeenRMIService.SOFTWARE_REPOSITORY));
			//swRepoURL= "rmi://localhost:1099" + SoftwareRepositoryInterface.URL;
			swRepo = (SoftwareRepositoryInterface) getTasksPort().serviceFind(SoftwareRepositoryService.SERVICE_NAME,  Service.RMI_MAIN_IFACE);
		} catch (Exception e) {
			throw new TaskException("Cannot lookup Software Repository", e);
		}
		
		// setup socket
		ServerSocketChannel serverSocketChan = null;
		ServerSocket serverSocket = null;
		try {
			serverSocketChan = ServerSocketChannel.open();
			serverSocket = serverSocketChan.socket();
			serverSocket.bind(null);
		} catch (IOException e) {
			throw new TaskException("Cannot open socket", e);
		}
		
		// setup input file
		File inputFile = new File(path);
		FileChannel fileChannel = null;
		try {
			 fileChannel = new FileInputStream(inputFile).getChannel();
		} catch (FileNotFoundException e) {
			throw new TaskException("File not found: " + PKG_FILE, e);
		}
		
		UploadThread uploadThread = new UploadThread(serverSocketChan, fileChannel);
		uploadThread.start();
		// give the upload thread some time to initialize
		try {
			Thread.sleep(UPLOAD_THREAD_INIT_TIME);
		} catch (InterruptedException e) {
			// nothing bad happened
			Thread.currentThread().interrupt();
		}
		
		/*
		 * start the upload
		 */
		
		try {
			UploadHandle handle = swRepo.beginPackageUpload(InetAddress.getLocalHost(), serverSocket.getLocalPort());
			
			UploadStatus status = UploadStatus.ERROR;
			while (true) {
				status = swRepo.getPackageUploadStatus(handle);
				if ((status == UploadStatus.INITIALIZING) || (status == UploadStatus.UPLOADING)) {
					// still uploading
					try {
						Thread.sleep(UPLOAD_WAIT_TIME);
					} catch (InterruptedException e) {
						// nothing bad happened
						Thread.currentThread().interrupt();
					}
				} else {
					// upload finished
					break;
				}
			}
			
			if (status == UploadStatus.REJECTED) {
				// package was rejected by the Software Repository
				if (uploadThread.isAlive()) {
					uploadThread.interrupt();
				}
				endUpload(handle, serverSocketChan);
				throw new TaskException("Package was rejected by the Software Repository");
			}
			
			if (status == UploadStatus.ERROR) {
				// error occured in the Software Repository
				if (uploadThread.isAlive()) {
					uploadThread.interrupt();
				}
				endUpload(handle, serverSocketChan);
				throw new TaskException("Error occured in the Software Repository");
			}

			if (status == UploadStatus.ACCEPTED) {
				// package was succesfuly uploaded to the Software Repository
				if (uploadThread.isAlive()) {
					for (int i = 0; i < UPLOAD_THREAD_FINISH_TIME / 1000; i++) {
						boolean threadFinished = false;
						if (uploadThread.isAlive()) {
							try {
								Thread.sleep(1000);
							} catch (InterruptedException e) {
								// nothing bad happened
								Thread.currentThread().interrupt();
							}
						} else {
							uploadThread.interrupt();
							threadFinished = true;
							break;
						}
						
						if (!threadFinished) {
							logWarning("Upload thread not finished");
						}
					}
				}
				
				endUpload(handle, serverSocketChan);
			}
		} catch (java.net.UnknownHostException e) {
			throw new TaskException("IP adress of this host not found: ", e);
		} catch (RemoteException e) {
			throw new TaskException("Cannot connect to the Software Repository: ", e);
		}
	}
	
	/**
	 * Cleans up after uploading 
	 * @param serverSocketChan channel used to communicate with Software Repository
	 */
	private void endUpload(UploadHandle handle, ServerSocketChannel serverSocketChan) {
		try {
			swRepo.endPackageUpload(handle);
		} catch (RemoteException e) {
			e.printStackTrace();
			logError("The Software Repository cannot end the package upload");
		}
		swRepo = null;
		try {
			serverSocketChan.close();
		} catch (IOException e) {
			e.printStackTrace();
			logError("Cannot close socket");
		}
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{DIRECTORY, METADATA_FILE});
	}
}
