/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Jaroslav Urban
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.upload.metadata;

import java.io.File;
import java.util.ArrayList;
import java.util.StringTokenizer;

import cz.cuni.mff.been.common.Version;
import cz.cuni.mff.been.softwarerepository.PackageMetadata;
import cz.cuni.mff.been.softwarerepository.PackageType;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

/**
 * <p>Creates metadata file for a BEEN package in its working directory.</p>
 * 
 * <b>Task properties:</b><br>
 * <ul>
 * <li>{@value #PACKAGE_NAME}: <br>
 * 		- name of the package<br>
 * <li>{@value #PACKAGE_HUMAN_NAME}: <br>
 * 		- human readable, or a more descriptive package name<br>
 * <li>{@value #VERSION}: <br>
 * 		- version of the package<br>
 * <li>{@value #HARDWARE_PLATFORMS}: <br>
 * 		- hardware platforms supported by the package, separated by spaces<br>
 * <li>{@value #SOFTWARE_PLATFORMS}: <br>
 * 		- software platforms supported by the package, separated by spaces<br>
 * <li>{@value #TYPE}: <br>
 * 		- type of the package, valid values are {@value #TYPE_BINARY},
 * 		 {@value #TYPE_SOURCE}, {@value #TYPE_DATA} and {@value #TYPE_TASK}<br>
 * </ul>
 * 
 * @author Jaroslav Urban
 */
public class CreatePackageMetadata extends Job {
	/** Property name for the name of the package */
	public static final String PACKAGE_NAME = "package-name";
	/** Property name for the human name of the package */
	public static final String PACKAGE_HUMAN_NAME = "package-human-name";
	/** Property name for the version of the package */
	public static final String VERSION = "version";
	/** Property name for the hardware platforms of the package */
	public static final String HARDWARE_PLATFORMS = "hardware-platforms";
	/** Property name for the software platforms of the package */
	public static final String SOFTWARE_PLATFORMS = "software-platforms";
	/** Property name for the binary identifier of the package */
	public static final String BINARY_IDENTIFIER = "binary-identifier";
	/** Property name for the type of the package */
	public static final String TYPE = "type";
	/** Property value of the {@value #TYPE} property for a binary package */
	public static final String TYPE_BINARY = "binary";
	/** Property value of the {@value #TYPE} property for a source package */
	public static final String TYPE_SOURCE = "source";
	/** Property value of the {@value #TYPE} property for a task package */
	public static final String TYPE_TASK = "task";
	/** Property value of the {@value #TYPE} property for a data package */
	public static final String TYPE_DATA = "data";
	
		
	private String packageName;
	private String packageHumanName;
	private Version version;
	private ArrayList<String> hardwarePlatforms;
	private ArrayList<String> softwarePlatforms;
	private PackageType type;
	private String binaryIdentifier;
	
	/**
	 * Allocates a new <code>CreatePackageMetadata</code> object.
	 *
	 * @throws TaskInitializationException
	 */
	public CreatePackageMetadata() throws TaskInitializationException {
		super();
	}

	private void init() throws TaskException {
		packageName = getTaskProperty(PACKAGE_NAME);
		if (packageName != null) {
			logInfo("Package name: " + packageName);
		}
		packageHumanName = getTaskProperty(PACKAGE_HUMAN_NAME);
		if (packageHumanName != null) {
			logInfo("Human readable package name: " + packageHumanName);
		}
		String versionString = null;
		if ((versionString = getTaskProperty(VERSION)) != null) {
			version = new Version(versionString);
			logInfo("Version: " + versionString);
		}
		String typeString = getTaskProperty(TYPE);
		if (typeString != null) {
			logInfo("Type: " + typeString);
			type = PackageType.realValueOf(typeString);
		}
		
		String hwPlatforms = getTaskProperty(HARDWARE_PLATFORMS);
		if (hwPlatforms != null) {
			logInfo("Hardware platforms: " + hwPlatforms);
			hardwarePlatforms = new ArrayList<String>();
			StringTokenizer tokenizer = 
				new StringTokenizer(hwPlatforms, " ");
			while (tokenizer.hasMoreTokens()) {
				hardwarePlatforms.add(tokenizer.nextToken());
			}
		}

		String swPlatforms = getTaskProperty(SOFTWARE_PLATFORMS);
		if (swPlatforms != null) {
			logInfo("Software platforms: " + swPlatforms);
			softwarePlatforms = new ArrayList<String>();
			StringTokenizer tokenizer = 
				new StringTokenizer(swPlatforms, " ");
			while (tokenizer.hasMoreTokens()) {
				softwarePlatforms.add(tokenizer.nextToken());
			}
		}
		
		String binaryId = getTaskProperty(BINARY_IDENTIFIER);
		if (binaryId != null) {
			logInfo("Binary identifier:" + binaryId);
			binaryIdentifier = binaryId;
		}
	}
	
	/* (non-Javadoc)
	 * @see cz.cuni.mff.been.task.Job#run()
	 */
	@Override
	protected void run() throws TaskException {
		init();
		
		PackageMetadata metadata = new PackageMetadata(
				null,
				0,
				packageName,
				version,
				hardwarePlatforms,
				softwarePlatforms,
				type,
				packageHumanName,
				null,
				null,
				null,
				binaryIdentifier,
				null);
		
		String path = getWorkingDirectory() + File.separator + "metadata.xml";
		try {
			metadata.saveToFile(path);
		} catch (Exception e) {
			throw new TaskException("Cannot save metadata to file: " + path, e);
		}
		
		checkPointReached("finish", "1");
	}
	
	@Override
	protected void checkRequiredProperties() throws TaskException {
		// all properties are optional
	}
}
