/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Michal Tomcanyi
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.task.xampler.compile.linux;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;

import cz.cuni.mff.been.common.BeenUtils;
import cz.cuni.mff.been.common.OutputReader;
import cz.cuni.mff.been.common.OutputType;
import cz.cuni.mff.been.common.StringUtils;
import cz.cuni.mff.been.common.Version;
import cz.cuni.mff.been.common.anttasks.AntTaskException;
import cz.cuni.mff.been.common.anttasks.Move;
import cz.cuni.mff.been.common.anttasks.Replace;
import cz.cuni.mff.been.common.anttasks.Untar;
import cz.cuni.mff.been.task.Job;
import cz.cuni.mff.been.task.TaskException;
import cz.cuni.mff.been.task.TaskInitializationException;

public class XamplerLinuxCompile extends Job {

	// directory where Xampler archive can be found
	private static final String PROPERTY_XAMPLER_DIR = "xampler.dir";
	// directory under which omniorb libraries can be found
	private static final String PROPERTY_OMNIORB_ROOT  = "omniorb.root";
	// version of omniorb we compile with
	private static final String PROPERTY_OMNIORB_VERSION = "omniorb.version";
	
	private static final String[][] TEST_CONSTANTS = new String[][] {
		new String[]{"SAM_TestSamples", "5"},
		new String[]{"SAM_MaxItems", "100000"},
		new String[]{"SAM_MaxOctets", "100000"},
		new String[]{"SAM_MaxThreads", "100"},
		new String[]{"SAM_MaxInstances", "100000"},
		new String[]{"MEA_WarmUpTimeMin", "5"},
		new String[]{"MEA_WarmUpTimeMax", "20"},
		new String[]{"MEA_RawCycles", "10000"},
		new String[]{"MEA_SoloCycles", "5000"},
		new String[]{"MEA_UsageCycles", "60"},
		new String[]{"MEA_SharedTime", "60"}
};
	
	private ProcessBuilder procBuilder = new ProcessBuilder();
	private File xamplerRoot;
	private File omniorbRoot;
	private String versionSuffix;
	
	public XamplerLinuxCompile() throws TaskInitializationException {
		super();
	}
	
	
	@Override
	protected void run() throws TaskException {
		try {
			omniorbRoot = findOmniorbRoot(getTaskProperty(PROPERTY_OMNIORB_ROOT));
			versionSuffix = determineVersionSuffix();
			dearchive();
			xamplerRoot = findXamplerRoot(getTempDirectory());
			writeBenchmarkConstants();
			setNamingMethod();
			make();
			install();
		} catch (Exception e) {
			e.printStackTrace();
			throw new TaskException("Build failed", e);
		}
	}
	
	protected void checkRequiredProperties() throws TaskException {
		checkRequiredProperties(new String[]{
			PROPERTY_XAMPLER_DIR, 
			PROPERTY_OMNIORB_ROOT,
			PROPERTY_OMNIORB_VERSION
		});
		
		File sourceDir = new File(getTaskProperty(PROPERTY_XAMPLER_DIR));
		if (!sourceDir.exists() || sourceDir.list().length == 0) {
			throw new TaskException("Source file does not exist in "
				+ sourceDir.getAbsolutePath());
		}
		
	}

	private String determineVersionSuffix() {
		String omniVersion = getTaskProperty(PROPERTY_OMNIORB_VERSION);
		String result = new Version(omniVersion).compareTo(new Version("4.1")) < 0 ? "4.0.X" : "4.1.X";
		logInfo("omniORB version detected: " + result);
		return result;
	}
	
	private void dearchive() throws AntTaskException, TaskException {
		logInfo("Extracting from archive");
		File archiveLocation = new File(getTaskProperty(PROPERTY_XAMPLER_DIR));
		File archiveFile = null;
		for (File f : archiveLocation.listFiles()) {
			if (f.isFile() && f.getName().endsWith(".tar.bz2")) {
				archiveFile = f;
				break;
			}
		}
		if (archiveFile == null) {
			throw new TaskException("Can't find xampler archive in " + archiveLocation.getPath());
		}
		Untar.untar(archiveFile.getAbsolutePath(), getTempDirectory(), "bzip2");
	}

	private void writeBenchmarkConstants() throws IOException {
		File target = new File(xamplerRoot, "C++/all_defs.h");
		// write defaults
		BufferedWriter writer = new BufferedWriter(new FileWriter(target));
		writer.write(
				"#define SAM_GlobalVersionMajor  1\n"
				+ "#define SAM_GlobalVersionMinor  10\n" 
				+ "#define SAM_GlobalVersion       \"post 1.10\"\n"
				+ "#define SAM_GlobalDate          \"post 04/02/02\"\n"
				+ "#define SAM_BenchmarkName       \"Xampler\"\n"
		);
		
		for (String[] item : TEST_CONSTANTS) {
			String value = getTaskProperty(item[0]);
			if (StringUtils.isEmpty(value)) {
				value = item[1];
			}
			writer.write("#define " + item[0] + " " + value + "\n");
		}
		
		writer.flush();
		writer.close();
		
	}
	
	private void setNamingMethod() throws AntTaskException {
		File target = new File(xamplerRoot,
				"C++/_Broker/omniORB/" + versionSuffix + "_Linux_GCC/make_ini.inc");
		logInfo("Setting naming: " + target.getAbsolutePath());
		Replace.replace(target.getAbsolutePath(),
				"LOC_BASE := loc_naming_service",
				"LOC_BASE := loc_file");
	}
	
	private void make() throws TaskException {
		logInfo("Running compilation");
		String pythonLibs = new File(omniorbRoot,
				"lib/python2.4/site-packages").getAbsolutePath();
		
		// we will run 'make raw' for each suite
		procBuilder.command(new String[]{"make", "raw"});
		Map<String, String> env = procBuilder.environment();
		
		// add omniorb dir and python libs to make environment
		String pythonPath = env.containsKey("PYTHONPATH")
							? env.get("PYTHONPATH") + ":" + pythonLibs
							: pythonLibs; 
		
		env.put("OMNIORB_DIR", omniorbRoot.getAbsolutePath());
		env.put("PYTHONPATH", pythonPath);
		
		File suiteRoot = new File(xamplerRoot, "C++/_Suites");
		if (!suiteRoot.exists() || ! suiteRoot.isDirectory()) {
			throw new TaskException("Invalid suites directory: " + suiteRoot.getPath());
		}
		
		for (File suite : suiteRoot.listFiles()) {
			// skip non-directory entries
			if (!suite.isDirectory()) {
				continue;
			}
			
			
			try {
				logInfo("Building '" + suite.getName() + "'");
				suite = new File(suite, "omniORB_" + versionSuffix + "_Linux_GCC");
				makeSuite(suite);
			} catch (IOException e) {
				logError("Couldn't start 'make' for suite: '" + suite + "'");
			} catch (InterruptedException e) {
				throw new TaskException("Interrupted");
			}
		}
		
		logInfo("Build finished");
	}
	
	private int makeSuite(File suiteDirectory) throws IOException, InterruptedException {
		procBuilder.directory(suiteDirectory);
		Process p = procBuilder.start();
		
		OutputReader stdOutReader = new OutputReader(p.getInputStream(), OutputType.STDOUT);
		stdOutReader.setOutputStream(System.out);
		stdOutReader.start();
		OutputReader stdErrReader = new OutputReader(p.getErrorStream(), OutputType.STDERR);
		stdErrReader.setOutputStream(System.err);
		stdErrReader.start();
	
		int result = -1;
		try {
			result = p.waitFor();
			if (result != 0) {
				logWarning("Build finished with non-null return value");
			} else {
				logDebug("Suite built OK");
			}
		} catch (InterruptedException e) {
			p.destroy();
			throw e;
		} finally {
			// stop reader threads in any case
			if (stdOutReader.isAlive()) {
				stdOutReader.interrupt();
			}
			if (stdErrReader.isAlive()) {
				stdErrReader.interrupt();
			}
		}
		
		return result;
	}

	private void install() throws TaskException {
		try {
			Move.filesetMove(getTempDirectory(), "**/Server", getWorkingDirectory());
			Move.filesetMove(getTempDirectory(), "**/Client", getWorkingDirectory());
		} catch (AntTaskException e) {
			logFatal("Unable to install binaries");
			throw new TaskException("Unable to install binaries", e);
		}
	}
	
	/*
	 * Searches for omniorb root directory under 'rootDir'
	 * according to presence of 'configure' script
	 */
	private static File findOmniorbRoot(String rootDir) throws TaskException {
		File root = BeenUtils.findRootDir(rootDir,"lib");
		if (root == null) {
			throw new TaskException("Can't find omniOrb root directory!");
		}
		
		return root;
	}
	
	/*
	 * Searches for xampler root directory according to presence of 'C++'
	 * directory
	 */
	private static File findXamplerRoot(String rootDir) throws TaskException {
		File root = BeenUtils.findRootDir(rootDir,"C++");
		if (root == null) {
			throw new TaskException("Can't find Xampler root directory");
		}
		
		return root;
	}
	
}
