/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Antonin Tomecek
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.taskmanager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import cz.cuni.mff.been.common.rsl.Condition;
import cz.cuni.mff.been.common.rsl.ParseException;
import cz.cuni.mff.been.common.rsl.ParserWrapper;
import cz.cuni.mff.been.taskmanager.data.TaskExclusivity;

/**
 * Class for parsing XML file with TaskDescriptor.
 * 
 * @author Antonin Tomecek
 */
public class TaskDescriptorXmlParser {
	
	private TaskDescriptorXmlParser() {
		// Do nothing... (overwrites default constructor...)
	}
	
	/**
	 * Parse PCDATA from element containing it.
	 * 
	 * @param elementWithPCDATA Element containing CDATA.
	 * @return <code>String</code> representation of CDATA.
	 */
	private static String parsePCDATA(Element elementWithPCDATA) {
		NodeList childNodes = elementWithPCDATA.getChildNodes();
		
		for (int i = 0; i < childNodes.getLength(); i++) {
			Node childNode = childNodes.item(i);
			
			if (childNode.getNodeName().equals("#text")) {
				return childNode.getNodeValue();
			}
		}
		
		return null;  // should never occur
	}
	
	/**
	 * Parse element <code>loadMonitoring</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param loadMonitoringElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementLoadMonitoring(
			Element loadMonitoringElement, TaskDescriptor taskDescriptor) {
		/* Process attribute detailedLoad. */
		String attributeDetailedLoad;
		attributeDetailedLoad
			= loadMonitoringElement.getAttribute("detailedLoad");
		if (!attributeDetailedLoad.equals("")) {
			taskDescriptor.setDetailedLoad(
					(attributeDetailedLoad.equals("yes")) ? true : false);
		}
		
		/* Process attribute detailedLoad. */
		String attributeDetailedLoadInterval;
		attributeDetailedLoadInterval
			= loadMonitoringElement.getAttribute("detailedLoadInterval");
		if (!attributeDetailedLoadInterval.equals("")) {
			taskDescriptor.setDetailedLoadInterval(
					Long.parseLong(attributeDetailedLoad));
		}
		
		/* ... There are no elements... */
	}
	
	/**
	 * Parse element <code>failurePolicy</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param failurePolicyElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementFailurePolicy(Element failurePolicyElement,
			TaskDescriptor taskDescriptor) {
		/* Process attribute restartMax. */
		String attributeRestartMax;
		attributeRestartMax = failurePolicyElement.getAttribute("restartMax");
		
		/* Process attribute timeoutRun. */
		String attributeTimeoutRun;
		attributeTimeoutRun = failurePolicyElement.getAttribute("timeoutRun");
		
		if (attributeRestartMax.equals("") || attributeTimeoutRun.equals("")) {
			System.err.println("Warning: All restartMax and timeoutRun "
					+ "attributes should be set within the "
					+ "\"failurePolicy\" element!");
		} else {
			taskDescriptor.setRestartMax(Integer.parseInt(attributeRestartMax));
			taskDescriptor.setTimeoutRun(Long.parseLong(attributeTimeoutRun));
		}
		
		/* ... There are no elements... */
	}
	
	/**
	 * Parse element <code>dependencyCheckPoint</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param dependencyCheckPointElement DOM <code>Element</code> to be
	 * 	parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementDependencyCheckPoint(
			Element dependencyCheckPointElement,
			TaskDescriptor taskDescriptor) {
		/* Process attribute tid. */
		String attributeTid;
		attributeTid = dependencyCheckPointElement.getAttribute("tid");
		
		/* Process attribute type. */
		String attributeType;
		attributeType = dependencyCheckPointElement.getAttribute("type");
		
		/* Process attribute value. */
		String attributeValue;
		attributeValue = dependencyCheckPointElement.getAttribute("value");
		
		if (attributeTid.equals("")
				|| attributeType.equals("") || attributeValue.equals("")) {
			System.err.println("Warning: All tid, type and value "
					+ "attributes should be set within the "
					+ "\"dependencyCheckPoint\" element!");
		} else {
			taskDescriptor.addDependencyCheckPoint(new Dependency(
					attributeTid, attributeType, attributeValue));
		}
		
		/* ... There are no elements... */
	}
	
	/**
	 * Parse element <code>dependencies</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param dependenciesElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementDependencies(
			Element dependenciesElement,
			TaskDescriptor taskDescriptor) {
		/* ... There are no attributes... */
		
		/* Process element dependencyCheckPoint. */
		NodeList dependencyCheckPointList
			= dependenciesElement.getElementsByTagName("dependencyCheckPoint");
		for (int i = 0; i < dependencyCheckPointList.getLength(); i++) {
			Element dependencyCheckPointElement =
				(Element) dependencyCheckPointList.item(i);
			parseElementDependencyCheckPoint(
					dependencyCheckPointElement, taskDescriptor);
		}
	}
	
	/**
	 * Parse element <code>hostRuntime</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param hostRuntimesElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementHostRuntimes(
			Element hostRuntimesElement,
			TaskDescriptor taskDescriptor) {
		/* Process attribute options. */
		String attributeAsTask;
		attributeAsTask = hostRuntimesElement.getAttribute("asTask");
		if (!attributeAsTask.equals("")) {
			taskDescriptor.setHostRuntimesAsTask(attributeAsTask);
		}
		
		/* Process element name. */
		NodeList nameElementList
			= hostRuntimesElement.getElementsByTagName("name");
		for (int i = 0; i < nameElementList.getLength(); i++) {
			Element nameElement = (Element) nameElementList.item(i);
			String name = parsePCDATA(nameElement);
			taskDescriptor.addHostRuntimesName(name);
		}
		
		/* Process element rsl. */
		NodeList rslElementList
			= hostRuntimesElement.getElementsByTagName("rsl");
		if (rslElementList.getLength() > 0) {
			if (rslElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"rsl\" element within the "
						+ "\"hostRuntimes\" element. Only the first occurence "
						+ "will be used!");
			}
			Element rslElement = (Element) rslElementList.item(0);
			String rsl = parsePCDATA(rslElement);
			Condition rslCondition;
			try {
				rslCondition = ParserWrapper.parseString(rsl);
			} catch (ParseException e) {
				throw new IllegalArgumentException("rsl is not valid", e);
			}
			taskDescriptor.setHostRuntimesRsl(rslCondition);
		}
	}
	
	/**
	 * Parse element <code>package</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param packageElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementPackage(
			Element packageElement,
			TaskDescriptor taskDescriptor) {
		/* ... There are no attributes... */
		
		/* Process element name. */
		NodeList nameElementList
			= packageElement.getElementsByTagName("name");
		if (nameElementList.getLength() > 0) {
			if (nameElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"name\" element within the "
						+ "\"package\" element. Only the first occurence will "
						+ "be used!");
			}
			Element nameElement = (Element) nameElementList.item(0);
			String name = parsePCDATA(nameElement);
			taskDescriptor.setPackageName(name);
		}
		
		/* Process element rsl. */
		NodeList rslElementList
			= packageElement.getElementsByTagName("rsl");
		if (rslElementList.getLength() > 0) {
			if (rslElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"rsl\" element within the "
						+ "\"package\" element. Only the first occurence will "
						+ "be used!");
			}
			Element rslElement = (Element) rslElementList.item(0);
			String rsl = parsePCDATA(rslElement);
			Condition rslCondition;
			try {
				rslCondition = ParserWrapper.parseString(rsl);
			} catch (ParseException e) {
				throw new IllegalArgumentException("rsl is not valid", e);
			}
			taskDescriptor.setPackageRsl(rslCondition);
		}
	}
	
	/**
	 * Parse element <code>taskProperty</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param taskProperty DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementTaskProperty(
			Element taskPropertyElement,
			TaskDescriptor taskDescriptor) {
		/* Process attribute key. */
		String attributeKey;
		attributeKey = taskPropertyElement.getAttribute("key");
		
		/* Process attribute value. */
		String attributeValue;
		attributeValue = taskPropertyElement.getAttribute("value");
		
		/* ... set TaskDescriptor... */
		if (attributeKey.equals("") || attributeValue.equals("")) {
			System.err.println("Warning: Booth key and value attribute"
					+ "should be set within the \"taskProperty\" element!");
		} else {
			taskDescriptor.addTaskProperties(attributeKey, attributeValue);
		}
		
		/* There are no elements... */
	}
	
	/**
	 * Parse element <code>taskProperties</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param taskProperties DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementTaskProperties(
			Element taskPropertiesElement,
			TaskDescriptor taskDescriptor) {
		/* ... There are no attributes... */
		
		/* Process element taskProperty. */
		NodeList taskPropertyElementList
			= taskPropertiesElement.getElementsByTagName("taskProperty");
		if (taskPropertyElementList.getLength() < 1) {
			System.err.println("Error: There must be at least one "
					+ "occurence of the \"taskProperty\" element within the "
					+ "\"taskProperties\" element!");
		} else {
			for (int i = 0; i < taskPropertyElementList.getLength(); i++) {
				Element taskPropertyElement
					= (Element) taskPropertyElementList.item(i);
				parseElementTaskProperty(taskPropertyElement, taskDescriptor);
			}
		}
	}
	
	/**
	 * Parse element <code>JavaOptions</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param javaOptionsElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementJavaOptions(
			Element javaOptionsElement,
			TaskDescriptor taskDescriptor) {
		/* ... There are no attributes... */
		
		/* Process element javaOption. */
		NodeList javaOptionElementList
			= javaOptionsElement.getElementsByTagName("javaOption");
		if (javaOptionElementList.getLength() < 1) {
			System.err.println("Error: There must be at least one "
					+ "occurence of the \"javaOption\" element within the "
					+ "\"javaOptions\" element!");
		} else {
			for (int i = 0; i < javaOptionElementList.getLength(); i++) {
				Element javaOptionElement
					= (Element) javaOptionElementList.item(i);
				String javaOption = parsePCDATA(javaOptionElement);
				taskDescriptor.addJavaOption(javaOption);
			}
		}
	}
	
	/**
	 * Parse element <code>java</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param javaElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementJava(
			Element javaElement,
			TaskDescriptor taskDescriptor) {
		/* Process attribute options. */
		String attributeOptions;
		attributeOptions = javaElement.getAttribute("options");
		if (!attributeOptions.equals("")) {
			taskDescriptor.setJavaOptions(attributeOptions, false);
		}
		
		/* Process element javaOptions. */
		NodeList javaOptionsElementList
			= javaElement.getElementsByTagName("javaOptions");
		if (javaOptionsElementList.getLength() > 0) {
			if (javaOptionsElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"javaOptions\" element within "
						+ "the \"java\" element. Only the first occurence "
						+ "will be used!");
			}
			Element javaOptionsElement
				= (Element) javaOptionsElementList.item(0);
			parseElementJavaOptions(javaOptionsElement, taskDescriptor);
		}
	}
	
	/**
	 * Parse element <code>arguments</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param argumentsElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementArguments(
			Element argumentsElement, TaskDescriptor taskDescriptor) {
		/* ... There are no attributes... */
		
		/* Process element argument. */
		NodeList argumentElementList
			= argumentsElement.getElementsByTagName("argument");
		if (argumentElementList.getLength() < 1) {
			System.err.println("Error: There must be at least one "
					+ "occurence of the \"argument\" element within the "
					+ "\"arguments\" element!");
		} else {
			for (int i = 0; i < argumentElementList.getLength(); i++) {
				Element argumentElement
					= (Element) argumentElementList.item(i);
				String argument = parsePCDATA(argumentElement);
				taskDescriptor.addArguments(argument);
			}
		}
	}
	
	/**
	 * Parse element <code>taskDescriptor</code>.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param taskDescriptorElement DOM <code>Element</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseElementTaskDescriptor(
			Element taskDescriptorElement, TaskDescriptor taskDescriptor) {
		/* Process attribute taskId. */
		String attributeTaskId;
		attributeTaskId = taskDescriptorElement.getAttribute("taskId");
		if (!attributeTaskId.equals("")) {
			taskDescriptor.setTaskId(attributeTaskId);
		}
		
		/* Process attribute contextId. */
		String attributeContextId;
		attributeContextId = taskDescriptorElement.getAttribute("contextId");
		if (!attributeContextId.equals("")) {
			taskDescriptor.setContextId(attributeContextId);
		}
		
		/* Process attribute name. */
		String attributeName;
		attributeName = taskDescriptorElement.getAttribute("name");
		if (!attributeName.equals("")) {
			taskDescriptor.setTaskName(attributeName);
		}
		
		/* Process attribute description. */
		String attributeDescription;
		attributeDescription
			= taskDescriptorElement.getAttribute("description");
		if (!attributeDescription.equals("")) {
			taskDescriptor.setTaskDescription(attributeDescription);
		}
		
		/* Process attribute exclusive. */
		String attributeExclusive;
		attributeExclusive = taskDescriptorElement.getAttribute("exclusive");
		if (!attributeExclusive.equals("")) {
			if (attributeExclusive
					.equals(TaskExclusivity.NON_EXCLUSIVE.toString())) {
				taskDescriptor.setTaskExclusive(TaskExclusivity.NON_EXCLUSIVE);
			} else if (attributeExclusive
					.equals(TaskExclusivity.CONTEXT_EXCLUSIVE.toString())) {
				taskDescriptor
					.setTaskExclusive(TaskExclusivity.CONTEXT_EXCLUSIVE);
			} else if (attributeExclusive
					.equals(TaskExclusivity.EXCLUSIVE.toString())) {
				taskDescriptor.setTaskExclusive(TaskExclusivity.EXCLUSIVE);
			} else {
				System.err.println("Warning: Attribute \"exclusive\" "
						+ "within \"task\" element has unexpected value!");
			}
		}
		
		/* Process element arguments. */
		NodeList argumentsElementList
			= taskDescriptorElement.getElementsByTagName("arguments");
		if (argumentsElementList.getLength() > 0) {
			if (argumentsElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"arguments\" element within the "
						+ "\"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element argumentsElement = (Element) argumentsElementList.item(0);
			parseElementArguments(argumentsElement, taskDescriptor);
		}
		
		/* Process element java. */
		NodeList javaElementList
			= taskDescriptorElement.getElementsByTagName("java");
		if (javaElementList.getLength() > 0) {
			if (javaElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"java\" element within the "
						+ "\"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element javaElement = (Element) javaElementList.item(0);
			parseElementJava(javaElement, taskDescriptor);
		}
		
		/* Process element taskProperties. */
		NodeList taskPropertiesElementList
			= taskDescriptorElement.getElementsByTagName("taskProperties");
		if (taskPropertiesElementList.getLength() > 0) {
			if (taskPropertiesElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"taskProperties\" element within "
						+ "the \"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element taskPropertiesElement
				= (Element) taskPropertiesElementList.item(0);
			parseElementTaskProperties(taskPropertiesElement, taskDescriptor);
		}
		
		/* Process element package. */
		NodeList packageElementList
			= taskDescriptorElement.getElementsByTagName("package");
		if (packageElementList.getLength() < 1) {
			System.err.println("Error: There must be just one occurence "
					+ "of the \"package\" element within the "
					+ "\"taskDescriptor\" element!");
		} else {
			if (packageElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"package\" element within the "
						+ "\"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element packageElement = (Element) packageElementList.item(0);
			parseElementPackage(packageElement, taskDescriptor);
		}
		
		/* Process element hostRuntimes. */
		NodeList hostRuntimesElementList
			= taskDescriptorElement.getElementsByTagName("hostRuntimes");
		if (hostRuntimesElementList.getLength() < 1) {
			System.err.println("Error: There must be just one occurence "
					+ "of the \"hostRuntimes\" element within the "
					+ "\"taskDescriptor\" element!");
		} else {
			if (hostRuntimesElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"hostRuntimes\" element within "
						+ "the \"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element hostRuntimesElement
				= (Element) hostRuntimesElementList.item(0);
			parseElementHostRuntimes(hostRuntimesElement, taskDescriptor);
		}
		
		/* Process element dependencies. */
		NodeList dependenciesElementList
			= taskDescriptorElement.getElementsByTagName("dependencies");
		if (dependenciesElementList.getLength() > 0) {
			if (dependenciesElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"dependencies\" element within "
						+ "the \"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element dependenciesElement
				= (Element) dependenciesElementList.item(0);
			parseElementDependencies(dependenciesElement, taskDescriptor);
		}
		
		/* Process element failurePolicy. */
		NodeList failurePolicyElementList
			= taskDescriptorElement.getElementsByTagName("failurePolicy");
		if (failurePolicyElementList.getLength() > 0) {
			if (failurePolicyElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"failurePolicy\" element within "
						+ "the \"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element failurePolicyElement
				= (Element) failurePolicyElementList.item(0);
			parseElementFailurePolicy(failurePolicyElement, taskDescriptor);
		}
		
		/* Process element loadMonitoring. */
		NodeList loadMonitoringElementList
			= taskDescriptorElement.getElementsByTagName("loadMonitoring");
		if (loadMonitoringElementList.getLength() > 0) {
			if (loadMonitoringElementList.getLength() > 1) {
				System.err.println("Warning: There is more than one "
						+ "occurence of the \"loadMonitoring\" element within "
						+ "the \"taskDescriptor\" element. Only the first "
						+ "occurence will be used!");
			}
			Element loadMonitoringElement
				= (Element) loadMonitoringElementList.item(0);
			parseElementLoadMonitoring(loadMonitoringElement, taskDescriptor);
		}
	}
	
	/**
	 * Parse DOM <code>Document</code> representation of TaskDescriptor and do
	 * appropriate settings of TaskDescriptor.
	 * Causes parsing of all descendant elements of this element.
	 * 
	 * @param document DOM <code>Document</code> to be parsed.
	 * @param taskDescriptor <code>TaskDescriptor</code> to be set.
	 */
	private static void parseDocument(
			Document document, TaskDescriptor taskDescriptor) {
		/* Process element taskDescriptor (root element). */
		Element taskDescriptorElement = document.getDocumentElement();
		if (taskDescriptorElement.getTagName().equals("taskDescriptor")) {
			parseElementTaskDescriptor(taskDescriptorElement, taskDescriptor);
		}
	}
	
	/**
	 * Gets XML file representation of TaskDescriptor and returns object
	 * representation of it.
	 * 
	 * @param xmlFile input stream containing the XML representation of the
	 *         TaskDescriptor
	 * @return Object representation of TaskDescriptor.
	 * @throws TaskDescriptorException If something failed.
	 */
	public static TaskDescriptor parseXmlTaskDescriptor(InputStream xmlFile)
	throws TaskDescriptorException {
		/* Prepare DOM. */
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setValidating(false);
		
		Document document;
		try {
			document = factory.newDocumentBuilder().parse(xmlFile);
		} catch (SAXException e) {
			throw new TaskDescriptorException("Parse error occured.", e);
		} catch (IOException e) {
			throw new TaskDescriptorException(e);
		} catch (ParserConfigurationException e) {
			throw new TaskDescriptorException("DocumentBuilder can not be "
					+ "created", e);
		}
		
		/* Create a new (empty) instance of TaskDescriptor. */
		TaskDescriptor taskDescriptor = new TaskDescriptor();
		
		/* Inicialize instance of TaskDescriptor in agreement with XML file. */
		parseDocument(document, taskDescriptor);
		
		/* Return initialized instance of TaskDescriptor. */
		return taskDescriptor;
	}

	/**
	 * Gets XML file representation of TaskDescriptor and returns object
	 * representation of it.
	 * 
	 * @param xmlFile XML file representation of TaskDescriptor.
	 * @return Object representation of TaskDescriptor.
	 * @throws TaskDescriptorException If something failed.
	 */
	public static TaskDescriptor parseXmlTaskDescriptor(File xmlFile)
	throws TaskDescriptorException {
		try {
			return parseXmlTaskDescriptor(new FileInputStream(xmlFile));
		} catch (FileNotFoundException e) {
			throw new TaskDescriptorException(e);
		}
	}
}
