/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: Antonin Tomecek
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.taskmanager;

import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import cz.cuni.mff.been.logging.LogLevel;

/**
 * Runner class for Task Manager.
 * 
 * @author Antonin Tomecek
 */
public class TaskManagerRunner {

	private TaskManagerRunner() {
		// Do nothing... (overwrites default constructor...)
	}
	
	public static void main(String[] args) {
		/* Check command-line arguments. */
		if (args.length < 1) {
			System.out.println("Usage: java cz.cuni.mff.been."
				+ "taskmanager.TaskManagerRunner "
				+ "<TRACE | DEBUG | INFO | WARN | ERROR | FATAL> "
				+ "<path_to_TaskManager_directory>[ "
				+ "<XML_task_descriptor>[ "
				+ "<XML_task_descriptor>[ ...]]]");
			System.exit(1);
		}
		/* Number of currently (lastly) processed argument. */
		int argNo = 0;
		
		/* Set log level. */
		LogLevel level = LogLevel.valueOf(args[0]);
		System.out.println("Log level: " + level);
		argNo = 1;
		
		try {
			LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
		} catch (RemoteException e) {
			System.err.println("Note: Can't start the RMI registry - "
					+ "another instance is probably running. Usually you can "
					+ "ignore this.");
		}
		
		TaskManagerInterface taskManager;
		try {
			/* Root directory for Task Manager. */
			String rootDirectory = args[argNo++];
			
			/* Create a new Task Manager. */
			taskManager = new TaskManagerImplementation(
					rootDirectory, level);
			
			/* Register main RMI interface of this Task Manager. */
			Naming.rebind("rmi://localhost:1099"
				+ TaskManagerInterface.URL, taskManager);
			
			/* Print message to the standard output stream. */
			System.out.println("Task Manager started...");
		} catch (Exception e) {
			throw new RuntimeException("Unexpected failure!", e);
		}
		
		try {
			/* Array containing paths to TaskDescriptors to run. */
			String[] taskDescriptors = new String[args.length - argNo];

			for (int i = 0; i < taskDescriptors.length; i++) {
				taskDescriptors[i] = args[argNo++];
			}

			/* Run all specified tasks by their task descriptors. */
			taskManager.runTask(taskDescriptors);
		} catch (RemoteException e) {
			System.err.println("Running of tasks specified by their task "
					+ "descriptors failed: " + e.getMessage());
		}
	}
}
