/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.webinterface.benchmarks;

import cz.cuni.mff.been.benchmarkmanager.BeenPluginDescriptor;
import cz.cuni.mff.been.benchmarkmanager.Entity;
import cz.cuni.mff.been.benchmarkmanager.ExperimentEntity;
import cz.cuni.mff.been.benchmarkmanager.gui.Role;
import cz.cuni.mff.been.benchmarkmanager.gui.SchedulerInfo;
import cz.cuni.mff.been.benchmarkmanager.gui.Screen;
import cz.cuni.mff.been.common.id.AID;
import cz.cuni.mff.been.common.id.BRID;
import cz.cuni.mff.been.common.id.EID;
import cz.cuni.mff.been.resultsrepository.gui.CallbackRScripts;
import cz.cuni.mff.been.taskmanager.TaskDescriptor;

/**
 * Contains state information used when navigating through the benchmark run
 * wizard. Instances of this object are saved in the session.  
 * 
 * @author David Majda
 */
public class BenchmarkRunState {
	/**
	 * Prefix used in name of the session attributes, where instances of
	 * <code>BenchmarkRunState</code> class are stored.
	 */
	public static final String SESSION_PREFIX = "benchmark-run-state-";
	
	/**
	 * Enumerates screens of the wizard.
	 * 
	 * @author David Majda
	 */
	public enum WizardScreen {
		SELECT_PLUGIN,
		SELECT_CREATE_EXPERIMENT,
		SCHEDULING_INFO,
		CONFIGURE_PLUGIN,
		SELECT_HOSTS_FOR_ROLES,
		CALLBACK_R_SCRIPTS,
		CONFIRM,
		TASKS
	}
	
	/** Benchmark run identifier. */
	private BRID brid;
	/** Analysis identifier. */
	private AID aid;
	/** Information whether the analysis is regression. */
	private boolean isRegression;
	/** Information about selected plugin. Also contains some configuration options. */
	private BeenPluginDescriptor plugin;
	/** 
	 * Last screen in the wizard whose results were processed successfully, or
	 * <code>null</code> if we are in the beginning of the wizard.
	 */
	private WizardScreen lastWizardScreen;
	/** Currently displayed configurator screen. */
	private Screen currentConfiguratorScreen;
	/** Task descriptors of generated benchmark tasks. */
	private TaskDescriptor[] tasks;
	/** Name of the experiment. */
	private String experimentName;
	/** Description of the experiment. */
	private String experimentDescription;
	/** Information about scheduling of the experiment. */
	private SchedulerInfo schedulerInfo;
	/** Information about assignment of hosts to roles. */
	private Role[] roles;
	/** Callback R scripts. */
	private CallbackRScripts callbackRScripts;
	/** Configured experiment. */
	private ExperimentEntity experiment;
	
	/** @return the benchmark run identifier */
	public BRID getBrid() {
		return brid;
	}

	/** @return the analysis identifier */
	public AID getAid() {
		return aid;
	}

	/**
	 * Sets the analysis identifier.
	 * 
	 * @param aid analysis identifier
	 */
	public void setAid(AID aid) {
		this.aid = aid;
	}

	/** @return the information whether the analysis is regression */
	public boolean getIsRegression() {
		return isRegression;
	}

	/**
	 * Sets the information whether the analysis is regression.
	 * 
	 * @param isRegression information whether the analysis is regression to set
	 */
	public void setIsRegression(boolean isRegression) {
		this.isRegression = isRegression;
	}

	/** @return information about selected plugin */
	public BeenPluginDescriptor getPlugin() {
		return plugin;
	}
	
	/**
	 * Sets the information about selected plugin.
	 * 
	 * @param plugin information about selected plugin to set
	 */
	public void setPlugin(BeenPluginDescriptor plugin) {
		this.plugin = plugin;
	}

	/** @return the currently displayed configurator screen */
	public Screen getCurrentScreen() {
		return currentConfiguratorScreen;
	}
	
	/**
	 * Sets the currently displayed configurator screen.
	 * 
	 * @param currentScreen currently displayed configurator screen to set
	 */
	public void setCurrentScreen(Screen currentScreen) {
		this.currentConfiguratorScreen = currentScreen;
	}

	/**
	 * @return the last screen in the wizard whose results were processed
	 *          successfully, or <code>null</code> if we are in the beginning of
	 *          the wizard.
	 */
	public WizardScreen getLastWizardScreen() {
		return lastWizardScreen;
	}
	
	/**
	 * Sets the last screen in the wizard whose results were processed
	 * successfully.
	 * 
	 * @param lastWizardScreen last screen in the wizard whose results were
	 *         processed successfully to set
	 */
	public void setLastWizardScreen(WizardScreen lastWizardScreen) {
		this.lastWizardScreen = lastWizardScreen;
	}

	/** @return task descriptors of generated benchmark tasks */
	public TaskDescriptor[] getTasks() {
		return tasks;
	}
	
	/**
	 * Sets the task descriptors of generated benchmark tasks.
	 * 
	 * @param tasks task descriptors of generated benchmark tasks
	 */
	public void setTasks(TaskDescriptor[] tasks) {
		this.tasks = tasks;
	}
	
	/** @return name of the experiment */
	public String getExperimentName() {
		return experimentName;
	}

	/**
	 * Sets the name of the experiment.
	 * 
	 * @param experimentName name of the experiment to set
	 */
	public void setExperimentName(String experimentName) {
		this.experimentName = experimentName;
	}

	/** @return the experimentDescription description of the experiment */
	public String getExperimentDescription() {
		return experimentDescription;
	}

	/**
	 * Sets the description of the experiment.
	 * 
	 * @param experimentDescription description of the experiment to set
	 */
	public void setExperimentDescription(String experimentDescription) {
		this.experimentDescription = experimentDescription;
	}

	/** @return the information about scheduling of the experiment */
	public SchedulerInfo getSchedulerInfo() {
		return schedulerInfo;
	}

	/**
	 * Sets the information about scheduling of the experiment
	 * 
	 * @param schedulerInfo information about scheduling of the experiment to set
	 */
	public void setSchedulerInfo(SchedulerInfo schedulerInfo) {
		this.schedulerInfo = schedulerInfo;
	}

	/** @return information about assignment of hosts to roles */
	public Role[] getRoles() {
		return roles;
	}

	/**
	 * Sets the information about assignment of hosts to roles.
	 * 
	 * @param roles information about assignment of hosts to roles to set
	 */
	public void setRoles(Role[] roles) {
		this.roles = roles;
	}

	/** @return callback R scripts */
	public CallbackRScripts getCallbackRScripts() {
		return callbackRScripts;
	}

	/**
	 * Sets the callback R scripts.
	 * 
	 * @param callbackRScripts R scripts to set
	 */
	public void setCallbackRScripts(CallbackRScripts callbackRScripts) {
		this.callbackRScripts = callbackRScripts;
	}

	/** @return callback R scripts */
	public Entity<EID> getExperiment() {
		return experiment;
	}

	/**
	 * Sets the experiment.
	 * 
	 * @param experiment the experiment.
	 */
	public void setExperiment(ExperimentEntity experiment) {
		this.experiment = experiment;
	}

	/**
	 * Allocates a new <code>BenchmarkRunState</code> object.
	 * 
	 * @param brid benchmark run identifier
	 */
	public BenchmarkRunState(BRID brid) {
		super();
		this.brid = brid;
	}
}
