/*
 *  BEEN: Benchmarking Environment
 *  ==============================
 *
 *  File author: David Majda
 *
 *  GNU Lesser General Public License Version 2.1
 *  ---------------------------------------------
 *  Copyright (C) 2004-2006 Distributed Systems Research Group,
 *  Faculty of Mathematics and Physics, Charles University in Prague
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 */
package cz.cuni.mff.been.webinterface.packages;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import cz.cuni.mff.been.common.Version;
import cz.cuni.mff.been.softwarerepository.PackageType;

/**
 * Abstract class representing an operator used in the package query form (and maybe
 * in other forms later). Should be specialized for each operator used. 
 * 
 * Operator can be viewed as a relation between tho objects. It contains
 * abstract method <code>apply</code>, which is executed when the operator 
 * needs to be applied on some operads during the queries. This method shoud tell,
 * whether two objects passed as arguments are in relation represended by this 
 * operator, or not. 
 *  
 * @author David Majda
 */
public abstract class Operator implements Serializable {
	/** String operators. */
	private static final Operator[] STRING_OPERATORS = new Operator[] {
			new Operator("is", "is") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((String) metadataValue).equals((String) formValue);
				}
			},      
			new Operator("is-not", "is not") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return !((String) metadataValue).equals((String) formValue);
				}
			},      
			new Operator("contains", "contains") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((String) metadataValue).indexOf((String) formValue) != -1;
				}
			},      
			new Operator("does-not-contain", "does not contain") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((String) metadataValue).indexOf((String) formValue) == -1;
				}
			},      
	};
    
	/** Date operators. */
	private static final Operator[] DATE_OPERATORS = new Operator[] {
			new Operator("eq", "equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Date) metadataValue).equals((Date) formValue);
				}
			},      
			new Operator("ne", "not equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return !((Date) metadataValue).equals((Date) formValue);
				}
			},      
			new Operator("gt", "greater than") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Date) metadataValue).compareTo((Date) formValue) > 0;
				}
			},      
			new Operator("ge", "greater or equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Date) metadataValue).compareTo((Date) formValue) >= 0;
				}
			},      
			new Operator("lt", "smaller than") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Date) metadataValue).compareTo((Date) formValue) < 0;
				}
			},      
			new Operator("le", "smaller or equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Date) metadataValue).compareTo((Date) formValue) <= 0;
				}
			},      
	};
	
	/** ArrayList operators. */
	private static final Operator[] ARRAY_LIST_OPERATORS = new Operator[] {
			new Operator("contains", "contains") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((ArrayList) metadataValue).contains((String) formValue);
				}
			},      
			new Operator("does-not-contain", "does not contain") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return !((ArrayList) metadataValue).contains((String) formValue);
				}
			},      
	};
	
	/** Version operators. */
	private static final Operator[] VERSION_OPERATORS = new Operator[] {
			new Operator("eq", "equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Version) metadataValue).equals((Version) formValue);
				}
			},      
			new Operator("ne", "not equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return !((Version) metadataValue).equals((Version) formValue);
				}
			},      
			new Operator("gt", "greater than") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Version) metadataValue).compareTo((Version) formValue) > 0;
				}
			},      
			new Operator("ge", "greater or equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Version) metadataValue).compareTo((Version) formValue) >= 0;
				}
			},      
			new Operator("lt", "smaller than") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Version) metadataValue).compareTo((Version) formValue) < 0;
				}
			},      
			new Operator("le", "smaller or equal to") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((Version) metadataValue).compareTo((Version) formValue) <= 0;
				}
			},      
	};
	
	/** PackageType operators. */
	private static final Operator[] PACKAGE_TYPE_OPERATORS = new Operator[] {
			new Operator("is", "is") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return ((PackageType) metadataValue).equals(((PackageType) formValue));
				}
			},      
			new Operator("is-not", "is not") {
				public boolean apply(Object metadataValue, Object formValue) {
					if (metadataValue == null) {
						return false;
					}
					return !((PackageType) metadataValue).equals(((PackageType) formValue));
				}
			},      
	};

	private static HashMap<Class, Operator[]> operatorMap = new HashMap<Class, Operator[]>();
	
	/** Operator name (identification). */
	private String name;
	/** Operator title (intended to be displayed in the UI). */
	private String title;
	
	/** @return returns the name */
	public String getName() {
		return name;
	}
	
	/** @return returns the title */
	public String getTitle() {
		return title;
	}
	
	static {
		operatorMap.put(String.class, STRING_OPERATORS);
		operatorMap.put(Date.class, DATE_OPERATORS);
		operatorMap.put(ArrayList.class, ARRAY_LIST_OPERATORS);
		operatorMap.put(Version.class, VERSION_OPERATORS);
		operatorMap.put(PackageType.class, PACKAGE_TYPE_OPERATORS);
	}
	
	/**
	 * Finds operators for given class name.
	 * 
	 * @param klass class name
	 * @return te operators
	 */
	public static Operator[] forKlass(Class klass) {
		for (Class k: operatorMap.keySet()) {
			if (k.equals(klass)) {
				return operatorMap.get(k);
			}
		}
		return null;
	}
	
	/**
	 * Returns a set of classes for which we have operators.
	 * 
	 * @return set of classes
	 */
	public static Set<Class> getKlasses() {
		return operatorMap.keySet();
	}
	
	/**
	 * Allocates a new <code>Operator</code> object.
	 * 
	 * @param name operator name (identification)
	 * @param title operator title (intended to be displayed in the UI)
	 */
	public Operator(String name, String title) {
		super();
		this.name = name;
		this.title = title;
	}
    
	/**
	 * Function executed during the queries. It tells whether two objects
	 * are in relation according to this oeprator.
	 * 
	 * Note that descendats can constrain allowed argument types.
	 * 
	 * <code>metadataValue</code> parameter can be null, because packages do not
	 * necessairly have all metadata attributes specified. On the contrary,
	 * <code>formvalue</code> can't be null, as there is no way for the web
	 * interface user to specify <code>null</code> value.
	 * 
	 * All operators should return <code>false</code> if the
	 * <code>metadataValue</code> parameter is <code>null</code>, because it is
	 * the only consistent behaviour.
	 * 
	 * @param metadataValue value of the package metadata attribute, can be <code>null</code> 
	 * @param formValue value of the form field, can't be <code>null</code>
	 * @return <code>true</code> if both arguments are in the relation
	 *          represented by this operator;
	 *          <code>false</code> otherwise
	 */
	public abstract boolean apply(Object metadataValue, Object formValue);
}
